<?php

namespace WPEP\Admin\Coordinator\DataSystem;

use WPEP\Container\FormFields as WPEP_FormFieds;
use WPEP\Container\FileSystem as WPEP_FileSystem;
use WPEP\Container\DataWalker as WPEP_DataWalker;

class Activity {

  public function setup() {
    add_action('wp_ajax_wpep_data_system_activity_report', [ $this, 'activity_report' ] );
    add_action('wp_ajax_wpep_data_system_activity_report_users', [ $this, 'activity_report_users' ] );
  }

  public function activity_report() {
    if( !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      exit( '<div class="wpep-notification">' . __( "Your account is not allowed to complete this operation", "wpep" ) . '</div>');

    $this->_output_activity_report( [ wpep_controller()->dataSystem->userActivity, 'get_chart_map_by_time_slots' ] );
  }

  public function activity_report_users() {
    if( !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      exit( '<div class="wpep-notification">' . __( "Your account is not allowed to complete this operation", "wpep" ) . '</div>');

    $this->_output_activity_report( [ wpep_controller()->dataSystem->userActivity, 'get_user_activity_chart_map_by_time_slots' ] );
  }

  private function _output_activity_report( $data_system_report_func_call ) {
    $current_time      = time();

    $today_noon_time          = strtotime( 'today 00:00', $current_time );
    $yesterday_noon_time      = strtotime('-1 day', $today_noon_time );
    $tomorrow_noon_time       = strtotime('+1 day', $today_noon_time );;
    $base_time_weekly         = strtotime('-1 week', $today_noon_time );
    $base_time_monthly        = strtotime('-1 month', $today_noon_time );
    $base_time_yearly         = strtotime('first day of this month -1 year', $today_noon_time );

    $activity_report_map = [
      $this->_build_date_slots_string( $today_noon_time, 3600 * 4, $tomorrow_noon_time )     => __( "Daily", "wpep" ),
      $this->_build_date_slots_string( $yesterday_noon_time, 3600 * 4, $today_noon_time + 3600 * 4 )    => __( "Yesterday", "wpep" ),
      $this->_build_date_slots_string( $base_time_weekly, '+1 day', $tomorrow_noon_time )    => __( "Weekly", "wpep" ),
      $this->_build_date_slots_string( $base_time_monthly, '+1 day', $tomorrow_noon_time )   => __( "Monthly", "wpep" ),
      $this->_build_date_slots_string( $base_time_yearly, '+1 month', $tomorrow_noon_time )  => __( "Yearly", "wpep" )
    ];

    $date_range_active = false;
    $current_activity_report = isset( $_POST['sync_stage'] ) ? $_POST['sync_stage'] : 'default';

    if( $current_activity_report === 'default' ) {
      $current_activity_report = key( $activity_report_map );
    } elseif( $current_activity_report === 'from_date_range' && isset( $_POST['date_range'] ) ) {
      $date_range_from = strtotime( $_POST['date_range']['from'] );
      $date_range_to   = strtotime( $_POST['date_range']['to'] );

      $difference = ( $date_range_to - $date_range_from ) / 12;

      $current_activity_report = $date_range_from;

      for( $i = $date_range_from; $i <= $date_range_to; $i += $difference )
        $current_activity_report .= ',' . $i;

      $date_range_active = true;
    }

    $time_slots = explode( ',', $current_activity_report );

    if( !isset( $date_range_from ) || !isset( $date_range_to ) ) {
      reset( $time_slots );
      $date_range_from = $time_slots[ key( $time_slots ) ];
      end( $time_slots );
      $date_range_to = $time_slots[ key( $time_slots ) ];

      reset( $time_slots );
    }

    $data_system_report = call_user_func( $data_system_report_func_call, $time_slots );

    $activity_report_chart_data = [
      'labels' => [],
      'data'   => []
    ];

    if( is_array( $data_system_report ) ) {
      foreach( $data_system_report as $data_key => $data_value ) {
        $activity_report_chart_data['labels'][ $data_key ] = wpep_format_date_with_time( $data_key );

        foreach( $data_value as $chart_data_label => $chart_data_size ) {
          if( !isset( $activity_report_chart_data['data'][$chart_data_label] ) )
            $activity_report_chart_data['data'][$chart_data_label] = [];

          $activity_report_chart_data['data'][$chart_data_label][] = $chart_data_size;
        }
      }

      foreach( $activity_report_chart_data['data'] as $label => $data ) {
        $array_sum = array_sum( $data );

        if( $array_sum == 0 )
          unset( $activity_report_chart_data['data'][$label] );
      }
    }

    exit( wpep_template_load( 'admin/statistics/data-system-activity-report.php', [
      'activity_report_chart_data' => $activity_report_chart_data,
      'activity_report_map'        => $activity_report_map,
      'current_report'             => $current_activity_report,
      'yesterday_noon_time'        => $yesterday_noon_time,
      'tomorrow_noon_time'         => $tomorrow_noon_time,
      'date_range_from'            => $date_range_from,
      'date_range_to'              => ( isset( $date_range_to ) ? $date_range_to : '' ),
      'date_range_active'          => $date_range_active
    ] ) );
  }

  private function _build_date_slots_string( $start_time, $increment, $time_limit ) {
    $response = $start_time;

    $last_time = $start_time;

    while( $last_time < $time_limit ) {
      $last_time = ( is_numeric( $increment ) ? $last_time + $increment : strtotime( $increment, $last_time ) );

      $response .= ',' . $last_time;
    }

    return $response;
  }

}
