<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Entity\Discussion as WPEP_Entity_Discussion;

/**
 * Class Discussion
 * @package WPEP\Admin\Coordinator
 */
class Discussion {

  public function _register_menu() {
    if( !wpep_controller()->discussion->has_integrations() )
      return;

    wpep_admin_register_menu(
      __( "WPEP - Discussions", 'wpep'),
      __( "Discussions", 'wpep'),
      WPEP_CAP_MANAGE_MEMBERS,
      "wpep-discussion-management",
      [ $this, "display" ]
    );
  }

  public function display() {
    $discussion_integrations = wpep_controller()->discussion->all();

    foreach( $discussion_integrations as $discussion_integration )
      $discussion_integration->pre_administration_discussions_query();

    $entries_per_page = ( isset( $_GET['pagination-per-page'] ) ? intval( $_GET['pagination-per-page'] ) : 15 );
    $current_page     = max( 1, ( isset( $_GET['pagination'] ) ? intval( $_GET['pagination'] ) : 1 ) );

    $discussion_data = WPEP_Entity_Discussion::instance()->get_discussions( [], [
      'count_total' => true,
      'limit'       => $entries_per_page,
      'offset'      => ( $entries_per_page * ( $current_page - 1 ) )
    ] );

    if( !isset( $discussion_data['results'] ) || empty( $discussion_data['results'] ) ) {
      wpep_template_load( 'admin/discussion-management.php', [
        'discussion_list_with_comments' => [],
        'discussion_pagination'         => []
      ] );

      return;
    }

    $discussions_total = $discussion_data['total'];
    $discussion_list   = $discussion_data['results'];

    foreach( $discussion_list as $discussion_key => $discussion_details )
      $discussion_list[$discussion_key]->comments = WPEP_Entity_Discussion::instance()->get_discussion_comments(
        [
          'discussion_id'          => $discussion_details->id
        ],
        [
          'do_not_order_by_pinned' => true,
          'count_total'            => false,
          'limit'                  => 2,
          'offset'                 => 0
        ]
      );

    wpep_template_load( 'admin/discussion-management.php', [
      'discussion_list_with_comments' => $discussion_list,
      'entries_per_page'              => $entries_per_page,
      'pagination_current'            => $current_page,
      'pagination_total'              => ceil( $discussions_total / $entries_per_page )
    ] );
  }

}