<?php

namespace WPEP\Admin\Coordinator;

class Embed {

  public function setup() {
    if( !wpep_has_embed_support() )
      return;

    if( wpep_has_engagement_embed_support() ) {
      add_filter('wpep_meta_boxes', [ $this, '_init_custom_meta_boxes' ] );
    }

    add_filter( WPEP_TAXONOMY_CATEGORY . "_row_actions", [ $this, 'course_category_table_row_actions' ], 15, 2 );

    add_filter('manage_posts_columns', [ $this, '_table_set_columns' ] );
    add_filter('manage_posts_custom_column', [ $this, '_table_custom_columns' ] );

    if( in_array( 'page', $this->_external_embed_allowed_post_types() ) ) {
      add_filter('manage_pages_columns', [ $this, '_table_set_columns' ] );
      add_filter('manage_pages_custom_column', [ $this, '_table_custom_columns' ] );
    }
  }

  public function _init_custom_meta_boxes(array $meta_boxes) {
    $meta_boxes['wpep_embed']  = [
      'id'         => 'wpep_embed',
      'title'      => __('Knowledge Broadcaster - Embed Settings', "wpep"),
      'post_types' => [ WPEP_POST_TYPE_COURSE ],
      'context'    => 'advanced',
      'priority'   => 'high',
      'fields'     => [
        'wpep_embed_status' => [
          'name'    => __( 'Status', "wpep" ),
          'id'      => 'wpep_embed_status',
          'type'    => 'select',
          'options' => [
            ''  =>  sprintf(
              __( "Default Setting : %s", "wpep" ),
              ( wpep_get_setting_flag( 'user-content-embed-box-course-status' ) ? __( "Enabled", "wpep" ) : __( "Disabled", "wpep" ) )
            ),
            '0' => __( "Disabled", "wpep" ),
            '1' => __( "Enabled", "wpep" )
          ]
        ],
        'wpep_embed_title' => [
          'name'        => __( 'Title', "wpep" ),
          'id'          => 'wpep_embed_title',
          'type'        => 'text',
          'placeholder' => wpep_get_setting( 'user-content-embed-box-title' ),
          'description' => __( "Leave this field empty to use the Title from the Sharing & Embed Settings", "wpep" )
        ],
        'wpep_embed_content' => [
          'name'        => __( 'Content', "wpep" ),
          'id'          => 'wpep_embed_content',
          'type'        => 'editor',
          'description' => __( "Leave this field empty to use the Content from the Sharing & Embed Settings", "wpep" )
        ],
        'wpep_embed_button' => [
          'name'        => __( 'Button', "wpep" ),
          'id'          => 'wpep_embed_button',
          'type'        => 'text',
          'placeholder' => wpep_get_setting( 'user-content-embed-box-button' ),
          'description' => __( "Leave this field empty to use the Button from the Sharing & Embed Settings", "wpep" )
        ]
      ]
    ];

    return $meta_boxes;
  }

  public function course_category_table_row_actions( $actions, $term ) {
    $actions['wpep_external_embed'] = wpep_get_external_page_embed_copy_button(
      $term->term_id, get_term_link( $term ), 'wpep-external-embed-text aria-button-if-js'
    );

    return $actions;
  }

  public function _table_set_columns($columns) {
    global $post_type;

    if ( !in_array( $post_type,  $this->_external_embed_allowed_post_types() ) )
      return $columns;

    unset(
      $columns['date']
    );

    $new_columns = [
      'external_embed' => __( 'External Embed', "wpep")
    ];

    $new_columns['date'] = __('Date', "wpep");

    return array_merge($columns, $new_columns);
  }

  public function _table_custom_columns($column) {
    global $post,$post_type;

    if ( !in_array( $post_type,  $this->_external_embed_allowed_post_types() ) )
      return;

    if( $column == 'external_embed' ) {
      echo wpep_get_external_page_embed_copy_button( $post->ID );

      return;
    }
  }

  private function _external_embed_allowed_post_types() {
    return apply_filters( "wpep_embed_post_types", [ WPEP_POST_TYPE_COURSE, 'page' ] );
  }

}