<?php

namespace WPEP\Admin\Coordinator;

class FieldLibrary {

  public $post;
  public $term;

  public function __construct() {
    $this->post = new FieldLibrary\Post();
    $this->term = new FieldLibrary\Term();
  }

  public function setup() {
    $this->post->setup();
    $this->term->setup();
  }

  /**
   * @param \WPEP\Container\FormFields $containerConfigurationInstance
   * @param $field_list
   */
  public function display_fields( $containerConfigurationInstance, $field_list ) {
    $remaining_fields = count( $field_list );

    echo '<div class="wpep-meta-box-container">';

    foreach( $field_list as $field_information ) {
      $input_name = isset( $field_information[ 'input_name' ] ) ? $field_information[ 'input_name' ] : false;

      if( isset( $field_information[ 'content_before' ] ) )
        echo $field_information[ 'content_before' ];

      if( isset( $field_information['name'] ) )
        echo '<p class="label">' . $field_information['name'] . '</p>';

      if( isset( $field_information['top_description'] ) || isset( $field_information['top_desc'] ) )
        echo '<div class="description">' . wpautop( isset( $field_information['top_description'] ) ? $field_information['top_description'] : $field_information['top_desc'] ) . '</div>';

      if( $field_information['type'] == 'radio_inline' ) {
        $containerConfigurationInstance->display_radio_fields(
          $field_information['id'],
          isset( $field_information['options'] ) ? $field_information['options'] : $field_information['values'],
          isset( $field_information['default'] ) ? $field_information['default'] : 0
        );
      } else if( $field_information['type'] == 'select' ) {
        $containerConfigurationInstance->display_select_field(
          $field_information['id'],
          isset( $field_information['options'] ) ? $field_information['options'] : $field_information['values'],
          $input_name,
          null,
          isset( $field_information['class'] ) ? $field_information['class'] : 'wpep-select-field',
          isset( $field_information['default'] ) ? $field_information['default'] : '',
          isset( $field_information['attributes'] ) ? $field_information['attributes'] : []
        );
      } else if( $field_information['type'] == 'multiple_select' ) {
        $containerConfigurationInstance->display_multiple_select_field(
          $field_information['id'],
          isset( $field_information['options'] ) ? $field_information['options'] : $field_information['values'],
          $input_name,
          null,
          'wpep-multiple-select',
          isset( $field_information['attributes'] ) ? $field_information['attributes'] : []
        );
      } else if( $field_information['type'] == 'wysiwyg' || $field_information['type'] == 'editor' ) {
        $containerConfigurationInstance->display_editor_field( $field_information['id'] );
      } else if( $field_information['type'] == 'textarea' ) {
        $containerConfigurationInstance->display_textarea_field(
          $field_information['id'],
          isset( $field_information['input_name'] ) ? $field_information['input_name'] : false,
          isset( $field_information['height'] )     ? $field_information['height']     : false
        );
      } else if( $field_information['type'] == 'colorpicker' || $field_information['type'] == 'input_color' ) {
        $containerConfigurationInstance->display_color_field( $field_information['id'] );
      } else if( $field_information['type'] == 'image' ) {
        $containerConfigurationInstance->display_image_field( $field_information['id'], $field_information['name'] );
      } else if( $field_information['type'] == 'file' ) {
        $containerConfigurationInstance->display_file_field( $field_information['id'] );
      } else if( $field_information['type'] == 'input' || $field_information['type'] == 'text' ) {
        $containerConfigurationInstance->display_text_field(
          $field_information['id'],
          $input_name,
          null,
          isset( $field_information['class'] ) ? $field_information['class'] : false,
          isset( $field_information['placeholder'] ) ? $field_information['placeholder'] : '',
          isset( $field_information['data'] ) ? $field_information['data'] : [],
          isset( $field_information['default'] ) ? $field_information['default'] : ''
        );
      } else if( $field_information['type'] == 'attributes' ) {
        $containerConfigurationInstance->display_attributes_field(
          $field_information['id'],
          $input_name,
          null,
          isset( $field_information['default'] ) ? $field_information['default'] : []
        );
      } else if( $field_information['type'] == 'list' ) {
        $containerConfigurationInstance->display_list_field(
          $field_information['id'],
          $input_name,
          null,
          isset( $field_information['default'] ) ? $field_information['default'] : []
        );
      } else if( $field_information['type'] == 'learning_journey' ) {
        $containerConfigurationInstance->display_select_learning_journey_sequence_field(
          $field_information['id'],
          $input_name,
          isset( $field_information['value'] ) ? $field_information['value'] : false
        );
      } else if( $field_information['type'] == 'attributes_field_filters' ) {
        $containerConfigurationInstance->display_attributes_field_filters(
          $field_information['id'],
          $input_name,
          null,
          isset( $field_information['default'] ) ? $field_information['default'] : []
        );
      }

      if( isset( $field_information['description'] ) || isset( $field_information['desc'] ) )
        echo '<div class="description">' . wpautop( isset( $field_information['description'] ) ? $field_information['description'] : $field_information['desc'] ) . '</div>';

      if( $remaining_fields > 1 && $field_information['type'] != 'none' )
        echo '<hr/>';

      if( isset( $field_information[ 'content_after' ] ) )
        echo $field_information[ 'content_after' ];

      if( $field_information['type'] != 'none' )
        $remaining_fields--;
    }

    echo '</div>';
  }

  /**
   * @param \WPEP\Container\FormFields $containerConfigurationInstance
   * @param $field_list
   */
  public function display_inner_table( $containerConfigurationInstance, $field_list ) {
    foreach( $field_list as $field_information ) {
      $label = $field_information['name'];
      $description = ( isset( $field_information['description'] ) ? $field_information['description'] : ( isset( $field_information['desc'] ) ? $field_information['desc'] : '' ) );

      if( $field_information['type'] == 'select' ) {
        $containerConfigurationInstance->display_table_select_field(
          $label,
          $field_information['id'],
          ( isset( $field_information['options'] ) ? $field_information['options'] : $field_information['values'] ) ,
          $description,
          ( isset( $field_information['class'] ) ? $field_information['class'] : 'wpep-select-field' )
        );
      } else if( $field_information['type'] == 'multiple_select' ) {
        $containerConfigurationInstance->display_table_multiple_select_field(
          $label,
          $field_information['id'],
          ( isset( $field_information['options'] ) ? $field_information['options'] : $field_information['values'] ),
          $description
        );
      } else if( $field_information['type'] == 'wysiwyg' || $field_information['type'] == 'editor' ) {
        $containerConfigurationInstance->display_table_editor_field(
          $label,
          $field_information['id'],
          $description
        );
      } else if( $field_information['type'] == 'textarea' ) {
        $containerConfigurationInstance->display_table_textarea_field(
          $label,
          $field_information['id'],
          $description,
          ( isset( $field_information['input_name'] ) ? $field_information['input_name'] : false ),
          ( isset( $field_information['height'] )     ? $field_information['height']     : false )
        );
      } else if( $field_information['type'] == 'colorpicker' || $field_information['type'] == 'input_color' ) {
        $containerConfigurationInstance->display_table_color_field(
          $label,
          $field_information['id'],
          $description
        );
      } else if( $field_information['type'] == 'image' || $field_information['type'] == 'file' ) {
        $containerConfigurationInstance->display_table_image_field(
          $label,
          $field_information['id'],
          $description
        );
      } else if( $field_information['type'] == 'input' || $field_information['type'] == 'text' ) {
        $containerConfigurationInstance->display_table_text_field(
          $label,
          $field_information['id'],
          $description
        );
      } else if( $field_information['type'] == 'attributes' ) {
        $containerConfigurationInstance->display_table_attributes_field(
          $label,
          $field_information['id'],
          $description,
          false,
          null,
          ( isset( $field_information['default'] ) ? $field_information['default'] : [] )
        );
      } else if( $field_information['type'] == 'html' ) {
        $containerConfigurationInstance->display_table_separator(
          $label,
          $field_information['html']
        );
      } else {
        $containerConfigurationInstance->display_table_separator(
          "<hr/>",
          sprintf( __( "Field Type %s is not available", "wpep" ), $field_information['type'] )
        );
      }
    }
  }

}