<?php

namespace WPEP\Admin\Coordinator\FieldLibrary;

use WPEP\Controller as WPEP_Controller;
use WPEP\Container\FormFields as WPEP_FormFields;

/**
 * Class Post
 * @package WPEP\Admin\Coordinator
 */
class Post {

  use WPEP_FormFields;

  private $_post_id = 0;
  private $_displayed_metabox_post_id = false;

  public function __construct() {
    $this->set_field_prefix( 'wpep_meta_value' );
  }

  public function setup() {
    add_action( 'add_meta_boxes', [ $this, 'register_meta_boxes'  ] );
    add_action( 'save_post', [ $this, '_handle_save' ], 10 );

    add_filter( 'wpep_admin_save_post_ajax', [ $this, '_wpep_admin_save_post_ajax' ], 20, 2 );
  }

  public function register_meta_boxes() {
    $meta_boxes = apply_filters( 'wpep_meta_boxes', [] );

    foreach( $meta_boxes as $meta_box ) {
      $meta_box['post_types'] = ( isset( $meta_box['post_types'] ) ? $meta_box['post_types'] : $meta_box['pages'] );
      unset($meta_box['pages']);

      add_meta_box(
        $meta_box['id'],
        $meta_box['title'],
        [ $this, '_display_fields' ],
        $meta_box['post_types'],
        $meta_box['context'],
        $meta_box['priority'],
        $meta_box['fields']
      );
    }
  }

  public function _display_fields( $post, $information ) {
    $this->_post_id = $post->ID;

    if( $this->_displayed_metabox_post_id == false ) {
      $this->display_hidden_field( 'wpep_metabox_post_id', $this->_post_id, 'wpep_metabox_post_id' );
      $this->_displayed_metabox_post_id = true;
    }

    WPEP_Controller::instance()->adminController->fieldLibrary->display_fields( $this, $information['args'] );
  }

  public function get_value( $field_key, $field_default = '', $single = null ) {
    if( $single == null ) {
      if( is_array( $field_default ) && strpos( $field_key, '_' ) !== 0 )
        $single = false;
      else
        $single = true;
    }

    $field_value = get_post_meta( $this->_post_id, $field_key, $single );

    return ( ( $single && $field_value === '' ) || !$single && empty( $field_value ) ) ? $field_default : $field_value;
  }

  public function _handle_save( $post_id ) {
    if( !isset( $_POST[ $this->field_prefix ] ) )
      return;

    if( !isset( $_POST[ 'wpep_metabox_post_id' ] ) )
      return;

    if( !is_array( $_POST[ 'wpep_metabox_post_id' ] ) && intval( $_POST[ 'wpep_metabox_post_id' ] ) !== intval( $post_id ) )
      return;
    else if( is_array( $_POST[ 'wpep_metabox_post_id' ] ) && !in_array( $post_id, $_POST[ 'wpep_metabox_post_id' ] ) )
      return;

    do_action( "wpep_administration_meta_boxes_save_before" );

    $information = apply_filters( "wpep_administration_meta_boxes_save_information", $_POST[ $this->field_prefix ] );

    foreach( $information as $field_key => $field_value ) {
      if( is_array( $field_value ) ) {
        if( strpos( $field_key, '_' ) !== 0 ) {
          $database_information = get_post_meta( $post_id, $field_key, false );

          if( !empty( $database_information ) ) {
            if( is_array( $field_value[ key( $field_value ) ] ) && is_array( $database_information[ key( $database_information ) ] ) ) {
              $field_value          = array_values( $field_value );
              $database_information = array_values( $database_information );

              if( json_encode( $field_value ) === json_encode( $database_information ) ) {
                $add_meta_values = [];
                $remove_meta_values = [];
              } else {
                $add_meta_values    = $field_value;
                $remove_meta_values = $database_information;
              }
            } else {
              $add_meta_values    = @array_diff( $field_value, $database_information );
              $remove_meta_values = @array_diff( $database_information, $field_value );
            }

            if( !empty( $remove_meta_values ) ) {
              foreach( $remove_meta_values as $remove_meta_value )
                delete_post_meta( $post_id, $field_key, $remove_meta_value );
            }
          } else {
            $add_meta_values = $field_value;
          }

          if( !empty( $add_meta_values ) ) {
            foreach( $add_meta_values as $add_meta_value ) {
              if( $add_meta_value == 'a:0:{}' )
                continue;

              add_post_meta( $post_id, $field_key, $add_meta_value );
            }
          }

          continue;
        }
      }

      if( $field_value === '' ) {
        delete_post_meta( $post_id, $field_key );

        continue;
      }

      update_post_meta( $post_id, $field_key, $field_value );
    }

    do_action( "wpep_administration_meta_boxes_save_after" );
  }

  /**
   * @param array $entity_id_map
   * @param $post_id
   * @return mixed
   */
  public function _wpep_admin_save_post_ajax( $entity_id_map, $post_id ) {
    $this->_handle_save( $post_id );

    return $entity_id_map;
  }

}