<?php
namespace WPEP\Admin\Coordinator;

class LearningCenter {

  /**
   * @var LearningCenter\Skeleton[]
   */
  public $pages = [];

  public function __construct() {
    $this->_setup_pages();
  }

  public function _init() {
    foreach( $this->pages as $page ) {
      if( method_exists( $page, '_init' ) )
        $page->_init();
    }
  }

  public function _register_menu() {
    wpep_admin_register_menu(
      __("WPEP Learning Center", 'wpep'),
      __( "Learning Center", 'wpep'),
      WPEP_CAP_LEARNING_CENTER,
      "wpep-learning-center",
      [ $this, "display" ]
    );
  }

  public function display() {
    echo '<div class="wrap wpep-learning-center-container" data-wpep-page="learning-center">';
    echo '<h2 class="wpep-title-page">' . __( "WPEP Learning Center", 'wpep') . '</h2>';

    $current_platform_health_page = $this->get_pages( $this->get_current_tab() );

    $this->show_tabs( $this->get_current_tab() );

    $current_platform_health_page->display();

    echo '</div>';
  }

  public function show_tabs( $current = 'homepage' ) {
    echo '<h2 class="wpep-navigation wpep-navigation-primary" data-wpep-navigation="learning-center">';
    foreach( $this->get_pages() as $tab => $setting_page ){
      $class = ( $tab == $current ) ? ' wpep-navigation-item-active' : '';
      echo '<a data-wpep-tab="' . $tab . '" class="wpep-navigation-item' . $class . '" href="' . wpep_admin_url_learning_center( [ 'tab' => $tab ] ). '">';
      echo    '<span>' . $setting_page->get_name() . '</span>';
      echo '</a>';
    }

    echo '</h2>';

    echo '<div class="wpep-clearfix"></div>';
  }

  public function get_current_tab() {
    $statistics_pages = $this->get_pages();

    $current_tab = ( isset( $_GET['tab'] ) ? $_GET['tab'] : key( $statistics_pages ) );

    if( !isset( $statistics_pages[ $current_tab ] ) )
      $current_tab = key( $statistics_pages );

    return $current_tab;
  }

  public function get_pages( $index = false ) {
    if( !is_array( $this->pages ) )
      $this->_setup_pages();

    return ( $index == false ? $this->pages : $this->pages[$index] );
  }

  private function _setup_pages() {
    $this->pages = [];

    foreach( [
               new LearningCenter\WalkTrough(),
               new LearningCenter\Content(),
             ] as $current_instance )
      $this->pages[ $current_instance->get_alias() ] = $current_instance;

    $this->pages = apply_filters( 'wpep_administration_learning_center_pages', $this->pages );
  }

}