<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;

class PlatformHealth {

  public $tabs;


  public function setup() {
    add_action( 'activated_plugin', [ $this, '_any_plugin_activation_deactivation' ], 10 );
    add_action( 'deactivated_plugin', [ $this, '_any_plugin_activation_deactivation' ], 10 );
  }

  public function _register_menu() {
    $menu_title = __( "Platform Health", 'wpep');

    wpep_admin_register_menu(
      __("WPEP Platform Health", 'wpep'),
      $menu_title,
      WPEP_CAP_MANAGE_SETTINGS,
      "wpep-platform-health",
      [ $this, "display" ]
    );
  }

  public function _init() {
    if( !is_admin() )
      return;

    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return;

    if( isset( $_GET['page'] ) && $_GET['page'] == 'wpep-platform-health' ) {
      $this->_setup_pages();

      return;
    }
  }

  public function display() {
    echo '<div class="wrap" data-wpep-page="platform-health">';
    echo '<h2 class="wpep-title-page">' . __( "WPEP - Platform Health", 'wpep') . '</h2>';

    $current_platform_health_page = $this->get_pages( $this->get_current_tab() );

    $this->show_tabs( $this->get_current_tab() );

    $current_platform_health_page->display();

    echo '</div>';
  }

  public function show_tabs( $current = 'homepage' ) {
    echo '<h2 class="wpep-navigation wpep-navigation-primary" data-wpep-navigation="platform-health">';
    foreach( $this->get_pages() as $tab => $setting_page ){
      $class = ( $tab == $current ) ? ' wpep-navigation-item-active' : '';
      echo '<a data-wpep-tab="' . $tab . '" class="wpep-navigation-item' . $class . '" href="' . wpep_admin_url_platform_health( [ 'tab' => $tab ] ). '">';
      echo    '<span>' . $setting_page->get_name() . '</span>';
      echo '</a>';
    }

    echo '</h2>';

      echo '<div class="wpep-clearfix"></div>';
  }

  public function get_current_tab() {
    $statistics_pages = $this->get_pages();

    $current_tab = ( isset( $_GET['tab'] ) ? $_GET['tab'] : key( $statistics_pages ) );

    if( !isset( $statistics_pages[ $current_tab ] ) )
      $current_tab = key( $statistics_pages );

    return $current_tab;
  }

  public function get_pages( $index = false ) {
    if( !is_array( $this->tabs ) )
      $this->_setup_pages();

    return ( $index == false ? $this->tabs : $this->tabs[$index] );
  }

  private function _setup_pages() {
    $this->tabs = [];

    foreach( [
               new PlatformHealth\Dashboard(),
               new PlatformHealth\DataSystem(),
               new PlatformHealth\CleanupUtilities(),
               new PlatformHealth\Tools(),
             ] as $current_instance )
      $this->tabs[ $current_instance->get_alias() ] = $current_instance;

    $this->tabs = apply_filters( 'wpep_administration_platform_health_pages', $this->tabs );
  }

  public function _any_plugin_activation_deactivation() {
    WPEP_Controller::instance()->platformHealth->delete_service_status_notification_count_transient();
  }

}
