<?php

namespace WPEP\Admin\Coordinator\PlatformHealth;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;
use WPEPCronController as WPEP_Cron;

class Dashboard extends Skeleton {

  public function get_name() {
    return __( "Dashboard", "wpep" );
  }

  public function get_alias() {
    return 'dashboard';
  }

  public function display() {

    $this->_handle_operations();

    echo '<div class="wpep-system-report">';

    echo wpep_administration_platform_health_table(
      WPEP_Controller::instance()->platformHealth->get_service_status_report(),
      __( "Platform Health", "wpep" ),
      'wpep-modules-table',
      '<a class="wpep-button wpep-button-primary" data-wpep-ajax-content-export="wpep_admin_platform_health_export">' . __( "Export Report", "wpep" ) . '</a>'
    );

    foreach( WPEP_Controller::instance()->platformHealth->get_report() as $report_key => $report_group ) {

      echo '<table class="wp-list-table widefat striped wpep-' . $report_key . '-table">';
      echo    '<thead>';
      echo      '<tr>';
      echo        '<th colspan="2">' . $report_group['name'] . '</th>';
      echo      '</tr>';
      echo    '</thead>';
      echo    '<tbody>';

      foreach( $report_group['entries'] as $report_entry_key => $report_entry_data ) {
        echo wpep_format_report_data_to_line(
          $report_entry_key,
          $report_entry_data,
          '<tr class="%class_status%">' .
                    '<td>%label%</td>' .
                    '<td class="wpep-row-highlight-icon">%value%</td>' .
                  '</tr>',
          'wpep-row-td-highlight wpep-row-td-highlight-'
        );
      }

      echo    '</tbody>';
      echo '</table>';
    }

    echo '</div>';
  }

  private function _handle_operations() {
    if( !isset( $_GET[ 'operation'] ) )
      return;

    if( $_GET[ 'operation'] == 'refresh-user-roles' ) {
      $response = WPEP_Controller::instance()->capabilities->register_caps_and_roles();

      if( $response )
        echo '<div class="wpep-notification wpep-notification-success">' . __( "WPEP User Roles & User Capabilities have been refreshed.", "wpep" ) . '</div>';
      else
        echo '<div class="wpep-notification wpep-notification-danger">' . __( "There was a problem refreshing the WPEP User Roles & User Capabilities.", "wpep" ) . '</div>';
    } else if( $_GET[ 'operation'] == 'reset-permalinks' ) {
      flush_rewrite_rules();

      echo '<div class="wpep-notification wpep-notification-success">' . __( "WordPress Rewrite Rules have been refreshed.", "wpep" ) . '</div>';
    } else if( $_GET[ 'operation'] == 'set-custom-css-by-file' ) {
      WPEP_Settings::instance()->update_settings( [
        'wpep-custom-css-by-file' => 1
      ] );

      echo '<div class="wpep-notification wpep-notification-success">' . __( "Custom CSS & Color Variations will now be delivered from a file.", "wpep" ) . '</div>';
    } else if( $_GET[ 'operation'] == 'set-custom-css-by-html' ) {
      WPEP_Settings::instance()->update_settings( [
        'wpep-custom-css-by-file' => 0
      ] );

      echo '<div class="wpep-notification wpep-notification-info">' . __( "Custom CSS & Color Variations will now be delivered on page.", "wpep" ) . '</div>';
    } else if( $_GET[ 'operation'] == 'set-course-content-delivery-by-raw' ) {
      WPEP_Settings::instance()->update_settings( [
        'course-use-the-content-filter' => 0
      ] );

      echo '<div class="wpep-notification wpep-notification-info">' . __( "The Course Content will now be delivered without using the_content filter.", "wpep" ) . '</div>';
    } else if( $_GET[ 'operation'] == 'set-course-content-delivery-by-filter' ) {
      WPEP_Settings::instance()->update_settings( [
        'course-use-the-content-filter' => 1
      ] );

      echo '<div class="wpep-notification wpep-notification-success">' . __( "The Course Content will now be delivered using the_content filter.", "wpep" ) . '</div>';
    } else if( $_GET[ 'operation'] == 'enable-course-timing-options' ) {
      WPEP_Settings::instance()->update_settings( [
        'enable-course-timing-option' => 1
      ] );

      echo '<div class="wpep-notification wpep-notification-success">' . __( "The Course Timing Options have been enabled.", "wpep" ) . '</div>';
    } else if( $_GET[ 'operation'] == 'disable-course-timing-options' ) {
      WPEP_Settings::instance()->update_settings( [
        'enable-course-timing-option' => 0
      ] );

      echo '<div class="wpep-notification wpep-notification-info">' . __( "The Course Timing Options have been disabled.", "wpep" ) . '</div>';
    } else if( $_GET['operation'] == 'resolve-table-integrity-issues' ) {
      foreach( WPEP_Controller::instance()->adminController->migration->get_integrations() as $integration )
        $integration->fix_table_structure();

      global $wpdb;

      $table_assessment_question_answer_columns = wpep_utility_map_to_array_assoc(
        $wpdb->get_results( 'DESCRIBE `' . $wpdb->prefix . 'wpep_assessment_question_answer' . '`'),
        'Field',
        'Type'
      );
      $table_section_lesson_columns = wpep_utility_map_to_array_assoc(
        $wpdb->get_results( 'DESCRIBE `' . $wpdb->prefix . 'wpep_section_lesson' . '`'),
        'Field',
        'Type'
      );
      $table_assessment_user_data_columns = wpep_utility_map_to_array_assoc(
        $wpdb->get_results( 'DESCRIBE `' . $wpdb->prefix . 'wpep_assessment_user_data' . '`'),
        'Field',
        'Type'
      );

      if( !isset( $table_assessment_question_answer_columns['question_id'] ) ) {
        $sql = "ALTER TABLE `" . $wpdb->prefix . 'wpep_assessment_question_answer' . "` CHANGE `question` `question_id` INT(11) NOT NULL DEFAULT '0';";

        $wpdb->query( $sql );
      }

      if( !isset( $table_section_lesson_columns['post_id'] ) ) {
        $sql = "ALTER TABLE `" . $wpdb->prefix . 'wpep_section_lesson' . "` ADD `post_id` INT(11) NOT NULL DEFAULT 0 AFTER `id`;";

        $wpdb->query( $sql );
      }

      if( $table_assessment_user_data_columns['value'] != 'longtext' ) {
        $sql = 'ALTER TABLE `' . $wpdb->prefix . 'wpep_assessment_user_data` CHANGE `value` `value` LONGTEXT;';

        $wpdb->query( $sql );
      }


      echo '<div class="wpep-notification wpep-notification-info">' . __( "Table Integrity Issues have been resolved, if the issue persists, ensure the DB user has the right to modify tables.", "wpep" ) . '</div>';
    } else if( $_GET[ 'operation' ] === 'reset-cron-jobs' ) {
      WPEP_Cron::reset_cron_events();
    }

    WPEP_Controller::instance()->platformHealth->delete_service_status_notification_count_transient();
  }

}