<?php

namespace WPEP\Admin\Coordinator\PlatformHealth\Tools;

use WPEP\Container\FormFields as WPEP_FormFields;
use WPEP\Admin\Coordinator\PlatformHealth\Skeleton as WPEP_PH_Skeleton;

class TTFFonts extends WPEP_PH_Skeleton {

  use WPEP_FormFields;

  public function get_name() {
    return __( "TTF Fonts", "wpep" );
  }

  public function get_alias() {
    return 'ttf-fonts';
  }

  public function display() {
    $font_list = wpep_image_builder_get_font_list( false );

    echo '<form method="post" enctype="multipart/form-data">';

    $this->display_table_start();

    if( isset( $_GET['delete'] ) && isset( $font_list[ urldecode( $_GET['delete'] ) ] ) ) {
      $delete_font = urldecode( $_GET['delete'] );

      $this->display_table_html(
        wpep_template_get( 'admin/platform-health/tools-ttf-fonts-delete.php', [
          'font_list'   => $font_list,
          'delete_font' => $delete_font
        ] )
      );
    } elseif( isset( $_GET['confirm-delete'] ) && isset( $font_list[ urldecode( $_GET['confirm-delete'] ) ] ) ) {
      $deleted_font = urldecode( $_GET['confirm-delete'] );

      if( $deleted_font === 'arial.ttf' || $deleted_font === 'arial' ) {
        $this->display_table_html(
          '<div class="wpep-notification wpep-notification-danger">' . __( "This operation is not allowed.", "wpep" ) . '</div>'
        );
      } else {
        unlink( wpep_image_builder_get_font_path( $deleted_font ) );

        $this->display_table_html(
          wpep_template_get( 'admin/platform-health/tools-ttf-fonts-deleted.php', [
            'font_list'    => $font_list,
            'deleted_font' => $deleted_font
          ] )
        );
      }

      $font_list = wpep_image_builder_get_font_list( false );
    } elseif( isset( $_GET[ 'operation' ] ) && $_GET[ 'operation' ] == 'download-fonts' ) {
      wpep_image_builder_import_font_directory_assets();

      $this->display_table_html(
        wpep_template_get( 'admin/platform-health/tools-ttf-fonts-downloaded.php' )
      );

      $font_list = wpep_image_builder_get_font_list( false );
    }

    $this->display_table_html(
      wpep_template_get( 'admin/platform-health/tools-ttf-fonts-list.php', [
        'font_list' =>  $font_list
      ] )
    );

    $this->display_table_end();
    echo '</form>';
  }

  public function get_value( $key, $default = '' ) {
    if( $key == 'custom_css_converter'
      && isset( $_POST['wpep_platform_health_tools'] )
      && isset( $_POST['wpep_platform_health_tools']['custom_css_converter'] )
    ) {
      $custom_css = $_POST['wpep_platform_health_tools']['custom_css_converter'];
      $custom_css = sanitize_textarea_field( $custom_css );
      $custom_css = wpep_custom_style_format_legacy_rules( $custom_css );

      return $custom_css;
    }

    return $default;
  }

}