<?php

namespace WPEP\Admin\Coordinator\Settings;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;

class CoursePage extends Skeleton {

  public $tabs = [];

  public function _setup_tabs() {
   $this->tabs = [
     'general'        => __( 'General', "wpep" ),
     'execution_plan' => __( 'Execution Plan', "wpep" ),
     'standard'       => __( 'Standard Course', "wpep" ),
     'video'          => __( 'Video Course', "wpep" ),
     'one_off'        => __( 'One-Off Course', "wpep" ),
     'challenge'      => __( 'Challenge Course', "wpep" ),
     'assessments'    => __( 'Assessments', "wpep" ),
     'certification'  => __( 'Certification', "wpep" )
   ];
  }

  public function get_name() {
    return __( 'Course Page', "wpep" );
  }

  public function get_alias() {
    return 'course_page';
  }

  public function display() {
    $tab = ( isset( $_GET[ 'tab_inner' ] ) ? $_GET[ 'tab_inner' ] : key( $this->tabs ) );

    $this->display_tabs( $tab );

    if( $tab == 'standard' )
      $this->display_standard();
    else if( $tab == 'video' )
      $this->display_video();
    else if( $tab == 'one_off' )
      $this->display_one_off();
    else if( $tab == 'execution_plan' )
      $this->display_execution_plan();
    else if( $tab == 'challenge' )
      $this->display_challenge();
    else if( $tab == 'assessments' )
      $this->display_assessments();
    else if( $tab == 'certification' )
      $this->display_certification();
    else
      $this->display_general();

  }

  public function display_general() {
    $this->display_table_start();

    $this->display_table_checkbox_field( __('Enable Lesson Incomplete', "wpep"),  'enable-lesson-incomplete-option', __( 'Allow Lessons within Courses to be Marked as incomplete, after they have been completed, Execution Plans allow this by default.', "wpep" ) );

    $this->display_table_checkbox_field( __('Enable Timing Options', "wpep"),  'enable-course-timing-option', __( 'Enable Content Dripfeeding & Course Timing Options.', "wpep" ) );

    $this->display_table_checkbox_field( __('Course Tracking Code', "wpep"),  'enable-course-tracking-code-option', __( 'Allow Adding Course Completion Tracking Codes.', "wpep" ) );

    $this->display_table_checkbox_field( __('Resize Wistia Videos', "wpep"),  'course-resize-wistia-embeds', __( "Depending on how you are embeding Wistia Videos, you'll want to check this option.", "wpep" ) );

    $this->display_table_checkbox_field(
      __('The Content Filter', "wpep"),
      'course-use-the-content-filter',
      __( 'WPEP makes use of the standard the_content filter to display the course contents, allowing other plugins to interact with it, disabling this option is situational.', "wpep" )
    );

    $this->display_table_checkbox_field(
      __('Pre Authentication Tracking', "wpep"),
      'course-pre-authentication-tracking',
      __( 'Track User Course Progress trough Cookies if they are not logged in.', "wpep" )
    );

    $this->display_table_text_translated_field(
      __('Progress Text', "wpep"),
      'opt-course-progressbar',
      __('To personalize the text below the WPEP progress bar enter it here', "wpep")
    );

    $this->display_table_checkbox_field(
      __('Progress Percentage Bar', "wpep"),
      'opt-course-percentage-bar',
      __('Display a course progress percentage bar, you may personalize this for each course.', "wpep")
    );

    $this->display_table_editor_field(
      __( "Coming Soon Access Restricted", "wpep" ),
      'course-coming-soon-student-notification',
      '<p>' . __( "While a Course is Coming Soon, students may not do the Course earlier by guessing the URL, this is the error message.", "wpep" ) . '</p>' .
      '<p>' . __( "This does not apply for shortcode embeds.", "wpep" ) . '</p>'
    );

    $this->display_table_separator( __( "Deliverables Settings", "wpep" ) );

    $this->display_table_checkbox_field(
      __('Plain List Mode', "wpep"),
      'course-downloads-basic'
    );

    $this->display_table_text_translated_field(
      __('Title', "wpep"),
      'course-downloads-list-title'
    );

    $this->display_table_text_translated_field(
      __('Button Text', "wpep"),
      'course-downloads-list-download'
    );

    $this->display_table_select_field(
      __('Position', "wpep"),
      'course-downloads-position',
      [
        'bottom'  => __( "Bottom", "wpep" ),
        'top'     => __( "Top", "wpep" )
      ]
    );

    $this->display_table_separator( __( "Prerequisites", "wpep" ) );

    $this->display_table_checkbox_field(
      __( 'Lock Course', "wpep"),
      'prerequisite-course-button-lock',
      __('Enable this option if you want to restrict students from accessing courses they have not completed the pre-requisites requirement, from the Index Page.', "wpep")
    );

    $this->display_table_text_translated_field(
      __( 'Locked Course Button', "wpep"),
      'prerequisite-course-button-lock-text',
      __('The text displayed on the Button on the Index Page when the Course is Locked from the above option.', "wpep")
    );

    $this->display_table_text_translated_field(
      __('Suggested Notification', "wpep"),
      'prerequisite-suggested-notification',
      __('This Notification is displayed at the top of a given Course if the Student has not yet gone thru items set as prerequisites . %s will be replaced with the title of the course(s) the student has not completed yet.', "wpep")
    );

    $this->display_table_text_translated_field(
      __('Required Notification', "wpep"),
      'prerequisite-required-notification',
      __('This Notification is displayed at the top of a given Course if the Student has not yet gone thru prerequisite courses, %s will be replaced with the course the student wanted to access.', "wpep")
    );

    $this->display_table_text_translated_field(
      __('Suggested Courses Title', "wpep"),
      'prerequisite-next-courses-title',
      __('Based on the prerequisites at the end of a given Course the student will get a suggested next content notification.', "wpep" )
    );

    $this->display_table_separator( __( "Progress Repeat", "wpep" ) );

    $this->display_table_checkbox_field(
      __( 'Status', "wpep" ),
      'enable-progress-repeat-support'
    );

    $this->display_table_text_translated_field(
      __( 'Button Text', "wpep" ),
      'progress-repeat-headline'
    );

    $this->display_table_text_translated_field(
      __( 'Title', "wpep" ),
      'progress-repeat-button'
    );

    $this->display_table_end();
  }

  public function display_execution_plan() {
    $this->display_table_start();

    $this->display_table_checkbox_field(
      __( 'Lesson Hover Color', "wpep" ),
      'show-lesson-hover-color',
      __( 'Show a different color when the mouse click is on the Lesson.', "wpep" )
    );

    $this->display_table_text_translated_field(
      __('Learning Path Placeholder', "wpep"),
      'lesson-learning-path-placeholder',
      __('If you are building courses that have learning paths, this placeholder should reflect a call to action to select one, for example : Choose Your Learning Path', "wpep")
    );

    $this->display_table_separator( 'Lesson Quick Continue' );

    $this->display_table_checkbox_field(
      __( 'Enable Quick Continue', "wpep"),
      'show-lesson-continue-button',
      __('Display a button at the end of each lesson that allows students to easily continue to the next lesson.', "wpep")
    );

    $this->display_table_text_translated_field(
      __( 'Button Text', "wpep"),
      'lesson-quick-continue-button-text'
    );

    $this->display_table_text_translated_field(
      __( 'Final Button Text', "wpep"),
      'lesson-quick-continue-final-button-text'
    );

    $this->display_table_select_field(
      __( 'Button Alignment', "wpep"),
      'lesson-continue-button-position',
      [ "default" => __( "Default" ), 'left' => __( 'Left' ), 'center' => __( 'Center' ), 'right' => __( 'Right' ) ],
      __('Customize the position of the Lesson Quick Continue button.', "wpep")
    );

    $this->display_table_end();
  }

  public function display_video() {
    $this->display_table_start();

    $this->display_table_separator( __( "General Options", "wpep" ) );

    $this->display_table_checkbox_field(  __('Course Overview Page', "wpep"), 'course-video-allow-index-page' );
    $this->display_table_checkbox_field(  __('Redirect on Completion', "wpep"), 'course-video-redirect-on-completion', __( "Redirect the Student to the next Video on Completion.", "wpep" ) );

    $this->display_table_separator( __( "Text Options", "wpep" ) );

    $this->display_table_text_translated_field( __('Course Status', "wpep"), 'course-video-course-status' );
    $this->display_table_text_translated_field( __('Course Status Pending', "wpep"), 'course-video-course-status-pending' );
    $this->display_table_text_translated_field( __('Course Status Progressing', "wpep"), 'course-video-course-status-progress' );
    $this->display_table_text_translated_field( __('Course Status Completed', "wpep"), 'course-video-course-status-completed' );
    $this->display_table_text_translated_field( __('Course Progress', "wpep"), 'course-video-course-progress' );
    $this->display_table_text_translated_field( __('Table Sections', "wpep"), 'course-video-course-table-sections' );
    $this->display_table_text_translated_field( __('Table Status', "wpep"), 'course-video-course-table-status' );
    $this->display_table_text_translated_field( __('Table Start Section', "wpep"), 'course-video-start-section' );
    $this->display_table_text_translated_field( __('Available Downloads', "wpep"), 'course-video-lesson-description' );


    $this->display_table_text_translated_field( __('Back to Overview Page', "wpep"), 'course-video-lesson-back' );
    $this->display_table_text_translated_field( __('Mark video as watched', "wpep"), 'course-video-lesson-completed' );
    $this->display_table_text_translated_field( __('Video has been watched', "wpep"), 'course-video-lesson-complete' );
    $this->display_table_text_translated_field( __('Previous Video', "wpep"), 'course-video-previous-lesson' );
    $this->display_table_text_translated_field( __('Next Video', "wpep"), 'course-video-next-lesson' );

    $this->display_table_end();
  }

  public function display_one_off() {
    $this->display_table_start();

    $this->display_table_text_translated_field( __('Complete Course', "wpep"), 'course-basic-complete-course' );
    $this->display_table_text_translated_field( __('Course Completed', "wpep"), 'course-basic-course-completed' );
    $this->display_table_select_field(
      __( "Video Description", "wpep" ),
      'course-basic-enable-video-description',
      wpep_flag_setting_field_options(),
      __( "Enable an extra editor field on the One-Off Course Builder, that enables you to add content under the video.", "wpep" )
    );


    $this->display_table_text_translated_field( __('Jump Links Title', "wpep"), 'course-basic-jump-link-title' );

    $this->display_table_end();
  }

  public function display_standard() {
    $this->display_table_start();

    $this->display_table_separator( __( "General Options", "wpep" ) );

    $this->display_table_checkbox_field(  __('Overview Page', "wpep"), 'course-standard-allow-index-page' );
    $this->display_table_checkbox_field(  __('Overview Page Widgets', "wpep"), 'course-standard-allow-index-page-widgets' );
    $this->display_table_checkbox_field(  __('Redirect on Completion', "wpep"), 'course-standard-redirect-on-completion', __( "Redirect the Student to the next Lesson on Completion.", "wpep" ) );

    $this->display_table_separator( __( "Text Options", "wpep" ) );

    $this->display_table_text_translated_field( __('Section', "wpep"), 'course-standard-section' );
    $this->display_table_text_translated_field( __('Section List Title', "wpep"), 'course-standard-section-list' );
    $this->display_table_text_translated_field( __('Lesson', "wpep"), 'course-standard-lesson' );
    $this->display_table_text_translated_field( __('Course Progress', "wpep"), 'course-standard-course-progress' );
    $this->display_table_text_translated_field( __('Course Navigation', "wpep"), 'course-standard-course-navigation' );
    $this->display_table_text_translated_field( __('Start Lesson', "wpep"), 'course-standard-start-lesson' );
    $this->display_table_text_translated_field( __('Previous Lesson', "wpep"), 'course-standard-previous-lesson' );
    $this->display_table_text_translated_field( __('Next Lesson', "wpep"), 'course-standard-next-lesson' );
    $this->display_table_text_translated_field( __('Complete Lesson', "wpep"), 'course-standard-complete-lesson' );
    $this->display_table_text_translated_field( __('Message Lesson Completed', "wpep"), 'course-standard-lesson-completed' );

    $this->display_table_end();
  }

  public function display_challenge() {
    $this->display_table_start();

    $this->display_table_text_field(  __('URL Slug', "wpep"), 'course-challenge-section-slug', __( "No spaces allowed", "wpep" ) );
    $this->display_table_editor_field(  __( 'Introduction Text', "wpep" ), 'course-challenge-introduction-text' );

    $this->display_table_separator( __( "Social Share", "wpep" ) );

    $this->display_table_select_field( __( "Status", "wpep" ), 'course-challenge-social-share-status', wpep_flag_setting_field_options() );
    $this->display_table_editor_field(  __( 'CTA', "wpep" ), 'course-challenge-social-share-cta' );
    $this->display_table_multiple_select_field(
      __( "Networks", "wpep" ),
      'course-challenge-social-share-networks',
      wpep_utility_map_to_array_assoc( wpep_utilities_social_share_map(), 'alias', 'alias' )
    );

    $this->display_table_end();
  }

  public function display_assessments() {
    $this->display_table_start();

    $this->display_table_checkbox_field( __('Enable Course Assessments', "wpep"),  'enable-course-assessments', __( 'Course Assessments allow you to quiz your students and gather information.', "wpep" ) );

    if( defined( "WPEP_BETA_TESTING" ) ) {
      $this->display_table_checkbox_field( __('Streamlined Experience', "wpep"),  'assessment-course-page-embed', __( "Students will not leave the course page in order to complete assessments", "wpep" ) );
    }

    $this->display_table_checkbox_field( __('Enable Ajax', "wpep"),  'assessment-ajax-enabled', __( 'Enabling this option will make the assessment experience happen without page reloads.', "wpep" ) );
    $this->display_table_checkbox_field( __('Enable Protection', "wpep"),  'assessment-course-protection-rules', __( 'Enabling this option will protect the Assessment Page as the Course Page, disable this option if you want to configure each assessment protection rules individually.', "wpep" ) );

    $this->display_table_text_translated_field( __( "Assessment", "wpep" ), 'assessment' );
    $this->display_table_text_translated_field( __( "View Assessment", "wpep" ), 'assessment-view' );

    $this->display_table_separator( __( "General", "wpep" ) );
    $this->display_table_text_translated_field( __( "Input Placeholder", "wpep" ), 'assessment-input-placeholder' );
    $this->display_table_text_translated_field( __( "Assessment Score", "wpep" ), 'assessment-score' );
    $this->display_table_text_translated_field( __( 'Required to Pass', "wpep" ), 'assessment-score-requirement' );
    $this->display_table_text_translated_field( __( 'Questions Correct', "wpep" ), 'assessment-questions-correct' );

    $this->display_table_separator( __( "Interaction", "wpep" ) );

    $this->display_table_text_translated_field( __( "File Upload", "wpep" ), 'assessment-file-upload' );
    $this->display_table_text_translated_field( __( "Paginated Prev", "wpep" ), 'assessment-part-prev' );
    $this->display_table_text_translated_field( __( "Paginated Next", "wpep" ), 'assessment-part-next' );
    $this->display_table_text_translated_field( __( "Save Answers", "wpep" ), 'assessment-save' );
    $this->display_table_text_translated_field( __( "Submit Answers", "wpep" ), 'assessment-submit' );
    $this->display_table_text_translated_field( __( "Continue to Course", "wpep" ), 'assessment-go-to-course' );
    $this->display_table_text_translated_field( __( "Return to Course", "wpep" ), 'assessment-return-to-course' );

    $this->display_table_separator( __( "Completed Assessment", "wpep" ) );
    $this->display_table_editor_field( __( "Default Completed Message", "wpep" ), 'assessment-success-message' );
    $this->display_table_text_translated_field( __( "Questions", "wpep" ), 'assessment-questions' );
    $this->display_table_text_translated_field( __( 'Of', "wpep" ), 'assessment-answer-breakdown-separator' );
    $this->display_table_text_translated_field( __( "View Questions", "wpep" ), 'assessment-view-questions' );
    $this->display_table_text_translated_field( __( "Retake Assessment", "wpep" ), 'assessment-retake' );

    $this->display_table_separator( __( "Time Remaining Settings", "wpep" ) );

    $this->display_table_text_translated_field( __( "Time Allowed", "wpep" ), 'assessment-time-allowed' );
    $this->display_table_text_translated_field( __( "Countdown Text", "wpep" ), 'assessment-time-remaining' );
    $this->display_table_editor_field( __( "Introduction Text", "wpep" ), 'assessment-time-text' );
    $this->display_table_text_translated_field( __( "Start Timed Assessment", "wpep" ), 'assessment-time-start' );

    $this->display_table_separator( __( "Status", "wpep" ) );
    $this->display_table_text_translated_field( __( "Status", "wpep" ), 'assessment-status' );
    $this->display_table_text_translated_field( __( "Status Pending", "wpep" ), 'assessment-status-label-pending' );
    $this->display_table_text_translated_field( __( "Status Pending Review", "wpep" ), 'assessment-status-label-pending_review' );
    $this->display_table_text_translated_field( __( "Status Completed", "wpep" ), 'assessment-status-label-completed' );
    $this->display_table_text_translated_field( __( "Status Failed", "wpep" ), 'assessment-status-label-failed' );

    $this->display_table_separator( __( "Status Notification", "wpep" ) );
    $this->display_table_text_translated_field( __( "Pending", "wpep" ), 'assessment-status-pending' );
    $this->display_table_text_translated_field( __( "Pending Review", "wpep" ), 'assessment-status-pending-review' );
    $this->display_table_text_translated_field( __( "Completed", "wpep" ), 'assessment-status-completed' );
    $this->display_table_text_translated_field( __( "Failed", "wpep" ), 'assessment-status-failed' );
    $this->display_table_text_translated_field( __( "Question Correct", "wpep" ), 'assessment-question-status-correct' );
    $this->display_table_text_translated_field( __( "Question Incorrect", "wpep" ), 'assessment-question-status-incorrect' );

    if( defined( "WPEP_BETA_TESTING" ) || defined( "WPEP_BETA_TESTING_TEACHER_NOTIFICATIONS" ) ) {
      $this->display_table_separator( __( "Teacher Notifications", "wpep" ) );

      $this->display_table_select_field(
        __( "Deliver To", "wpep" ),
        'assessment-admin-email-notification-deliver-to',
        [
          0 =>  __( "Don't send Email Notifications", "wpep" )
        ] + wpep_get_users_by_role_as_array_assoc( [ WPEP_USER_ROLE_AUDIENCE_MANAGER, WPEP_USER_ROLE_PLATFORM_MANAGER, 'administrator' ], 'ID', 'user_email' ),
        __( "The Receiver must have an admin account on the site, on one of the following roles : Administrator Audience Manager or Platform Manager.", "wpep" )
      );
      $this->display_table_text_translated_field( __( "Email Title", "wpep" ), 'assessment-admin-email-notification-title' );
      $this->display_table_editor_field(
        __( "Email Content", "wpep" ),
        'assessment-admin-email-notification-content',
        '<h2>' . __( "Assessment Specific Shortcodes", "wpep" ) . '</h2>' .
        '<ul>' .
          '<li>[assessment_name] - ' . __( "Name of the assessment that has been interacted with.", "wpep" ) . '</li>' .
          '<li>[assessment_student_name] - ' . __( "Name of the Student that interacted with the assessment.", "wpep" ) . '</li>' .
          '<li>[assessment_student_status] - ' . __( "Assessment Status of the Student that interacted with the assessment.", "wpep" ) . '</li>' .
          '<li>[assessment_student_score] - ' . __( "Assessment Score of the Student that interacted with the assessment.", "wpep" ) . '</li>' .
          '<li>[assessment_student_answers] - ' . __( "List of Answers the Student has given to each question.", "wpep" ) . '</li>' .
          '<li>[assessment_student_submission_link] - ' . __( "Link to the User Assessment Submission in the Statistics in the admin.", "wpep" ) . '</li>' .
        '</ul>' .
        wpep_controller()->userEngagement->get_available_shortcode_list( false )
      );
    }

    $this->display_table_end();
  }

  public function display_certification() {
    $this->display_table_start();

    $this->display_table_checkbox_field( __('Enable Certifications', "wpep"),  'enable-certifications-support' );

    $this->display_table_separator( __( "Download Box", "wpep" ) );

    $this->display_table_select_field( __( "Placement", "wpep" ), 'certification-download-box-place', [
      'top'     => __( "Before Course Content"  ),
      'bottom'  => __( "After Course Content" )
    ], __( "After earning an certificate, the student can download it from the Course Page, chose where to display the download section.") );

    $this->display_table_text_translated_field( __( "Title", "wpep" ), 'certification-download-box-title' );
    $this->display_table_text_translated_field( __( "Button", "wpep" ), 'certification-download-box-button' );

    $this->display_table_separator( __( "User Profile", "wpep" ) );

    $this->display_table_text_translated_field( __( "Title", "wpep" ), 'certification-user-profile-title' );
    $this->display_table_select_field(
      __( "Show Public", "wpep" ),
      'certification-user-profile-public',
      wpep_flag_setting_field_options(),
      __( "By default we don't show the certificates the student has earned on the public profile link, check this if you want to.", "wpep" )
    );

    $this->display_table_end();
  }

}

