<?php

namespace WPEP\Admin\Coordinator\Settings;

use WPEP\Admin\Coordinator\Settings;
use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;

class General extends Skeleton {

  public $tabs = [];

  public function _setup_tabs() {
    $this->tabs = [
      'design'     => __( 'Design Options', "wpep" ),
      'content'    => __( 'Content Options', "wpep" ),
      'auth'       => __( 'Auth Options', "wpep" ),
      'auth-emails'=> __( 'Auth Emails', "wpep" ),
      'advanced'   => __( 'Advanced Options', "wpep" ),
      'url'        => __( 'URL Settings', "wpep" ),
      'search'     => __( 'Search Settings', "wpep" )
    ];
  }

  public function get_name() {
    return __( 'General', "wpep" );
  }

  public function get_alias() {
    return 'general';
  }

  public function display() {
    $tab = ( isset( $_GET[ 'tab_inner' ] ) ? $_GET[ 'tab_inner' ] : key( $this->tabs ) );

    $this->display_tabs( $tab );

    if( $tab == 'advanced' )
      $this->display_advanced();
    else if ( $tab == 'url' )
      $this->display_url_options();
    else if( $tab == 'auth' )
      $this->display_auth();
    else if( $tab == 'auth-emails' )
      $this->display_auth_emails();
    else if( $tab == 'search' )
      $this->display_search();
    else if( $tab == 'content' )
      $this->display_content_options();
    else //if( $tab == 'design' )
      $this->display_design();
  }

  public function display_design() {
    $default_font_size   = ( isset( WPEP_Controller::instance()->templateStyle->get_current_theme()['defaults']['font_size'] )   ? WPEP_Controller::instance()->templateStyle->get_current_theme()['defaults']['font_size'] . 'px' : ' Error ' );
    $default_font_family = ( isset( WPEP_Controller::instance()->templateStyle->get_current_theme()['defaults']['font_family'] ) ? WPEP_Controller::instance()->templateStyle->get_current_theme()['defaults']['font_family']  : ' Error ' );

    $google_font_list = wpep_get_google_font_list();
    $native_font_list = wpep_get_native_font_list();

    echo $this->get_live_preview_container_start();

    $this->display_table_start();

    $this->display_table_select_field(
      __( 'Theme', "wpep" ),
      'wpep-theme',
      wpep_utility_map_to_array_assoc( WPEP_Controller::instance()->templateStyle->get_theme_list(), 'alias', 'name' )
    );

    $this->display_table_select_field(
      __( 'Base Font Size', "wpep" ),
      'wpep-custom-font-size',
      [
        ''   => 'Default - ' . $default_font_size,
      ] + wpep_get_indexed_number_list(10, 20, __( "%value%px", "wpep" ) )
    );

    $this->display_table_select_field(
      __( 'Font Family', "wpep" ),
      'wpep-custom-font-family',
      [
        ''              => 'Default - ' . $default_font_family,
        'inherit'       => 'Inherit Theme Fonts',
        'native_fonts'  => [
          'group_title'   => __( "Native Fonts", "wpep" ),
          'group_options' => array_combine( $native_font_list, $native_font_list )
        ],
        'google_fonts'  => [
          'group_title'   => __( "Google Fonts", "wpep" ),
          'group_options' => array_combine( $google_font_list, $google_font_list )
        ]
      ],
      false,
      'wpep-select-field wpep-js-select-element'
    );

    $this->display_table_color_field(
      __( 'Primary Color', "wpep" ),
      'wpep-custom-color'
    );

    $this->display_table_color_field(
      __( 'Secondary Color', "wpep" ),
      'wpep-custom-color-secondary',
      __( 'This should be a darker or lighter color similar to the primary one', "wpep" )
    );

    $this->display_table_wrapper_options();

    $this->display_table_separator( __( "Other Options", "wpep" ) );

    $this->display_table_image_field(
      __( "Default Image", "wpep" ),
      'wpep-placeholder-image',
      __( "The default image used within WPEP where one is required, this includes the default Course Image.", "wpep" )
    );

    echo        '<tr id="wpep-table-row-wpep-custom-css">';
    echo          '<th scope="row"><label for="wpep-custom-css">' . __('Custom CSS', "wpep") . '</label></th>';
    echo          '<td>';
    echo            '<textarea id="wpep-custom-css" name="wpep_settings[wpep-custom-css]" style="width:400px;height:100px;">' . WPEP_Settings::instance()->get_option( 'wpep-custom-css', '' ) . '</textarea>';
    echo            '<p>' . __('Enter any custom CSS you want to add into the field above.<br>CSS added here will override the default CSS rules of WPEP.', "wpep") . '</p>';
    echo          '</td>';
    echo        '</tr>';
    

    $this->display_table_end();

    echo    '<p class="submit"><input type="submit" value="' . __( "Save Changes", "wpep" ) . '" class="button button-primary" id="submit" name="wpep_settings_save"></p>';

    WPEP_Controller::instance()->adminController->settings->displayed_form_submit = true;

    echo $this->get_live_preview_container_end();
  }

  public function display_content_options() {
    $this->display_table_start();

    $this->display_table_separator( __( "Content Types", "wpep" ) );

    foreach( wpep_content_type()->switchableContentTypes as $contentType )
      $this->display_table_checkbox_field( $contentType->get_plural_name(),  'content-type-' . $contentType->get_post_type() . '-enabled' );

    $this->display_table_separator( __( "Course Options", "wpep" ) );

    $this->display_table_checkbox_field( __('Assessments', "wpep"),  'enable-course-assessments' );
    $this->display_table_checkbox_field( __('Certifications', "wpep"),  'enable-certifications-support' );
    $this->display_table_checkbox_field( __('Bookmarks', "wpep"),  'enable-course-bookmarks' );
    $this->display_table_checkbox_field( __('Deliverables', "wpep"),  'enable-course-deliverables' );
    $this->display_table_checkbox_field( __('SID Identification', "wpep"),  'enable-sid-support' );
    $this->display_table_checkbox_field( __('Enable Drip Options', "wpep"),  'enable-course-timing-option' );
    $this->display_table_checkbox_field( __('Progress Repeat', "wpep"),  'enable-progress-repeat-support' );

    $this->display_table_separator( __( "Content Modules", "wpep" ) );

    $this->display_table_checkbox_field( __('Enable Author Field', "wpep"),  'enable-post-author-field', __( "Practical when you use multiple Individual Content Managers." ) );
    $this->display_table_select_field(
      __('Post List For Individual Authors', "wpep"),
      'post-list-type-by-individual-author',
      [
        'all'   => __( "See all content created.", "wpep" ),
        'mine'  => __( "See content created by them.", "wpep" )
      ]
    );
    $this->display_table_checkbox_field( __('Enable Standard Categories', "wpep"),  'register-default-categories', __( "Practical when your membership plugin cannot protect Custom Taxonomies, and you want to create Course Bundles trough them." ) );

    $this->display_table_separator( __( "Member Magnet", "wpep" ) );

    $this->display_table_checkbox_field( __('Enable', "wpep"),  'enable-post-member-magnet-option' );
    $this->display_table_checkbox_field(
      __( 'Allow Preview', "wpep"),
      'enable-post-member-magnet-interaction',
      __( "For Execution Plan, Execution Plan Unfolded and Learn Map Course Types Enable Registration or Login upon Content Tab button click action.", "wpep" )
    );
    $this->display_table_select_field(
      __( 'Type', "wpep"),
      'post-member-magnet-type',
     [
       WPEP_MEMBER_MAGNET_TYPE_STANDARD => __( "Standard", "wpep" ),
       WPEP_MEMBER_MAGNET_TYPE_QUICK    => __( "Quick, requests Username / Email Address / Password", "wpep" )
     ]
    );

    $this->display_table_separator( __( "Categories Options", "wpep" ) );

    $this->display_table_checkbox_field(
      __( 'Presentation Card Labels', "wpep"),
      'index-page-category-presentation-labels',
      __( "Enabling this option will allow you to choose the status of a Category to Show as Tag, apart from all the standard functionality, it will also display a tag card on the Presentation Card.", "wpep" )
    );

    $this->display_table_end();
  }

  public function display_advanced() {
    $this->display_table_start();

    $this->display_table_separator( __( "Frontend", "wpep" ) );

    $this->display_table_checkbox_field(
      __( 'Disable Admin Bar', "wpep" ),
      'disable-admin-bar-for-students',
      __( "Disable the admin bar on the frontend for non-admin users, Administrators & Platform Managers will continue to see it.", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Custom CSS File', "wpep" ),
      'wpep-custom-css-by-file',
      __( "WPEP will generate a Custom CSS file to deliver the Color Variation & Custom CSS, disable this option to embed them directly on page if you encounter CSS problems.", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Enforce RTL Support', "wpep" ),
      'enforce-rtl-support',
      __( "NOTE: By default we're enabling RTL support if WordPress has been configured for RTL.", "wpep" )
    );

    $this->display_table_separator( __( "Administration", "wpep" ) );

    $this->display_table_select_field(
      __( 'Menu Position', "wpep" ),
      'menu-position',
      [
        3 => "Default, at the top",
        2 => "Under Dashboard",
        4 => "Under Dashboard Separator",
        5 => "Under Posts",
        10 => "Under Media",
        15 => "Under Links",
        20 => "Under Pages",
        25 => "Under Comments",
        59 => "Under Separator",
        60 => "Under Appearance",
        65 => "Under Plugins",
        70 => "Under Users",
        75 => "Under Tools",
        80 => "Under Settings",
        99 => "Under Bottom Separator",
      ]
    );

    if( wpep_content_type()->is_enabled( WPEP_POST_TYPE_COURSE ) ) {

      $this->display_table_select_field(
        __('New Course Sections', "wpep"),
        'admin-course-preset-sections',
        [
          1 => 1,
          2 => 2,
          3 => 3,
          4 => 4,
          5 => 5
        ],
        __('Choose the number of Sections that are automatically Pre-Loaded for you when adding a new Course.', "wpep")
      );

      $this->display_table_select_field(
        __('New Course Lessons', "wpep"),
        'admin-course-preset-lessons',
        [
          1 => 1,
          2 => 2,
          3 => 3,
          4 => 4,
          5 => 5,
          6 => 6,
          7 => 7,
          8 => 8,
          9 => 9,
          10 => 10
        ],
        __('Choose the number of Lessons that are automatically Pre-Loaded into Sections when adding a new Course.', "wpep")
      );

      $this->display_table_checkbox_field(
        __('Course Save Alert', "wpep"),
        'admin-course-save-alert',
        __("Enable keep your content save alert while editing a course.", "wpep")
      );

      $this->display_table_checkbox_field(
        __('Instant Lesson Editor', "wpep"),
        'lesson-wp-editor-preload',
        __('Disable this if your device behaves poorly while editing lessons.', "wpep")
      );

      $this->display_table_checkbox_field(
        __('Open New Lessons Editor', "wpep"),
        'lesson-wp-editor-enabled-new',
        __('Check this to automate your wysiwyg text editor to open by default when adding a new lesson.', "wpep")
      );

      $this->display_table_checkbox_field(
        __('Editor Integration', "wpep"),
        'admin-editor-integration',
        __('Check this to enable the WPEP editor improvements and shortcode builder.', "wpep")
      );


      $this->display_table_select_field(
        __('Text Editor', "wpep"),
        'wpep-editor-setup',
        $this->_get_wpep_editor_setup_options(),
        __('Chose the editor that will be displayed everywhere within WPEP.', "wpep")
      );

    }

    $this->display_table_end();
  }

  public function display_url_options() {
    $this->display_table_start();

    $this->display_table_separator( __( "Permalink Name", "wpep" ) );

    foreach( wpep_content_type()->libraryContentTypes as $libraryContentType ) {
      $this->display_table_text_field(
        $libraryContentType->get_plural_name(),
        'content-type-' . $libraryContentType->get_post_type() . '-rewrite-slug',
        sprintf( __( "The permalink prefix goes here, by default %s", "wpep" ), $libraryContentType->get_post_type() )
      );
    }

    if( wpep_assessments_enabled() ) {
      $this->display_table_text_field(
        __( 'Assessment', "wpep" ),
        'content-type-assessment-rewrite-slug',
        __( "The permalink prefix goes here, by default wpep_assessment", "wpep" )
      );
    }

    $this->display_table_text_field(
      __( 'Categories', "wpep" ),
      'rewrite-slug-categories',
      __( "The permalink prefix goes here, by default course_category", "wpep" )
    );

    $this->display_table_separator( __( "Archive Page", "wpep" ) );

    foreach( wpep_content_type()->libraryContentTypes as $libraryContentType ) {
      $this->display_table_checkbox_field(
        $libraryContentType->get_plural_name(),
        'content-type-' . $libraryContentType->get_post_type() . '-enable-archive',
        sprintf( __( "By default WordPress will create the Course Archive Page under /%s/ you can disable this here", "wpep" ), $libraryContentType->get_post_type() )
      );
    }

    $this->display_table_end();
  }

  public function display_auth() {
    $this->display_table_start();

    $this->display_table_separator( __( "General Settings", "wpep" ) );

    $this->display_table_select_field(
      __( 'Internal Auth Status', "wpep"),
      'user-auth-status-level',
      wpep_get_auth_status_level_options(),
      __( 'By default WPEP does not enforce users to login. Your WP Membership plugin will usually take care of this..', "wpep" )
    );

    $this->display_table_select_field(
      __( "Registration Status", "wpep" ),
      'user-auth-enable-registration',
      [
        0 => __( 'Inactive', "wpep" ),
        1 => __( 'Active', "wpep" ),
        2 => __( "Active - Display Registration instead of login", "wpep" )
      ]
    );

    $this->display_table_select_field(
      __( "Enable Marketing Content", "wpep" ),
      'user-auth-sidebar-marketing-status',
      [
        0 => __( "Disabled", "wpep" ),
        1 => __( "Enabled", "wpep" ),
        2 => __( "Enabled, hidden on register", "wpep" )
      ],
      __( "Enable a Marketing text on the right side of the Authentication Login Page. This applies only for the authentication page that WPEP creates for you..", "wpep" )
    );

    $this->display_table_editor_field(
      __( "Marketing Content", "wpep" ),
      'user-auth-sidebar-marketing-content'
    );

    $this->display_table_separator( __( "Form Fields & Actions", "wpep" ) );

    $this->display_table_text_translated_field(
      __( "Login Identifier", "wpep" ),
      'user-auth-field-login-identifier',
      __( "Username or Email Address", "wpep" )
    );

    $this->display_table_text_translated_field(
      __( "Username", "wpep" ),
      'user-auth-field-username'
    );
    $this->display_table_text_translated_field(
      __( "Email Address", "wpep" ),
      'user-auth-field-email-address'
    );
    $this->display_table_text_translated_field(
      __( "First Name", "wpep" ),
      'user-auth-field-first-name'
    );
    $this->display_table_text_translated_field(
      __( "Last Name" ),
      'user-auth-field-last-name'
    );
    $this->display_table_text_translated_field(
      __( "Password", "wpep" ),
      'user-auth-field-password'
    );
    $this->display_table_text_translated_field(
      __( "Remember me", "wpep" ),
      'user-auth-field-remember-me'
    );
    $this->display_table_text_translated_field(
      __( "Register Account", "wpep" ),
      'user-auth-open-tab-register'
    );
    $this->display_table_text_translated_field(
      __( "Reset Password", "wpep" ),
      'user-auth-open-tab-reset-password'
    );
    $this->display_table_text_translated_field(
      __( "I already have an Account", "wpep" ),
      'user-auth-open-tab-login'
    );
    $this->display_table_text_translated_field(
      __( "Return to Login", "wpep" ),
      'user-auth-return-to-login'
    );

    $this->display_table_text_translated_field(
      __( "Login", "wpep" ),
      'user-auth-operation-login'
    );
    $this->display_table_text_translated_field(
      __( "Register", "wpep" ),
      'user-auth-operation-register'
    );
    $this->display_table_text_translated_field(
      __( "Reset Password", "wpep" ),
      'user-auth-operation-reset-password'
    );

    $this->display_table_separator( __( "Other Fields", "wpep" ) );

    if( wpep_has_sid_support() ) {
      $this->display_table_text_translated_field(
        __( "SID", "wpep" ),
        'user-auth-field-sid'
      );
    }

    $this->display_table_text_translated_field(
      __( "Location", "wpep" ),
      'user-auth-field-location'
    );

    $this->display_table_text_translated_field(
      __( "Website", "wpep" ),
      'user-auth-field-website'
    );

    $this->display_table_separator( __( "Form Notifications", "wpep" ) );

    $this->display_table_text_translated_field(
      __( "Registration Notice", "wpep" ),
      'user-auth-notification-please-register',
      __( "Displays this message when the form defaults to register.", "wpep" )
    );
    $this->display_table_text_translated_field(
      __( "Login Details Required", "wpep" ),
      'user-auth-notification-login-details-required'
    );
    $this->display_table_text_translated_field(
      __( "Incorrect Password", "wpep" ),
      'user-auth-notification-password-incorrect'
    );
    $this->display_table_text_translated_field(
      __( "Successfully Logged In", "wpep" ),
      'user-auth-notification-logged-in'
    );
    $this->display_table_text_translated_field(
      __( "Account Registered", "wpep" ),
      'user-auth-notification-account-registered'
    );
    $this->display_table_text_translated_field(
      __( "Password Reset", "wpep" ),
      'user-auth-notification-password-reset'
    );
    $this->display_table_text_translated_field(
      __( "Password Reset Not Registered", "wpep" ),
      'user-auth-notification-recovery-not-registered'
    );
    $this->display_table_text_translated_field(
      __( "Already Logged In", "wpep" ),
      'user-auth-notification-already-logged-in'
    );
    $this->display_table_text_translated_field(
      __( "Notification Page Expired", "wpep" ),
      'user-auth-notification-page-expired'
    );

    $this->display_table_end();
  }

  public function display_auth_emails() {
    $this->display_table_start();

    $this->display_table_separator( __( "Password Reset Email", "wpep" ) );

    $this->display_table_checkbox_field(
      __( "Enable Globally", "wpep" ),
      'email-reset-password-override-filter',
      __( "By default only the WPEP Authentication Form Password reset will use this email content.", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( "Use Delivery Template", "wpep" ),
      'email-reset-password-use-template',
      __( "Enable the WPEP Email Template when sending the password reset email notification - this applies for the global setting above, reset emails from the authentication form will always use the template.", "wpep" )
    );

    $this->display_table_text_field(
      __( "Email Title", "wpep" ),
      'email-reset-password-title',
      __( "%s will be replaced with the website name, using it is strictly optional, and is there to make the email personalized by default", "wpep" )
    );

    $this->display_table_editor_field(
      __( "Email Content", "wpep" ),
      'email-reset-password-content',
      '<h2>' . __( "Available ShortCodes") . '</h2>' .
      '<ul>' .
        '<li>[site_url] - ' . __( "The Site Link", "wpep" ) . '</li>' .
        '<li>[username] - ' . __( "The username of the student that has requested a password reset", "wpep" ) . '</li>' .
        '<li>[password_reset] - ' . __( "The password reset link", "wpep" ) . '</li>' .
      '</ul>'
    );

    $this->display_table_end();
  }

  public function display_search() {
    $this->display_table_start();

    $this->display_table_checkbox_field(
      __( "Image Ribbon" ),
      'search-image-ribbon',
      __( "Enable this option to display the presentation card ribbon text on the image.", "wpep" )
    );

    $this->display_table_text_field(
      __( "Max Results", "wpep" ),
      'search-results-number',
      __( "The maximum number of results displayed when a search is made.", "wpep" )
    );
    $this->display_table_text_translated_field(
      __( "No Results Found", "wpep" ),
      'search-no-results-found'
    );
    $this->display_table_text_translated_field(
      __( "Search Placeholder", "wpep" ),
      'search-input-placeholder'
    );

    $this->display_table_end();
  }

  private function _get_wpep_editor_setup_options() {
    return [
      'default'  => 'Default WP Editor - Visual',
      'visual'   => 'Visual WP Editor',
      'html'     => 'Default WP Editor - HTML',
      'textarea' => 'Textarea Only'
    ];
  }

}
