<?php

namespace WPEP\Admin\Coordinator\Settings;

use WPEP\Settings   as WPEP_Settings;

class IndexPage extends Skeleton {

  public $tabs = [];

  public function _setup_tabs() {
    $this->tabs = [
      'general'   => __( 'General Options', "wpep" )
    ];

    $libraryContentTypes = wpep_content_type()->libraryContentTypes;

    if( is_array( $libraryContentTypes ) && !empty( $libraryContentTypes ) ) {
      foreach( $libraryContentTypes as $post_type => $contentType ) {
        $this->tabs[ $contentType->get_post_type() ] = sprintf( __( '%s Options', "wpep" ), $contentType->get_singular_name() );
      }
    }

    $this->tabs['grid'] = __( 'Grid Options', "wpep" );
  }

  public function get_name() {
    return __( 'Index Page', "wpep" );
  }

  public function get_alias() {
    return 'index_page';
  }

  public function display() {
    $tab = ( isset( $_GET[ 'tab_inner' ] ) ? $_GET[ 'tab_inner' ] : key( $this->tabs ) );

    $this->display_tabs( $tab );

    if( $tab == 'grid' )
      $this->display_grid_options();
    else if( isset( wpep_content_type()->libraryContentTypes[ $tab ] ) )
      wpep_content_type()->libraryContentTypes[ $tab ]->display_settings_fields( $this );
    else
      $this->display_general_options();

  }

  public function display_general_options() {
    $this->display_table_start();

    echo        '<tr class="wpep-table-row">';
    echo          '<th scope="row"><label for="wpep-opt-course-index">' . __('Index Page', "wpep") . '</label></th>';
    echo          '<td>';
    echo            '<select id="wpep-opt-course-index" name="wpep_settings[opt-course-index]" class="wpep-select-field">';
    echo              '<option value="0">' .
                         __( ( WPEP_Settings::instance()->get_option( 'opt-course-index', 0 ) == 0 ? "Select a Page" : "Ignore This Feature" ), "wpep" ) .
                      '</option>';

    foreach( get_posts( [ "post_type" => 'page', 'numberposts' => -1 ] ) as $page ) {
      echo '<option value="' . $page->ID . '" ' . ( selected( WPEP_Settings::instance()->get_option( 'opt-course-index', 0 ), $page->ID, false ) ) . '>' .
              $page->post_title .
           '</option>';
    }

    echo            '</select>';
    echo            '<p>' . sprintf(
                            __( 'You can also use %s to display the course grid on any page.', "wpep" ),
                            wpep_admin_shortcode_builder(
                              'wpep_index',
                              [ 'wpep_admin_shortcode_builder_wpep_index_options' ],
                              '[wpep_index]',
                              false
                            )
                           ). '</p>';
    echo          '</td>';
    echo        '</tr>';

    $this->display_table_select_field(
      'Categories Per Row',
      'index-page-categories-per-row',
      wpep_content_categories_row_options()
    );

    $this->display_table_text_translated_field ( __('Secondary Filter', "wpep"),  'opt-course-secondary-filter', __( 'Personalize the text of the secondary category level filter.', "wpep" ) );

    $this->display_table_select_field( __( "Single Category View", "wpep" ), 'index-page-single-category-primary-view', [
      0 => __( "Show secondary category navigation" ),
      1 => __( "Show primary category navigation" )
    ] );

    $this->display_table_separator( __( "All Categories Filter", "wpep" ) );

    $this->display_table_select_field(
      __( 'Status', "wpep"),
      'index-page-enable-filter-all-categories',
      wpep_flag_setting_field_options(),
      __( "Add an Category Filter tab that will show content from all categories.", "wpep" )
    );

    $this->display_table_text_translated_field( __('Title', "wpep"),  'index-page-filter-all-categories-title' );

    $this->display_table_image_field( __('Image', "wpep"),  'index-page-filter-all-categories-image' );

    $this->display_table_separator( __( "Introduction Text", "wpep" ) );

    $this->display_table_text_field( __('Main heading', "wpep"),  'opt-course-heading', __( 'Enter the main heading of your course index page.', "wpep" ) );

    $this->display_table_editor_field(
      __('Sub Heading', "wpep"),
      'opt-course-sub-heading',
      __('Enter the sub heading of your course index page.', "wpep")
    );

    $this->display_table_separator( __( "Filtering", "wpep" ) );

    $this->display_table_text_translated_field(
      __( 'Search', "wpep"),
      'index-page-filter-search'
    );

    $this->display_table_checkbox_field(
      __( 'Require All Categories', "wpep"),
      'index-page-categories-filter-require-all',
      __( "When filtering by categories, the content must be placed in all categories to be displayed", "wpep" )
    );

    if( defined( "WPEP_BETA_TESTING" ) || defined( "WPEP_BETA_PAGINATION" ) ) {

      $this->display_table_separator( __( "Pagination", "wpep" ) );

      $this->display_table_select_field(
        __( 'Status', "wpep"),
        'index-page-enable-pagination',
        [
          0 => __( 'Inactive', "wpep" ),
          1 => __( 'Active', "wpep" )
        ],
        __( "Enabling Pagination will speed up the Index Page load time dramatically.", "wpep" ),
        'wpep-select-field',
        [
          'data-wpep-conditional-display' => 'data-wpep-ip-enable-pagination'
        ]
      );

      $this->set_table_row_data_attributes( [
        'wpep-ip-enable-pagination'  => 1
      ] );

      $this->display_table_text_field(
        __( 'Entries Per Page', "wpep"),
        'index-page-entries-per-page'
      );

      $this->set_table_row_data_attributes([]);
    }

    $this->display_table_end();
  }

  public function display_grid_options() {
    $this->display_table_start();

    $this->display_table_checkbox_field( __( 'Enforce Ordering', "wpep" ), 'index-page-enforce-ordering', __('Check this to resolve ordering issues on the Index Page', "wpep" ) );

    echo '<tr>';
    echo    '<td colspan="2" class="wpep-entity-order-functionality-container">';

    $tabs_list = wpep_content_type()->library_post_type_list() + [
        WPEP_TAXONOMY_CATEGORY => __( "Categories", "wpep" )
      ];

    $tab = ( isset( $_GET[ 'tab_inner_section' ] ) ? $_GET[ 'tab_inner_section' ] : false );

    if( $tab === false || !isset( $tabs_list[ $tab ] ) )
      $tab = key( $tabs_list );

    $this->display_tabs_section( $tabs_list, $tab, 'grid' );

    if( post_type_exists( $tab ) )
      $this->_post_ordering_functionality( $tab, $tabs_list[ $tab ] );
    else if( taxonomy_exists( $tab ) )
      $this->_taxonomy_ordering_functionality( $tab, $tabs_list[ $tab ] );

    echo    '</td>';
    echo '</tr>';

    $this->display_table_end();
  }

  private function _post_ordering_functionality( $post_type, $post_name = false ) {
    global $wpdb;

    $post_name = ( $post_name == false ? wpep_alias_to_label( $post_name ) : $post_name );
    $wpep_categories_map = wpep_utilities_map_object( get_terms( [ 'taxonomy' => WPEP_TAXONOMY_CATEGORY, 'hide_empty' => false ] ), 'term_id' );

    $query = 'SELECT p.ID,
                     p.post_title,
                     CONVERT( IFNULL( pom.meta_value, 0 ), UNSIGNED INTEGER ) as wpep_order,
                     IFNULL( ( SELECT GROUP_CONCAT(ptr.term_taxonomy_id SEPARATOR "," ) FROM ' . $wpdb->term_relationships . ' ptr WHERE ptr.object_id = p.ID ), "" ) as wpep_categories
                       FROM ' . $wpdb->posts . ' p
                  LEFT JOIN ' . $wpdb->postmeta . ' pom ON pom.post_id = p.ID AND pom.meta_key = "wpep_order"
                      WHERE post_type = "' . $post_type . '" 
                        AND post_status IN ("' . implode( '","', wpep_post_monitored_post_status_list() ). '")
                   ORDER BY wpep_order ASC, p.post_date DESC';

    $post_list = $wpdb->get_results( $query );

    if( !empty( $post_list ) ) {
      echo '<ul class="wpep-entity-order-functionality">';

      $position = 0;
      foreach( $post_list as $post ) {
        echo '<li>';
        echo   '<div>';
        echo   '<input type="hidden" name="wpep_settings_post_order[' . $post->ID . ']" value="' . $position . '">';
        echo   '<span class="wpep-title">' . $post->post_title . '</span>';

        if( $post->wpep_categories !== "" ) {
          $cat_ids = explode( ",", $post->wpep_categories );

          echo '<div class="wpep-post-categories">';

          foreach( $cat_ids as $category_id ) {
            if( !isset( $wpep_categories_map[ $category_id ] ) )
              continue;

            echo '<span>' . $wpep_categories_map[$category_id]->name . '</span>';
          }

          echo '</div>';
          echo '</div>';
        }

        echo '</li>';

        $position++;
      }

      echo '</ul>';
    } else {
      echo  '<p class="wpep-notification wpep-notification-info">' . __('No ' . $post_name . ' available to order.', 'wpep') . '</p>';
    }
  }

  private function _taxonomy_ordering_functionality( $taxonomy, $taxonomy_name = false ) {
    $taxonomy_name = ( $taxonomy_name == false ? wpep_alias_to_label( $taxonomy_name ) : $taxonomy_name );

    $taxonomy_list = get_terms( [ 'taxonomy' => $taxonomy, 'parent' => 0, 'hide_empty' => false ] );

    if( !empty( $taxonomy_list ) ) {
      $this->_taxonomy_single_ordering_functionality( $taxonomy_list, 0 );
    } else {
      echo  '<p class="wpep-notification wpep-notification-info">' . __('No ' . $taxonomy_name . ' available to order.', 'wpep') . '</p>';
    }
  }

  private function _taxonomy_single_ordering_functionality( $taxonomy_list, $position = 0, $wpep_attribute = 'top' ) {
    if( empty( $taxonomy_list ) )
      return $position;

    $taxonomy_list = wpep_term_list_ordering( $taxonomy_list );

    echo '<ul class="wpep-entity-order-functionality">';

    $position = 0;
    foreach( $taxonomy_list as $taxonomy_item ) {
      echo '<li data-wpep-taxonomy-level="' . $wpep_attribute . '">';
      echo   '<div>';
      echo    '<input type="hidden" name="wpep_settings_taxonomy_order[' . $taxonomy_item->term_id . ']" value="' . $position . '">';
      echo    '<span class="wpep-title">' . $taxonomy_item->name . '</span>';
      echo    '</div>';

      $position = $this->_taxonomy_single_ordering_functionality(
        get_terms( [ 'taxonomy' => $taxonomy_item->taxonomy, 'parent' => $taxonomy_item->term_id, 'hide_empty' => false ] ),
        ++$position,
        'secondary'
      );

      echo '</li>';
    }

    echo '</ul>';

    return $position;
  }

}
