<?php

namespace WPEP\Admin\Coordinator\Settings;

use WPEP\Controller as WPEP_Controller;

class LearnerEngagement extends Skeleton {

  public $tabs = [];

  public function _setup_tabs() {
    $this->tabs = [
      'data_privacy'             => __( 'Data & Privacy', 'wpep' ),
      'delivery_settings'        => __( 'Delivery Settings', "wpep" ),
      'weekly_recap'             => __( 'Weekly Recap', "wpep" ),
      'discussions'              => __( "Discussions", "wpep" )
    ];

    if( defined( "WPEP_BETA_TESTING" ) )
      $this->tabs['auto-login-links'] = __( "Auto Login Links", "wpep" );

    if( wpep_has_embed_support() )
      $this->tabs[ 'share_embed' ] = __( 'Sharing & Embed', "wpep" );
  }

  public function get_name() {
    return __( 'Learner Engagement', "wpep" );
  }

  public function get_alias() {
    return 'learner_engagement';
  }

  public function display() {
    $current_tab = $this->get_current_tab();

    $this->display_tabs( $current_tab );

    if( $current_tab == 'weekly_recap' )
      $this->display_weekly_recap_options();
    else if( $current_tab == 'share_embed' )
      $this->display_share_embed_options();
    else if( $current_tab == 'discussions' )
      $this->display_discussion_options();
    else if( $current_tab == 'data_privacy' )
      $this->display_data_privacy_options();
    else if( $current_tab == 'auto-login-links' )
      $this->display_auto_login_links();
    else
      $this->display_delivery_settings_options();
  }

  public function get_current_tab() {
    $current_tab = ( isset( $_GET['tab_inner'] ) ? $_GET['tab_inner'] : key( $this->tabs ) );

    if( !isset( $this->tabs[ $current_tab ] ) )
      $current_tab = key( $this->tabs );

    return $current_tab;
  }

  public function display_weekly_recap_options() {
    $this->display_table_start();

    $this->display_table_select_field(
      __( 'Status', "wpep" ) ,
      'email-weekly-recap-status',
      [
        0 => __( 'Inactive', "wpep" ),
        1 => __( 'Active', "wpep" )
      ]
    );

    $weekly_recap_user_ids = get_option( 'wpep_weekly_recap_que', [] );

    if( in_array( get_current_user_id(), $weekly_recap_user_ids ) ) {
      $weekly_recap_preview_content = '<div class="wpep-notification wpep-notification-success">' .
                                        sprintf( __( "Weekly Recap Test has been added to the que, you should receive an email at %s within 5 minutes.", "wpep" ), wp_get_current_user()->user_email ) .
                                      '</div>';
    } else {
      $weekly_recap_preview_content = '<p class="wpep-operations-line-container">';
      $weekly_recap_preview_content .=  '<a class="wpep-button wpep-button-primary" target="_blank" href="' . add_query_arg( 'wpep-preview-email', 'weekly-recap-browser', get_site_url() ) . '">' . __( "Preview", "wpep" ) . '</a>' .
                                        '<a class="wpep-button wpep-button-secondary" href="' . add_query_arg( 'wpep-preview-email', 'weekly-recap-instant-email', get_site_url() ) . '">' . __( "Send Instant Email", "wpep" ) . '</a>' .
                                        '<a class="wpep-button wpep-button-secondary" href="' . add_query_arg( 'wpep-preview-email', 'weekly-recap-email', get_site_url() ) . '">' . __( "Send Email", "wpep" ) . '</a>';
      $weekly_recap_preview_content .= '</p>';

    }

    $this->display_table_separator(
      __( 'Preview', "wpep" ),
      $weekly_recap_preview_content
    );

    $this->display_table_text_translated_field( __( 'Title', "wpep" ),  'email-weekly-recap-title', __( 'This is the email title, you can optionally use shortcodes, for example [user_name], [date]', "wpep" ) );

    $this->display_table_editor_field(
      __('Content', "wpep"),
      'email-weekly-recap-content',
      wpep_controller()->userEngagement->get_available_shortcode_list( true ),
      300
    );

    $this->display_table_end();
  }

  public function display_delivery_settings_options() {
    $this->display_table_start();

    $this->display_table_separator( __( 'Branding', "wpep" ) );

    $this->display_table_image_field(
      __( 'Email Logo', "wpep" ),
      'email-logo',
      __( 'The Logo used in the Email Template', "wpep" ),
      false,
      null,
      [
        'width'  => WPEP_EMAIL_LOGO_WIDTH,
        'height' => WPEP_EMAIL_LOGO_HEIGHT
      ]
    );

    $this->display_table_text_field( __( 'Company Name', "wpep" ), 'email-company-name' );
    $this->display_table_text_field( __( 'Company Address', "wpep" ), 'email-company-address' );
    $this->display_table_text_field( __( 'Copyright', "wpep" ), 'email-copyright-text' );

    $this->display_table_separator( __( 'Email Delivery', "wpep" ) );

    $this->display_table_text_field( __( 'From Name', "wpep" ), 'email-from-name', __( 'The default name will be used if left empty.', "wpep" ) );
    $this->display_table_text_field( __( 'From Email', "wpep" ), 'email-from-email',
      '<p>' . __( 'The default email address will be used if left empty.', "wpep" ) . '</p>' .
      '<p>' . __( 'Note : To avoid getting the emails flagged as spam, you must use an address from the current domain.', 'wpep' ) . '</p>'
    );

    $this->display_table_separator( __( 'Email Delivery', "wpep" ) );

    $token = ( wpep_get_setting( 'email-tracking-salt' ) !== '' ? NULL : wpep_generate_random_token( 24 ) );

    $this->display_table_text_field(
      __( 'Encryption Token', "wpep" ),
      'email-tracking-salt',
      ( $token !== null ? '' : __( 'Changing this setting will invalidate all existing tracking events, existent data stays.', "wpep" ) ),
      false,
      $token
    );

    $this->display_table_end();
  }

  public function display_discussion_options() {
    $this->display_table_start();

    $this->display_table_separator( __( 'List', "wpep" ) );

    $this->display_table_text_field( __( 'Search', "wpep" ), 'comments-search-label' );
    $this->display_table_text_field( __( 'Show My Comments', "wpep" ), 'comments-filter-mine', __( "The text displayed alongside to display only the comments the current user has posted", "wpep" ) );
    $this->display_table_text_field( __( 'Comments Per Page', "wpep" ), 'comments-per-page' );
    $this->display_table_text_field( __( 'Comment Replies Per Page', "wpep" ), 'comment-replies-per-page' );
    $this->display_table_text_field( __( 'Show More Replies', "wpep" ), 'comment-replies-show-more', __( "%s will be replaced with the number of replies that haven't been shown already.", "wpep" ) );
    $this->display_table_text_field( __( 'Reply', "wpep" ), 'comments-button-reply' );
    $this->display_table_text_field( __( 'Previous', "wpep" ), 'comments-navigation-previous' );
    $this->display_table_text_field( __( 'Next', "wpep" ), 'comments-navigation-next' );
    $this->display_table_text_field( __( 'Authority Badge', "wpep" ), 'comments-authority-badge-name' );

    $this->display_table_separator( __( 'List Admin', "wpep" ) );

    $this->display_table_text_field( __( 'Delete', "wpep" ), 'comments-button-delete' );
    $this->display_table_text_field( __( 'Confirm Delete', "wpep" ), 'comments-button-confirm-delete' );
    $this->display_table_text_field( __( 'Feature Comment', "wpep" ), 'comments-button-feature-comment' );

    $this->display_table_separator( __( 'Form', "wpep" ) );

    $this->display_table_text_field( __( 'Content Required', "wpep" ), 'comments-notification-content-empty' );
    $this->display_table_text_field( __( 'Cancel Reply', "wpep" ), 'comments-button-cancel-reply' );
    $this->display_table_text_field( __( 'Submit Comment', "wpep" ), 'comments-button-submit' );
    $this->display_table_text_field( __( 'Submit Reply', "wpep" ), 'comments-button-reply-submit' );

    $this->display_table_separator( __( 'Notifications', "wpep" ) );

    $this->display_table_text_field(
      __( 'Manage Notifications Title', "wpep" ),
      'comments-discussion-notifications-title',
      __( "Displayed on the User Profile, allowing students to unsubscribe from Email Notifications on various discussions", "wpep" )
    );

    $this->display_table_separator( __( 'Notifications Email', "wpep" ) );

    $this->display_table_text_field(
      __( 'Title', "wpep" ),
      'comments-email-notification-title'
    );

    $this->display_table_editor_field(
      __( 'Content', "wpep" ),
      'comments-email-notification-content',
      '<h2>' . __( "Discussion ShortCodes") . '</h2>' .
      '<ul>' .
        '<li>[discussion_activity_preview] - ' . __( "Displays a Preview of all the new Activity within Discussions.", "wpep" ) . '</li>' .
        '<li>[discussion_activity_number] - ' . __( "The Number of Discussions there's new activity within", "wpep" ) . '</li>' .
      '</ul>' .
      wpep_controller()->userEngagement->get_available_shortcode_list( false )
    );

    $this->display_table_select_field(
      __( 'Frequency', "wpep" ),
      'comments-email-notification-frequency',
      wpep_discussion_notifications_intervals(),
      __( "When there's activity within a Discussion, students will be notified maximum once per frequency cycle.", "wpep" )
    );

    $this->display_table_text_field(
      __( 'View Discussion', "wpep" ),
      'comments-email-notification-button-view',
      __( "The [discussion_activity_preview] shortcode will display a button for each Discussion.", "wpep" )
    );

    if( defined( "WPEP_BETA_TESTING" ) || defined( "WPEP_BETA_TESTING_COMMENTS_USER_LIST" ) ) {
      $this->display_table_separator( __( 'User List', "wpep" ) );

      $this->display_table_checkbox_field( __( "Show User List", "wpep" ), 'comments-user-list-status', __( "Display a list of students who've engaged in the discussion at the top", "wpep" ) );

      $this->display_table_text_field( __( 'Title', "wpep" ),  'comments-user-list-title' );

      $this->display_table_text_field( __( 'View Profile', "wpep" ),  'comments-user-list-view-profile', __( "Students need to have a Public Profile in order for other students to have the rights to view them.", "wpep" ) );

      $this->display_table_text_field( __( 'Show List', "wpep" ), 'comments-user-list-show-container' );
    }

    $this->display_table_separator( __( 'Advanced Settings', "wpep" ) );
    $this->display_table_text_field( __( "Sync Comments", "wpep" ), 'comments-refresh-ms-interval',
      '<p>' . __( "In seconds, how frequent WPEP should check if new comments were posted while the page is open for a student, minimum 3.", "wpep" ) . '</p>' .
      '<p>' . __( "Pro Tip : This process is highly optimized to not overlap requests.", "wpep" ) . '</p>'
    );

    $this->display_table_end();
  }

  public function display_data_privacy_options() {
    $this->display_table_start();

    $this->display_table_separator( __( 'Terms of Service', "wpep" ) );

    $this->display_table_select_field(
      __( "Status", "wpep" ),
      'user-service-agreement-enabled',
      wpep_flag_setting_field_options(),
      __( "Require Students to comply with the terms of service.", "wpep" )
    );

    $this->display_table_text_field(
      __( "Title", "wpep" ),
      'user-service-agreement-title'
    );

    $this->display_table_editor_field(
      __( "Description", "wpep" ),
      'user-service-agreement-content'
    );

    $this->display_table_text_field(
      __( "Error Notification", "wpep" ),
      'user-service-agreement-error'
    );

    $this->display_table_date_field(
      __( "Last Change", "wpep" ),
      'user-service-agreement-last-change',
      '<p>' . __( "If you require your students to re-confirm compliance after changes where made to the policy, change this date, anyone who approved the terms before will be asked to approve again.", "wpep" ) . '</p>' .
      '<p>' . __( "Leave this empty to ignore this feature", "wpep" ) . '</p>',
      false,
      wpep_format_date( $this->get_value( 'user-service-agreement-last-change' ) ),
      __( "Never", "wpep" ),
      [
        'wpep-date-format' => wpep_date_format_to_js()
      ]
    );

    $this->display_table_separator( __( 'Privacy Policy', "wpep" ) );

    $this->display_table_select_field(
      __( "Status", "wpep" ),
      'user-privacy-agreement-enabled',
      wpep_flag_setting_field_options(),
      __( "Require Students to comply with the privacy policy terms.", "wpep" )
    );

    $this->display_table_text_field(
      __( "Title", "wpep" ),
      'user-privacy-agreement-title'
    );

    $this->display_table_editor_field(
      __( "Description", "wpep" ),
      'user-privacy-agreement-content'
    );

    $this->display_table_text_field(
      __( "Error Notification", "wpep" ),
      'user-privacy-agreement-error'
    );

    $this->display_table_date_field(
      __( "Last Change", "wpep" ),
      'user-privacy-agreement-last-change',
      '<p>' . __( "If you require your students to re-confirm compliance after changes where made to the policy, change this date, anyone who approved the terms before will be asked to approve again.", "wpep" ) . '</p>' .
      '<p>' . __( "Leave this empty to ignore this feature", "wpep" ) . '</p>',
      false,
      wpep_format_date( $this->get_value( 'user-privacy-agreement-last-change' ) ),
      __( "Never", "wpep" ),
      [
        'wpep-date-format' => wpep_date_format_to_js()
      ]
    );

    $this->display_table_separator( __( 'General Settings', "wpep" ) );

    $this->display_table_editor_field(
      'Intermediate Step',
      'user-agreement-introduction-text',
      '<p>' . __( "If you've made changes to your Terms of Service or Privacy Policy & set the Last Change Date accordingly, the students wil be asked to confirm them again.", "wpep" ) . '</p>' .
      '<p>' . __( "Text displayed before the Terms of Service or Privacy Policy.", "wpep" ) . '</p>'
    );

    $this->display_table_text_field(
      'Accept Terms',
      'user-agreement-button-accept'
    );

    $this->display_table_text_field(
      'Decline Terms',
      'user-agreement-button-decline'
    );

    $this->display_table_text_field(
      'Decline Terms Redirect',
      'user-agreement-button-decline-page',
      __( "The page where the user will be taken when he declines Terms of Service or Privacy Policy", "wpep" ),
      false,
      null,
      get_site_url()
    );

    $this->display_table_separator( __( 'Account Delete', "wpep" ) );

    $this->display_table_select_field(
      __( "Status", "wpep" ),
      'user-account-delete-enabled',
      wpep_flag_setting_field_options(),
      '<p>' . __( "Allow Students to delete their account permanently from the User Profile.", "wpep" ) . '</p>' .
      '<p>' . __( "WPEP is compliant with any other plugin that deletes an User Account, and will follow-up by deleting data it stores", "wpep" ) . '</p>'
    );

    $this->display_table_text_field(
      __( "Button", "wpep" ),
      'user-account-delete-button'
    );

    $this->display_table_editor_field(
      __( "Confirmation Message", "wpep" ),
      'user-account-delete-confirmation'
    );

    $this->display_table_text_field(
      __( "Confirmation Button", "wpep" ),
      'user-account-delete-confirmation-button'
    );

    $this->display_table_text_field(
      __( "Cancel Button", "wpep" ),
      'user-account-delete-confirmation-cancel-button'
    );

    $this->display_table_text_field(
      __( "Success Redirect", "wpep" ),
      'user-account-delete-success-page',
      __( "Redirect the User to a specific page after his account will be deleted, by default the homepage.", "wpep" ),
      false,
      null,
      get_site_url()
    );

    $this->display_table_separator( __( 'Account Export', "wpep" ) );

    foreach( [
               'user-account-export-student-information' => __( "Student Information", "wpep" )
             ] as $setting_name => $setting_label )
      $this->display_table_text_field( $setting_label, $setting_name );



    $this->display_table_end();
  }

  public function display_share_embed_options() {
    $this->display_table_start();

    $this->display_table_select_field(
      __( 'Allow Students to Embed Content', "wpep" ),
      'user-content-embed-box-enabled',
      [
        0 => __( 'Inactive', "wpep" ),
        1 => __( 'Active', "wpep" )
      ],
      __( "Enable this option to allow anyone to share your courses, keeping it inactive will allow you to limit the domains allowed to embed content.", "wpep" ),
      'wpep-select-field',
      [
        'data-wpep-conditional-display' => 'data-wpep-le-embed-box-status'
      ]
    );

    $this->set_table_row_data_attributes( [
      'wpep-le-embed-box-status'  => 0
    ] );

    $this->display_table_separator( __( 'Whitelist Domains', "wpep" ) );

    $this->display_table_select_field(
      __( "Status", "wpep" ),
      'user-content-embed-domain-verify',
      [
        0 => __( "Any domain is allowed to embed content", "wpep" ),
        1 => __( "Only specific domains are allowed to embed content", "wpep" )
      ],
      false,
      'wpep-select-field',
      [
        'data-wpep-conditional-display' => 'data-wpep-le-domain-verify-status'
      ]
    );

    $this->set_table_row_data_attributes( [
      'wpep-le-domain-verify-status' => 1
    ] );

    $this->display_table_list_field(
      __( "List", "wpep" ),
      'user-content-embed-domain-list',
      false,
      false,
      [],
      [
        get_site_url()
      ]
    );

    $this->display_table_editor_field(
      __( "Error Notification", "wpep" ),
      'user-content-embed-error-notification',
      __( "If a domain embeds content and is not part of the whitelist, it will be presented with this error.", "wpep" )
    );

    $this->set_table_row_data_attributes( [ 'wpep-le-embed-box-status'  => 1 ] );

    $this->display_table_separator( __( 'Call To Action', "wpep" ) );

    $this->display_table_select_field( __( 'Default Status', "wpep" ), 'user-content-embed-box-course-status', [
      0 => __( 'Inactive', "wpep" ),
      1 => __( 'Active', "wpep" )
    ], __( "The Default Setting, you can disable / enable the CTA Individually for each course.", "wpep" ) );

    $this->display_table_text_field( __( 'Title', "wpep" ), 'user-content-embed-box-title' );
    $this->display_table_editor_field( __( 'Description', "wpep" ), 'user-content-embed-box-description' );
    $this->display_table_text_field( __( 'Button Text', "wpep" ), 'user-content-embed-box-button' );
    $this->display_table_text_field( __( 'Button Text Clicked', "wpep" ), 'user-content-embed-box-button-clicked' );
    $this->display_table_select_field(
      __( 'Show Inside Embeds', "wpep" ),
      'user-content-embed-box-inside-embeds',
      [
        0 => __( 'Inactive', "wpep" ),
        1 => __( 'Active', "wpep" )
      ]
    );

    $this->display_table_end();
  }

  public function display_auto_login_links() {
    $this->display_table_start();

    if( wpep_get_setting_flag( 'enable-auto-login-link' ) ) {
      $download_url     = add_query_arg( 'wpep-dynamic-login-links', 'download-csv', admin_url() );
      $reset_tokens_url = add_query_arg( 'wpep-dynamic-login-links', 'reset-tokens', admin_url() );
      $last_token_reset = wpep_get_setting( 'auto-login-link-last-generate' );

      $content = '';

      $content .= '<p class="wpep-operations-line-container">';
      $content .=   '<a class="wpep-button wpep-button-primary" href="' . $download_url . '">' . __( "Download", "wpep" ) . '</a>';
      $content .=   '<a class="wpep-button wpep-button-secondary" href="' . $reset_tokens_url . '">' . __( "Reset Tokens & Download", "wpep" ) . '</a>';
      $content .=   '<span class="wpep-operation-status">' . sprintf( __( "Last : %s", "wpep" ), '<strong>' . ( $last_token_reset == 0 ? "Never" : wpep_format_date_with_time( $last_token_reset ) ) . '</strong>' ) . '</span>';
      $content .= '</p>';

      $this->display_table_separator(
        __( "Actions", "wpep" ),
        $content
      );
    } else {
      $this->display_table_separator(
        __( "Actions", "wpep" ),
        __( "Enable Dynamic Login Links and save the settings in order to enable this", "wpep" )
      );
    }

    $this->display_table_select_field(
      __( 'Dynamic Login Links', "wpep" ),
      'enable-auto-login-link',
      [
        0 => __( 'Inactive', "wpep" ),
        1 => __( 'Active', "wpep" )
      ],
      '<p>' . __( "Dynamic Login Links are a sensitive feature, some security experts consider it dangerous, that's why we allow to enable / disable it on demand.", "wpep" ) . '</p>' .
      '<p>' . __( "It's also possible to regenerate all the login tokens anytime, invalidating existent URLs, all login tokens that are generated are always unique, across all WPEP installations & site users, it is also recommended to regenerate the tokens & grabbing a new CSV file every time you send the login links out.", "wpep" ) . '</p>' .
      '<p>' . __( "If you disable this feature, existent login links will not work anymore until you active it back.", "wpep" ) . '</p>'
    );

    $this->display_table_text_field(
      __( 'URL Slug', "wpep" ),
      'auto-login-link-token',
      '<p>' . __( "If you change this parameter all existent dynamic login links will stop working, remember to Save Settings before you export if you've changed this.", "wpep" )  . '</p>'
    );

    $this->display_table_text_field(
      __( 'Token Salt', "wpep" ),
      'auto-login-link-salt',
      '<p>' . __( "When Login Tokens are generated, this seed is used in order to ensure they're unique.", "wpep" )  . '</p>',
      false,
      ( wpep_get_setting( 'auto-login-link-salt' ) !== '' ? NULL : wpep_generate_random_token( 24 ) )
    );



    $this->display_table_end();
  }

}
