<?php

namespace WPEP\Admin\Coordinator\Settings;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;

class LearnerProfile extends Skeleton {

  public $tabs = [];

  public function _setup_tabs() {
    $this->tabs = [
      'general'         => __( 'General', "wpep" ),
      'tab_management'  => __( 'Tab Management', "wpep" ),
      'public'          => __( 'Public Profiles', "wpep" ),
      'social-networks' => __( 'Social Networks', "wpep" )
    ];

    if( defined( "WPEP_BETA_TESTING" ) )
      $this->tabs['student-supervisor'] = __( "Student Supervisor", "wpep" );

  }

  public function get_name() {
    return __( 'Learner Profile', "wpep" );
  }

  public function get_alias() {
    return 'learner_profile';
  }

  public function display() {
    $current_tab = $this->get_current_tab();

    $this->display_tabs( $current_tab );

    if( $current_tab == 'public' )
      $this->display_public_settings();
    else if( $current_tab == 'social-networks' )
      $this->display_social_network_settings();
    else if( $current_tab == 'tab_management' )
      $this->display_tab_management_settings();
    else if( $current_tab == 'student-supervisor' && defined( "WPEP_BETA_TESTING" ) )
      $this->display_tab_student_supervisor_settings();
    else
      $this->display_general_settings();
  }

  public function get_current_tab() {
    $current_tab = ( isset( $_GET['tab_inner'] ) ? $_GET['tab_inner'] : key( $this->tabs ) );

    if( !isset( $this->tabs[ $current_tab ] ) )
      $current_tab = key( $this->tabs );

    return $current_tab;
  }

  public function display_general_settings() {
    $this->display_table_start();

    $this->display_table_select_field(
      __( 'Courses Per Row', "wpep"),
      'user-profile-courses-per-row',
      wpep_content_primary_row_options( WPEP_POST_TYPE_COURSE ),
      __( "How many courses per row will be displayed", "wpep" )
    );

    $this->display_table_select_field(
      __( "Cover Picture", "wpep" ),
      'user-profile-cover-picture-status',
      wpep_flag_setting_field_options(),
      false,
      'wpep-select-field',
      [
        'data-wpep-conditional-display' => 'data-wpep-lp-enable-cover-picture'
      ]
    );

    $this->display_table_text_translated_field(
      __('Change Picture', "wpep"),
      'user-profile-change-picture-button'
    );

    $this->display_table_text_translated_field(
      __('Change Profile Picture', "wpep"),
      'user-profile-change-picture',
      __( "Displayed during the upload & file selection modal / popup")
    );

    $this->set_table_row_data_attributes( [
      'wpep-lp-enable-cover-picture'  => 1
    ] );

    $this->display_table_text_translated_field(
      __('Change Cover Picture', "wpep"),
      'user-profile-change-picture-cover',
      __( "Displayed during the upload & file selection modal / popup")
    );

    $this->set_table_row_data_attributes( [] );

    $this->display_table_text_translated_field(
      __('Update Button', "wpep"),
      'user-profile-update-button'
    );

    $this->display_table_separator( __( 'Marketing Content', "wpep" ) );

    $this->display_table_select_field(
      __( 'Status', "wpep"),
      'user-profile-marketing-content-status',
      wpep_flag_setting_field_options(),
      __( "Display the Marketing Content from the Authentication Form on the left side of the Profile Page", "wpep" )
    );

    $this->display_table_editor_field(
      __( 'Content', "wpep"),
      'user-profile-marketing-content'
    );

    $this->display_table_separator( __( 'Default Settings', "wpep" ) );

    $this->display_table_image_field(
      __( "Profile Picture", "wpep" ), 'user-profile-default-profile-picture', false,
      false, null, [
        'width' => '256px',
        'height'=> '256px'
      ],
      false
    );

    $this->set_table_row_data_attributes( [
      'wpep-lp-enable-cover-picture'  => 1
    ] );

    $this->display_table_image_field(
      __( "Cover Picture", "wpep" ), 'user-profile-default-cover-picture', false,
      false, null, [], false
    );

    $this->set_table_row_data_attributes( [] );

    $this->display_table_separator( __( 'Email Notifications', "wpep" ) );

    $this->display_table_text_translated_field(
      __('Weekly Recap', "wpep"),
      'user-profile-email-notification-weekly-recap'
    );

    $this->display_table_end();
  }

  public function display_public_settings() {
    $this->display_table_start();

    $profile_link_example = wpep_user_profile_public_link();

    $description = '';

    if( wpep_public_profiles_enabled() ) {
      $description .= '<p>' . sprintf(
        __( "Your Public Profile :  %s", "wpep" ),
        '<a target="_blank" href="' . $profile_link_example . '">' . $profile_link_example . '</a>'
      ) . '</p>' . '<p>' . __( "If the above link doesn't work, reset permalinks from the Platform Health.", "wpep" ) . '</p>';
    } else {
      $description .= __( "Activating the Public Profiles will automatically provide every WordPress User with a Profile Page.", "wpep" );
    }

    $this->display_table_checkbox_field(
      __('Enable', "wpep"),
      'user-profile-public-active',
      $description,
      NULL,
      wpep_public_profiles_enabled() && wpep_get_setting_flag( 'user-profile-public-active' ) == false
    );

    $this->display_table_text_translated_field(
      __( 'Profile Page Title', "wpep"),
      'user-profile-heading',
      __( "Adding the %s identifier will replace it with the First Name & Last Name of the student", "wpep" )
    );

    $this->display_table_text_field(
      __('Profile Slug', "wpep"),
      'user-profile-public-slug',
      '',
      false,
      WPEP_Controller::instance()->content->userProfile->publicProfile->user_identifier
    );

    $this->display_table_separator( __( "Profile Privacy", "wpep" ) );

    $this->display_table_select_field(
      __( "Default Setting"),
      'user-profile-privacy-setting-default',
      [
        WPEP_USER_PROFILE_PRIVACY_TYPE_PRIVATE => __( "Private, students can only see their own public profile.", "wpep" ),
        WPEP_USER_PROFILE_PRIVACY_TYPE_PUBLIC  => __( "Public, anyone with a Link can see the Public Profiles.", "wpep" )
      ],
      __( "Admin & Staff accounts will be able to see Private User Profiles while they're logged in", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Allow Switching', "wpep"),
      'user-profile-privacy-setting-status',
      __( "Allow Students to choose their own Privacy Setting from their Profile, by default the one above will be applied.", "wpep" )
    );

    $this->display_table_text_field(
      __( 'Private Profile', "wpep"),
      'user-profile-privacy-setting-private'
    );

    $this->display_table_text_field(
      __( 'Public Profile', "wpep"),
      'user-profile-privacy-setting-public'
    );

    $this->display_table_textarea_field(
      __( 'Profile Types', "wpep"),
      'user-profile-privacy-setting-tooltip',
      __( "This tooltip is displayed on the right side of the profile privacy type setting box", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Public Email Address', "wpep"),
      'user-profile-course-public-email-address',
      __( "Enable this option to display the Email Address of the Students on Public Profiles", "wpep" )
    );

    $this->display_table_separator( __( "Course Options", "wpep" ) );

    $this->display_table_checkbox_field(
      __( 'Display Courses', "wpep"),
    'user-profile-course-public-available',
      __( "Progressing, Completed & Bookmarked Courses will be displayed on the Public Profiles if this option is enabled. Admin & Staff accounts will still see them.", "wpep" )
    );

    $this->display_table_checkbox_field(
      __( 'Display Progress Bars', "wpep"),
      'user-profile-course-public-progress-bars'
    );

    $this->display_table_end();
  }

  public function display_social_network_settings() {
    $this->display_table_start();

    foreach( wpep_user_profile_social_network_platform_list() as $social_platform_key => $social_platform_information ) {
      $this->display_table_checkbox_array_field(
        '<span class="' . $social_platform_information['icon']. '"></span>' . $social_platform_information['name'],
        'user-profile-enabled-social-networks',
        $social_platform_key
      );
    }

    $this->display_table_end();
  }

  public function display_tab_management_settings() {
    $this->display_table_start();

    $this->display_table_separator( __( "General Options", "wpep" ) );

    $this->display_table_select_field(
      __('Tab Links', "wpep"),
      'user-profile-tab-link-format',
      [
        'label'   => __( "Tab Title", "wpep" ),
        'alias'   => __( "Tab Alias", "wpep" ),
        'disabled'=> __( "Disable Navigation", "wpep" )
      ],
      __( "The URL Format of the links generated for the tabs when clicking on them.", "wpep" )
    );

    $this->display_table_separator( __( "Default Tabs", "wpep" ) );

    $this->display_table_select_field(
      __('Display Default Tabs', "wpep"),
      'user-profile-tab-default-tabs-enabled',
      wpep_flag_setting_field_options(),
      __( "This includes Progressing Courses, Completed Courses and Bookmarked Courses", "wpep" )
    );

    $this->display_table_text_translated_field(
      __( 'Progressing Courses', "wpep" ),
      'user-profile-tab-progressing-courses'
    );
    $this->display_table_text_translated_field(
      __( 'Completed Courses', "wpep" ),
      'user-profile-tab-completed-courses'
    );
    $this->display_table_text_translated_field(
      __( 'Bookmarked Courses', "wpep" ),
      'user-profile-tab-bookmarked-courses'
    );

    if( WPEP_Controller::instance()->contentLibrary->is_integrated() ) {
      $this->display_table_separator( __( "Advanced Tabs", "wpep" ) );

      $this->display_table_select_field(
        __('Display My Content Tab', "wpep"),
        'user-profile-tab-my-content-enabled',
        wpep_flag_setting_field_options(),
        __( "My Content Tab will display an Index Page that is not paginated, that will show all the Content the Student has access to.", "wpep" )
      );

      $this->display_table_text_translated_field(
        __( 'My Content', "wpep" ),
        'user-profile-tab-my-content'
      );
    }

    $this->display_table_separator( __( "Tab Ordering", "wpep" ) );

    echo '<tr>';
    echo    '<th>' . __( "All Available Tabs", "wpep" ) . '</th>';
    echo    '<td class="wpep-entity-order-functionality-container" style="margin:0 !important;">';
    echo      '<ul class="wpep-entity-order-functionality wpep-entity-order-functionality-keys">';

    foreach( wpep_content_user_profile_tabs( 0 ) as $tab_key => $tab_information ) {
      echo '<li>';
      echo    '<div>';
      echo      '<input type="hidden" name="wpep_settings[user-profile-tab-ordering][]" value="' . $tab_key . '">';
      echo      '<span class="wpep-title">' . ( isset( $tab_information['title'] ) ? $tab_information['title'] : $tab_key ) . '</span>';
      echo    '</div>';
      echo '</li>';
    }

    echo      '</ul>';
    echo    '</td>';
    echo '</tr>';

    $this->display_table_separator( __( "Custom Tabs", "wpep" ) );

    $this->display_table_select_field(
      __( 'Status', "wpep"),
      'user-profile-custom-tabs-enabled',
      wpep_flag_setting_field_options(),
      __( "New Custom Tabs will appear in the ordering above, only after you Save Changes." )
    );

    $this->display_table_attributes_field_content_editor(
      __( 'Configure Tabs', "wpep"),
      'user-profile-custom-tabs',
      false,
      false,
      [],
      [
        'Example Tab' => __( "You can add anything here, and it will be displayed within this tab.", "wpep" )
      ]
    );

    $this->display_table_end();
  }

  public function display_tab_student_supervisor_settings() {
    $this->display_table_start();

    $this->display_table_checkbox_field(
      __( 'Student Supervisor', "wpep" ),
      'enable-student-supervisor',
      __( "Allows students to choose a supervisor from their Learner Profile.", "wpep" )
    );

    $this->display_table_separator( __( "Student Profile", "wpep" ) );

    $this->display_table_text_translated_field(
      __( 'Choose Supervisor', "wpep"),
      'user-profile-student-supervisor-placeholder'
    );

    $this->display_table_checkbox_field(
      __( 'Allow Student Switch', "wpep"),
      'user-profile-student-supervisor-allow-change',
      __( "Allows your students to change their supervisor from the User Profile, they can choose from all the existent Student Supervisors.", "wpep" )
    );

    $this->display_table_separator( __( "Supervisor Tab", "wpep" ) );

    $this->display_table_text_translated_field(
      __( 'Name', "wpep" ),
      'user-profile-tab-my-students',
      __( "Displayed on the supervisor Learner Profile", "wpep" )
    );

    $this->display_table_editor_field(
      __( 'Content', "wpep" ),
      'user-profile-tab-my-students-content',
      __( "Displayed above the List of Students this supervisor has access to.", "wpep" )
    );

    $this->display_table_editor_field(
      __( 'Welcome Message', "wpep" ),
      'user-profile-tab-my-students-content-pre',
      __( "This is displayed to supervisors until they have at least 1 student that have selected them as their supervisor, the message above will not be displayed.", "wpep" )
    );

    $this->display_table_end();
  }

}