<?php

namespace WPEP\Admin\Coordinator\SetupProcess;

use WPEP\Controller as WPEP_Controller;

class Request {

  public function __construct() {
    add_action( 'wp_ajax_wpep_setup_process', [ $this, 'setup_process' ] );
  }

  public function setup_process() {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS )  )
      exit( '<div class="wpep-notification">' . __( "Your account is not allowed to complete this operation", "wpep" ) . '</div>');

    $steps = WPEP_Controller::instance()->adminController->setupProcess->get_step_list();

    $current_stage_alias = ( isset( $_POST[ 'sync_stage' ] ) ? $_POST[ 'sync_stage' ] : key( $steps ) );

    if( !isset( $steps[ $current_stage_alias ] ) )
      exit( '<div class="wpep-notification">' . __( "You are not allowed to run this sync_stage.", "wpep" ) . '</div>');

    $response = $this->_handle_sync_stage( $current_stage_alias, $steps[ $current_stage_alias ] );

    if( $response !== true )
      exit;

    $trigger_next_step = false;

    foreach( $steps as $step_alias => $step_data ) {
      if( $trigger_next_step ) {
        $response = $this->_handle_sync_stage( $step_alias, $step_data );

        if( $response !== true )
          exit;
      } else if( $step_alias === $current_stage_alias ) {
        $trigger_next_step = true;
      }
    }

    exit;
  }

  private function _handle_sync_stage( $step_alias, $step_data ) {
    if( isset( $step_data[ 'process' ] )
        && is_callable( $step_data[ 'process' ] ) ) {
      $process_response = $step_data[ 'process' ]();

      if( $process_response === true )
        return $process_response;
    }

    if( isset( $step_data['title'] ) ) {
      echo '<h2>' . $step_data['title'] . '</h2>';
    }

    if( isset( $process_response ) && is_string( $process_response ) ) {
      echo '<div class="wpep-notification wpep-notification-danger">' . $process_response . '</div>';
    }

    // Skip Step
    if( !isset( $step_data[ 'display' ] ) || !is_callable( $step_data[ 'display' ] ) )
      return true;

    $response = $step_data[ 'display' ]( $step_alias );

    if( $response === true )
      return true;

    echo $response;

    return false;
  }

}