<?php

namespace WPEP\Admin\Coordinator\SetupProcess;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

class Steps {

  public function get_step_list() {
    $response = [
      'license'             => $this->_get_step_license(),
      'license_validated'   => $this->_get_step_license_validated(),
      'content_types'       => $this->_get_step_content_types(),
      'completed'           => $this->_get_step_completed()
    ];

    return $response;
  }

  public function _get_step_license() {
    return [
      'title'   => __( "License Key Verification", "wpep" ),
      'display' => function( $sync_step ) {
        $response =  '<div class="wpep-utility-data-sync-row wpep-utility-data-sync-row-wide">';
        $response .=   '<label for="#wpep_license_key">' . __( "License Key", "wpep" ) . '</label>';
        $response .=   '<input type="text" 
                               id="wpep_license_key" 
                               name="wpep_license_key" 
                               value="' . WPEP_Controller::instance()->adminController->license->get_license() .  '"/>';
        $response .= '</div>';

        $response .= '<div class="wpep-clearfix"></div>';

        $response .= '<input class="wpep-button wpep-button-center wpep-button-primary" 
                             type="submit" 
                             data-switch-sync-step="' . $sync_step . '"
                             value="' . __( "Verify License", "wpep" ) . '"/>';

        $response .= '<h3>' . __( "Locate Your License Key", "wpep" ) . '</h3>';

        $response .= '<div class="wpep-utility-data-sync-row wpep-utility-data-sync-row-wide">';
        $response .= '<ol class="wpep-instructional-steps">' .
                      '<li><a href="https://codestore.codeiscode.com/my-account/">' . __( "Visit Customer Area", "wpep" ) . '</a></li>' .
                      '<li>' . __( "Login to Customer Area", "wpep" ) . '</li>' .
                      '<li>' . __( "Copy The License Key", "wpep" ) . '</li>' .
                      '<li>' . __( "Paste The License Key on the field above", "wpep" ) . '</li>' .
                      '<li>' . __( "Press on the Verify License button", "wpep" ) . '</li>' .
                     '</ol>';
        $response .= '</div>';

        return $response;
      },
      'process' => function() {
        if( isset( $_POST['wpep_license_key'] ) ) {
          $license = trim( $_POST['wpep_license_key'] );

          $is_valid_license = WPEP_Controller::instance()->adminController->license->activate_license( $license, true );

          if( $is_valid_license === true ) {
            return true;
          } else {
            $response = __( "Warning! Your license key is not valid. Please contact our support.", "wpep" );

            if( is_wp_error( $is_valid_license ) )
              $response .= '<br/>' . __( "Request Error : " ) . $is_valid_license->get_error_message();
            else if( isset( $is_valid_license->error ) )
              $response .= '<br/>' . __( "License Error : " ) . ( $is_valid_license->error == "missing" ? __( "License Key doesn't exist", "wpep" ) : $is_valid_license->error );

            return $response;
          }
        }

        return false;
      }
    ];
  }

  public function _get_step_license_validated() {
    return [
      'title'   => __( "License Key Verified", "wpep" ),
      'display' => function( $sync_step ) {
        $response =  '';

        $response .= '<div class="wpep-utility-data-sync-row wpep-utility-data-sync-row-wide">';
        $response .=     '<p>' . __( "Configure your E-Learning within minutes, with our Setup Process", "wpep" ) . '</p>';
        $response .=     '<p>' . __( "You'll decide what type of functionality will be enabled", "wpep" ) . '</p>';
        $response .=     '<p class="wpep-notice">' . sprintf( __( "Estimated setup time is %s minutes", "wpep" ), '2' ) . '</p>';
        $response .= '</div>';

        $response .= '<input type="hidden" name="wpep_license_verified_next_step" value="1"/>';
        $response .= '<input class="wpep-button wpep-button-center wpep-button-primary" 
                             type="submit" 
                             data-switch-sync-step="' . $sync_step . '"
                             value="' . __( "Start Setup Process", "wpep" ) . '"/>';

        if( !empty( wpep_content_type()->libraryContentTypes ) ) {
          $response .= '<p class="wpep-choice-notice">';
          $response .=  '<a href="' . wpep_admin_url_dashboard() . '">' . __( "Skip Setup Process", "wpep" ) . '</a>';
          $response .= '</p>';
        }

        return $response;
      },
      'process' => function() {
        return ( isset( $_POST[ 'wpep_license_verified_next_step' ] ) && $_POST[ 'wpep_license_verified_next_step' ] );
      }
    ];
  }

  public function _get_step_content_types() {
    return [
      'title'   => __( "Setup Content Types", "wpep" ),
      'display' => function( $sync_step ) {
        $settings_helper = new \WPEP\Admin\Coordinator\Settings\General();

        $response = '';

        $response .= '<div class="wpep-utility-data-sync-row wpep-utility-data-sync-row-wide">';
        $response .=     '<p>' . __( "What Content Types do you want to use ?", "wpep" ) . '</p>';
        $response .= '</div>';

        $response .= '<div class="wpep-utility-data-sync-row wpep-utility-data-sync-row-wide">';
        $response .=    '<table class="wpep-utility-data-table">';
        $response .=      '<thead>';
        $response .=        '<tr>';
        $response .=          '<th></th>';
        $response .=          '<th>' . __( "Enable Content Type", "wpep" ) . '</th>';
        $response .=          '<th>' . __( "Default Archive Page", "wpep" ) . '</th>';
        $response .=          '<th>' . __( "WordPress URL Slug", "wpep" ) . '</th>';
        $response .=        '</tr>';
        $response .=      '</thead>';
        $response .=      '<tbody>';

        foreach( wpep_content_type()->switchableContentTypes as $switchableContentType ) {
          $response .=  '<tr>';
          $response .=    '<td>' . $switchableContentType->get_plural_name() . '</td>';
          $response .=    '<td>' . $settings_helper->get_checkbox_field( 'content-type-' . $switchableContentType->get_post_type() . '-enabled' ) . '</td>';
          $response .=    '<td>' . $settings_helper->get_checkbox_field( 'content-type-' . $switchableContentType->get_post_type() . '-enable-archive' ) . '</td>';
          $response .=    '<td>' . $settings_helper->get_text_field(
              'content-type-' . $switchableContentType->get_post_type() . '-rewrite-slug',
              false,
              '',
              false,
              wpep_get_setting( 'content-type-' . $switchableContentType->get_post_type() . '-rewrite-slug', $switchableContentType->get_post_type() )
            ) .
            '</td>';
          $response .=  '</tr>';
        }

        $response .=      '</tbody>';
        $response .=    '</table>';
        $response .= '</div>';

        $response .= '<input class="wpep-button wpep-button-center wpep-button-primary" 
                              type="submit" 
                              data-switch-sync-step="' . $sync_step . '"
                              value="' . __( "Confirm Choice", "wpep" ) . '"/>';
        $response .= '<p class="wpep-choice-notice">' . __( "New Content Types can be added anytime from the Settings, avoid enabling too many.", "wpep" ) . '</p>';

        return $response;
      },
      'process' => function() {
        if( isset( $_POST[ 'wpep_settings' ] ) ) {
          $settings = $_POST[ 'wpep_settings' ];

          foreach( $settings as $setting_key => $setting_value ) {
            if( wpep_string_ends_with( $setting_key, '-rewrite-slug' ) && $setting_value == '' )
              unset( $settings[ $setting_key ] );
          }

          WPEP_Settings::instance()->update_settings( $settings );

          return true;
        }
        return false;
      }
    ];
  }

  public function _get_step_completed() {
    return [
      'title'   => __( "Platform Configured", "wpep" ),
      'display' => function( $sync_step ) {
        $response =  '';

        $response .= '<div class="wpep-utility-data-sync-row wpep-utility-data-sync-row-wide">';
        $response .=     '<p>' . __( "Your Platform has been configured", "wpep" ) . '</p>';
        $response .=     '<p>' . __( "You can always change anything from the settings, you can locate them on the left side menu.", "wpep" ) . '</p>';
        $response .=     '<p class="wpep-final-step">';
        $response .=        '<a href="' . wpep_admin_url_dashboard() . '" class="wpep-button wpep-button-primary">' . __( "Complete Setup Process", "wpep" ) . '</a>';
        $response .=     '</p>';
        $response .= '</div>';

        $response .= '<input type="hidden" name="wpep_license_verified_next_step" value="1" data-switch-sync-step="completed"/>';


        return $response;
      },
      'process' => function() {
        return ( isset( $_POST[ 'wpep_license_verified_next_step' ] ) && $_POST[ 'wpep_license_verified_next_step' ] );
      }
    ];
  }

}