<?php

namespace WPEP\Admin\Coordinator\Statistics;

use WPEP\ContentType\Skeleton\Integration\Statistics as WPEP_ContentType_Statistics_Integration;

class ContentTypes extends Skeleton {

  /**
   * @var WPEP_ContentType_Statistics_Integration[]
   */
  protected $_integrations = [];
  public $tabs = [];
  public $current_tab = '';

  public function get_name() {
    return __( 'Content Types', "wpep" );
  }

  public function get_alias() {
    return 'content-types';
  }

  private function _setup_data() {
    foreach( wpep_content_type()->standardContentTypes as $standardContentType ) {
      $current_statistics_instance = $standardContentType->get_statistics_instance();

      if( $current_statistics_instance == false )
        continue;

      $this->_integrations[ $standardContentType->get_post_type() ] = $current_statistics_instance;
      $this->tabs[ $standardContentType->get_post_type() ] = $standardContentType->get_plural_name();
    }

    if( isset( $_GET[ 'content-type' ] ) && isset( $this->_integrations[ $_GET[ 'content-type' ] ] ) )
      $this->current_tab = $_GET[ 'content-type' ];
    else
      $this->current_tab = key( $this->tabs );
  }

  public function display( $filter_by = [] ) {
    $this->_setup_data();

    $this->display_tabs( $this->current_tab );

    $this->_integrations[ $this->current_tab ]->display( $filter_by );
  }

  public function download( $filter_by = [] ) {
    $this->_setup_data();

    $this->_integrations[ $this->current_tab ]->download( $filter_by );
  }

  public function display_tabs( $current = false ) {
    echo '<h3 class="wpep-navigation wpep-navigation-secondary" data-wpep-navigation="statistics" data-wpep-navigation-secondary="content-types">';

    foreach( $this->tabs  as $tab => $tab_name ){
      $class = ( $tab == $current ) ? 'wpep-navigation-item-active' : '';
      echo '<a class="wpep-navigation-item ' . $class . '" href="' . wpep_admin_url_statistics( [ 'tab' => $this->get_alias(), 'content-type' => $tab ] ) . '" data-wpep-tab="' . $tab . '">' . $tab_name . '</a>';
    }

    echo '</h3>';

    echo '<div class="wpep-clearfix"></div>';
  }

}
