<?php

namespace WPEP\Admin\Coordinator\Statistics\Emails;

class Dashboard {

  protected $table_tabs = [];
  private   $_current_tab = false;

  public function __construct() {
    foreach( [
               new Dashboard\Delivery(),
               new Dashboard\Entries(),
               new Dashboard\EntriesByTitle(),
               new Dashboard\Clear()
             ] as $tab )
      $this->table_tabs[ $tab->get_alias() ] = $tab;

    $this->table_tabs = apply_filters( "wpep_administration_statistics_emails_dashboard_tabs", $this->table_tabs );
  }

  /**
   * @return Dashboard\Delivery|Dashboard\Entries|Dashboard\EntriesByTitle
   */
  private function _get_current_tab() {
    if( $this->_current_tab !== false )
      return $this->table_tabs[ $this->_current_tab ];

    $this->_current_tab = ( isset( $_GET['view'] ) ? $_GET['view'] : false );
    $this->_current_tab = ( isset( $this->table_tabs[ $this->_current_tab ] ) ? $this->_current_tab : key( $this->table_tabs ) );

    return $this->table_tabs[ $this->_current_tab ];
  }

  public function display( $filter_by = [] ) {
    $table_navigation = [];

    $entries_per_page = 30;
    $current_page     = max( 1, ( isset( $_GET['pagination'] ) ? intval( $_GET['pagination'] ) : 1 ) );

    $table_information = $this->_get_current_tab()->get_table_data( $entries_per_page, $current_page, $filter_by );

    if( !is_array( $table_information ) ) {
      echo $table_information;
      return;
    }

    foreach( $this->table_tabs as $table_tab ) {
      if( method_exists( $table_tab, 'is_public' ) && $table_tab->is_public() === false ) {
        if( $this->_current_tab != $table_tab->get_alias() )
          continue;
      }

      $key = ( $this->_current_tab != $table_tab->get_alias()
        ? wpep_admin_url_statistics( [ 'tab' => 'emails', 'view' => $table_tab->get_alias() ] )
        : false
      );

      $table_navigation[ $key ] = $table_tab->get_name();
    }

    wpep_admin_display_table(
      $table_navigation,
      $table_information['head'],
      $table_information['body'],
      true,
      $table_information['page_current'],
      $table_information['page_total']
    );
  }

  public function download( $filter_by = [] ) {
    $table_information = $this->_get_current_tab()->get_table_data(
      1000000,
      1,
      $filter_by
    );

    do_action(
      'wpep_deliver_csv_export_file',
      $this->_get_current_tab()->get_file_alias( $filter_by ),
      $table_information['head'],
      $table_information['body']
    );
  }

}