<?php

namespace WPEP\Admin\Coordinator\Statistics\Emails\Dashboard;

use WPEP\Controller as WPEP_Controller;

class Delivery {

  public function get_name() {
    return __( 'Delivery', "wpep" );
  }

  public function get_alias() {
    return 'delivery';
  }

  public function get_file_alias( $filter_by = [] ) {
    return 'emails-delivery-' . ( isset( $filter_by['user_role'] ) && $filter_by['user_role'] !== false ? $filter_by['user_role'] . '-' : '' ) . wpep_format_date();
  }

  public function get_table_data( $entries_per_page, $current_page = 1, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => [],
      'page_current'  => null,
      'page_total'    => null
    ];

    $courses_offset = ( $current_page - 1 ) * $entries_per_page;

    $entries_query_information = WPEP_Controller::instance()->email->get_delivery_history_by_title(
      $entries_per_page,
      $courses_offset,
      true,
      $filter_by
    );

    if( empty( $entries_query_information['entries'] ) )
      return $table_information;

    $table_information['head'] = [
      'title'           => __( "Title", 'wpep' ),
      'sent_open_rate'  => [
        'type'  => 'digits',
        'value' => __( "Open Rate", "wpep" )
      ],
      'sent_opened'      => [
        'type'  => 'digits',
        'value' => __( "Emails Opened", "wpep" )
      ],
      'sent_total'      => [
        'type'  => 'digits',
        'value' => __( "Emails Sent", "wpep" )
      ],
      'sent_last_date'  => __( "Last Sent", "wpep" ),
      'sent_first_date' => __( "First Sent", "wpep" ),
      'clear_history'   => [
        'type'  => 'link-item',
        'value' => ''
      ],
    ];

    foreach( $entries_query_information['entries'] as $entry ) {
      $body = [
        'title'       => [
          'type'      => 'link',
          'value'     => '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'emails', 'view' => 'emails-by-title', 'title' => urlencode( $entry['title']) ] ) . '">' . $entry['title'] . '</a>',
          'value_raw' => $entry['title']
        ],
        'sent_open_rate'        => [
          'type'      => 'digits',
          'value'     => wpep_get_percentage( $entry['open_count'], $entry['sent_count'] )
        ],
        'sent_opened'        => [
          'type'      => 'digits',
          'value'     => $entry['open_count']
        ],
        'sent_total'        => [
          'type'      => 'digits',
          'value'     => $entry['sent_count']
        ],
        'sent_last_date' => wpep_format_date_with_time( $entry['last_date'] ),
        'sent_first_date' => wpep_format_date_with_time( $entry['first_date'] ),
        'clear_history'   => [
          'type'  => 'link-item',
          'value' =>
            '<a class="button button-secondary" href="' . wpep_admin_url_statistics( [ 'tab' => 'emails', 'view' => 'clear', 'title' => urlencode( $entry['title'] ) ] ) . '">' . __( "Clear Statistics", "wpep" ) . '</a>'
        ],
      ];

      $table_information['body'][] = $body;
    }

    $table_information['page_current'] = $current_page;
    $table_information['page_total']   = ceil( $entries_query_information['entries_total'] / $entries_per_page );

    return $table_information;
  }

}