<?php

namespace WPEP\Admin\Coordinator\Statistics\Emails\Dashboard;

use WPEP\Controller as WPEP_Controller;

class Entries {

  public function get_name() {
    return __( 'All Emails', "wpep" );
  }

  public function get_alias() {
    return 'all';
  }

  public function get_file_alias( $filter_by = [] ) {
    return 'emails-all-' . ( isset( $filter_by['user_role'] ) && $filter_by['user_role'] !== false ? $filter_by['user_role'] . '-' : '' ) . wpep_format_date();
  }

  public function get_table_data( $entries_per_page, $current_page = 1, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => [],
      'page_current'  => null,
      'page_total'    => null
    ];

    $courses_offset = ( $current_page - 1 ) * $entries_per_page;

    $entries_query_information = WPEP_Controller::instance()->email->get_delivery_history(
      $entries_per_page,
      $courses_offset,
      true,
      $filter_by
    );

    if( empty( $entries_query_information['entries'] ) )
      return $table_information;

    $table_information['head'] = [
      'title'      => __( "Title", 'wpep' ),
      'name'       => __( "Name", 'wpep' ),
      'email'      => __( "Email", 'wpep' ),
      'created_at' => __( "Sent At", "wpep" ),
      'opened_at'  => __( "Opened At", "wpep" )
    ];

    foreach( $entries_query_information['entries'] as $entry ) {
      $body = [
        'title'      => $entry['title'],
        'name'       => wpep_content_user_profile_format_name( $entry['user_id'] ),
        'email'      => $entry['user_email'],
        'created_at' => wpep_format_date_with_time( $entry['created_at'] ),
        'opened_at'  => ( $entry['opened_at'] === NULL ? '-' : wpep_format_date_with_time( $entry['opened_at'] ) )
      ];

      $table_information['body'][] = $body;
    }

    $table_information['page_current'] = $current_page;
    $table_information['page_total']   = ceil( $entries_query_information['entries_total'] / $entries_per_page );

    return $table_information;
  }

}