<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Settings as WPEP_Settings;
use WPEP\Controller as WPEP_Controller;

class UserEdit {

  public function setup() {
    if( wpep_get_setting_flag( 'email-weekly-recap-status' ) == false
        && wpep_get_setting_flag( 'user-service-agreement-enabled' ) == false
        && wpep_get_setting_flag( 'user-privacy-agreement-enabled' ) == false
        && wpep_get_setting_flag( 'enable-student-supervisor' ) == false )
      return;

    add_action( 'show_user_profile', [ $this, 'user_profile_access_management' ] );
    add_action( 'edit_user_profile', [ $this, 'user_profile_access_management' ] );

    add_action( 'personal_options_update', [ $this, 'user_profile_access_management_save' ] );
    add_action( 'edit_user_profile_update', [ $this, 'user_profile_access_management_save' ] );
  }

  public function user_profile_access_management( $user ) {
    wpep_template_load( 'admin/core-user-edit.php', [
      'user'  => $user
    ] );
  }

  public function user_profile_access_management_save( $user_id ) {
    if ( !current_user_can( 'edit_user', $user_id ) )
      return false;

    if( wpep_get_setting_flag( 'email-weekly-recap-status' ) && isset( $_POST['wpep_disable_email_weekly_recap'] ) ) {
      update_user_meta( $user_id, 'wpep_disable_email_weekly_recap', intval( $_POST['wpep_disable_email_weekly_recap'] ) );
    }

    if( wpep_get_setting_flag( 'user-privacy-agreement-enabled' ) && isset( $_POST[WPEP_USER_META_AGREEMENT_PRIVACY_DATE] ) ) {
      update_user_meta( $user_id, WPEP_USER_META_AGREEMENT_PRIVACY_DATE, $this->_get_date_field_value( $_POST[WPEP_USER_META_AGREEMENT_PRIVACY_DATE] ) );
    }

    if( wpep_get_setting_flag( 'user-service-agreement-enabled' ) && isset( $_POST[WPEP_USER_META_AGREEMENT_TOS_DATE] ) ) {
      update_user_meta( $user_id, WPEP_USER_META_AGREEMENT_TOS_DATE, $this->_get_date_field_value( $_POST[WPEP_USER_META_AGREEMENT_TOS_DATE] ) );
    }

    if( wpep_get_setting_flag( 'enable-student-supervisor' ) && isset( $_POST[ 'wpep_student_supervisor'] ) ) {
      update_user_meta( $user_id, WPEP_USER_META_SUPERVISOR_USER_ID, intval( $_POST['wpep_student_supervisor'] ) );
    }
  }

  private function _get_date_field_value( $value ) {
    if( $value == '' )
      return $value;

    $value = strtotime( $value );
    $value = date( "Y-m-d", $value );

    return $value;
  }

}
