<?php

namespace WPEP\Container;

trait FileSystem {

  public function file_system_get_file_list($source) {
    if (!is_dir($source))
      return false;

    $return = [];
    $source = str_replace('\\', '/', realpath($source));

    if (is_dir($source) === true) {
      $files = new \RecursiveIteratorIterator(new \RecursiveDirectoryIterator($source), \RecursiveIteratorIterator::SELF_FIRST);

      foreach ($files as $file) {
        $file = str_replace('\\', '/', $file);

        if (is_file($file) === true)
          $return[] = $file;
      }
    } else if (is_file($source) === true) {
      $return[] = $source;
    }

    return $return;
  }

  public function file_system_get_base_directory_list($source, $nameOnly = false) {
    if (!is_dir($source))
      return false;

    $return = [];
    $source = str_replace('\\', '/', realpath($source));

    if (is_dir($source) === true)
      if ($directory_handler = opendir($source))
        while (false !== ($request_handler = readdir($directory_handler)))
          if($request_handler !== '.' && $request_handler !== '..')
            if(is_dir($source . DIRECTORY_SEPARATOR . $request_handler))
              $return[] = ($nameOnly ? $request_handler : $source . DIRECTORY_SEPARATOR . $request_handler);

    return $return;
  }

  public function file_system_copy_dir($src, $dest) {
    if(!is_dir($src)) return false;
    if(!is_dir($dest)) if(!mkdir($dest)) return false;

    $i = new \DirectoryIterator($src);

    foreach($i as $f)
      if($f->isFile())
        copy($f->getRealPath(), $dest. "/" . $f->getFilename());
      else if(!$f->isDot() && $f->isDir())
        $this->file_system_copy_dir($f->getRealPath(), $dest . "/" . $f);

    return true;
  }

  public function file_system_remove_dir($dir) {
    $this->file_system_remove_dir_children( $dir );

    rmdir($dir);
  }

  public function file_system_remove_dir_children($dir, $ignore = [] ) {
    $ignore = array_merge( $ignore, [ '.', '..' ] );


    $it = new \RecursiveDirectoryIterator($dir);
    $it = new \RecursiveIteratorIterator($it, \RecursiveIteratorIterator::CHILD_FIRST);

    foreach($it as $file) {
      if ( in_array( $file->getBasename(), $ignore ) ) continue;
      if ($file->isDir()) rmdir($file->getPathname());
      else unlink($file->getPathname());
    }
  }

}