<?php

namespace WPEP\Container;

use WPEP\Settings as WPEP_Settings;

trait FormFields {

  public $field_prefix = 'wpep_configuration';
  private $_table_row_class = 'wpep-table-row';
  private $_table_row_data_attr = [];

  public function set_field_prefix( $field_prefix ) {
    $this->field_prefix = $field_prefix;
  }

  public function set_table_row_class( $class = '' ) {
    $this->_table_row_class = 'wpep-table-row' . ( $class !== '' ? ' ' . trim( $class ) : '' );
  }

  public function set_table_row_data_attributes( $attributes = [] ) {
    $this->_table_row_data_attr = $attributes;

    if( !is_array( $this->_table_row_data_attr ) )
      $this->_table_row_data_attr = [];
  }

  public function display_table_wrapper_options() {
    $this->display_table_separator( __( "Wrapper Options", "wpep" ) );

    $this->display_table_select_field(
      __('Width', "wpep"),
      'layout-max-width',
      [
        1024 => sprintf( __( "Default, up to %s", "wpep" ), '1024px' ),
        1200 => sprintf( __( "Default, up to %s", "wpep" ), '1200px' ),
        1440 => sprintf( __( "Up to %s", "wpep" ), '1440px' ),
        0    => __( "Unlimited", "wpep" ),
        240  => sprintf( __( "Situational : Up to %s", "wpep" ), '240px' ),
        360  => sprintf( __( "Situational : Up to %s", "wpep" ), '360px' ),
        480  => sprintf( __( "Situational : Up to %s", "wpep" ), '480px' ),
        640  => sprintf( __( "Situational : Up to %s", "wpep" ), '640px' ),
        960  => sprintf( __( "Situational : Up to %s", "wpep" ), '960px' ),
      ],
      __( "Set the max. width of WPEP's layout, not all options may be supported by your theme.", "wpep" )
    );

    $this->display_table_color_field(
      __( 'Background', "wpep" ),
      'layout-background-color',
      __( 'The background color, leave this empty for the default.', "wpep" )
    );

    $this->display_table_select_field(
      __( 'Padding', "wpep" ),
      'layout-padding',
      [
        'default'   => __( 'Skin Default', "wpep" ),
        0           => __( "None", "wpep" ),
        10          => __( "10 Pixels", "wpep" ),
        20          => __( "20 Pixels", "wpep" ),
        '20px 10px' => sprintf( __( "%s Pixels Top & Bottom. %s Pixels Sides", "wpep" ), '20', '10' ),
        30          => __( "30 Pixels", "wpep" ),
        '30px 10px' => sprintf( __( "%s Pixels Top & Bottom. %s Pixels Sides", "wpep" ), '30', '10' ),
        '40px 10px' => sprintf( __( "%s Pixels Top & Bottom. %s Pixels Sides", "wpep" ), '40', '10' ),
        '40px 20px' => sprintf( __( "%s Pixels Top & Bottom. %s Pixels Sides", "wpep" ), '40', '20' ),
        '60px 30px' => sprintf( __( "%s Pixels Top & Bottom. %s Pixels Sides", "wpep" ), '60', '30' ),
      ],
      __( 'The space space around content.', "wpep" )
    );

    $this->display_table_select_field(
      __( 'Box Shadow', "wpep" ),
      'layout-box-shadow',
      [
        'default'                      => __( 'Skin Default'  , "wpep" ),
        '0 0 0'                        => __( "None"  , "wpep" ),
        '0 0 2px rgba(0, 0, 0, 0.33)'  => __( "Very Light"  , "wpep" ),
        '0 0 5px rgba(0, 0, 0, 0.33)'  => __( "Light" , "wpep" ),
        '0 0 10px rgba(0, 0, 0, 0.33)' => __( "Medium"  , "wpep" ),
      ],
      __( 'Adjust the shadow of the content wrapper.', "wpep" )
    );
  }

  public function get_value( $field_key, $field_default = '' ) {
    return WPEP_Settings::instance()->get_option( $field_key, $field_default );
  }

  public function get_value_translated( $field_key ) {
    return WPEP_Settings::instance()->get_option_translated( $field_key );
  }

  public function display_table_start( $table_class = 'form-table' ) {
    echo    '<table class="' . $table_class . '">';
    echo      '<tbody>';
  }

  public function display_table_end() {
    echo      '</tbody>';
    echo    '</table>';

    $this->set_table_row_data_attributes([]);
    $this->set_table_row_class();
  }

  public function display_table_hidden_field( $field_key, $field_value, $field_name = false ) {
    echo        '<tr style="display:none;">';
    echo          '<td>';
      $this->display_hidden_field( $field_key, $field_value, $field_name );
    echo          '</td>';
    echo        '</tr>';
  }

  private function _table_row_start( $field_key, $class = '' ) {
    $response = '<tr';

    if( $field_key !== false )
      $response .= ' id="wpep-table-row-' . $field_key . '"';

    $response .= ' class="' . $this->_table_row_class . ( $class !== '' ? ' ' . $class : '' ) . '"';

    foreach( $this->_table_row_data_attr as $data_attribute_key => $data_attribute_value )
      $response .= ' data-' . $data_attribute_key . '="' . $data_attribute_value . '" ';

    $response .= '>';

    return $response;
  }

  private function _table_row_end() {
    return '</tr>';
  }

  public function display_table_textarea_field( $field_label, $field_key, $field_description = false, $field_name = false, $height = false ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_textarea_field( $field_key, $field_name, $height );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_text_translated_field( $field_label, $field_key, $field_description = false, $field_name = false ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
    $this->display_text_field( $field_key, $field_name, $this->get_value_translated( $field_key ) );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_text_field( $field_label, $field_key, $field_description = false, $field_name = false, $field_value = null, $field_placeholder = "" ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_text_field( $field_key, $field_name, $field_value, false, $field_placeholder );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_date_field( $field_label, $field_key, $field_description = false, $field_name = false, $field_value = null, $field_placeholder = "", $data_attributes = [] ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_date_field( $field_key, $field_name, $field_value, false, $field_placeholder, $data_attributes );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_numeric_field( $field_label, $field_key, $field_description = false, $field_name = false, $field_value = null, $field_placeholder = "" ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_numeric_field( $field_key, $field_name, $field_value, false, $field_placeholder );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_image_field( $field_label, $field_key, $field_description = false, $field_name = false, $field_value = null, $image_css_attributes = [], $allow_image_remove = true ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_image_field( $field_key, $field_label, $field_name, $field_value, $image_css_attributes, $allow_image_remove );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_file_field( $field_label, $field_key, $field_description = false, $field_name = false, $field_value = null, $class = false, $field_placeholder = '' ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
    $this->display_file_field( $field_key, $field_name, $field_value, $class, $field_placeholder );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_attributes_field_content_editor( $field_label, $field_key, $field_description = false, $field_name = false, $field_map = [], $field_default = [] ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_attributes_field_content_editor( $field_key, $field_name, $field_map, $field_default );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_attributes_field( $field_label, $field_key, $field_description = false, $field_name = false, $field_map = [], $field_default = [] ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_attributes_field( $field_key, $field_name, $field_map, $field_default );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_select_sequence_field( $field_label, $field_key, $field_description = false, $field_name = false, $field_map = [], $field_options = [] ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
    $this->display_select_sequence_field( $field_key, $field_name, $field_map, $field_options );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_list_field( $field_label, $field_key, $field_description = false, $field_name = false, $field_map = [], $field_default = [] ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_list_field( $field_key, $field_name, $field_map, $field_default );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_editor_field( $field_label, $field_key, $field_description = false, $editor_height = 150, $field_name = null, $field_value = null ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_editor_field( $field_key, $editor_height, $field_name, $field_value );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_color_field( $field_label, $field_key, $field_description = false ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_color_field( $field_key );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_checkbox_field( $field_label, $field_key, $field_description = false, $is_checked = null, $force_checked = null ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_checkbox_field( $field_key, $is_checked, null, 0, $force_checked );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_checkbox_array_field( $field_label, $field_key, $field_value ) {
    $identifier = 'wpep-' . $field_key . '-' . $field_value . '';

    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
    $this->display_checkbox_array_field( $field_key, $field_value, null, $identifier );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_select_field( $field_label, $field_key, $field_options = [], $field_description = false, $select_class = 'wpep-select-field', $attributes = [] ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_select_field( $field_key, $field_options, false, null, $select_class, '', $attributes );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_multiple_select_field( $field_label, $field_key, $field_options = [], $field_description = false, $class = 'wpep-multiple-select', $attributes = [] ) {
    echo $this->_table_row_start( $field_key );

    if( $field_label !== false )
      echo '<th scope="row"><label for="wpep-' . $field_key . '">' . $field_label . '</label></th>';

    echo '<td colspan="' . ( $field_label === false ? 2 : 1 ) . '">';
          $this->display_multiple_select_field( $field_key, $field_options, false, null, $class, $attributes );
    echo $this->_field_description( $field_description );
    echo '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_separator( $name, $description = '<hr/>' ) {
    $alias = sanitize_title( $name );

    echo $this->_table_row_start( false, 'wpep-table-separator' );
    echo '<th scope="row"><label for="wpep-' . $alias . '">' . $name . '</label></th>';
    echo '<td>' . $description . '</td>';
    echo $this->_table_row_end();
  }

  public function display_table_html( $description ) {
    echo $this->_table_row_start( false, 'wpep-table-html' );
    echo '<td colspan="2">' . $description . '</td>';
    echo $this->_table_row_end();
  }

  public function display_image_field( $field_key, $image_name, $field_name = false, $image_path = null, $image_css_attributes = [], $allow_image_remove = true ) {
    $field_name = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    $image_path = ( $image_path === null ? $this->get_value( $field_key, '' ) : $image_path );

    $image_style = '';

    foreach( $image_css_attributes as $attribute_key => $attribute_value )
      $image_style .= $attribute_key . ':' . $attribute_value . ';';

    echo '<div id="wpep-image-field-' . wpep_label_to_alias( $field_key ) . '" class="wpep-image-field' . ( $image_path == '' ? '' : ' uploaded' ) . '">';
    echo    '<input type="hidden" value="' . $image_path . '" name="' . $field_name . '">';
    echo    '<img src="' . $image_path . '" style="' . $image_style . '"/>';
    echo    '<p class="wpep-upload-image button button-primary">' . sprintf( __( "Set %s", "wpep" ), $image_name ) . '</p>';
    echo    '<p class="wpep-instructions-image">' . __( "Click the image to edit or update", "wpep" ) . '</p>';

    if( $allow_image_remove )
      echo    '<p class="wpep-remove-image button button-secondary">' . sprintf( __( "Remove %s", "wpep" ), $image_name ) . '</p>';

    echo '</div>';
  }

  public function display_image_builder_field(
    $available_fields,
    $image_field_name,
    $content_field_name,
    $default_image,
    $default_content = [],
    $available_fonts = []
  ) {

    $image   = $this->get_value( $image_field_name, $default_image, true );
    $content = $this->get_value( $content_field_name, $default_content, false );

    echo '<div class="wpep-image-builder-field">';
    echo    '<p class="wpep-change-image button button-primary">' . __( "Change Image", "wpep" ) . '</p>';
    echo    '<div class="wpep-clearfix"></div>';
    echo    '<input type="hidden" class="image-link-field" value="' . $image . '" name="' . $this->field_prefix . '[' . $image_field_name . ']' . '">';
    echo    '<div class="image-builder-container">';
    echo        '<div class="image-inner-container">';
    echo            '<img src="' . $image . '"/>';
    echo            '<div class="content">';

    $i = 1; foreach( $content as $content_part ) {
      $this->_image_builder_field_content_part($content_part, $content_field_name, $i, 'name', $available_fonts );
      $i++;
    }

    $this->_image_builder_field_content_part([], $content_field_name, 'template', 'data-name', $available_fonts );

    echo            '</div>';
    echo        '</div>';
    echo        '<div class="image-options-container">';
    echo          '<ul class="image-options-list">';

    foreach( $available_fields as $field_key => $field_value )
    echo            '<li data-field-name="' .$field_key . '">' . $field_value . '</li>';

    echo          '</ul>';
    echo        '</div>';
    echo        '<div class="wpep-clearfix"></div>';
    echo    '</div>';
    echo '</div>';
  }

  private function _image_builder_field_content_part( $content_part, $content_field_name, $position, $name_attribute = 'name', $available_fonts = [] ) {
    $content_part  = $this->image_builder_field_sanitize_content_part( $content_part );
    $current_style = '';
    $input_name_prefix = $name_attribute . '="' . $this->field_prefix . '[' . $content_field_name . '][' . $position . ']';

    foreach( $content_part as $style_property => $style_value ) {
      if( in_array( $style_property, [ 'content' ] ) )
        continue;

      $current_style .= ( $style_property . ':' . $style_value . ';' );
    }

    if( $position == 'template' )
      $current_style .= 'display:none;';

    echo '<div class="content-part" style="' . $current_style . '">';

    foreach( $content_part as $style_property => $style_value ) {
      if( in_array( $style_property, [ 'content', 'color', 'font-family' ] ) )
        continue;

      echo '<input type="hidden" 
                   ' . $input_name_prefix . '[' . $style_property . ']' . '" 
                   value="' . $style_value . '" 
                   data-property-target="' . $style_property . '"
               />';
    }

    echo    '<span class="input-container"><input autocomplete="off" type="text" ' . ( $input_name_prefix . '[content]' ) . '" value="' . $content_part['content'] . '" data-property-target="content"/></span>';
    echo    '<span class="dragging-handler"></span>';
    echo    '<span class="customize-handler"></span>';
    echo    '<div class="customize-container">';
    echo      '<span class="delete"></span>';
    echo      '<span class="close"></span>';

    if( isset( $content_part['font-family'] ) && !empty( $available_fonts ) ) {
      echo '<select class="wpep-content-part-font-family"
                   ' . $input_name_prefix . '[font-family]' . '" 
                   data-property-target="font-family">';

      foreach( $available_fonts as $available_font_file => $available_font )
        echo '<option value="' . $available_font . '" ' . wpep_utility_selected( $available_font, $content_part['font-family'], false ) . '>' . $available_font . '</option>';

      echo  '</select>';
    }

    if( isset( $content_part['color'] ) )
      echo '<input type="text" class="wpep-content-part-color-picker"
                   ' . $input_name_prefix . '[color]' . '" 
                   value="' . $content_part['color'] . '" 
                   data-property-target="color"
               />';

    echo    '</div>';
    echo '</div>';
  }

  public function image_builder_field_sanitize_content_part( $input_current_values ) {
    if( is_string( $input_current_values ) ) {
      $current_values = json_decode( $input_current_values, true );

      if( $current_values === NULL )
        $current_values = unserialize( $input_current_values );
    } else {
      $current_values = $input_current_values;
      unset( $input_current_values );
    }

    if( !is_array( $current_values ) ) {
      $current_values = [];
    } else {
      foreach( $current_values as $key => $value ) {
        if( !is_array( $value ) )
          continue;

        $current_values[$key] = array_shift( $value );
      }
    }


    return $current_values + [
      'content'     => 'write your content here',
      'left'        => '300px',
      'top'         => '200px',
      'width'       => '200px',
      'height'      => '22px',
      'font-size'   => '19px',
      'font-family' => 'Arial',
      'font-weight' => 300,
      'color'       => '#FFFFFF'
    ];
  }

  public function display_hidden_field( $field_key, $field_value, $field_name = false, $class = '' ) {
    $field_name = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    echo '<input type="hidden" id="wpep-' . $field_key . '" value="' . $field_value . '" name="' . $field_name . '" class="' . $class . '">';
  }

  public function display_textarea_field( $field_key, $field_name = false, $height = false, $placeholder = "" ) {
    $field_name = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    echo '<textarea class="large-text" 
                    placeholder="' . $placeholder . '"
                    ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . '
                    name="' . $field_name . '" ';

    if( $height != false )
      echo ' style="height:' . $height . '" ';

    echo '>' . $this->get_value( $field_key, '' ) . '</textarea>';
  }

  public function display_text_field( $field_key, $field_name = false, $field_value = null, $class = false, $field_placeholder = '', $data_attributes = [], $default_value = '' ) {
    echo $this->get_text_field( $field_key, $field_name, $field_value, $class, $field_placeholder, $data_attributes, $default_value );
  }

  public function display_numeric_field( $field_key, $field_name = false, $field_value = null, $class = false, $field_placeholder = '', $data_attributes = [], $default_value = '' ) {
    echo $this->get_numeric_field( $field_key, $field_name, $field_value, $class, $field_placeholder, $data_attributes, $default_value );
  }

  public function display_date_field( $field_key, $field_name = false, $field_value = null, $class = false, $field_placeholder = '', $data_attributes = [] ) {
    $class = ( $class == false ? 'regular-text wpep-datepicker-field' : $class );

    $this->display_text_field( $field_key, $field_name, $field_value, $class, $field_placeholder, $data_attributes );
  }

  public function display_file_field( $field_key, $field_name = false, $field_value = null, $class = false, $field_placeholder = '' ) {
    if( $field_placeholder === '' )
      $field_placeholder = __( "Click to upload your file", "wpep" );

    echo $this->get_file_field( $field_key, $field_name, $field_value, $class, $field_placeholder );
  }

  public function display_attributes_field_content_editor( $field_key, $field_name = false, $field_map = [], $field_default = []) {
    $field_map         = ( !empty( $field_map ) ? $field_map : $this->get_value( $field_key, [] ) );
    $field_name        = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    if( empty( $field_map ) || !is_array( $field_map ) )
      $field_map = $field_default;

    echo '<div class="wpep-attributes-field-container wpep-attributes-field-type-content-editor" data-wpep-container-type="title_and_content" data-name="' . $field_name . '">';
    echo '<table class="wp-list-table fixed widefat">';
    echo  '<thead>';
    echo    '<tr>';
    echo      '<th class="wpep-attributes-field-row-drag"></th>';
    echo      '<th class="wpep-attributes-field-row-key">' . __( "Title", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-value">' . __( "Content", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-remove"></th>';
    echo    '</tr>';
    echo  '</thead>';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_map as $key => $value ) {
      echo '<tr class="wpep-attributes-field-group">';
      echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
      echo  '<td class="wpep-attributes-field-row-key">';
      echo    '<textarea class="wpep-attributes-field" style="display:none;" name="' . $field_name . '[' . $key . ']">' . $value . '</textarea>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . $key . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-value">';
      echo  '<div class="wpep-content-editor-container">';
      wpep_editor( $value, 'wpep-afv-' . md5( $key ), false, [
        'editor_class'  => "wpep-attributes-field-value",
        'textarea_rows' => 5,
        'editor_css'    => '<style>body#tinymce { padding : 0 !important; }; </style>'
        ]
      );
      echo  '</div>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';
    }

    echo '<tr class="wpep-attributes-field-group" style="display:none;">';
    echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
    echo  '<td class="wpep-attributes-field-row-key">';
    echo    '<textarea class="wpep-attributes-field" style="display:none;"></textarea>';
    echo    '<input type="text" class="wpep-attributes-field-key" value=""/>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-value">';
    echo    '<div class="wpep-content-editor-container">';
    echo    '<div class="wpep-notification wpep-notification-info">' . __( "Content Box will be available shortly", "wpep" ). '</div>';
    echo    '</div>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
    echo '</tr>';

    echo  '</tbody>';
    echo '</table>';
    echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "New Field", "wpep" ) . '</button>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  public function display_attributes_field_downloads( $field_key, $field_name = false, $field_map = [] ) {
    $field_map         = ( !empty( $field_map ) ? $field_map : $this->get_value( $field_key, [] ) );
    $field_name        = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    if( empty( $field_map ) || ( !is_array( $field_map ) && !is_object( $field_map ) ) )
      $field_map = [];

    echo '<div class="wpep-attributes-field-container wpep-attributes-field-type-downloads" data-wpep-container-type="downloads" data-name="' . $field_name . '">';
    echo  '<input type="hidden" name="' . $field_name . '" value=""/>';
    echo '<table class="wp-list-table fixed widefat" ' . ( empty( $field_map ) ? 'style="display:none;"' : '' ) . '>';
    echo  '<thead>';
    echo    '<tr>';
    echo      '<th class="wpep-attributes-field-row-drag"></th>';
    echo      '<th class="wpep-attributes-field-row-key">' . __( "Link", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-value">' . __( "Name", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-remove"></th>';
    echo    '</tr>';
    echo  '</thead>';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_map as $key => $value ) {
      echo '<tr class="wpep-attributes-field-group">';
      echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
      echo  '<td class="wpep-attributes-field-row-key">';
      echo    '<input type="hidden" class="wpep-attributes-field" name="' . $field_name . '[' . esc_attr( $key ) . ']" value="' . esc_attr( $value ) . '"/>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . esc_attr( $key ) . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-value">';
      echo    '<input type="text" class="wpep-attributes-field-value" value="' . esc_attr( $value ) . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';
    }

    echo '<tr class="wpep-attributes-field-group" style="display:none;">';
    echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
    echo  '<td class="wpep-attributes-field-row-key">';
    echo    '<input type="hidden" class="wpep-attributes-field"/>';
    echo    '<input type="text" class="wpep-attributes-field-key" value=""/>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-value">';
    echo    '<input type="text" class="wpep-attributes-field-value" value=""/>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
    echo '</tr>';

    echo  '</tbody>';
    echo '</table>';
    echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "Add Download", "wpep" ) . '</button>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  public function display_attributes_field_jump_links( $field_key, $field_name = false, $field_map = [] ) {
    $field_map         = ( !empty( $field_map ) ? $field_map : $this->get_value( $field_key, [] ) );
    $field_name        = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    if( empty( $field_map ) || !is_array( $field_map ) )
      $field_map = [];

    echo '<div class="wpep-attributes-field-container wpep-attributes-field-type-jump-links" data-name="' . $field_name . '" data-key-glue=":" data-key-default="00">';
    echo  '<input type="hidden" name="' . $field_name . '" value=""/>';
    echo '<table class="wp-list-table fixed widefat" ' . ( empty( $field_map ) ? 'style="display:none;"' : '' ) . '>';
    echo  '<thead>';
    echo    '<tr>';
    echo      '<th class="wpep-attributes-field-row-drag"></th>';
    echo      '<th class="wpep-attributes-field-row-key">' . __( "Timestamp", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-value">' . __( "Title", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-remove"></th>';
    echo    '</tr>';
    echo  '</thead>';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_map as $key => $value ) {
      $key_pieces = explode( ":", $key );

      echo '<tr class="wpep-attributes-field-group">';
      echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
      echo  '<td class="wpep-attributes-field-row-key wpep-jump-link-input-container">';
      echo    '<input type="hidden" class="wpep-attributes-field" name="' . $field_name . '[' . $key . ']" value="' . esc_attr( $value ) . '"/>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . ( isset( $key_pieces[0] ) ? $key_pieces[0] : '00' ) . '" placeholder="hh"/>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . ( isset( $key_pieces[1] ) ? $key_pieces[1] : '00' ) . '" placeholder="mm"/>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . ( isset( $key_pieces[2] ) ? $key_pieces[2] : '00' ) . '" placeholder="ss"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-value">';
      echo    '<input type="text" class="wpep-attributes-field-value" value="' . esc_attr( $value ) . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';
    }

    echo '<tr class="wpep-attributes-field-group" style="display:none;">';
    echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
    echo  '<td class="wpep-attributes-field-row-key wpep-jump-link-input-container">';
    echo    '<input type="hidden" class="wpep-attributes-field"/>';
    echo    '<input type="text" class="wpep-attributes-field-key" value="00" placeholder="hh"/>';
    echo    '<input type="text" class="wpep-attributes-field-key" value="00" placeholder="mm"/>';
    echo    '<input type="text" class="wpep-attributes-field-key" value="00" placeholder="ss"/>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-value">';
    echo    '<input type="text" class="wpep-attributes-field-value" value=""/>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
    echo '</tr>';

    echo  '</tbody>';
    echo '</table>';
    echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "Add JumpLink", "wpep" ) . '</button>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  public function display_attributes_field( $field_key, $field_name = false, $field_map = [], $field_default = [] ) {
    $field_map         = ( !empty( $field_map ) ? $field_map : $this->get_value( $field_key, [] ) );
    $field_name        = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    if( empty( $field_map ) || !is_array( $field_map ) )
      $field_map = $field_default;

    echo '<div class="wpep-attributes-field-container" data-wpep-container-type="standard" data-name="' . $field_name . '">';
    echo '<table class="wp-list-table fixed widefat">';
    echo  '<thead>';
    echo    '<tr>';
    echo      '<th class="wpep-attributes-field-row-key">' . __( "Field Type", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-value">' . __( "Field Value", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-remove"></th>';
    echo    '</tr>';
    echo  '</thead>';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_map as $key => $value ) {
      echo '<tr class="wpep-attributes-field-group">';
      echo  '<td class="wpep-attributes-field-row-key">';
      echo    '<input type="hidden" class="wpep-attributes-field" name="' . $field_name . '[' . $key . ']" value="' . esc_attr( $value ) . '"/>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . esc_attr( $value ) . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-value">';
      echo    '<input type="text" class="wpep-attributes-field-value" value="' . esc_attr( $value ) . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';
    }

    echo  '</tbody>';
    echo '</table>';
    echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "New Field", "wpep" ) . '</button>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  public function display_attributes_field_filters( $field_key, $field_name = false, $field_map = [], $field_default = [] ) {
    $field_map         = ( !empty( $field_map ) ? $field_map : $this->get_value( $field_key, [] ) );
    $field_name        = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    if( empty( $field_map ) || !is_array( $field_map ) )
      $field_map = $field_default;

    echo '<div class="wpep-attributes-field-container" data-wpep-container-type="filters" data-name="' . $field_name . '">';
    echo '<table class="wp-list-table fixed widefat">';
    echo  '<thead>';
    echo    '<tr>';
    echo      '<th class="wpep-attributes-field-row-drag"></th>';
    echo      '<th class="wpep-attributes-field-row-key">' . __( "Title", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-value">' . __( "Icon", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-remove"></th>';
    echo    '</tr>';
    echo  '</thead>';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_map as $key => $value ) {
      echo '<tr class="wpep-attributes-field-group">';
      echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
      echo  '<td class="wpep-attributes-field-row-key">';
      echo    '<input type="hidden" class="wpep-attributes-field" name="' . $field_name . '[' . $key . ']" value="' . $value . '"/>';
      echo    '<input type="text" class="wpep-attributes-field-key" value="' . $key . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-value">';
      echo    '<input type="text" class="wpep-attributes-field-value wpep-js-select-element-icon" value="' . $value . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';
    }

    echo '<tr class="wpep-attributes-field-group" style="display:none;">';
    echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
    echo  '<td class="wpep-attributes-field-row-key">';
    echo    '<input type="hidden" class="wpep-attributes-field" name="" value=""/>';
    echo    '<input type="text" class="wpep-attributes-field-key" value=""/>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-value">';
    echo    '<input type="text" class="wpep-attributes-field-value wpep-js-select-element-icon" value=""/>';
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
    echo '</tr>';

    echo  '</tbody>';
    echo '</table>';
    echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "New Filter", "wpep" ) . '</button>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  public function display_learning_journey_for_field_map( $field_prefix, $field_options, $object_information ) {
    echo '<div class="wpep-attributes-field-container" data-wpep-container-type="learning_journey" data-wpep-container-segment="field_map" data-name="' . $field_prefix . '">';
    echo '<table class="wp-list-table fixed widefat">';
    echo  '<thead>';
    echo    '<tr>';
    echo      '<th class="wpep-attributes-field-row">' . __( "Name", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row">' . __( "Delivery", "wpep" ) . '</th>';
    echo    '</tr>';
    echo  '</thead>';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_options as $field_journey_key => $field_journey_data ) {
      $current_field_type    = $field_journey_key . '_type';
      $current_field_segment = $field_journey_key . '_segment';

      $current_delivery_type    = ( isset( $object_information->$current_field_type ) ? $object_information->$current_field_type : '' );
      $current_delivery_segment = ( isset( $object_information->$current_field_segment ) ? $object_information->$current_field_segment : '' );

      echo '<tr class="wpep-attributes-field-group" data-wpep-name-prefix="' . $field_prefix . '" data-wpep-learning-journey-delivery-type="' . $current_delivery_type . '">';
      echo  '<td class="wpep-attributes-field-row">' . $field_journey_data['name'] . '</td>';
      echo  '<td class="wpep-attributes-field-row">' . $this->_learning_journey_delivery( $field_prefix, $current_delivery_type, $current_delivery_segment, [], $field_journey_data['options'], $field_journey_key ) . '</td>';
      echo '</tr>';
    }
    echo  '</tbody>';
    echo '</table>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  public function display_select_learning_journey_sequence_field( $field_key, $field_name = false, $field_value = false ) {
    $field_name       = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );
    $learning_journey = ( $field_value !== false ? $field_value : $this->get_value( $field_key, [] ) );

    echo '<div class="wpep-attributes-field-container" data-wpep-container-type="learning_journey" data-name="' . $field_name . '">';
    echo '<table class="wp-list-table fixed widefat">';
    echo  '<thead>';
    echo    '<tr>';
    echo      '<th class="wpep-attributes-field-row-drag"></th>';
    echo      '<th class="wpep-attributes-field-row">' . __( "Name", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row">' . __( "Delivery", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-remove"></th>';
    echo    '</tr>';
    echo  '</thead>';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $learning_journey as $learning_journey_key => $learning_journey_data ) {
      if( is_object( $learning_journey_data ) )
        $learning_journey_data = get_object_vars( $learning_journey_data );

      $id = isset( $learning_journey_data['id'] ) ? $learning_journey_data['id'] : uniqid( 'new_' );

      $field_prefix = $field_name . '[' . $id . ']';

      $post_information = get_post( $learning_journey_data['journey_post_id'] );

      $text = '';

      if( isset(wpep_content_type()->contentTypes[ $post_information->post_type ] ) )
        $text .= wpep_content_type()->contentTypes[$post_information->post_type]->get_singular_name() . ' - ';

      $text .= $post_information->post_title . ' #' . $post_information->ID;

      echo '<tr class="wpep-attributes-field-group" data-wpep-name-prefix="' . $field_prefix . '" data-wpep-learning-journey-delivery-type="' . $learning_journey_data['journey_delivery_type'] . '">';
      echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
      echo  '<td class="wpep-attributes-field-row">' . $this->_learning_journey_select_post_id( $field_prefix, $post_information->ID, $text ) . '</td>';
      echo  '<td class="wpep-attributes-field-row">' . $this->_learning_journey_delivery( $field_prefix, $learning_journey_data['journey_delivery_type'], $learning_journey_data['journey_delivery_segment'] ) . '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';
    }

    echo '<tr class="wpep-attributes-field-group" data-wpep-name-prefix="' . $field_name . '[%rand%]' . '" style="display:none;">';
    echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
    echo  '<td class="wpep-attributes-field-row">' .
              $this->_learning_journey_select_post_id( false, false, __( "Press to Assign Content", "wpep" ), [ 'data-wpep-prevent-js-select' => '1' ] ) .
          '</td>';
    echo  '<td class="wpep-attributes-field-row">' . $this->_learning_journey_delivery( false, WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_INSTANT, '', [ 'data-wpep-prevent-js-select' => '1' ] ) . '</td>';
    echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
    echo '</tr>';

    echo  '</tbody>';
    echo '</table>';
    echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "Add New", "wpep" ) . '</button>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  private function _learning_journey_select_post_id( $field_prefix, $option_value, $option_text = 0, $additional_select_attributes = [] ) {
    $select_field_attributes = ( is_array( $additional_select_attributes ) ? $additional_select_attributes : [] ) + [
      'data-wpep-name'          => "journey_post_id",
      'data-wpep-select-search' => json_encode( [
        'action'    => 'wpep_admin_post_learning_journey_search'
      ] )
    ];

    if( $field_prefix != false )
      $select_field_attributes['name'] = $field_prefix . '[journey_post_id]';

    $response = '';
    $response .= '<select class="wpep-js-select-element"';
    foreach( $select_field_attributes as $attribute_key => $attribute_value ) {
      $response .= ' ' . $attribute_key . '="' . esc_attr( $attribute_value ) . '"';
    }
    $response .= '>';
    $response .=   '<option value="' . $option_value . '" selected="selected">' . $option_text . '</option>';
    $response .= '</select>';

    return $response;
  }

  private function _learning_journey_delivery(
    $field_prefix,
    $journey_delivery_type,
    $journey_delivery_segment = '',
    $additional_select_attributes = [],
    $journey_delivery_types = [],
    $field_prefix_inner = 'journey_delivery'
  ) {
    $journey_delivery_types = ( empty( $journey_delivery_types ) ? wpep_learning_journey_delivery_types() : $journey_delivery_types );

    $delivery_type_field_attributes    = $additional_select_attributes + [
        'data-wpep-name'                        => $field_prefix_inner . "_type",
        'data-wpep-learning-journey-field-type' => 'type',
      ];
    $delivery_segment_field_attributes = $additional_select_attributes + [
      'data-wpep-name'                        => $field_prefix_inner . "_segment",
      'data-wpep-learning-journey-field-type' => 'segment',
      'value'                                 => ( $journey_delivery_type === WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_SPECIFIC_DATE ? wpep_format_date_with_time( $journey_delivery_segment ) : $journey_delivery_segment )
    ];

    if( $field_prefix != false ) {
      $delivery_type_field_attributes['name']    = $field_prefix . '[' . $field_prefix_inner . '_type]';
      $delivery_segment_field_attributes['name'] = $field_prefix . '[' . $field_prefix_inner . '_segment]';
    }

    $response = '';

    $response .= '<select class="wpep-select-field"';
    foreach( $delivery_type_field_attributes as $attribute_key => $attribute_value ) {
      $response .= ' ' . $attribute_key . '="' . esc_attr( $attribute_value ) . '"';
    }
    $response .= '>';

    foreach( $journey_delivery_types as $delivery_type => $delivery_type_text )
      $response .=   '<option value="' . $delivery_type . '" ' . selected( $delivery_type, $journey_delivery_type, false ). '>' . $delivery_type_text . '</option>';

    $response .= '</select>';

    $response .= '<input';
    foreach( $delivery_segment_field_attributes as $attribute_key => $attribute_value ) {
      $response .= ' ' . $attribute_key . '="' . esc_attr( $attribute_value ) . '"';
    }
    $response .= '>';

    return $response;
  }

  public function display_select_sequence_field( $field_key, $field_name = false, $field_map = [], $field_options = [], $select_field_attributes = [] ) {
    $field_map         = ( !empty( $field_map ) ? $field_map : $this->get_value( $field_key, [] ) );
    $field_name        = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    echo '<div class="wpep-attributes-field-container" data-wpep-container-type="select_sequence" data-name="' . $field_name . '">';
    echo '<table class="wp-list-table fixed widefat">';
    echo  '<thead>';
    echo    '<tr>';
    echo      '<th class="wpep-attributes-field-row-drag"></th>';
    echo      '<th class="wpep-attributes-field-row-key">' . __( "Name", "wpep" ) . '</th>';
    echo      '<th class="wpep-attributes-field-row-remove"></th>';
    echo    '</tr>';
    echo  '</thead>';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_map as $selected_field_from_map ) {
      echo '<tr class="wpep-attributes-field-group">';
      echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';

      echo  '<td class="wpep-attributes-field-row-value">';
      echo    '<input type="hidden" class="wpep-attributes-field" name="' . $field_name . '[' . $selected_field_from_map . ']" value="' . $selected_field_from_map . '"/>';

      $this->_sequence_field_select( $field_options, $selected_field_from_map, $select_field_attributes );

      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';
    }

    echo '<tr class="wpep-attributes-field-group" style="display:none;">';
    echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
    echo  '<td class="wpep-attributes-field-row-value">';
    echo    '<input type="hidden" class="wpep-attributes-field" name="" value=""/>';
    $this->_sequence_field_select( $field_options, 0, $select_field_attributes + [ 'data-wpep-prevent-js-select' => '1' ] );
    echo  '</td>';
    echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
    echo '</tr>';

    echo  '</tbody>';
    echo '</table>';
    echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "Add New", "wpep" ) . '</button>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  private function _sequence_field_select( $field_list, $selected_field_key = 0, $select_field_attributes = [] ) {
    echo '<select class="wpep-attributes-field-key wpep-attributes-field-value ';
    echo   ( isset( $select_field_attributes['class'] ) ? $select_field_attributes['class'] : 'wpep-select-field' ) . '"';

    if( is_array( $select_field_attributes ) ) {
      foreach( $select_field_attributes as $attribute_key => $attribute_value ) {
        if( $attribute_key === 'class' )
          continue;

        echo ' ' . $attribute_key . '="' . esc_attr( $attribute_value ) . '"';
      }
    }

    echo '>';
    echo      '<option value="">' . __( "Select an option", "wpep" ) . '</option>';
    foreach( $field_list as $field_key => $field_value )
      echo    '<option value="' . $field_key . '" ' . ( selected( $field_key == $selected_field_key, true, false ) ) . '>' . $field_value . '</option>';

    echo '</select>';
  }

  public function display_list_field( $field_key, $field_name = false, $field_map = [], $field_default = [] ) {
    $field_map         = ( !empty( $field_map ) ? $field_map : $this->get_value( $field_key, [] ) );
    $field_name        = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    if( empty( $field_map ) || !is_array( $field_map ) )
      $field_map = $field_default;

    echo '<div class="wpep-attributes-field-container" data-wpep-container-type="list" data-name="' . $field_name . '">';
    echo '<table class="wp-list-table fixed widefat">';
    echo  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_map as $key => $value ) {
      echo '<tr class="wpep-attributes-field-group">';
      echo  '<td class="wpep-attributes-field-row-drag"><span class="wpep-attributes-field-group-order-handle"></span></td>';
      echo  '<td class="wpep-attributes-field-row-value">';
      echo    '<input type="text" class="wpep-attributes-field-value" name="' . $field_name . '[]" value="' . $value . '"/>';
      echo  '</td>';
      echo  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      echo '</tr>';
    }

    echo  '</tbody>';
    echo '</table>';
    echo '<button class="button button-primary wpep-attributes-field-group-add">' . __( "Add", "wpep" ) . '</button>';
    echo '<div class="clear"></div>';
    echo '</div>';
  }

  public function display_editor_field( $field_key, $editor_height = 150, $field_name = null, $field_value = null ) {
    $field_name  = ( $field_name === null  ? '' . $this->field_prefix . '[' . $field_key . ']' : $field_name );
    $field_value = ( $field_value === null ? $this->get_value( $field_key, '' ) : $field_value );

    wpep_editor(
      $field_value,
      'wpep-' . $field_key,
      $field_name,
      [
        'editor_height' => $editor_height
      ]
    );
  }

  public function display_color_field( $field_key ) {
    echo  '<input type="text" class="wpep-color-field" id="wpep-' . $field_key . '" value="' . $this->get_value( $field_key, '' ) . '" name="' . $this->field_prefix . '[' . $field_key . ']">';
  }

  public function display_checkbox_field( $field_key, $is_checked = null, $field_name = null, $default_value = 0, $force_checked = null ) {
    echo $this->get_checkbox_field( $field_key, $is_checked, $field_name, $default_value, $force_checked );
  }

  public function display_checkbox_array_field( $field_key, $field_value, $field_name = null, $identifier = null ) {
    $field_name = ( $field_name === null ? $this->field_prefix . '[' . $field_key . '][]' : $field_name );
    $identifier = ( $identifier === null ? 'wpep-' . $field_key . '-' . $field_value : $identifier );

    echo  '<input type="checkbox"
                  id="' . $identifier . '"
                  value="' . $field_value . '"
                  name="' . $field_name . '"
                  ' . checked( in_array( $field_value, $this->get_value( $field_key ) ), true, false ) . '
                  >';
  }

  public function display_radio_fields( $field_key, $field_options = [], $default_value = '' ) {
    echo '<div class="wpep-radio-container">';

    foreach( $field_options as $field_option_key => $field_option_label ) {
      echo '<div class="wpep-radio">';
      echo '<input type="radio"
                   ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . '
                   value="' . $field_option_key . '"
                   name="' . $this->field_prefix . '[' . $field_key . ']"
                   ' . checked( $this->get_value( $field_key, $default_value ), $field_option_key, false ) . '
                   >';
      echo $field_option_label;
      echo '</div>';
    }

    echo '</div>';
  }

  public function display_select_field( $field_key, $field_options = [], $field_name = false, $current_field_value = null, $class = 'wpep-select-field', $default_value = '', $attributes = [] ) {
    $current_field_value = ( $current_field_value === null ? $this->get_value( $field_key, $default_value ) : $current_field_value );
    $field_name          = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );

    if( $field_key !== false )
      $class .= ' wpep-id-' . $field_key;

    echo '<select class="' . $class . '" ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . ' name="' . $field_name . '"';

    if( is_array( $attributes ) ) {
      foreach( $attributes as $attribute_key => $attribute_value )
        echo ' ' . $attribute_key . '="' . esc_attr( $attribute_value ) . '"';
    }

    echo '>';

    $this->_display_select_field_options( $field_options, $current_field_value );

    echo '</select>';
  }

  private function _display_select_field_options( $field_options, $current_field_value ) {
    foreach( $field_options as $field_option_key => $field_option_label ) {
      $attributes = [];

      if( is_array( $field_option_label ) ) {
        // This is a Field Option Group
        if( isset( $field_option_label[ 'group_title' ] ) && isset( $field_option_label['group_options'] ) ) {
          echo '<optgroup label="' . $field_option_label[ 'group_title' ] . '">';

          foreach( $field_option_label['group_options'] as $inner_value => $inner_label )
            $this->_display_select_field_option( $inner_value, $current_field_value, $inner_label, [] );

          echo '</optgroup>';

          continue;
        }

        $attributes = $field_option_label;

        unset( $attributes['label'] );

        $field_option_label = isset( $field_option_label['label'] ) ? $field_option_label['label'] : $field_option_key;
      }

      $this->_display_select_field_option( $field_option_key, $current_field_value, $field_option_label, $attributes );
    }
  }

  private function _display_select_field_option( $field_option_key, $field_value, $field_option_label, $attributes = [] ) {
    echo '<option value="' . $field_option_key . '" ';

    foreach( $attributes as $attribute_key => $attribute_value )
      echo 'data-' . str_replace( '_', '-', $attribute_key ) . '="' . esc_attr( $attribute_value ) . '" ';

    if( is_array( $field_value ) )
      selected( in_array( $field_option_key, $field_value ) );
    else
      selected( $field_option_key, $field_value );

    echo '>' . $field_option_label . '</option>';
  }

  public function display_multiple_select_field( $field_key, $field_options = [], $field_name = false, $field_values = null, $class = 'wpep-multiple-select', $attributes = [] ) {
    $field_values = ( $field_values === null ? $this->get_value( $field_key, [] ) : $field_values );
    $field_values = ( is_array( $field_values ) ? $field_values : [] );
    $field_name   = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . '][]' );

    echo '<input type="hidden" 
                 name="' . ( wpep_string_ends_with( $field_name, '[]' ) && strpos( $field_key, '_' ) === 0 ? substr( $field_name, 0, -2 ) : $field_name ) . '" 
                 value="' . serialize( [] ) . '">';


    echo '<select ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . ' 
                  name="' . $field_name . '" 
                  class="' . $class . '" 
                  multiple="multiple"';

    if( is_array( $attributes ) ) {
      foreach( $attributes as $attribute_key => $attribute_value )
        echo ' ' . $attribute_key . '="' . esc_attr( $attribute_value ) . '"';
    }

    echo '>';

    $this->_display_select_field_options( $field_options, $field_values );

    echo '</select>';
  }

  public function display_time_field( $field_key, $field_name = false, $field_value = null ) {
    $field_name  = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );
    $field_value = intval( $field_value === null ? $this->get_value( $field_key, '' ) : $field_value );

    $dhms = wpep_time_to_dhms( $field_value );

    echo '<div class="wpep-time-field-container">';
    echo    '<input class="wpep-time-field-value" type="hidden" name="' . $field_name . '" value="' . $field_value . '"/>';

    echo    '<div class="wpep-time-field-input-container wpep-time-field-input-days-container">';
    echo      '<input type="number" class="wpep-time-field-days" min="0" max="360" value="' . $dhms['days'] . '" data-field-multiplier="' . ( 60 * 60 * 24 ) . '"/>';
    echo      '<span>' . __( "Days", "wpep" ) . '</span>';
    echo    '</div>';
    echo    '<div class="wpep-time-field-input-container wpep-time-field-input-hours-container">';
    echo      '<input type="number" class="wpep-time-field-hours" min="0" max="24" value="' . $dhms['hours'] . '" data-field-multiplier="' . ( 60 * 60 ) . '"/>';
    echo      '<span>' . __( "Hours", "wpep" ) . '</span>';
    echo    '</div>';
    echo    '<div class="wpep-time-field-input-container wpep-time-field-input-minutes-container">';
    echo      '<input type="number" class="wpep-time-field-minutes" min="0" max="60" value="' . $dhms['minutes'] . '" data-field-multiplier="60"/>';
    echo      '<span>' . __( "Minutes", "wpep" ) . '</span>';
    echo    '</div>';
    echo    '<div class="wpep-time-field-input-container wpep-time-field-input-seconds-container">';
    echo      '<input type="number" class="wpep-time-field-seconds" min="0" max="60" value="' . $dhms['seconds'] . '" data-field-multiplier="1"/>';
    echo      '<span>' . __( "Seconds", "wpep" ) . '</span>';
    echo    '</div>';

    echo '</div>';
  }

  public function get_text_field( $field_key, $field_name = false, $field_value = null, $class = false, $field_placeholder = '', $data_attributes = [], $default_value = '' ) {
    $field_name = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );
    $class = ( $class == false ? 'regular-text' : $class );

    if( $field_key !== false )
      $class .= ' wpep-id-' . $field_key;

    $response = '<input type="text" 
                        class="' . $class . '" 
                        ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . '
                        value="' . esc_attr( $field_value === null ? $this->get_value( $field_key, $default_value ) : $field_value ) . '" 
                        name="' . $field_name . '" 
                        placeholder="' . $field_placeholder . '" ';

    foreach( $data_attributes as $data_attribute_key => $data_attribute_value )
      $response .= ' data-' . $data_attribute_key . '="' . $data_attribute_value . '" ';

    $response .= '  >';

    return $response;
  }

  public function get_file_field( $field_key, $field_name = false, $field_value = null, $class = false, $field_placeholder = '' ) {
    $class = ( $class == false ? 'regular-text' : $class ) . ' wpep-file-field';

    return $this->get_text_field( $field_key, $field_name, $field_value, $class, $field_placeholder );
  }

  public function get_numeric_field( $field_key, $field_name = false, $field_value = null, $class = false, $field_placeholder = '', $data_attributes = [], $default_value = '' ) {
    $field_name = ( $field_name != false ? $field_name : $this->field_prefix . '[' . $field_key . ']' );
    $class = ( $class == false ? 'regular-text regular-number' : $class );

    $response = '<input type="number" 
                        class="' . $class . '" 
                        ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . '
                        value="' . esc_attr( $field_value === null ? $this->get_value( $field_key, $default_value ) : $field_value ) . '" 
                        name="' . $field_name . '" 
                        placeholder="' . $field_placeholder . '" ';

    foreach( $data_attributes as $data_attribute_key => $data_attribute_value )
      $response .= ' data-' . $data_attribute_key . '="' . $data_attribute_value . '" ';

    $response .= '  >';

    return $response;
  }

  public function get_checkbox_field( $field_key, $is_checked = null, $field_name = null, $default_value = 0, $force_checked = null ) {
    $field_name = ( $field_name === null ? $this->field_prefix . '[' . $field_key . ']' : $field_name );

    if( is_array( $is_checked ) ) {
      $is_checked = array_reverse($is_checked);
      $is_checked = array_shift( $is_checked );
    }

    if( $force_checked == true )
      $is_checked = true;

    $response = '';
    $response .=  '<input type="hidden" value="0" name="' . $field_name . '">';
    $response .=  '<input type="checkbox"
                          ' . ( $field_key !== false ? 'id="wpep-' . $field_key . '" ' : '' ) . '
                          value="1"
                          name="' . $field_name . '"
                          ' . checked( ( $is_checked === null || $is_checked == '' ? $this->get_value( $field_key, $default_value ) : $is_checked ), true, false ) . ' '
              . ( $force_checked ? disabled( true, true, false ) : '' ) .
              '>';

    return $response;
  }

  public function get_attributes_field_with_select( $field_key, $select_field_options, $options = [] ) {
    $field_map         = ( isset( $options['field_map' ] ) && !empty( $options['field_map' ] ) ? $options['field_map' ] : ( isset( $options[ 'field_key'] ) ? $this->get_value( $field_key, [] ) : [] ) );
    $field_name        = ( isset( $options['field_name'] ) && $options['field_name'] != false  ? $options['field_name'] : $this->field_prefix . '[' . $field_key . ']' );

    if( ( empty( $field_map ) || !is_array( $field_map ) ) && isset( $options['field_default' ] ) )
      $field_map = $options['field_default' ];

    $response = '';

    $response .= '<div class="wpep-attributes-field-container" data-name="' . $field_name . '">';
    $response .= '<table class="wp-list-table fixed widefat">';
    $response .=  '<thead>';
    $response .=    '<tr>';
    $response .=      '<th class="wpep-attributes-field-row-key">' . ( isset( $options[ 'label_key' ] ) ? $options[ 'label_key' ] : __( "Field Type", "wpep" ) ) . '</th>';
    $response .=      '<th class="wpep-attributes-field-row-value">' . ( isset( $options[ 'label_value' ] ) ? $options[ 'label_value' ] : __( "Field Value", "wpep" ) ) . '</th>';
    $response .=      '<th class="wpep-attributes-field-row-remove"></th>';
    $response .=    '</tr>';
    $response .=  '</thead>';
    $response .=  '<tbody class="wpep-attributes-field-group-container">';

    foreach( $field_map as $key => $value ) {
      $response .= '<tr class="wpep-attributes-field-group">';
      $response .=  '<td class="wpep-attributes-field-row-key">';
      $response .=    '<input type="hidden" class="wpep-attributes-field" name="' . $field_name . '[' . $key . ']" value="' . $value . '"/>';
      $response .=    '<input type="text" class="wpep-attributes-field-key" value="' . $key . '"/>';
      $response .=  '</td>';
      $response .=  '<td class="wpep-attributes-field-row-value">';
      $response .=    '<select class="wpep-attributes-field-value">';
      foreach( $select_field_options as $select_field_option_key => $select_field_option_value )
        $response .=      '<option value="' . $select_field_option_key . '" ' . selected( $value, $select_field_option_key, false ) . '>' . $select_field_option_value . '</option>';
      $response .=    '</select>';
      $response .=  '</td>';
      $response .=  '<td class="wpep-attributes-field-row-remove"><span class="button wpep-attributes-field-group-delete">' . __( "X" ) . '</span></td>';
      $response .= '</tr>';
    }

    $response .=  '</tbody>';
    $response .= '</table>';
    $response .= '<button class="button button-primary wpep-attributes-field-group-add">';
    $response .=    ( isset( $options[ 'label_add_new' ] ) ? $options[ 'label_add_new' ] : __( "New Field", "wpep" ) );
    $response .= '</button>';
    $response .= '<div class="clear"></div>';
    $response .= '</div>';

    return $response;
  }

  public function get_live_preview_container_start( $preview_items = [] ) {
    $preview_items = ( $preview_items === false || !empty( $preview_items ) ? $preview_items : [
      [
        'title'   => __( "WPEP Index", "wpep" ),
        'content' => '[wpep_index]'
      ]
    ] );

    $response  = '<div class="wpep-live-preview-container">';
    $response .=   wp_nonce_field( 'wpep-preview-content', 'wpep-preview-content', false, false );

    if( $preview_items !== false ) {
      $response .=  '<textarea name="wpep-content" style="display:none;">' . $preview_items[0]['content'] . '</textarea>';
    }

    $response .= '<div class="wpep-live-preview-settings">';

    return $response;
  }

  public function get_live_preview_container_end() {
    $base_url = get_site_url();

    $response = '';

    $response .=    '</div>'; // end wpep-live-preview-settings
    $response .=  '<div class="wpep-live-preview-screen-container" 
                        data-wpep-preview-layout-options="' . wpep_controller()->templateStyle->get_wpep_wrapper_style() . '"
                        data-wpep-preview-base-url="' . $base_url . '">';
    $response .=  '</div>';
    $response .=    '<div class="wpep-clearfix"></div>';
    $response .= '</div>'; // end wpep-live-preview-container
    $response .= '<div class="wpep-clearfix"></div>';

    return $response;
  }

  private function _field_description( $field_description = false ) {
    if( $field_description == false || $field_description == '' )
      return '';

    if( strlen( strip_tags( $field_description ) ) != strlen( $field_description ) )
      return $field_description;

    return '<p>' . $field_description . '</p>';
  }

}
