<?php

namespace WPEP\Container;

abstract class LicenseAndUpdates {

  public $_top_level = true;

  public function __construct() {
    $this->api = new LicenseAndUpdates\API( $this );

    wpep_controller()->licenseLibrary->register( $this );
  }

  abstract public function get_product_name();

  public function get_plugin_alias() {
    return wpep_label_to_alias( $this->get_product_name() );
  }

  abstract public function get_plugin_version();

  abstract public function get_product_id();

  abstract public function get_options_prefix();

  public function get_license() {
    return get_option( $this->get_option_license_key(), '' );
  }

  public function prepare_license_status($licenseStatus) {
    return is_string($licenseStatus) ? md5($licenseStatus . '-wpep') : $licenseStatus;
  }

  public function is_valid_license() {
    $status = get_option( $this->get_option_license_key_status() );

    return $status !== false && $status == $this->prepare_license_status('valid' );
  }

  public function get_option_license_key() {
    return $this->get_options_prefix() . 'key';
  }

  public function get_option_license_key_bundled_products() {
    return $this->get_options_prefix() . 'bundled_products';
  }

  public function get_option_license_key_status() {
    return $this->get_options_prefix() . 'status_information';
  }

  /**
   * @var LicenseAndUpdates\Administration
   */
  public $administration;
  /**
   * @var LicenseAndUpdates\API
   */
  public $api;
  /**
   * @var LicenseAndUpdates\Updater
   */
  public $updater;

  final public function setup_license_and_updates( $plugin_base_file_path = false, $administration_ajax_request = false ) {
    if( !is_admin() )
      return;

    if( $administration_ajax_request !== false )
      $this->administration = new LicenseAndUpdates\Administration( $this, $administration_ajax_request );

    if( $plugin_base_file_path !== false )
      $this->updater = new LicenseAndUpdates\Updater( $this, $plugin_base_file_path );
  }

  final public function activate_license( $license ) {
    $this->set_license( $license );

    $license_data = $this->api->activate_license();

    if( isset( $license_data->license ) ) {
      if( $license_data->license == 'valid' ) {
        $this->set_license_status( 'valid' );
      } else {
        $this->set_license_status( 'invalid' );

        return $license_data;
      }
    } else {
      if( is_wp_error( $license_data ) )
        $this->set_license_status( $license_data->get_error_message() );

      return $license_data;
    }

    if( $this->_top_level )
      $this->set_license_bundled_products(isset( $license_data->bundled_products ) ? $license_data->bundled_products : false );

    return $this->is_valid_license();
  }

  /**
   * @param $license
   * @return $this
   */
  public function set_license( $license ) {
    update_option( $this->get_option_license_key(), $license );

    return $this;
  }

  /**
   * @param string $status
   * @return $this
   */
  final public function set_license_status( $status = 'valid' ) {
    update_option( $this->get_option_license_key_status(), $this->prepare_license_status( $status ) );

    return $this;
  }

  final public function get_license_bundled_products() {
    return get_option( $this->get_option_license_key_bundled_products(), [] );
  }

  /**
   * @param bool|array $product_list
   * @return $this
   */
  final public function set_license_bundled_products( $product_list ) {
    if( $product_list == false )
      delete_option( $this->get_option_license_key_bundled_products() );
    else
      update_option( $this->get_option_license_key_bundled_products(), $product_list );

    return $this;
  }

  /**
   * @return $this
   */
  final public function refresh_license_status() {
    $license_data = $this->api->get_license_status();

    if( !isset( $license_data->license ) ) {
      if( is_wp_error( $license_data ) )
        $this->set_license_status( $license_data->get_error_message() );

      return $this;
    }

    if( in_array( $license_data->license, [ 'site_inactive', 'inactive' ] ) ) {
      $this->activate_license( $this->get_license() );
    } else if( $license_data->license == 'valid' ) {
      $this->set_license_status( 'valid' );
    } else {
      $this->set_license_status( 'invalid' );
    }

    if( $this->_top_level )
      $this->set_license_bundled_products(
        isset( $license_data->bundled_products ) ? $license_data->bundled_products : false
      );

    return $this;

  }

}