<?php

namespace WPEP\Container\LicenseAndUpdates;

class Administration {

  /**
   * @var \WPEP\Container\LicenseAndUpdates
   */
  protected $_controller;
  private $_ajax_request_name = '';

  public function __construct( $controller_instance, $administration_ajax_request ) {
    $this->_controller        = $controller_instance;
    $this->_ajax_request_name = $administration_ajax_request;

    if( !wpep_controller()->request->is_request_type( 'admin' ) )
      return;

    if( $this->_handle_menu_registration() )
      add_action( 'admin_menu', [ $this, '_register_menu' ], 15 );

    add_action( 'admin_init', [ $this, '_admin_init' ] );
  }

  public function _admin_init() {
    if( $this->_controller->is_valid_license() )
      return;

    if( isset( $_GET['page'] ) && ( $_GET['page'] == $this->_controller->get_plugin_alias() . '-migration' || $_GET['page'] == $this->_controller->get_plugin_alias() . '-license' ) )
      return;

    if( isset( $_GET['post_type'] )
      && in_array( $_GET['post_type'], [ WPEP_POST_TYPE_COURSE, WPEP_POST_TYPE_VIDEO, WPEP_POST_TYPE_E_BOOK, WPEP_POST_TYPE_OFFER, WPEP_POST_TYPE_ASSESSMENT ] ) ) {
      wpep_redirect( $this->get_configuration_page() );
      exit;
    }

    add_action('admin_notices', [ $this, 'admin_notices' ] );
  }

  public function _register_menu() {
    wpep_admin_register_menu(
      sprintf( __( "%s License", 'wpep'), $this->_controller->get_product_name() ),
      __( "License", 'wpep'),
      'manage_options',
      $this->_controller->get_plugin_alias() . "-license",
      [ $this, "display_page_content" ]
    );
  }

  private function _handle_menu_registration() {
    if( isset( $_GET['page'] ) && $_GET['page'] == $this->_controller->get_plugin_alias() . '-license' )
      return true;

    return !$this->_controller->is_valid_license();
  }

  public function display_page_content() {
    echo '<div class="wpep-setup-process-container">';
    echo '<form class="wpep-utility-data-sync" 
                data-wpep-sync-ajax="' . $this->_ajax_request_name . '"
                data-wpep-sync-stage="' . ( isset( $_GET[ 'wpep-license-step' ] ) ? $_GET[ 'wpep-license-step' ] : 'license' ) . '"
                method="POST">';
    echo   '<div class="wpep-content-wrapper"></div>';
    echo   wpep_spinner_loader( __( "Please wait...", "wpep" ) );
    echo '</form>';
    echo '</div>';
    echo '<style>#adminmenumain { display:none !important; } #wpcontent, #wpfooter { margin-left : 0 !important; } .wpep-setup-process-container { max-width : 1240px; margin : 0 auto; }</style>';
  }

  public function get_configuration_page() {
    return wpep_admin_url( $this->_controller->get_plugin_alias() . '-license' );
  }

  public function admin_notices($hook) {
    echo '<div class="error">';
    echo    '<p>';
    echo      sprintf( __( 'Warning! You have not yet activated the license for %s.', "wpep" ), $this->_controller->get_product_name() );
    echo      ' <strong>' . sprintf( __( 'To activate your license %splease click here%s', "wpep" ), '<a href="' . $this->get_configuration_page() . '">', '</a>' ) . '</strong>';
    echo    '</p>';
    echo '</div>';
  }

}