<?php

namespace WPEP\ContentType;

use WPEP\ContentType\Course\Container\PostDependency as Course_PostDependency;
use WPEP\ContentType\Assessment\Container\Data as Assessment_Data;

class Assessment
  extends Skeleton\Standard
  implements Course_PostDependency {

  use Assessment_Data;

  /**
   * @var Assessment\Admin
   */
  public $admin;

  /**
   * @var Assessment\Block
   */
  public $block;

  /**
   * @var Assessment\Delivery
   */
  public $delivery;

  /**
   * @var Assessment\Privacy
   */
  public $privacy;

  public function init() {
    $this->admin    = new Assessment\Admin();
    $this->block    = new Assessment\Block();
    $this->delivery = new Assessment\Delivery();
    $this->privacy  = new Assessment\Privacy();

    add_action( 'delete_post', [ $this, "_on_delete_post" ] );
    add_action( 'delete_user', [ $this, "_on_delete_user" ] );
  }

  public function has_duplicate() {
    return false;
  }

  public function _on_delete_post( $post_id ) {
    if( get_post_type( $post_id ) != WPEP_POST_TYPE_ASSESSMENT )
      return;

    global $wpdb;

    $wpdb->delete( $wpdb->prefix . 'wpep_assessment_question', [ 'post_id' => $post_id ] );
    $wpdb->delete( $wpdb->prefix . 'wpep_assessment_question_answer', [ 'post_id' => $post_id ] );
    $wpdb->delete( $wpdb->prefix . 'wpep_assessment_user_data', [ 'post_id' => $post_id ] );
  }

  public function _on_delete_user( $user_id ) {
    global $wpdb;

    $wpdb->delete( $wpdb->prefix . 'wpep_assessment_user_data', [ 'user_id' => $user_id ] );
  }

  /**
   * @return Assessment\Integration\Statistics
   */
  public function get_statistics_instance() {
    return new Assessment\Integration\Statistics();
  }

  /**
   * @return Assessment\Integration\LearningJourney
   */
  public function get_learning_journey_instance() {
    return new Assessment\Integration\LearningJourney( $this );
  }

  public function get_post_supports() {
    return [ 'title' ];
  }

  public function get_post_type() {
    return WPEP_POST_TYPE_ASSESSMENT;
  }

  public function get_cap_name() {
    return WPEP_POST_CAP_ASSESSMENT;
  }

  public function get_singular_name() {
    return 'Assessment';
  }

  public function get_plural_name() {
    return 'Assessments';
  }

  public function register_post_type() {
    $post_labels = [
      'name'               => _x( $this->get_singular_name(), 'Post Type General Name', "wpep"),
      'singular_name'      => _x( $this->get_singular_name(), 'Post Type Singular Name', "wpep"),
      'menu_name'          => $this->get_label_all_items(),
      'name_admin_bar'     => __( $this->get_singular_name(), "wpep"),
      'parent_item_colon'  => __('Parent ' . $this->get_singular_name() . ':', "wpep"),
      'add_new'            => __('New ' . $this->get_singular_name(), "wpep"),
      'add_new_item'       => __('Add New ' . $this->get_singular_name(), "wpep"),
      'new_item'           => __('New ' . $this->get_singular_name(), "wpep"),
      'edit_item'          => __('Edit ' . $this->get_singular_name(), "wpep"),
      'update_item'        => __('Update ' . $this->get_singular_name(), "wpep"),
      'view_item'          => __('View ' . $this->get_singular_name(), "wpep"),
      'all_items'          => $this->get_label_all_items(),
      'search_items'       => __('Search ' . $this->get_singular_name(), "wpep"),
      'not_found'          => __('No ' . $this->get_singular_name() . ' found', "wpep"),
      'not_found_in_trash' => __('No ' . $this->get_singular_name() . ' found in Trash', "wpep")
    ];

    $post_params = [
      'supports'            => $this->get_post_supports(),
      'taxonomies'          => [],
      'hierarchical'        => false,
      'public'              => true,
      'show_ui'             => true,
      'show_in_menu'        => WPEP_ADMIN_BASE_MENU_SLUG,
      'show_in_nav_menus'   => true,
      'show_in_admin_bar'   => true,
      'menu_position'       => 3,
      'menu_icon'           => '',
      'can_export'          => true,
      'has_archive'         => false,
      'exclude_from_search' => false,
      'publicly_queryable'  => true,
      'capabilities' => [
        'create_posts'          => 'do_not_allow'
      ],
      'capability_type'     => WPEP_POST_CAP_ASSESSMENT,
      'map_meta_cap'        => true,
      'rewrite' => [
        'slug'  => wpep_get_setting( 'content-type-assessment-rewrite-slug', WPEP_POST_TYPE_ASSESSMENT )
      ],
      'label'               => __( strtolower( $this->get_singular_name() ), "wpep" ),
      'labels'              => $post_labels
    ];

    $post_params = apply_filters( 'wpep_content_type_' . $this->get_post_type() . '_register_args', $post_params );

    register_post_type( $this->get_post_type(), $post_params);
  }

  public function is_enabled() {
    if( !wpep_get_setting( 'content-type-' . WPEP_POST_TYPE_COURSE . '-enabled', false ) )
      return false;

    return wpep_get_setting( 'enable-course-assessments', 0 );
  }

  public function data_export_array() {
    return false;
  }

  public function data_asset_map() {
    return [
      'meta'              => [
        'wpep_success_message',
        'wpep_failed_message'
      ],
      'question_list'     => [
        'content',
        'answers' => [
          'content'
        ]
      ],
      'student_activity'  => [
        'value'
      ]
    ];
  }



}