<?php

namespace WPEP\ContentType\Assessment;

use WPEP\Controller as WPEP_Controller;

class Block {

  public function __construct() {
    add_shortcode( 'wpep_assessment', [ $this, 'single' ] );

    add_action( 'wpep_user_assessment_box', [ $this, 'display_assessment_box' ], 10, 2 );
    add_action( 'wpep_current_user_assessment_information_box', [ $this, 'display_information_box' ] );
  }

  public function single( $attributes ) {
    $post_id     = intval( isset($attributes['id']) ? $attributes['id'] : get_the_ID() );

    if( $post_id == false )
      return __( "You need to offer the id param for this shortcode. ", "wpep" );

    $post = get_post( $post_id );

    if( $post == null || get_post_type( $post ) != WPEP_POST_TYPE_ASSESSMENT ) {
      return sprintf( __( "This Assessment has gone missing, please verify the following assessment id %s ", "wpep" ), $post_id );
    }

    if( !is_user_logged_in() && in_array( 'course-shortcode', wpep_get_auth_status_page_restrictions() ) ) {
      return WPEP_Controller::instance()->authentication->get_form( $attributes );
    }

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    $request = new Delivery\Request( $post_id, get_current_user_id() );

    $request->serve( 'view-assessment' );

    add_action( 'wpep_single_assessment_content', [ $request, 'display_content' ], 10 );

    $content = WPEP_Controller::instance()->template->get_template( 'content-assessment-single.php', [
      'assessment_id'      => $post_id,
      'include_header'     => false,
      'include_footer'     => false,
      'is_ajax_experience' => true
    ] );

    remove_action( 'wpep_single_assessment_content', [ $request, 'display_content' ], 10 );

    $content = wpep_format_content_autoembed( $content );

    return $content;
  }

  public function display_assessment_box( $assessment_id, $is_visible = true ) {
    if( $assessment_id == 0 )
      return;

    if( wpep_get_setting_flag( 'assessment-course-page-embed' ) ) {
      wpep_template_load( 'assessment/ajax-embed.php', [
        'is_visible'     => $is_visible,
        'assessment_id'  => $assessment_id
      ] );
      return;
    }

    echo $this->get_information_box(
      $assessment_id,
      get_current_user_id(),
      ( $is_visible ? 'wpep-visible' : 'wpep-hidden' )
    );
  }

  public function display_information_box( $information ) {
    echo $this->get_information_box(
      $information[0],
      get_current_user_id(),
      ( isset( $information[1] ) ? $information[1] : false )
    );
  }

  public function get_information_box( $assessment_id, $user_id = false, $container_class = false ) {
    $user_id         = ( $user_id == false ? get_current_user_id() : $user_id );
    $container_class = 'wpep-assessment-information-box ' . ( $container_class == false ? '' : $container_class );

    if( $user_id == 0 || $assessment_id == 0 )
      return '';

    $user_status = wpep_user_assessment_status( $assessment_id, $user_id );

    $container_class .= ' wpep-assessment-status-' . $user_status;

    return WPEP_Controller::instance()->template->get_template( 'assessment/user-status-box.php', [
      'container_class'   => $container_class,
      'assessment_id'     => $assessment_id,
      'assessment_title'  => get_the_title( $assessment_id ),
      'assessment_url'    => get_the_permalink( $assessment_id ),
      'user_status'       => $user_status
    ] );
  }

}
