<?php

namespace WPEP\ContentType\Assessment\Container\Admin;

trait General {

  public function setup_general_functionality() {
    add_filter('manage_posts_columns', [ $this, '_table_set_columns' ] );
    add_filter('manage_posts_custom_column', [ $this, '_table_custom_columns' ] );

    add_action( 'save_post', [ $this, '_save_post_action' ] );
  }

  public function _save_post_action( $assessment_post_id ) {
    global $post_type;

    if ( !(is_admin() && ($post_type == WPEP_POST_TYPE_ASSESSMENT) ) )
      return;

    $this->_save_post( $assessment_post_id );
  }

  private function _save_post( $post_id, $entity_id_map = [] ) {
    if( get_post_type( $post_id ) != WPEP_POST_TYPE_ASSESSMENT )
      return $entity_id_map;

    $this->handle_assessment_delete_functionality();

    $entity_id_map = $this->save_assessment_questions( $_POST['wpep_assessment']['question'], $post_id, $entity_id_map );

    foreach( $_POST['wpep_assessment']['meta_information'] as $assessment_meta_key => $assessment_meta_value )
      update_post_meta( $post_id, $assessment_meta_key, $assessment_meta_value );

    return $entity_id_map;
  }

  public function _table_set_columns($columns) {
    global $post_type;

    if ( $post_type != WPEP_POST_TYPE_ASSESSMENT)
      return $columns;

    unset(
      $columns['date']
    );

    $new_columns = [
      'course'    => __('Course', "wpep"),
      'shortcode' => __('Shortcode', "wpep"),
    ];

    if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) )
      $new_columns['statistics'] = __('Statistics', "wpep");

    $new_columns['date'] = __('Date', "wpep");

    return array_merge($columns, $new_columns);
  }

  public function _table_custom_columns($column) {
    global $post;

    if( $post->post_type != WPEP_POST_TYPE_ASSESSMENT )
      return;

    if( $column == 'course' ) {
      $course_id = wpep_get_assessment_course_id( $post->ID );

      echo '<a href="' . admin_url( 'post.php?post=' . $course_id . '&action=edit' ) . '">' . get_the_title( $course_id ) . '</a>';
    }

    if ( $column == 'shortcode' ) {
      echo wpep_admin_shortcode_builder(
        'wpep_assessment',
        [
          'id'  => $post->ID,
          'wpep_admin_shortcode_builder_wpep_assessment_options'
        ],
        __( "ShortCode", "wpep" ),
        true
      );

      return;
    }

    if( $column == 'statistics' )
      echo '<a class="button button-primary" href="' . wpep_admin_url_statistics_content_type( WPEP_POST_TYPE_ASSESSMENT, [ 'assessment_id' => $post->ID ] ) . '">' . __( "View Statistics", "wpep" )    . '</a>';
  }

}
