<?php

namespace WPEP\ContentType\Assessment\Container;

use WPEP\Entity\Assessment as Entity_Assessment;

trait Data {

  /**
   * @param $args
   * @return array|null
   */
  public function data_user_activity_report( $args ) {
    $response = parent::data_user_activity_report( $args );

    global $wpdb;

    $sql = 'SELECT aud.value as status, COUNT( aud.value ) as entries
              FROM ' . $wpdb->prefix . 'wpep_assessment_user_data aud
             WHERE aud.key = "' . WPEP_USER_ASSESSMENT_STATUS . '"';

    if( isset( $args['time_from'] ) && isset( $args['time_to'] ) )
      $sql .= ' AND aud.created_at >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '" AND aud.created_at <= "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';

    $sql .= ' GROUP BY aud.value';

    $response = $response + [
      $this->get_plural_name() . ' ' . wpep_get_setting_translated( 'assessment-status-label-' . WPEP_USER_ASSESSMENT_STATUS_COMPLETED )      => 0,
      $this->get_plural_name() . ' ' . wpep_get_setting_translated( 'assessment-status-label-' . WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW ) => 0,
      $this->get_plural_name() . ' ' . wpep_get_setting_translated( 'assessment-status-label-' . WPEP_USER_ASSESSMENT_STATUS_FAILED )         => 0,
      $this->get_plural_name() . ' ' . wpep_get_setting_translated( 'assessment-status-label-' . WPEP_USER_ASSESSMENT_STATUS_PENDING )        => 0
    ];

    foreach( $wpdb->get_results( $sql ) as $data ) {
      $response[ $this->get_plural_name() . ' ' . wpep_get_setting_translated( 'assessment-status-label-' . $data->status ) ] = absint( $data->entries );
    }

    return $response;
  }

  public function attach_data_asset_map_for_course( $response ) {
    $response['assessment'] = $this->data_asset_map();
    $response['section_list']['assessment'] = $this->data_asset_map();

    return $response;
  }

  public function attach_data_export_array_for_course( $response, $include_user_activity = true ) {
    $response = $this->attach_data_export_array_for_course_section( $response, $response['post_id'], 0, $include_user_activity );

    return $response;
  }

  /**
   * @param $response
   * @param $course_id
   * @param int $section_id
   * @param bool $include_user_activity
   * @return mixed
   */
  public function attach_data_export_array_for_course_section( $response, $course_id, $section_id = 0, $include_user_activity = true ) {
    $assessment_id = wpep_get_course_assessment_id( $course_id, $section_id );

    if( $assessment_id == 0 )
      return $response;

    $response['assessment'] = $this->data_export_single_post_array( get_post( $assessment_id ) );

    $response['assessment']['question_list'] = Entity_Assessment::instance()->get_questions_by_assessment_id( $assessment_id, true, false );

    foreach( $response['assessment']['question_list'] as $question_index => $question ) {
      $question = ( is_object( $question ) ? get_object_vars( $question ) : $question );

      unset( $question['post_id'] );

      if( isset( $question['answers'] ) && is_array( $question['answers'] ) ) {
        foreach( $question['answers'] as $answer_index => $answer ) {
          $question['answers'][$answer_index] = ( is_object( $answer ) ? get_object_vars( $answer ) : $answer );

          unset( $question['answers'][$answer_index]['post_id'] );
          unset( $question['answers'][$answer_index]['question_id'] );
        }
      }

      $response['assessment']['question_list'][$question_index] = $question;
    }

    if( !$include_user_activity )
      return $response;

    $assessment_user_data = wpep_controller()->userActivity->assessment->get_user_data(
      false, $assessment_id, false, false, false, false, false
    );

    if( empty( $assessment_user_data ) )
      return $response;

    $response['assessment']['student_activity'] = [];

    $user_list = wpep_controller()->dataSystem->get_users();

    foreach( $assessment_user_data as $assessment_user_data_row ) {
      if( !isset( $user_list[ $assessment_user_data_row->user_id ] ) )
        continue;

      $response['assessment']['student_activity'][] = [
        'user_id'             => $assessment_user_data_row->user_id,
        'user_email_address'  => $user_list[ $assessment_user_data_row->user_id ]['email_address'],
        'question_id'         => $assessment_user_data_row->question_id,
        'question_answer_id'  => $assessment_user_data_row->question_answer_id,
        'key'                 => $assessment_user_data_row->key,
        'value'               => $assessment_user_data_row->value,
        'created_at'          => $assessment_user_data_row->created_at
      ];
    }

    return $response;
  }

  public function data_import_single_post_array( $post, $id_mapping = [] ) {
    $new_post_id = parent::data_import_single_post_array( $post, $id_mapping = []  );

    if( isset( $post['question_list'] ) && !empty( $post['question_list'] ) ) {
      $assessment_map_data = $this->_data_import_single_post_question_list( $post['question_list'], $new_post_id );

      if( isset( $post['student_activity'] ) && is_array( $post['student_activity'] ) && !empty( $post['student_activity'] ) ) {
        $student_list = wpep_controller()->dataSystem->get_user_ids_mapped_by_email();

        foreach( $post['student_activity'] as $student_activity ) {
          if( !isset( $student_list[ $student_activity['user_email_address'] ] ) )
            continue;

          wpep_controller()->userActivity->assessment->add_user_data(
            $student_activity['key'],
            $student_activity['value'],
            $new_post_id,
            ( isset( $assessment_map_data['assessment_question'][ $student_activity['question_id'] ] )      ? $assessment_map_data['assessment_question'][ $student_activity['question_id'] ]      : 0 ),
            ( isset( $assessment_map_data['assessment_answer'][ $student_activity['question_answer_id'] ] ) ? $assessment_map_data['assessment_answer'][ $student_activity['question_answer_id'] ] : 0 ),
            $student_list[ $student_activity['user_email_address'] ]
          );
        }
      }
    }
  }

  private function _data_import_single_post_question_list( $question_list, $post_id ) {
    global $wpdb;

    $response = [];

    foreach( $question_list as $question_information ) {
      $current_information = [
        'post_id'                   => $post_id,
        'title'                     => $question_information['title'],
        'content'                   => str_replace('\"', '"', ( isset( $question_information['content'] ) ? $question_information['content'] : '' ) ),
        'type'                      => ( isset( $question_information['type'] )   ? $question_information['type']   : 0 ),
        'weight'                    => ( isset( $question_information['weight'] ) ? $question_information['weight'] : 0 ),
        'order'                     => ( isset( $question_information['order'] )  ? $question_information['order']  : 0 ),
      ];

      $wpdb->insert( $wpdb->prefix . 'wpep_assessment_question', $current_information );
      $question_id = $wpdb->insert_id;

      if( isset( $question_information['id'] ) )
        $response['assessment_question'][ $question_information['id'] ] = $question_id;

      if( !isset( $question_information['answers'] ) )
        continue;

      if( !is_array( $question_information['answers'] ) )
        continue;

      foreach( $question_information['answers'] as $answer_information ) {
        $current_information = [
          'post_id'          => $post_id,
          'question_id'      => $question_id,
          'title'            => $answer_information['title'],
          'content'          => str_replace('\"', '"', ( isset( $answer_information['content'] ) ? $answer_information['content'] : '' ) ),
          'is_correct'       => ( isset( $answer_information['is_correct'] ) ? $answer_information['is_correct']   : 0 ),
          'order'            => ( isset( $answer_information['order'] )      ? $answer_information['order'] : 0 ),
        ];

        $wpdb->insert( $wpdb->prefix . 'wpep_assessment_question_answer', $current_information );

        $answer_id = $wpdb->insert_id;

        if( isset( $answer_information['id'] ) )
          $response['assessment_answer'][ $answer_information['id'] ] = $answer_id;
      }
    }

    return $response;
  }

}
