<?php

namespace WPEP\ContentType\Assessment;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Assessment as EntityAssessment;

class Delivery {

  /**
   * @var Delivery\Request;
   */
  public $request;

  public function __construct() {
    add_action( 'wp_ajax_wpep_assessment_content', [ $this, 'request_wpep_assessment_form' ] );
    add_action( 'init', [ $this, '_wp_init' ] );
  }

  public function _wp_init() {
    if( wpep_get_current_post_type() != WPEP_POST_TYPE_ASSESSMENT )
      return;

    $assessment_id = wpep_get_current_post_id();

    if( $assessment_id === 0 )
      return;

    $course_id = wpep_get_assessment_course_id( $assessment_id );

    if( wpep_get_setting_flag( 'assessment-course-protection-rules' ) ) {
      if( !WPEP_Controller::instance()->contentLibrary->has_access_to_post( $course_id ) ) {
        $course_permalink = apply_filters( 'wpep_primary_content_item_link', wpep_course_permalink( $course_id ), $course_id );

        if( $course_permalink == ''
            || $course_permalink == '#' )
          $course_permalink = wpep_course_permalink( $course_id );

        wp_redirect( $course_permalink );
        exit;
      }
    }
    $this->request = new Delivery\Request( $assessment_id, get_current_user_id() );

    $response = $this->request->serve( $this->_get_request_type(), [
      'answers'           => isset( $_POST['wpep_assessment_answer'] ) ? $_POST['wpep_assessment_answer'] : [],
      'current_step'      => isset( $_GET['step'] ) ? $_GET['step'] : 0,
      'display_questions' => ( isset( $_GET['display-questions'] ) && $_GET['display-questions'] == 1 )
    ] );

    if( $response['type'] == 'switch_assessment' ) {
      wpep_redirect( get_permalink( $response[ 'assessment_id' ] ) );
      exit;
    } elseif( $response['type'] == 'refresh_assessment' ) {
      $permalink = get_permalink( $response[ 'assessment_id' ] );

      if( isset( $response['step'] ) && $response['step'] != 0 )
        $permalink = add_query_arg( 'step', $response['step'], $permalink );

      wpep_redirect( $permalink );
      exit;
    }

    add_action( 'wpep_single_assessment_content', [ $this->request, 'display_content' ], 10 );
  }

  public function request_wpep_assessment_form() {
    if( !isset( $_POST[ 'assessment_id' ] ) )
      exit;

    $assessment = get_post( intval( $_POST[ 'assessment_id' ] ) );

    if( $assessment === null )
      exit;

    $request = new Delivery\Request( $assessment->ID, get_current_user_id() );

    $response = $request->serve( $this->_get_request_type(), [
      'answers'           => isset( $_POST['wpep_assessment_answer'] ) ? $_POST['wpep_assessment_answer'] : [],
      'current_step'      => isset( $_POST['current_step'] ) ? $_POST['current_step'] : 0,
      'display_questions' => ( isset( $_POST['display-questions'] ) && $_POST['display-questions'] == 1 )
    ] );

    while( $response['type'] != 'view_assessment' ) {
      if( $response['type'] == 'switch_assessment' )
        $request = new Delivery\Request( $response[ 'assessment_id' ], get_current_user_id() );

      if( in_array( $response[ 'type' ], [ 'switch_assessment', 'refresh_assessment' ] ) ) {
        $response = $request->serve( 'view-assessment', [
          'current_step'  =>  ( isset( $response['step'] ) ? $response['step'] : 0 )
        ] );
      }
    }

    remove_all_actions( 'wpep_before_main_content' );
    remove_all_actions( 'wpep_after_main_content' );
    add_action( 'wpep_single_assessment_content', [ $request, 'display_content' ], 10 );

    $response = [];

    $response['assessment_id'] = $assessment->ID;
    $response['content']       = WPEP_Controller::instance()->template->get_template( 'content-assessment-single.php', [
      'assessment_id'      => $assessment->ID,
      'include_header'     => false,
      'include_footer'     => false,
      'is_ajax_experience' => true
    ] );
    $response['user_status'] = wpep_user_assessment_status( $assessment->ID, get_current_user_id() );

    echo json_encode( $response );
    exit;
  }

  private function _get_request_type() {
    if( isset( $_GET['retake_assessment'] ) && wp_verify_nonce( $_GET['retake_assessment'], 'retake_assessment' ) != 0 )
      return 'retake-assessment';

    if( isset( $_POST['retake_assessment'] ) && wp_verify_nonce( $_POST['retake_assessment'], 'retake_assessment' ) != 0 )
      return 'retake-assessment';

    if( isset( $_POST['wpep_assessment_submit'] ) )
      return 'verify-assessment';

    if( isset( $_POST['wpep_start_assessment'] ) )
      return 'start-assessment';

    if( isset( $_POST['wpep_prev_part'] ) )
      return 'navigation-previous';

    if( isset( $_POST['wpep_next_part'] ) )
      return 'navigation-next';

    if( isset( $_POST['wpep_assessment_answer'] ) )
      return 'save-assessment';

    return 'view-assessment';
  }

}
