<?php

namespace WPEP\ContentType\Assessment\Integration\Statistics;

class Single {

  protected $table_tabs = [];
  private   $_current_tab = false;

  public function __construct( $assessment_id ) {
    foreach( ( defined( "WPEP_BETA_TESTING" ) ? [
               new Single\Dashboard(),
               new Single\AnswerOverview()
             ] : [ new Single\Dashboard() ] ) as $tab )
      $this->table_tabs[ $tab->get_alias() ] = $tab;

    $this->table_tabs = apply_filters( "wpep_administration_statistics_assessments_single_tabs", $this->table_tabs, $assessment_id );
  }

  private function _get_current_tab() {
    if( $this->_current_tab !== false )
      return $this->table_tabs[ $this->_current_tab ];

    $this->_current_tab = ( isset( $_GET['view'] ) ? $_GET['view'] : false );
    $this->_current_tab = ( isset( $this->table_tabs[ $this->_current_tab ] ) ? $this->_current_tab : key( $this->table_tabs ) );

    return $this->table_tabs[ $this->_current_tab ];
  }

  public function display( $assessment_id = false, $filter_by = [] ) {
    $bread_crumbs = [
      wpep_admin_url_statistics_content_type( WPEP_POST_TYPE_ASSESSMENT ) => __( "Course List", "wpep" ),
      wpep_admin_url_statistics_content_type( WPEP_POST_TYPE_ASSESSMENT, [ 'assessment_id' => $assessment_id ] ) => get_the_title( $assessment_id ),
      false => $this->_get_current_tab()->get_name( $assessment_id )
    ];

    echo wpep_breadcrumb( $bread_crumbs );

    $table_information = $this->_get_current_tab()->get_table_data(
      $assessment_id,
      $filter_by
    );

    $table_navigation = [];

    foreach( $this->table_tabs as $table_tab ) {
      if( method_exists( $table_tab, 'is_public' ) && $table_tab->is_public() === false ) {
        if( $this->_current_tab != $table_tab->get_alias() )
          continue;
      }

      $key = ( $this->_current_tab != $table_tab->get_alias()
        ? wpep_admin_url_statistics_content_type( WPEP_POST_TYPE_ASSESSMENT, [ 'assessment_id' => $assessment_id, 'view' => $table_tab->get_alias() ] )
        : false
      );

      $table_navigation[ $key ] = $table_tab->get_name( $assessment_id );
    }

    wpep_admin_display_table(
      $table_navigation,
      $table_information['head'],
      $table_information['body'],
      true
    );
  }

  public function download( $assessment_id = false, $filter_by = [] ) {
    $table_information = $this->_get_current_tab()->get_table_data(
      $assessment_id,
      $filter_by
    );

    do_action(
      'wpep_deliver_csv_export_file',
      $this->_get_current_tab()->get_file_alias(
        $assessment_id,
        $filter_by
      ),
      $table_information['head'],
      $table_information['body']
    );
  }

}