<?php

namespace WPEP\ContentType\Assessment\Integration\Statistics\Single;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Assessment as WPEP_Entity_Assessment;

class AnswerOverview {

  public function get_alias() {
    return 'answer_overview';
  }

  public function get_name() {
    return __( "Answer Overview", "wpep" );
  }

  public function get_file_alias( $assessment_id, $filter_by = [] ) {
    return 'assessment-' . get_the_title( $assessment_id ) . '-' .
      ( isset( $filter_by['user_role'] ) && $filter_by['user_role'] !== false ? $filter_by['user_role'] . '-' : '' ) .
      'answer-overview-' . wpep_format_date();
  }

  public function get_table_data( $assessment_id, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => []
    ];

    $question_list = WPEP_Entity_Assessment::instance()->get_questions_by_assessment_id( $assessment_id, false, false );

    $user_answers_map = WPEP_Controller::instance()->userActivity->assessment->get_user_answers_map_by_assessment_id( $assessment_id, $filter_by );

    if( empty( $user_answers_map ) )
      return $table_information;

    $table_information['head'] = [
      'name'                    => __( "Name", 'wpep' ),
      'student_identification'  => __( "SID", 'wpep' )
    ];

    foreach( $question_list as $question )
      $table_information['head']['question_' . $question->id ] = $question->title;

    $table_information['head']['view_submissions'] = [
      'type'  => 'link-item',
      'value' => ''
    ];

    if( !wpep_has_sid_support() )
      unset( $table_information['head']['student_identification'] );

    foreach( $user_answers_map as $user_id => $entry ) {

      $user_information = get_user_by( 'ID', $user_id );

      if( $user_information == null )
        continue;

      $body = [
        'name'                    => [
          'type'      => 'link',
          'value'     => '<a href="' . wpep_admin_url_statistics_content_type( WPEP_POST_TYPE_ASSESSMENT, [ 'user_id' => $user_id ] ) . '">' . wpep_content_user_profile_format_name( $user_information ) . '</a>',
          'value_raw' => wpep_content_user_profile_format_name( $user_information )
        ],
        'view_submissions'        => [
          'type'  => 'link-item',
          'value' => '<a class="button button-primary" style="width: 100px;" href="' . wpep_admin_url_statistics_content_type( WPEP_POST_TYPE_ASSESSMENT, [ 'assessment_id' => $assessment_id, 'user_id' => $user_id ] ) . '">' . __( "View Report", "wpep" )    . '</a>'
        ],
      ];

      $body += $entry;

      if( wpep_has_sid_support() )
        $body['student_identification'] = get_user_meta( $user_information->ID, WPEP_USER_META_STUDENT_IDENTIFICATION, true );

      $table_information['body'][] = $body;
    }

    return $table_information;
  }

}