<?php

namespace WPEP\ContentType\Assessment;

use WPEP\Entity\Assessment as EntityAssessment;

class Privacy {

  public function __construct() {
    add_filter( 'wp_privacy_personal_data_exporters', [ $this, 'register_exporters' ], 7 );
  }

  /**
   * Integrate this exporter implementation within the WordPress core exporters.
   *
   * @param array $exporters List of exporter callbacks.
   * @return array
   */
  public function register_exporters( $exporters = [] ) {
    $exporters[ 'wpep-assessment' ] = [
      'exporter_friendly_name' => wpep_get_setting_translated( 'assessment' ),
      'callback'               => [ $this, '_export_information' ],
    ];

    return $exporters;
  }

  public function _export_information( $email_address, $page ) {
    $number = 2;
    $page   = (int) $page;

    $data_to_export = [];

    $user = get_user_by( 'email' , $email_address );

    if ( false === $user ) {
      return array(
        'data' => $data_to_export,
        'done' => true,
      );
    }

    $started_assessment_list = wpep_utility_map_to_array_assoc( $this->_assessment_data()->get_user_data( 'start_time', false, false, false, $user->ID, false ), 'post_id', 'value' );

    $post_query = new \WP_Query( [
      'posts_per_page' => $number,
      'paged'          => $page,
      'post_type'      => WPEP_POST_TYPE_ASSESSMENT,
      'numberposts'    => -1,
      'orderby'        => 'post__in',
      'order'          => 'ASC',
      'post__in'       => array_keys( $started_assessment_list ),
      'post_status'    => 'any',
    ] );

    foreach ( (array) $post_query->posts as $post ) {
      $submission_time = $this->_assessment_data()->get_user_data_value( 'submission_time', $post->ID, false, false, $user->ID );
      $score           = $this->_assessment_data()->get_user_data_value( 'score', $post->ID, false, false, $user->ID );

      $post_data_to_export = [
        [
          'name'  => __( "Assessment" ),
          "value" => get_the_title( $post )
        ],
        [
          'name'  => wpep_get_setting_translated( 'assessment-status' ),
          'value' => wpep_get_setting_translated( 'assessment-status-label-' . wpep_user_assessment_status( $post->ID, $user->ID ) )
        ],
        [
          'name'  => __( 'Started At' ),
          'value' => wpep_format_date_with_time( $started_assessment_list[ $post->ID ] )
        ],
      ];

      if( $submission_time !== false )
        $post_data_to_export[] = [
          'name'  => __( 'Submission Time' ),
          'value' => wpep_format_date_with_time( $submission_time )
        ];

      if( $score !== false )
        $post_data_to_export[] = [
          'name'  => wpep_get_setting_translated( 'assessment-score' ),
          'value' => wpep_get_percentage( $score )
        ];

      $questions = EntityAssessment::instance()->get_questions_by_assessment_id( $post->ID, true, false );

      $user_answers = wpep_utility_map_to_array_assoc( $this->_assessment_data()->get_user_data(
        'answer', $post->ID, false, false, $user->ID, false
      ), 'question_id', 'value', true );

      $question_answer_status_list = wpep_utility_map_to_array_assoc( $this->_assessment_data()->get_user_data(
        'is_correct', $post->ID, false, false, $user->ID, false
      ), 'question_id', 'value', true );

      foreach( $questions as $question ) {
        $post_data_to_export[] = [
          'name'  => $question->title,
          'value' => ( is_array( $user_answers[$question->id] ) ? implode( ", ", $user_answers[$question->id] ) : $user_answers[$question->id] )
        ];
        $post_data_to_export[] = [
          'name'  => '',
          'value' => ( isset( $question_answer_status_list[$question->id] ) ? $question_answer_status_list[$question->id] : 0 )
            ? wpep_get_setting_translated( 'assessment-question-status-correct' )
            : wpep_get_setting_translated( 'assessment-question-status-incorrect' )
        ];
      }


      $data_to_export[] = [
        'group_id'    => 'wpep-assessment',
        'group_label' => wpep_get_setting_translated( 'assessment' ),
        'item_id'     => "post-{$post->ID}",
        'data'        => $post_data_to_export,
      ];
    }

    $done = $post_query->max_num_pages <= $page;

    return [
      'data' => $data_to_export,
      'done' => $done,
    ];
  }

  private function _assessment_data() {
    return wpep_controller()->userActivity->assessment;
  }

}