<?php

namespace WPEP\ContentType;

use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

class Certification extends Skeleton\Standard {

  /**
   * @var Certification\Admin
   */
  public $admin;

  /**
   * @var Certification\Delivery
   */
  public $delivery;

  public function init() {
    $this->admin    = new Certification\Admin();
    $this->delivery = new Certification\Delivery();

    $this->admin->init();
    $this->delivery->init();
  }

  public function get_statistics_instance() {
    return new Certification\Integration\Statistics();
  }

  public function get_learning_journey_instance() {
    return false;

    return new Certification\Integration\LearningJourney( $this );
  }

  public function has_duplicate() {
    return false;
  }

  public function get_post_supports() {
    return [ 'title' ];
  }

  public function get_post_type() {
    return WPEP_POST_TYPE_CERTIFICATE;
  }

  public function get_cap_name() {
    return WPEP_POST_CAP_CERTIFICATE;
  }

  public function get_singular_name() {
    return 'Certificate';
  }

  public function get_plural_name() {
    return 'Certifications';
  }

  public function register_post_type() {
    $post_labels = [
      'name'               => _x( $this->get_singular_name(), 'Post Type General Name', "wpep"),
      'singular_name'      => _x( $this->get_singular_name(), 'Post Type Singular Name', "wpep"),
      'menu_name'          => $this->get_label_all_items(),
      'name_admin_bar'     => __( $this->get_singular_name(), "wpep"),
      'parent_item_colon'  => __('Parent ' . $this->get_singular_name() . ':', "wpep"),
      'add_new'            => __('New ' . $this->get_singular_name(), "wpep"),
      'add_new_item'       => __('Add New ' . $this->get_singular_name(), "wpep"),
      'new_item'           => __('New ' . $this->get_singular_name(), "wpep"),
      'edit_item'          => __('Edit ' . $this->get_singular_name(), "wpep"),
      'update_item'        => __('Update ' . $this->get_singular_name(), "wpep"),
      'view_item'          => __('View ' . $this->get_singular_name(), "wpep"),
      'all_items'          => $this->get_label_all_items(),
      'search_items'       => __('Search ' . $this->get_singular_name(), "wpep"),
      'not_found'          => __('No ' . $this->get_singular_name() . ' found', "wpep"),
      'not_found_in_trash' => __('No ' . $this->get_singular_name() . ' found in Trash', "wpep")
    ];

    $post_params = [
      'supports'            => $this->get_post_supports(),
      'taxonomies'          => [],
      'hierarchical'        => false,
      'public'              => true,
      'show_ui'             => true,
      'show_in_menu'        => WPEP_ADMIN_BASE_MENU_SLUG,
      'show_in_nav_menus'   => true,
      'show_in_admin_bar'   => true,
      'menu_position'       => 3,
      'menu_icon'           => '',
      'can_export'          => true,
      'has_archive'         => false,
      'exclude_from_search' => false,
      'publicly_queryable'  => true,
      'capabilities' => [
        'create_posts'          => 'do_not_allow'
      ],
      'capability_type'     => WPEP_POST_CAP_CERTIFICATE,
      'map_meta_cap'        => true,
      'rewrite' => [
        'slug'  => wpep_get_setting( 'content-type-' . WPEP_POST_TYPE_CERTIFICATE . '-rewrite-slug', WPEP_POST_TYPE_CERTIFICATE )
      ],
      'label'               => __( strtolower( $this->get_singular_name() ), "wpep" ),
      'labels'              => $post_labels
    ];

    $post_params = apply_filters( 'wpep_content_type_' . $this->get_post_type() . '_register_args', $post_params );

    register_post_type( $this->get_post_type(), $post_params);
  }

  public function is_enabled() {
    if( !wpep_get_setting( 'content-type-' . WPEP_POST_TYPE_COURSE . '-enabled', false ) )
      return false;

    return wpep_get_setting( 'enable-certifications-support', 0 );
  }

  public function data_export_array() {
    return false;
  }

  public function data_asset_map() {
    return [

    ];
  }

  public function data_user_activity_report( $args ) {
    $response = parent::data_user_activity_report( $args );

    $args['type'] = WPEP_ACTION_TRACKING_TYPE_DOWNLOAD;
    $args['post_type'] = $this->get_post_type();

    return $response + [
        __( "Certificate Download", "wpep" ) => WPEP_Entity_ActionTracking::instance()->get_event_count_by_args( $args )
      ];
  }



}