<?php

namespace WPEP\ContentType\Certification;

use WPEP\Container\FormFields as WPEP_FormFields;

class Admin {

  use WPEP_FormFields;

  public function __construct() {
    $this->set_field_prefix( 'wpep_meta_value' );
  }

  public function get_value( $field_key, $field_default = '', $single = true ) {
    $field_value = get_post_meta( get_the_ID(), $field_key, $single );

    return empty( $field_value ) ? $field_default : $field_value;
  }

  public function init() {
    add_action( 'wpep_administration_course_builder_tab_items_after', [ $this, 'navigation_action' ] );
    add_action( 'wpep_administration_course_builder_content_after', [ $this, 'content_action' ] );
  }

  public function navigation_action() {
    echo '<li class="wpep-tab-link wpep-tab-course-certifications"><h4>' . __( "Certify", "wpep_certifications" ) . '</h4></li>';
  }

  public function content_action() {
    $certification_status_options = [
      WPEP_COURSE_CERTIFICATE_STATUS_DISABLED => __( "Disabled", "wpep_certifications" ),
      WPEP_COURSE_CERTIFICATE_STATUS_ENABLED  => __( "Awarded On Course Completion", "wpep_certifications" ),
    ];

    if( get_post_type() === WPEP_POST_TYPE_COURSE )
      $certification_status_options[WPEP_COURSE_CERTIFICATE_STATUS_ENABLED_WITH_ASSESSMENTS]
        = __( "Awarded On Course & Assessments Completion", "wpep_certifications" );

    echo '<div class="wpep-tab-item wpep-tab-course-certifications">';

    if( get_post_type() != WPEP_POST_TYPE_COURSE ) {
      echo '<h2>' . __( "Certification Settings", "wpep_certifications" ) . '</h2>';
    }

    if( !function_exists( 'imagettfbbox' ) ) {
      echo '<div class="wpep-notification wpep-notification-danger">' . __( "PHP Function missing : imagettfbbox, please ask your Hosting Company to enable the PHP GD Library, or enable it if you're familiar with the process.", "wpep" ) . '</div>';
    }

    if( wpep_image_builder_get_font_directory_path() === false ) {
      echo '<div class="wpep-notification wpep-notification-danger">' . __( "Missing Font Files, this is caused by a server error, WPEP will attempt to download the font files until they're stored on your server successfully.", "wpep" ) . '</div>';
    }

    echo      '<div class="wpep-form-group">';

    echo        '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Status", "wpep_certifications" ) . '</label>';

    $this->display_select_field(
      'wpep_certificate_status',
      $certification_status_options
    );
    echo        '</div>';

    echo        '<div class="wpep-form-group-row">';
    echo          '<label>' . __( "Lock After", "wpep_certifications" ) . '</label>';

    $this->display_date_field(
      'wpep_certificate_reset_date',
      false,
      null,
      'regular-text wpep-datepicker-field wpep-datepicker-field-default'
    );

    echo           '<div class="description">' . __( "All Certificates that have been created after the file lock day will stay as they are, even if you make template changes.", "wpep_certifications" ) . '</div>';
    echo        '</div>';

    echo      '</div>';

    echo      '<div class="wpep-form-group">';
    echo        '<div class="wpep-form-group-row">';

    $field_options = [
      '{full_name}'         => __( "Full Name", "wpep_certifications" ),
      '{first_name}'        => __( "First Name", "wpep_certifications" ),
      '{last_name}'         => __( "Last Name", "wpep_certifications" ),
      '{issue_date}'        => __( "Issue Date", "wpep_certifications" ),
      '{course_name}'       => __( "Course Name", "wpep_certifications" ),
    ];

    if( get_post_type() === WPEP_POST_TYPE_COURSE ) {
      $field_options += [
        '{course_started}'    => __( "Course Started Date", "wpep_certifications"),
        '{course_completed}'  => __( "Course Completed Date", "wpep_certifications"),
        '{assessment_grading}'=> __( "Assessment Grading", "wpep_certifications" ),
      ];
    }

    if( wpep_has_sid_support() )
      $field_options = [ '{student_identification}' => __( "Student Identification", "wpep_certifications" ) ] + $field_options;

    $field_options['enter text here'] = __( "Custom Text", "wpep_certifications" );

    $this->display_image_builder_field(
      $field_options,
      'wpep_certificate_image',
      'wpep_certificate_content_map',
      ( WPEP_BASE_URL_PATH . 'assets/img/default-certificate.jpg' ),
      [
        [
          'content'     => 'Certificate Example',
          'left'        => '222px',
          'top'         => '150px',
          'width'       => '394px',
          'height'      => '22px',
          'font-size'   => '18px',
          'font-family' => 'Arial',
          'font-weight' => 300,
          'color'       => '#FFFFFF'
        ],
        [
          'content'     => '{full_name} has successfully completed',
          'left'        => '127px',
          'top'         => '216px',
          'width'       => '618px',
          'height'      => '27px',
          'font-size'   => '23px',
          'font-family' => 'Arial',
          'font-weight' => 300,
          'color'       => '#000000'
        ],
        [
          'content'     => '{course_name}',
          'left'        => '215px',
          'top'         => '264px',
          'width'       => '411px',
          'height'      => '31px',
          'font-size'   => '27px',
          'font-family' => 'Arial',
          'font-weight' => 300,
          'color'       => '#000000'
        ],
        [
          'content'     => '...',
          'left'        => '108px',
          'top'         => '456px',
          'width'       => '200px',
          'height'      => '22px',
          'font-size'   => '19px',
          'font-family' => 'Arial',
          'font-weight' => 300,
          'color'       => '#000000'
        ],
        [
          'content'     => '...',
          'left'        => '561px',
          'top'         => '456px',
          'width'       => '200px',
          'height'      => '22px',
          'font-size'   => '19px',
          'font-family' => 'Arial',
          'font-weight' => 300,
          'color'       => '#000000'
        ]
      ],
      wpep_image_builder_get_font_list()
    );

    echo        '</div>';

    echo      '</div>';

    echo '</div>';
  }

}