<?php

namespace WPEP\ContentType\Course;

use WPEP\Container\FormFields as WPEP_FormFields_Configuration;
use WPEP\ContentType\Course\Container\Admin\General as WPEP_Course_General;
use WPEP\ContentType\Course\Container\Admin\Builder as WPEP_Course_Builder;
use WPEP\ContentType\Assessment\Container\Admin\Builder as WPEP_Assessment_Builder;

class Admin {

  use WPEP_FormFields_Configuration,
      WPEP_Course_General,
      WPEP_Course_Builder,
      WPEP_Assessment_Builder;

  public function init() {
    if( !is_admin() )
      return;

    $this->setup_general_functionality();
    $this->setup_course_builder();

    $this->set_field_prefix( 'wpep_meta_value' );

    add_action( 'edit_form_after_title', [ $this, '_display_builder' ] );
  }

  public function _display_builder() {
    global $post_type;

    if( $post_type !== WPEP_POST_TYPE_COURSE )
      return;

    $section_settings = '';

    foreach( $this->section_course_type as $key => $value )
      $section_settings .= ' data-option-' . $key . '="' . $value . '"';

    $current_primary_tab = apply_filters( "wpep_administration_course_builder_current_tab", intval( get_post_meta( get_the_ID(), 'wpep_current_primary_tab', true ) ), get_the_ID() );

    echo '<div id="wpep_builder"' . $section_settings . '>';
    echo    '<input type="hidden" id="wpep_current_primary_tab" name="wpep_current_primary_tab" value="' . $current_primary_tab . '"/>';

    $this->display_course_builder_tabs();
    $this->display_course_builder_content();

    echo '</div>';
  }

  public function get_meta_fields() {
    $wpep_course_options_fields = [];

    if( wpep_has_sid_support() ) {
      $wpep_course_options_fields[ 'wpep_course_identifier' ] = [
        'name'    => __( 'Course Reports Identifier', "wpep" ),
        'id'      => 'wpep_course_identifier',
        'type'    => 'text'
      ];
    }

    $wpep_course_options_fields[ 'wpep_social_share_link' ] = [
      'name'            => __( 'Social Share Link', "wpep" ),
      'id'              => 'wpep_social_share_link',
      'type'            => 'text',
      'description'     => __( "Leave this empty to share the course link instead.", "wppe" ),
      'content_before'  => '<div data-wpep-course-type="' . WPEP_COURSE_TYPE_CHALLENGE . '">',
      'content_after'   => '</div>'
    ];

    $wpep_course_options_fields += [
      'course_type' => [
        'name'    => __('Course Type', "wpep"),
        'desc'    => __('By default a course is an execution plan.', "wpep"),
        'id'      => 'course_type',
        'type'    => 'select',
        'options' => wpep_course_layout_types()
      ],
      'course_theme' => [
        'name'    => __('Course Theme', "wpep"),
        'desc'    => __('This option in not available for shortcode embeds of this course.', "wpep"),
        'id'      => 'course_theme',
        'type'    => 'select',
        'options' => wpep_get_theme_list_as_entity_layout_option()
      ],
      'wpep_course_enable_discussions' => [
        'name'    => __( 'Discussions', "wpep" ),
        'desc'    => __( 'Enable the WPEP Discussions feature within courses.', "wpep" ),
        'id'      => 'wpep_course_enable_discussions',
        'type'    => 'select',
        'options' => wpep_flag_setting_field_options()
      ],
      'wpep_standard_course_overview_page'  => [
        'name'    => __( 'Overview Page', "wpep" ),
        'desc'    => '<p>' . __( "Individually chose if this course should have an Index Page / Overview Page", "wpep" ) . '</p>',
        'id'      => 'wpep_standard_course_overview_page',
        'type'    => 'select',
        'options' => [
            'default' => sprintf( __( "Course Type Default ( %s )" ), ( wpep_get_setting_flag( 'course-standard-allow-index-page' ) ? __( 'Enabled', "wpep" ) : __( "Disabled", "wpep" ) ) ),
          ] + wpep_flag_setting_field_options(),
        'content_before'  => '<div data-wpep-course-type="' . WPEP_COURSE_TYPE_STANDARD . '">',
        'content_after'   => '</div>'
      ],
      'wpep_video_course_overview_page' => [
        'name'    => __( 'Overview Page', "wpep" ),
        'desc'    => '<p>' . __( "Individually chose if this course should have an Index Page / Overview Page", "wpep" ) . '</p>',
        'id'      => 'wpep_video_course_overview_page',
        'type'    => 'select',
        'options' => [
            'default' => sprintf( __( "Course Type Default ( %s )" ), ( wpep_get_setting_flag( 'course-video-allow-index-page' ) ? __( 'Enabled', "wpep" ) : __( "Disabled", "wpep" ) ) ),
          ] + wpep_flag_setting_field_options() ,
        'content_before'  => '<div data-wpep-course-type="' . WPEP_COURSE_TYPE_VIDEO . '">',
        'content_after'   => '</div>'
      ],
    ];

    if( $this->enable_assessments ) {
      $wpep_course_options_fields[ 'wpep_course_assessments_required' ] = [
        'name'    => __( 'Assessments Are', "wpep" ),
        'desc'    => '<p>' . __( "The student is required to complete the section / module assessment in order to continue the course.", "wpep" ) . '</p>',
        'id'      => 'wpep_course_assessments_required',
        'type'    => 'select',
        'options' => [
          1         => __( 'Mandatory', "wpep" ),
          0         => __( 'Optional', "wpep" ),
        ],
      ];
    }

    if( $this->enable_prerequisites_settings ) {
      $wpep_course_options_fields[ WPEP_PREREQUISITE_MODE_POST_META ] = [
        'name'    => __( 'Prerequisites Mode', "wpep" ),
        'desc'    => '<p>' . __( "How the prerequisites logic is handled.", "wpep" ) . '</p>',
        'id'      => WPEP_PREREQUISITE_MODE_POST_META,
        'type'    => 'select',
        'options' => wpep_get_prerequisite_mode_list()
      ];

      $selected_prerequisites = get_post_meta( get_the_ID(), 'wpep_prerequisite', false );

      if( !empty( $selected_prerequisites ) ) {
        $selected_prerequisites = get_posts( [
          'post_type' => WPEP_POST_TYPE_COURSE,
          'nopaging'  => true,
          'include'   => $selected_prerequisites
        ] );

        if( !empty( $selected_prerequisites ) )
          $selected_prerequisites = wpep_utility_map_to_array_assoc( $selected_prerequisites, 'ID', 'post_title' );
      }

      $wpep_course_options_fields[ 'wpep_prerequisite' ] = [
        'name'          => __( 'Prerequisites', "wpep" ),
        'desc'          => '<p>' . __( "The student must complete these courses in order to have access to this one.", "wpep" ) . '</p>',
        'id'            => 'wpep_prerequisite',
        'type'          => 'multiple_select',
        'options'       => $selected_prerequisites,
        'attributes'    => [
          'data-wpep-select-search' => json_encode( [
            'action'    => 'wpep_admin_post_list_search',
            'post_type' => WPEP_POST_TYPE_COURSE
          ] )
        ]
      ];
    }

    if( $this->enable_sidebar_settings ) {
      $wpep_course_options_fields[ 'wpep_sidebar_position' ] = [
        'name'    => __('Sidebar', "wpep"),
        'desc'    =>
          '<p data-wpep-course-type="' . WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . '">' . __( "An Execution Plan will not have a sidebar present if you haven't setup one yourself", "wpep" ) . '</p>' .
          '<p data-wpep-course-type="' . WPEP_COURSE_TYPE_CHALLENGE . '">' . __( "An Challenge Course Type will always have a sidebar active, by default on the left side.", "wpep" ) . '</p>' .
          '<p data-wpep-course-type="' . WPEP_COURSE_TYPE_STANDARD . '">' . __( "There will always have a sidebar, we highly recommend disabling it if you know what you're doing.", "wpep" ) . '</p>' .
          '<p data-wpep-course-type="' . WPEP_COURSE_TYPE_VIDEO . '">' . __( "There will always be a sidebar while viewing the overview, but disabled during the video library itself.", "wpep" ) . '</p>' .
          '<p data-wpep-course-type="' . wpep_course_layout_types_string( false, [ WPEP_COURSE_TYPE_CHALLENGE ] ) . '" class="wpep-not-important">' . sprintf( __( "Normally sidebars are present on the %s" ), ( wpep_is_rtl() ? 'left' : 'right' ) ) . '</p>',
        'id'      => 'wpep_sidebar_position',
        'type'    => 'select',
        'options' => [
          'default' => __( "Default", "wpep" ),
          'right'   => __( 'Right', "wpep" ),
          'bottom'  => __( 'Bottom', "wpep" ),
          'left'    => __( 'Left', "wpep" ),
          'disabled'=> __( 'Disabled', "wpep" )
        ],
        'content_before'  => '<div data-wpep-course-type="' . wpep_course_layout_types_string( false, [ WPEP_COURSE_TYPE_LEARN_MAP ] ) . '">',
        'content_after'   => '</div>'
      ];
    }

    $wpep_course_options_fields[ 'wpep_sequential_completion' ] = [
      'name'    => __( 'Lesson Completion', "wpep" ),
      'desc'    => '<p>' . __( "When the Course is Sequential, the Lessons have to be completed in the right order, and lessons cannot be skipped.", "wpep" ) . '</p>',
      'id'      => 'wpep_sequential_completion',
      'type'    => 'select',
      'options'         => [
        '0' => __( "Non Sequential", "wpep" ),
        '1' => __( "Sequential", "wpep" )
      ],
      'content_before'  => '<div data-wpep-course-type="' . wpep_course_layout_types_string( [ WPEP_COURSE_TYPE_EXECUTION_PLAN, WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED, WPEP_COURSE_TYPE_LEARN_MAP ] ) . '">',
      'content_after'   => '</div>'
    ];

    if( defined( "WPEP_BETA_TESTING" ) || defined( 'WPEP_BETA_TESTING_LESSON_CONTENT_ON_DEMAND' ) ) {
      $wpep_course_options_fields[ 'wpep_lesson_content_on_demand' ] = [
        'name'    => __( 'Load Lesson Content on', "wpep" ),
        'desc'    => '<p>' . __( "In normal circumstances you want to load the Lesson Content on Page Load, if the page load time is too slow, this option will dramatically increase performance, but may conflict with 3-rd party shortcode content.", "wpep" ) . '</p>',
        'id'      => 'wpep_lesson_content_on_demand',
        'type'    => 'select',
        'options'         => [
          '0' => __( "Page Load", "wpep" ),
          '1' => __( "Lesson Open", "wpep" )
        ],
        'content_before'  => '<div data-wpep-course-type="' . wpep_course_layout_types_string( [ WPEP_COURSE_TYPE_EXECUTION_PLAN, WPEP_COURSE_TYPE_LEARN_MAP ] ) . '">',
        'content_after'   => '</div>'
      ];
    }

    $wpep_course_options_fields[ 'wpep_read_only' ] = [
      'name'    => __( 'Read Only', "wpep" ),
      'desc'    => '<p>' . __( "This course cannot be completed, all features that track progress are hidden.", "wpep" ) . '</p>',
      'id'      => 'wpep_read_only',
      'type'    => 'select',
      'options' => wpep_flag_setting_field_options(),
    ];
    $wpep_course_options_fields[ 'wpep_show_progress_bar' ] = [
      'name'    => __('Show Progress Bar', "wpep"),
      'desc'    => __('Chose to hide or display the course progress bar specifically for this course.', "wpep"),
      'id'      => 'wpep_show_progress_bar',
      'type'    => 'radio_inline',
      'options' => [
        'show'   => __('Show', "wpep"),
        'hide'   => __('Hide', "wpep")
      ],
      'content_before'  => '<div data-wpep-course-type="' . wpep_course_layout_types_string( false, [ WPEP_COURSE_TYPE_ONE_OFF, WPEP_COURSE_TYPE_CHALLENGE ] ) . '">',
      'content_after'   => '</div>'
    ];

    if( $this->enable_progress_repeat ) {
      $wpep_course_options_fields[ 'wpep_progress_repeat_enabled' ] = [
        'name'    => __( 'Progress Repeat', 'wpep' ),
        'id'      => 'wpep_progress_repeat_enabled',
        'type'    => 'select',
        'options' => wpep_flag_setting_field_options(),
        'description' =>  __( "Allow the student to re-take this course", "wpep" )
      ];
    }

    if( $this->enable_presentation_card_settings ) {
      $wpep_course_options_fields[ 'wpep_thumb_image' ] =[
        'name' => __('Thumbnail Image', "wpep"),
        'desc' => __('Set a picture that will show on Course Index Page. ( recommended size : 300px width x 190px height )', "wpep"),
        'id'   => 'wpep_thumb_image',
        'type' => 'image',
      ];

      if( wpep_get_setting_flag( 'index-page-enable-hover-image' ) ) {
        $wpep_course_options_fields[ 'wpep_thumb_image_hover' ] = [
          'name' => __('Thumbnail Image - Hover', "wpep"),
          'desc' => __('Set a picture that will show on Course Index Page on hover.', "wpep"),
          'id'   => 'wpep_thumb_image_hover',
          'type' => 'image',
        ];
      }
    }

    $wpep_course_options_fields[ 'wpep_icon_image' ] =[
      'name'            => __('Icon Image', "wpep"),
      'desc'            => __('Set a picture that will show beside the Course Title. ( recommended size : 100px width x 100px height )', "wpep"),
      'id'              =>  'wpep_icon_image',
      'type'            => 'image',
      'content_before'  => '<div data-wpep-course-type="' . WPEP_COURSE_TYPE_EXECUTION_PLAN . ' ' . WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED . ' ' . WPEP_COURSE_TYPE_LEARN_MAP . '">',
      'content_after'   => '</div>'
    ];

    if( wpep_get_setting_flag( 'enable-course-tracking-code-option' ) ) {
      $wpep_course_options_fields['wpep_tracking_code']  = [
        'name'    => __('Tracking Code', "wpep"),
        'id'      => 'wpep_tracking_code_completed',
        'type'    => 'textarea',
        'height'  => '150px;',
        'desc'    => __(
          "This is a custom javascript code or HTML code executed when the student has completed the course.",
          "wpep"
        )
      ];
    }

    return $wpep_course_options_fields;
  }

  public function get_value( $field_key, $field_default = '', $single = true ) {
    global $post;

    $field_value = get_post_meta( $post->ID, $field_key, $single );

    return $field_value == '' ? $field_default : $field_value;
  }

}