<?php

namespace WPEP\ContentType\Course\Container\Admin;

trait General {

  public function setup_general_functionality() {
    add_action('post_row_actions', [ $this, 'row_actions' ], 10, 2);

    add_filter('manage_posts_columns', [ $this, '_table_set_columns' ] );
    add_filter('manage_posts_custom_column', [ $this, '_table_custom_columns' ] );

    add_action( 'save_post', [ $this, '_save_post_action' ], 20 );
    add_filter( 'wpep_admin_save_post_ajax', [ $this, '_wpep_admin_save_post_ajax' ], 10, 2 );
  }

  public function _save_post_action( $course_post_id ) {
    global $post_type;

    if ( !is_admin() || ( $post_type != WPEP_POST_TYPE_COURSE ) )
      return;

    $entity_id_map = ( isset( $_POST['wpep_entity_id_map'] ) ? $_POST['wpep_entity_id_map'] : [] );

    if( is_string( $entity_id_map ) ) {
      $entity_id_map = json_decode( stripslashes( $entity_id_map ), true );
    }

    if( !is_array( $entity_id_map ) )
      $entity_id_map = [];

    $entity_id_map = $entity_id_map + [
      'section' => [],
      'lesson'  => []
    ];

    $this->_save_post( $course_post_id, $entity_id_map );
  }

  public function _wpep_admin_save_post_ajax( $entity_id_map, $post_id ) {
    return $this->_save_post( $post_id, $entity_id_map );
  }

  public function _save_post( $course_post_id, $entity_id_map = [] ) {
    if( get_post_type( $course_post_id ) != WPEP_POST_TYPE_COURSE )
      return $entity_id_map;

    if( intval( get_post_meta( $course_post_id, 'wpep_order', true ) ) == false )
      update_post_meta( $course_post_id, 'wpep_order', 0 );

    $searchable_post_content = '';

    if (isset($_POST['wpep_course_description'])) {
      $course_description = stripcslashes ( $_POST['wpep_course_description'] );

      if( $course_description !== '' )
        update_post_meta( $course_post_id, 'wpep_course_description', $course_description );
      else
        delete_post_meta( $course_post_id, 'wpep_course_description' );

      $searchable_post_content .= $course_description . "\n";
    } else {
      $searchable_post_content .= get_post_meta( $course_post_id, 'wpep_course_description', true ) . "\n";
    }

    if( wpep_course_type_from_id( $course_post_id ) != WPEP_COURSE_TYPE_ONE_OFF ) {
      $entity_id_map = $this->handle_course_save_functionality( $course_post_id, $entity_id_map, $searchable_post_content );
    }

    $entity_id_map = $this->handle_assessment_save_functionality( $course_post_id, $entity_id_map );

    if( isset( $_POST['wpep_current_primary_tab'] ) )
      update_post_meta( $course_post_id, 'wpep_current_primary_tab', $_POST['wpep_current_primary_tab'] );

    $searchable_post_content = trim( $searchable_post_content );

    global $wpdb;

    $wpdb->update( $wpdb->posts, [
      'post_content' => $searchable_post_content
    ], [ 'ID' => $course_post_id ] );

    return $entity_id_map;
  }

  public function _table_set_columns($columns) {
    global $post_type;

    if ( $post_type != WPEP_POST_TYPE_COURSE )
      return $columns;

    unset(
      $columns['date']
    );

    $new_columns = [
      'shortcode' => __('Shortcode', "wpep"),
    ];

    if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) )
      $new_columns['statistics'] = __('Statistics', "wpep");

    $new_columns['date'] = __('Date', "wpep");

    return array_merge($columns, $new_columns);
  }

  public function _table_custom_columns($column) {
    global $post;

    if( $post->post_type != WPEP_POST_TYPE_COURSE )
      return;

    if ( $column == 'shortcode' ) {
      echo wpep_admin_shortcode_builder(
        'wpep_course',
        [
          'id'  => $post->ID,
          'wpep_admin_shortcode_builder_wpep_course_options'
        ],
        __( "ShortCode", "wpep" ),
        true
      );

      return;
    }

    if( $column == 'statistics' ) {
      echo '<a class="button button-primary" href="' . wpep_admin_url_statistics_content_type( WPEP_POST_TYPE_COURSE, [ 'course_id' => $post->ID ] ) . '">' . __( "View Statistics", "wpep" )    . '</a>';

      return;
    }
  }

  public function row_actions($actions, $post) {
    if( $post->post_type != WPEP_POST_TYPE_COURSE )
      return $actions;

    if( get_post_meta( $post->ID , 'course_type', true ) != WPEP_COURSE_TYPE_ONE_OFF ) {
      $actions['wpep_preview'] = wpep_admin_shortcode_builder(
        'wpep_course_preview',
        [
          'id'  => $post->ID,
          'wpep_admin_shortcode_builder_wpep_course_preview_options'
        ],
        __( "Course Preview", "wpep" ),
        false
      );
    }

    return $actions;

  }

}
