<?php

namespace WPEP\ContentType\Course\Container;

use WPEP\Entity\Course as Entity_Course;
use WPEP\Entity\Discussion as Entity_Discussion;

trait Data {

  /**
   * @param $args
   * @return array|null
   */
  public function data_user_activity_report( $args ) {
    $response = parent::data_user_activity_report( $args );

    global $wpdb;

    $courses_started_sql = 'SELECT COUNT( id ) FROM ' . $wpdb->prefix . 'wpep_course_user_data cud 
                                              WHERE `key` = "' . WPEP_USER_COURSE_STARTED . '"';
    $courses_completed_at_sql = 'SELECT COUNT( id ) FROM ' . $wpdb->prefix . 'wpep_course_user_data cud 
                                                   WHERE `key` = "' . WPEP_USER_COURSE_COMPLETED_AT . '"';
    $lessons_completed_sql    = 'SELECT COUNT( id ) FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed 
                                                   WHERE 1 = 1';

    if( isset( $args['time_from' ] ) ) {
      $courses_started_sql      .= ' AND CAST( `value` AS UNSIGNED ) >= ' . absint( $args['time_from'] );
      $courses_completed_at_sql .= ' AND CAST( `value` AS UNSIGNED ) >= ' . absint( $args['time_from'] );
      $lessons_completed_sql    .= ' AND `created_at` >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '"';
    }

    if( isset( $args['time_to' ] ) ) {
      $courses_started_sql .= ' AND CAST( `value` AS UNSIGNED ) < ' . absint( $args['time_to'] );
      $courses_completed_at_sql .= ' AND CAST( `value` AS UNSIGNED ) < ' . absint( $args['time_to'] );
      $lessons_completed_sql    .= ' AND `created_at` < "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';
    }

    $response[ $this->get_plural_name() . ' Started' ]  = absint( $wpdb->get_var( $courses_started_sql ) );
    $response[ $this->get_plural_name() . ' Completed' ] = absint( $wpdb->get_var( $courses_completed_at_sql ) );
    $response[ 'Lessons Completed' ] = absint( $wpdb->get_var( $lessons_completed_sql ) );
    $response[ $this->get_singular_name() . ' Comments' ] = absint( Entity_Discussion::instance()->get_comment_count_by_args( $this->discussionIntegration->get_namespace(), $args ) );

    return $response;
  }

  public function data_asset_map() {
    $response = [
      'meta'  => [
        'wpep_course_description',
        'wpep_thumb_image',
        'wpep_thumb_image_hover',
        'wpep_icon_image',
        'wpep_member_magnet_option_text_before',
        'wpep_member_magnet_option_text_after'
      ],
      'section_list' => [
        'content',
        'lessons' => [ 'content' ],
      ]
    ];

    foreach( $this->_post_dependencies as $post_dependency )
      $response = $post_dependency->attach_data_asset_map_for_course( $response );

    return $response;
  }

  public function data_post_connection() {
    return [
      'meta_wpep_prerequisite' => [
        'type'       => 'meta',
        'key'        => 'wpep_prerequisite',
        'value_type' => 'post_id',
        'location'   => 'internal'
      ]
    ];
  }

  public function data_export_single_post_array( $post, $include_user_data = true ) {
    $response = parent::data_export_single_post_array( $post, $include_user_data );

    $response = $this->_data_export_array_attach_course_data( $response, $post, $include_user_data );

    if( empty( $this->_post_dependencies ) )
      return $response;

    foreach( $this->_post_dependencies as $post_dependency ) {
      $response = $post_dependency->attach_data_export_array_for_course( $response, $include_user_data );
    }

    return $response;
  }

  public function data_import_single_post_array( $post, $id_mapping = [] ) {
    $new_post_id = parent::data_import_single_post_array( $post, $id_mapping );

    if( isset( $post['section_list'] ) && !empty( $post['section_list'] ) ) {
      $course_data_map = $this->_data_import_single_post_section_list( $post['section_list'], $new_post_id );

      if( isset( $post['student_activity'] ) && is_array( $post['student_activity'] ) && !empty( $post['student_activity'] ) ) {
        $student_list = wpep_controller()->dataSystem->get_user_ids_mapped_by_email();

        foreach( $post['student_activity'] as $student_activity ) {
          if( !isset( $student_list[ $student_activity['user_email_address'] ] ) )
            continue;

          wpep_controller()->userActivity->add_course_user_data(
            $student_activity['key'],
            $student_activity['value'],
            $new_post_id,
            ( isset( $course_data_map['course_section'][ $student_activity['section_id'] ] ) ? $course_data_map['course_section'][ $student_activity['section_id'] ] : 0 ),
            ( isset( $course_data_map['course_lesson'][ $student_activity['lesson_id'] ] )   ? $course_data_map['course_lesson'][ $student_activity['lesson_id'] ]   : 0 ),
            $student_list[ $student_activity['user_email_address'] ]
          );
        }
      }
    }

    if( isset( $post['assessment'] ) && isset( $this->_post_dependencies[ WPEP_POST_TYPE_ASSESSMENT ] ) ) {
      $assessment = $post['assessment'];

      if( !isset( $assessment['meta'] ) || !is_array( $assessment['meta'] ) )
        $assessment['meta'] = [];

      $assessment['meta']['wpep_course_id']  = $new_post_id;
      $assessment['meta']['wpep_section_id'] = 0;

      $this->_post_dependencies[ WPEP_POST_TYPE_ASSESSMENT ]->data_import_single_post_array( $assessment, $id_mapping );
    }

    if( isset( $post['discussions'] ) && !empty( $post['discussions'] ) ) {
      foreach( $post['discussions'] as $discussion_data ) {
        if( !isset( $discussion_data['identifier_args'] ) || !is_array( $discussion_data['identifier_args'] ) )
          continue;

        if( isset( $discussion_data['identifier_args']['course_id'] ) )
          $discussion_data['identifier_args']['course_id'] = $new_post_id;

        if( isset( $discussion_data['identifier_args']['section_id'] ) ) {
          if( !isset( $course_data_map[ 'course_section' ][ $discussion_data['identifier_args']['section_id'] ] ) )
            continue;

          $discussion_data['identifier_args']['section_id'] = $course_data_map[ 'course_section' ][ $discussion_data['identifier_args']['section_id'] ];
        }

        if( isset( $discussion_data['identifier_args']['lesson_id'] ) ) {
          if( !isset( $course_data_map[ 'course_lesson' ][ $discussion_data['identifier_args']['lesson_id'] ] ) )
            continue;

          $discussion_data['identifier_args']['lesson_id'] = $course_data_map[ 'course_lesson' ][ $discussion_data['identifier_args']['lesson_id'] ];
        }

        $this->data_import_single_post_discussion( $new_post_id, $discussion_data );
      }
    }

    return $new_post_id;
  }

  private function _data_import_single_post_section_list( $section_list, $post_id ) {
    global $wpdb;

    $response = [];

    foreach( $section_list as $section_information ) {
      $section_options = [];

      if( isset( $section_information['options'] ) ) {
        $section_options = $section_information['options'];

        if( is_object( $section_options ) )
          $section_options = get_object_vars( $section_options );
        else if( is_string( $section_options ) )
          $section_options = json_decode( $section_options, true );

        if( !is_array( $section_options ) )
          $section_options = [];
      }

      $current_information = [
        'post_id'                    => $post_id,
        'title'                      => $section_information['title'],
        'content'                    => str_replace('\"', '"', ( isset( $section_information['content'] ) ? $section_information['content'] : '' ) ),
        'journey_access_type'        => ( isset( $section_information['journey_access_type'] )  ? $section_information['journey_access_type']  : '' ),
        'journey_access_segment'     => ( isset( $section_information['journey_access_segment'] )  ? $section_information['journey_access_segment']  : '' ),
        'journey_expiration_type'    => ( isset( $section_information['journey_expiration_type'] )  ? $section_information['journey_expiration_type']  : '' ),
        'journey_expiration_segment' => ( isset( $section_information['journey_expiration_segment'] )  ? $section_information['journey_expiration_segment']  : '' ),
        'journey_execution_type'     => ( isset( $section_information['journey_execution_type'] )  ? $section_information['journey_execution_type']  : '' ),
        'journey_execution_segment'  => ( isset( $section_information['journey_execution_segment'] )  ? $section_information['journey_execution_segment']  : '' ),
        'order'                      => $section_information['order'],
        'options'                    => json_encode( $section_options )
      ];

      $wpdb->insert( $wpdb->prefix . 'wpep_section', $current_information );
      $section_id = $wpdb->insert_id;

      if( isset( $section_information['id'] ) )
        $response['course_section'][ $section_information['id'] ] = $section_id;

      if( isset( $section_information['assessment'] ) && isset( $this->_post_dependencies[ WPEP_POST_TYPE_ASSESSMENT ] ) ) {
        $assessment = $section_information['assessment'];

        if( !isset( $assessment['meta'] ) || !is_array( $assessment['meta'] ) )
          $assessment['meta'] = [];

        $assessment['meta']['wpep_course_id']  = $post_id;
        $assessment['meta']['wpep_section_id'] = $section_id;

        $this->_post_dependencies[ WPEP_POST_TYPE_ASSESSMENT ]->data_import_single_post_array( $assessment );
      }

      if( !isset( $section_information['lessons'] ) )
        continue;

      if( !is_array( $section_information['lessons'] ) )
        continue;

      foreach( $section_information['lessons'] as $lesson_information ) {
        $lesson_options = [];

        if( isset( $section_information['options'] ) ) {
          $lesson_options = $lesson_information['options'];

          if( is_object( $lesson_options ) )
            $lesson_options = get_object_vars( $lesson_options );
          else if( is_string( $lesson_options ) )
            $lesson_options = json_decode( $lesson_options, true );

          if( !is_array( $lesson_options ) )
            $lesson_options = [];
        }

        $current_information = [
          'post_id'                   => $post_id,
          'section_id'                => $section_id,
          'title'                     => $lesson_information['title'],
          'content'                   => str_replace('\"', '"', $lesson_information['content']),
          'journey_access_type'        => ( isset( $section_information['journey_access_type'] )  ? $section_information['journey_access_type']  : '' ),
          'journey_access_segment'     => ( isset( $section_information['journey_access_segment'] )  ? $section_information['journey_access_segment']  : '' ),
          'journey_expiration_type'    => ( isset( $section_information['journey_expiration_type'] )  ? $section_information['journey_expiration_type']  : '' ),
          'journey_expiration_segment' => ( isset( $section_information['journey_expiration_segment'] )  ? $section_information['journey_expiration_segment']  : '' ),
          'journey_execution_type'     => ( isset( $section_information['journey_execution_type'] )  ? $section_information['journey_execution_type']  : '' ),
          'journey_execution_segment'  => ( isset( $section_information['journey_execution_segment'] )  ? $section_information['journey_execution_segment']  : '' ),
          'order'                     => $lesson_information['order'],
          'options'                   => json_encode( $lesson_options )
        ];

        $wpdb->insert( $wpdb->prefix . 'wpep_section_lesson', $current_information );

        $lesson_id = $wpdb->insert_id;

        if( isset( $lesson_information['id'] ) )
          $response['course_lesson'][ $lesson_information['id'] ] = $lesson_id;

        if( !isset( $lesson_information['students_completed'] ) )
          continue;

        if( !is_array( $lesson_information['students_completed'] ) )
          continue;

        foreach( $lesson_information['students_completed'] as $student_completed ) {
          if( !isset( wpep_controller()->dataSystem->get_user_ids_mapped_by_email()[ $student_completed['user_email_address'] ] ) )
            continue;

          $wpdb->insert( $wpdb->prefix . 'wpep_section_lesson_completed', array(
            'user_id'    => wpep_controller()->dataSystem->get_user_ids_mapped_by_email()[ $student_completed['user_email_address'] ],
            'lesson_id'  => $lesson_id,
            'created_at' => $student_completed['created_at']
          ) );
        }
      }
    }

    return $response;
  }

  private function _data_export_array_attach_course_data( $response, $post, $include_user_data = true ) {
    global $wpdb;

    $user_list = wpep_controller()->dataSystem->get_users();

    $course_id = $post->ID;

    $section_list = Entity_Course::instance()->get_with_lessons( $course_id );

    $response['section_list'] = [];

    if( $include_user_data ) {
      $response['student_activity'] = [];

      foreach( wpep_controller()->userActivity->get_course_user_data( false, $course_id, false, false, false, false ) as $course_user_data_row ) {
        if( !isset( $user_list[ $course_user_data_row->user_id ] ) )
          continue;

        $response['student_activity'][] = [
          'user_id'             => $course_user_data_row->user_id,
          'user_email_address'  => $user_list[ $course_user_data_row->user_id ]['email_address'],
          'section_id'          => $course_user_data_row->section_id,
          'lesson_id'           => $course_user_data_row->lesson_id,
          'key'                 => $course_user_data_row->key,
          'value'               => $course_user_data_row->value
        ];
      }
    }


    foreach( $section_list as $section_key => $section_information ) {
      $export_section = [
        'id'                         => $section_information->id, // normally we don't care, but in case of the User Activity Retain we want this.
        'title'                      => $section_information->title,
        'content'                    => ( isset( $section_information->content ) ? $section_information->content : '' ),
        'journey_access_type'        => ( isset( $section_information->journey_access_type )  ? $section_information->journey_access_type  : '' ),
        'journey_access_segment'     => ( isset( $section_information->journey_access_segment )  ? $section_information->journey_access_segment  : '' ),
        'journey_expiration_type'    => ( isset( $section_information->journey_expiration_type )  ? $section_information->journey_expiration_type  : '' ),
        'journey_expiration_segment' => ( isset( $section_information->journey_expiration_segment )  ? $section_information->journey_expiration_segment  : '' ),
        'journey_execution_type'     => ( isset( $section_information->journey_execution_type )  ? $section_information->journey_execution_type  : '' ),
        'journey_execution_segment'  => ( isset( $section_information->journey_execution_segment )  ? $section_information->journey_execution_segment  : '' ),
        'order'                      => $section_information->order,
        'options'                    => ( is_object( $section_information->options ) ? get_object_vars( $section_information->options ) : $section_information->options ),
        'lessons'                    => []
      ];



      if( !empty( $this->_post_dependencies ) ) {
        foreach( $this->_post_dependencies as $post_dependency ) {
          $export_section = $post_dependency->attach_data_export_array_for_course_section( $export_section, $course_id, $section_information->id, $include_user_data );
        }
      }

      foreach( $section_information->lessons as $lesson_ley => $lesson_information ) {
        $export_section['lessons'][$lesson_information->id] = [
          'id'                         => $lesson_information->id, // normally we don't care, but in case of the User Activity Retain we want this.
          'title'                      => $lesson_information->title,
          'content'                    => $lesson_information->content,
          'journey_access_type'        => ( isset( $section_information->journey_access_type )  ? $section_information->journey_access_type  : '' ),
          'journey_access_segment'     => ( isset( $section_information->journey_access_segment )  ? $section_information->journey_access_segment  : '' ),
          'journey_expiration_type'    => ( isset( $section_information->journey_expiration_type )  ? $section_information->journey_expiration_type  : '' ),
          'journey_expiration_segment' => ( isset( $section_information->journey_expiration_segment )  ? $section_information->journey_expiration_segment  : '' ),
          'journey_execution_type'     => ( isset( $section_information->journey_execution_type )  ? $section_information->journey_execution_type  : '' ),
          'journey_execution_segment'  => ( isset( $section_information->journey_execution_segment )  ? $section_information->journey_execution_segment  : '' ),
          'order'                      => $lesson_information->order,
          'options'                    => ( is_object( $lesson_information->options ) ? get_object_vars( $lesson_information->options ) : $lesson_information->options ),
          'students_completed'         => []
        ];

        if( !$include_user_data )
          continue;

        $students_completed = $wpdb->get_results( 'SELECT user_id, created_at FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed WHERE lesson_id = ' . $lesson_information->id, OBJECT );

        foreach( $students_completed as $student_completed ) {
          if( !isset( $user_list[ $student_completed->user_id ] ) )
            continue;

          $export_section['lessons'][$lesson_information->id]['students_completed'][$student_completed->user_id] = [
            'user_id'            => $student_completed->user_id,
            'user_email_address' => $user_list[ $student_completed->user_id ]['email_address'],
            'created_at'         => $student_completed->created_at
          ];
        }
      }

      $response['section_list'][$section_key] = $export_section;
    }

    return $response;
  }

}
