<?php

namespace WPEP\ContentType\Course\Integration;

use WPEP\Entity\Discussion as WPEP_Discussion;
use WPEP\Entity\Course as WPEP_Course;

/**
 * Class PostComments
 * @package WPEPAddOnPersonalizedLearning\Integration
 */
class Discussion extends \WPEP_Discussion_Integration {

  public function get_namespace() {
    return 'course';
  }

  public function get_discussion_name( $discussion ) {
    if( !isset( $discussion->post_id ) || $discussion->post_id == 0 )
      return $discussion->identifier;

    if( strpos( $discussion->identifier, 'course_' . $discussion->post_id ) !== 0 )
      return get_the_title( $discussion->post_id );

    $discussion_sub_identifier = str_replace( 'course_' . $discussion->post_id, '', $discussion->identifier );

    if( $discussion_sub_identifier === '' )
      return get_the_title( $discussion->post_id );

    $discussion_sub_identifier = ltrim( $discussion_sub_identifier, '_' );

    if( strpos( $discussion_sub_identifier, 'section_' ) === 0 ) {
      $section = WPEP_Course::instance()->get_section( absint( str_replace( 'section_', '', $discussion_sub_identifier ) ) );

      if( isset( $section->title ) )
        return $section->title . ' ( ' . get_the_title( $discussion->post_id ) . ' ) ';
    }

    if( strpos( $discussion_sub_identifier, 'lesson_' ) === 0 ) {
      $lesson = WPEP_Course::instance()->get_lesson( absint( str_replace( 'lesson_', '', $discussion_sub_identifier ) ) );

      if( isset( $lesson->title ) )
        return $lesson->title . ' ( ' . get_the_title( $discussion->post_id ) . ' ) ';
    }

    return $discussion_sub_identifier . ' ( ' . get_the_title( $discussion->post_id ) . ' ) ';
  }

  public function get_discussion_link( $discussion, $user_id = false ) {
    if( !isset( $discussion->post_id ) )
      return '#';

    if( strpos( $discussion->identifier, 'course_' . $discussion->post_id ) !== 0 )
      return get_permalink( $discussion->post_id );

    $discussion_sub_identifier = str_replace( 'course_' . $discussion->post_id, '', $discussion->identifier );

    if( $discussion_sub_identifier === '' )
      return get_permalink( $discussion->post_id );

    $discussion_sub_identifier = ltrim( $discussion_sub_identifier, '_' );

    if( strpos( $discussion_sub_identifier, 'section_' ) === 0 ) {
      $section = WPEP_Course::instance()->get_section( absint( str_replace( 'section_', '', $discussion_sub_identifier ) ) );

      if( isset( $section->id ) )
        return add_query_arg( wpep_get_setting( 'course-challenge-section-slug' ), $section->id, get_permalink( $discussion->post_id ) );
    }

    if( strpos( $discussion_sub_identifier, 'lesson_' ) === 0 ) {
      $lesson = WPEP_Course::instance()->get_lesson( absint( str_replace( 'lesson_', '', $discussion_sub_identifier ) ) );

      if( isset( $lesson->id ) )
        return wpep_course_lesson_permalink( $lesson->id, $discussion->post_id );
    }

    return get_permalink( $discussion->post_id );
  }

  public function current_user_can_view( $discussion_id ) {
    return $this->current_user_can_post( $discussion_id );
  }

  public function current_user_can_post( $discussion_id ) {
    if( get_current_user_id() === 0 )
      return false;

    if( current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      return true;

    $discussion_data = ( !is_object( $discussion_id ) ? WPEP_Discussion::instance()->get_discussion( [
      'id' => $discussion_id
    ] ) : $discussion_id );

    return wpep_controller()->contentLibrary->has_access_to_post( $discussion_data->post_id );
  }

  public function get_email_notifications_status() {
    return true;
  }

  public function get_discussion_identifier( $args ) {
    if( isset( $args[ 'section_id'] ) )
      return 'course_' . ( isset( $args['course_id'] ) ? $args['course_id'] : 0 ) . '_section_' . $args[ 'section_id'];

    if( isset( $args[ 'lesson_id'] ) )
      return 'course_' . ( isset( $args['course_id'] ) ? $args['course_id'] : 0 ) . '_lesson_' . $args[ 'lesson_id'];

    return 'course_' . ( isset( $args['course_id'] ) ? $args['course_id'] : 0 );
  }

  /**
   * @param string $identifier
   * @return false|array
   */
  public function get_discussion_identifier_args_from_string( $identifier ) {
    if( strpos( $identifier, 'course_' ) !== 0 )
      return false;

    $identifier = substr( $identifier, strlen( 'course_' ) );

    if( strpos( $identifier, '_' ) === false )
      return [
        'course_id' => intval( $identifier )
      ];

    $course_id = intval( substr( $identifier, 0, strpos( $identifier, '_' ) ) );
    $identifier = str_replace( $course_id . '_', '', $identifier );

    if( strpos( $identifier, 'section_' ) === 0 )
      return [
        'course_id'  => $course_id,
        'section_id' => intval( str_replace( 'section_', '', $identifier ) )
      ];

    if( strpos( $identifier, 'lesson_' ) === 0 )
      return [
        'course_id' => $course_id,
        'lesson_id' => intval( str_replace( 'lesson_', '', $identifier ) )
      ];

    return [
      'course_id' => $course_id
    ];
  }
  
  public function get_comment_types() {
    return [];
  }

}