<?php

namespace WPEP\ContentType\Course\Integration;

use WPEP\ContentType\Skeleton\Integration\Statistics as Statistics_Integration;

class Statistics extends Statistics_Integration {

  /**
   * @var Statistics\Dashboard
   */
  public $dashboard;
  /**
   * @var Statistics\Single
   */
  public $single;
  /**
   * @var Statistics\User
   */
  public $user;
  /**
   * @var Statistics\UserSingle
   */
  public $userSingle;

  public function __construct() {
    $this->dashboard            = new Statistics\Dashboard();
    $this->single               = new Statistics\Single( ( isset( $_GET['course_id'] ) ? intval( $_GET['course_id'] ) : 0 ) );
    $this->user                 = new Statistics\User();
    $this->userSingle           = new Statistics\UserSingle();
  }

  public function get_name() {
    return __( 'Courses', "wpep" );
  }

  public function get_alias() {
    return WPEP_POST_TYPE_COURSE;
  }

  public function display( $filter_by = [] ) {
    $course_id = ( isset( $_GET['course_id'] ) ? intval( $_GET['course_id'] ) : 0 );

    if( $course_id != 0 ) {
      if( !wpep_user_has_statistics_access_for_post( $course_id ) ) {
        echo '<br/>';
        echo '<div class="wpep-notification wpep-notification-danger">' . __( "You do not have access to this page", "wpep" ) . '</div>';
        return;
      }


      if( isset( $_GET['user_id'] ) ) {
        $this->userSingle->display( $_GET['course_id'], $_GET['user_id'] );

        return;
      }

      $this->single->display( $_GET['course_id'], $filter_by );

      return;
    }

    if( isset( $_GET['user_id'] ) ) {
      $this->user->display( $_GET[ 'user_id' ], $filter_by );

      return;
    }

    $this->dashboard->display( $filter_by );
    return;
  }

  public function download( $filter_by = [] ) {
    $course_id = ( isset( $_GET['course_id'] ) ? intval( $_GET['course_id'] ) : 0 );

    if( $course_id != 0 ) {
      if( !wpep_user_has_statistics_access_for_post( $course_id ) ) {
        echo '<br/>';
        echo '<div class="wpep-notification wpep-notification-danger">' . __( "You do not have access to this page", "wpep" ) . '</div>';
        return;
      }

      if( isset( $_GET['user_id'] ) ) {
        return;
      }

      $this->single->download( $_GET['course_id'], $filter_by );

      return;
    }

    if( isset( $_GET['user_id'] ) ) {
      $this->user->download( $_GET[ 'user_id' ], $filter_by );

      return;
    }

    $this->dashboard->download( $filter_by );
    return;
  }
  
}
