<?php

namespace WPEP\ContentType\Course;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Course as WPEP_Entity_Course;

class Request {

  public function __construct() {
    add_action('wp_ajax_wpep_flag_course_started_at', [ $this, 'flag_course_started_at' ] );
    add_action('wp_ajax_wpep_get_lesson_content', [ $this, 'get_lesson_content' ] );

    add_action('wp_ajax_wpep_set_learning_path', [ $this, 'set_learning_path' ] );
    add_action('wp_ajax_wpep_set_bookmark_status', [ $this, 'set_bookmark_status' ] );
    add_action('wp_ajax_wpep_set_lesson_progress', [ $this, 'set_lesson_progress' ] );

    add_action('wp_ajax_nopriv_wpep_set_lesson_progress', [ $this, 'set_lesson_progress' ] );
  }

  public function flag_course_started_at() {
    if( !isset( $_POST['course_id'] )  )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "The course_id parameter is missing.", "wpep" )
      ] ) );

    $course_id = intval( $_POST[ 'course_id' ] );
    $user_id   = get_current_user_id();

    if( get_post_type( $course_id ) != WPEP_POST_TYPE_COURSE )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Invalid Course ID provided", "wpep" )
      ] ) );

    exit( json_encode( [
      'status'      => 'ok',
      'timestamp'   => wpep_user_course_started_event( $course_id, $user_id )
    ] ) );
  }

  public function get_lesson_content() {
    if( !isset( $_POST['lesson_id'] ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "The lesson_id parameter is missing.", "wpep" )
      ] ) );

    $lesson_id = intval( $_POST[ 'lesson_id' ] );
    $course_id = intval( WPEP_Entity_Course::instance()->get_course_id_by_lesson_id( $lesson_id ) );

    if( !WPEP_Controller::instance()->contentLibrary->has_access_to_post( $course_id ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "You don't have access to this Course", "wpep" )
      ] ) );

    $lesson = WPEP_Entity_Course::instance()->get_lesson( $lesson_id );

    exit( json_encode( [
      'status'         => 'ok',
      'lesson_content' => wpep_format_content_autoembed( wpep_format_content( $lesson->content ) )
    ] ) );
  }

  public function set_lesson_progress() {
    if( !isset( $_POST['id'] ) || !isset( $_POST['course_id'] ) || !isset( $_POST['is_completed'] ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Please Refresh your browser cache, you are not using the latest javascript version most likely.", "wpep" )
      ] ) );

    if( get_post_type( intval( $_POST[ 'course_id' ] ) ) != WPEP_POST_TYPE_COURSE )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Invalid Course ID provided", "wpep" )
      ] ) );


    if( isset( $_POST[ 'course_progress' ] ) ) {
      $course_is_completed = $this->_set_lesson_progress_eps();
    } else {
      WPEP_Controller::instance()->userActivity->set_lesson_status( $_POST['id'], $_POST['is_completed'], get_current_user_id() );

      $course_is_completed = wpep_user_course_is_completed( $_POST['id'], get_current_user_id() );
    }

    WPEP_Controller::instance()->userActivity->set_lesson_status(
      $_POST['id'], $_POST['is_completed'], get_current_user_id(), false, false
    );

    $response = [
      'status'           => 'ok',
      'course_completed' => $course_is_completed
    ];

    if( $course_is_completed )
      $response[ 'tracking_code' ] = wpep_prepare_tracking_code(
        get_post_meta( intval( $_POST[ 'course_id' ] ), 'wpep_tracking_code_completed', true ),
        'course-completed'
      );

    exit( json_encode( $response ) );
  }

  private function _set_lesson_progress_eps() {
    $course_is_completed = ( $_POST['course_progress'] == 100 ? 1 : 0 );

    WPEP_Controller::instance()->userActivity->set_course_user_data(
      WPEP_USER_COURSE_PROGRESS,
      floatval( $_POST['course_progress'] ),
      intval( $_POST[ 'course_id' ] ), 0, 0, get_current_user_id()
    );

    WPEP_Controller::instance()->userActivity->set_course_user_data(
      WPEP_USER_COURSE_COMPLETED,
      $course_is_completed,
      intval( $_POST[ 'course_id' ] ), 0, 0, get_current_user_id()
    );

    if( $course_is_completed ) {
      WPEP_Controller::instance()->userActivity->set_course_user_data(
        WPEP_USER_COURSE_COMPLETED_AT,
        time(),
        intval( $_POST[ 'course_id' ] ), 0, 0, get_current_user_id()
      );
    } else {
      WPEP_Controller::instance()->userActivity->remove_course_user_data(
        WPEP_USER_COURSE_COMPLETED_AT,
        null,
        intval( $_POST[ 'course_id' ] ), 0, 0, get_current_user_id()
      );
    }

    WPEP_Controller::instance()->userActivity->set_lesson_status(
      $_POST['id'], $_POST['is_completed'], get_current_user_id(), false, false
    );

    return $course_is_completed;
  }

  public function set_learning_path() {
    if( !isset( $_POST['course_id'] ) || !isset( $_POST['section_id'] ) || !isset( $_POST['learning_path'] ) )
      exit( json_encode( [
        'status'      => 'ok',
        'error'       => __( "One of the following parameters are missing : course_id, section_id, learning_path", "wpep" )
      ] ) );

    WPEP_Controller::instance()->userActivity->set_section_learning_path(
      intval( $_POST['course_id'] ), intval( $_POST['section_id'] ), intval( $_POST['learning_path'] ), get_current_user_id()
    );

    exit( json_encode( [
      'status'      => 'ok'
    ] ) );
  }

  public function set_bookmark_status() {
    if( !isset( $_POST['course_id'] ) || !isset( $_POST['is_bookmarked'] ) )
      exit( json_encode( [
        'status'      => 'ok',
        'error'       => __( "One of the following parameters are missing : course_id, is_bookmarked", "wpep" )
      ] ) );

    WPEP_Controller::instance()->userActivity->set_course_bookmark_status(
      intval( $_POST['course_id'] ),
      ( intval( $_POST['is_bookmarked'] ) ? 1 : 0 )
    );

    exit( json_encode( [
      'status'      => 'ok'
    ] ) );
  }

}