<?php

namespace WPEP\ContentType\Course;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Course as Entity_Course;

class UserProfile {

  public function __construct() {
    add_filter( 'wpep_content_user_profile_tab_list', [ $this, '_attach_tabs' ], 5, 2 );
  }

  public function _attach_tabs( $tab_list, $user_id ) {
    if( !wpep_get_setting_flag( 'user-profile-tab-default-tabs-enabled' ) )
      return $tab_list;

    if( !$this->_current_user_can_see_tabs( $user_id ) )
      return $tab_list;

    $progressing_course_list = ( $user_id == 0 ? [] : wpep_controller()->userActivity->statistics->get_progressing_course_list( $user_id ) );
    $completed_course_list   = ( $user_id == 0 ? [] : wpep_controller()->userActivity->statistics->get_completed_course_list( $user_id ) );

    if( !empty( $progressing_course_list ) || current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      $tab_list['progressing_course_list'] = $this->_get_progressing_courses_tab( $progressing_course_list, $user_id );

    if( !empty( $completed_course_list ) || current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      $tab_list['completed_course_list'] = $this->_get_completed_courses_tab( $completed_course_list, $user_id );

    if( wpep_course_has_bookmarks() ) {
      $bookmarked_course_list   = wpep_controller()->userActivity->statistics->get_bookmarked_course_list( $user_id, 1 );

      if( !empty( $bookmarked_course_list ) || current_user_can( WPEP_CAP_MANAGE_SETTINGS )  )
        $tab_list['bookmarked_course_list'] = $this->_get_bookmarked_courses_tab( $bookmarked_course_list, $user_id );
    }

    return $tab_list;
  }

  /**
   * @param $user_id
   * @return mixed
   */
  private function _current_user_can_see_tabs( $user_id ) {
    if( wpep_get_setting_flag( 'user-profile-course-public-available' ) )
      return true;

    return wpep_user_profile_current_user_can_view( $user_id );
  }

  private function _get_progressing_courses_tab( $progressing_course_list, $user_id ) {
    $response = [
      'title' => wpep_get_setting_translated( 'user-profile-tab-progressing-courses' )
    ];

    if( !empty( $progressing_course_list ) ) {
      $response['entries_count'] = count( $progressing_course_list );
      $response['content']       = wpep_content_primary_course_list_shortcode_by_ids( $progressing_course_list,
        [
          'course_progress_bars'  => (
          wpep_get_setting_flag( 'user-profile-course-public-progress-bars' )
            ? true
            : wpep_user_profile_current_user_can_view( $user_id )
          ),
          'courses_per_row'       => wpep_get_setting( 'user-profile-courses-per-row' ),
          'user_id'               => $user_id
        ]
      );
    } else if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) ) {
      $response['content'] = '<div class="wpep-notification wpep-notification-info">' . __( "Admin Notification : Students can see their progressing courses in this tab, if they do not have any progressing courses they don't see this tab.", "wpep" ) . '</div>';
    }

    return $response;
  }

  private function _get_completed_courses_tab( $completed_course_list, $user_id ) {
    $response = [
      'title' => wpep_get_setting_translated( 'user-profile-tab-completed-courses' )
    ];

    if( !empty( $completed_course_list ) ) {
      $response['entries_count'] = count( $completed_course_list );
      $response['content']       = wpep_content_primary_course_list_shortcode_by_ids( $completed_course_list, [ 'course_progress_bars' => 0, 'courses_per_row' => wpep_get_setting( 'user-profile-courses-per-row' ) ] );
    } else if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) ) {
      $response['content'] = '<div class="wpep-notification wpep-notification-info">' . __( "Admin Notification : Students can see their completed courses in this tab, if they do not have any completed courses they don't see this tab.", "wpep" ) . '</div>';
    }

    return $response;
  }

  private function _get_bookmarked_courses_tab( $bookmarked_course_list, $user_id ) {
    $response = [
      'title' => wpep_get_setting_translated( 'user-profile-tab-bookmarked-courses' )
    ];

    if( !empty( $bookmarked_course_list ) ) {
      $response['entries_count'] = count( $bookmarked_course_list );
      $response['content']       = wpep_content_primary_course_list_shortcode_by_ids( $bookmarked_course_list, [ 'courses_per_row' => wpep_get_setting( 'user-profile-courses-per-row' ) ] );
    } else if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) ) {
      $response['content'] = '<div class="wpep-notification wpep-notification-info">' . __( "Admin Notification : Students can see their bookmarked courses in this tab, if they do not have any bookmarked courses they don't see this tab.", "wpep" ) . '</div>';
    }

    return $response;
  }

}