<?php

namespace WPEP\ContentType\Skeleton\Integration;

use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

abstract class LearningJourneyFromActionTracking extends LearningJourney {

  abstract public function get_tracked_actions();

  /**
   * @param $post_id
   * @param $type
   * @return array [ link, link_title ]
   */
  abstract public function get_action_information( $post_id, $type );

  /**
   * The array format
   * @param int $post_id
   * @return array [ name, description, status, last_action_timestamp, link, link_title, design_attribute ]
   */
  public function get_step_map_for_current_user( $post_id ) {
    $required_actions  = $this->get_tracked_actions();
    $completed_actions = [];
    $last_action_timestamp = false;

    foreach( $required_actions as $required_action ) {
      $first_tracked_event = WPEP_Entity_ActionTracking::instance()->get_first_tracked_event( $post_id, get_current_user_id(), $required_action );

      if( $first_tracked_event === null )
        continue;

      $created_at_timestamp = strtotime( $first_tracked_event->created_at );

      if( $last_action_timestamp === false || $created_at_timestamp > $last_action_timestamp )
        $last_action_timestamp = $created_at_timestamp;

      $completed_actions[] = $required_action;
    }

    $status = ( count( $required_actions ) === count( $completed_actions ) ? WPEP_LEARNING_JOURNEY_STATUS_COMPLETED : ( $last_action_timestamp !== false ? WPEP_LEARNING_JOURNEY_STATUS_PROGRESSING : WPEP_LEARNING_JOURNEY_STATUS_PROGRESSING ) );

    $response = [
      'design_attribute'       => $this->_contentTypeInstance->get_post_type(),
      'name'                   => get_the_title( $post_id ),
      'status'                 => $status,
      'last_action_timestamp'  => $last_action_timestamp
    ];

    if( $status !== WPEP_LEARNING_JOURNEY_STATUS_COMPLETED ) {
      $pending_actions = array_diff( $required_actions, $completed_actions );

      $response += $this->get_action_information( $post_id, $pending_actions[ key( $pending_actions ) ] );
    }

    return $response;
  }

}