<?php

namespace WPEP\ContentType\Skeleton\Integration;

use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

abstract class StatisticsFromActionTracking extends Statistics {

  public $dashboard;

  public $single;

  public function __construct() {
    $this->dashboard            = new StatisticsFromActionTracking\Dashboard();
    $this->single               = new StatisticsFromActionTracking\Single();
  }

  abstract public function get_tracked_actions();

  abstract public function get_post_type();

  public function display( $filter_by = []  ) {
    $post_id = ( isset( $_GET['post_id'] ) ? intval( $_GET['post_id'] ) : 0 );

    if( $post_id != 0 ) {
      if( !wpep_user_has_statistics_access_for_post( $post_id ) ) {
        echo '<br/>';
        echo '<div class="wpep-notification wpep-notification-danger">' . __( "You do not have access to this page", "wpep" ) . '</div>';
        return;
      }

      $this->single->display( $this, $_GET['post_id'], $filter_by );
      return;
    }

    $this->dashboard->display( $this, $filter_by );
    return;
  }

  public function download( $filter_by = [] ) {
    $post_id = ( isset( $_GET['post_id'] ) ? intval( $_GET['post_id'] ) : 0 );

    if( $post_id != 0 ) {
      if( !wpep_user_has_statistics_access_for_post( $post_id ) ) {
        echo '<br/>';
        echo '<div class="wpep-notification wpep-notification-danger">' . __( "You do not have access to this page", "wpep" ) . '</div>';
        return;
      }

      $this->single->download( $this, $_GET['post_id'], $filter_by );
      return;
    }

    $this->dashboard->download( $this, $filter_by );
    return;
  }

  private function _get_table_data( $entries_per_page, $current_page, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => [],
      'page_current'  => null,
      'page_total'    => null
    ];

    $courses_offset = ( $current_page - 1 ) * $entries_per_page;

    $entries_query_information = WPEP_Controller::instance()->courseActivity->get_completed_history(
      $entries_per_page,
      $courses_offset,
      true,
      $filter_by
    );

    if( empty( $entries_query_information['entries'] ) )
      return $table_information;

    $table_information['head'] = [
      'name'                  => __( "Course Name", 'wpep' ),
      'course_identification' => __( "Course Identification", "wpep" ),
      'course_lessons'        => [
        'type'  => 'digits',
        'value' => __( "Course Lessons", 'wpep' )
      ],
      'completion_rate'   => [
        'type'  => 'digits',
        'value' => __( "Completion Rate", 'wpep' )
      ],
      'users_progressing' => [
        'type'  => 'digits',
        'value' => __( "Users progressing", 'wpep' )
      ],
      'users_completed'   => [
        'type'  => 'digits',
        'value' => __( "Users completed", 'wpep' )
      ],
      'view_submissions'        => [
        'type'  => 'link-item',
        'value' => ''
      ],
    ];

    if( !wpep_has_sid_support() )
      unset( $table_information['head']['course_identification'] );

    foreach( $entries_query_information['entries'] as $entry ) {
      if( isset( wpep_content_type_course_instance()->admin->section_course_type['course_management'] ) ) {
        if( !in_array(wpep_course_type_from_id( $entry['course_id'] ),
          explode( ' ', wpep_content_type_course_instance()->admin->section_course_type['course_management'] ) ) )
          $entry['course_lessons'] = '-';
      }

      $body = [
        'name'                    => [
          'type'      => 'link',
          'value'     => '<a href="' . wpep_admin_url_statistics_content_type( WPEP_POST_TYPE_COURSE, [ 'course_id' => $entry['course_id'] ] ) . '">' . $entry['course_title']    . '</a>',
          'value_raw' => $entry['course_title']
        ],
        'course_lessons'        => [
          'type'      => 'digits',
          'value'     => $entry['course_lessons']
        ],
        'completion_rate' => [
          'type'      => 'digits',
          'value'     => wpep_get_percentage( $entry['course_users_completed'], $entry['course_users_started'] )
        ],
        'users_progressing'      => [
          'type'      => 'digits',
          'value'     => ( $entry['course_users_started'] - $entry['course_users_completed'] )
        ],
        'users_completed'         => [
          'type'      => 'digits',
          'value'     => $entry['course_users_completed']
        ],
        'view_submissions'        => [
          'type'  => 'link-item',
          'value' => '<a class="button button-primary" style="width: 100px;" href="' . wpep_admin_url_statistics_content_type( WPEP_POST_TYPE_COURSE, [ 'course_id' => $entry['course_id'] ] ) . '">' . __( "View Report", "wpep" )    . '</a>'
        ]
      ];

      if( wpep_has_sid_support() )
        $body['course_identification'] = get_post_meta( $entry['course_id'], 'wpep_course_identifier', true );

      $table_information['body'][] = $body;
    }

    $table_information['page_current'] = $current_page;
    $table_information['page_total']   = ceil( $entries_query_information['entries_total'] / $entries_per_page );

    return $table_information;
  }

}