<?php

namespace WPEP\ContentType\Skeleton\Integration\StatisticsFromActionTracking;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;
use WPEP\ContentType\Skeleton\Integration\StatisticsFromActionTracking as WPEP_StatisticsFromTracking;

class Dashboard {

  /**
   * @param WPEP_StatisticsFromTracking $statisticsIntegration
   * @param array $filter_by
   */
  public function display( $statisticsIntegration, $filter_by = []  ) {
    $entries_per_page = 30;
    $current_page     = max( 1, ( isset( $_GET['pagination'] ) ? intval( $_GET['pagination'] ) : 1 ) );

    $table_information = $this->_get_table_data( $statisticsIntegration, $entries_per_page, $current_page, $filter_by );

    wpep_admin_display_table(
      sprintf( __( "%s List", "wpep" ), $statisticsIntegration->get_name() ),
      $table_information['head'],
      $table_information['body'],
      true,
      $table_information['page_current'],
      $table_information['page_total']
    );
  }

  /**
   * @param WPEP_StatisticsFromTracking $statisticsIntegration
   * @param array $filter_by
   */
  public function download( $statisticsIntegration, $filter_by = [] ) {
    $table_information = $this->_get_table_data( $statisticsIntegration, 1000000, 1, $filter_by );

    do_action(
      'wpep_deliver_csv_export_file',
      $statisticsIntegration->get_alias() . '-list-' . wpep_format_date(),
      $table_information['head'],
      $table_information['body']
    );
  }

  /**
   * @param WPEP_StatisticsFromTracking $statisticsIntegration
   * @param $entries_per_page
   * @param $current_page
   * @param array $filter_by
   * @return array
   */
  private function _get_table_data( $statisticsIntegration, $entries_per_page, $current_page, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => [],
      'page_current'  => null,
      'page_total'    => null
    ];

    $post_wp_query = new \WP_Query( [
      'post_type'   => $statisticsIntegration->get_post_type(),
      'numberposts' => -1,
      'offset'      => ( $current_page - 1 ) * $entries_per_page,
      'post_status' => wpep_post_monitored_post_status_list()
    ] );

    $post_list = $post_wp_query->get_posts();

    if( empty( $post_list ) )
      return $table_information;

    $table_information['head'] = [
      'name' => __( "Name", 'wpep' )
    ];

    foreach( $statisticsIntegration->get_tracked_actions() as $event_type ) {
      $event_name = WPEP_Controller::instance()->actionTracking->get_event_name( $event_type );

      $table_information[ 'head' ][ 'event_' . $event_type . '_count_lifetime' ]  = sprintf( __( "Total %s", "wpep" ), $event_name );
      $table_information[ 'head' ][ 'event_' . $event_type . '_last_date' ]       = sprintf( __( "Latest %s", "wpep" ), $event_name );
      $table_information[ 'head' ][ 'event_' . $event_type . '_first_date' ]      = sprintf( __( "First %s", "wpep" ), $event_name );
    }

    $table_information['head']['view_post'] = [
      'type'  => 'link-item',
      'value' => ''
    ];

    $post_ids = wpep_utility_map_to_array_assoc( $post_list, 'ID', 'ID' );

    $action_tracking_statistics_map = [];

    foreach( $statisticsIntegration->get_tracked_actions() as $event_type ) {
      $action_tracking_statistics_map[$event_type] = WPEP_Entity_ActionTracking::instance()->get_tracked_events_statistics_grouped_by_post( [
        'post_id' => [ 'IN' => $post_ids ],
        'type'    => $event_type,
      ] );
    }

    foreach( $post_list as $post ) {
      $body = [
        'name'       => [
          'type'      => 'link',
          'value'     => '<a href="' . wpep_admin_url_statistics_content_type( $statisticsIntegration->get_alias(), [ 'post_id' => $post->ID ] ) . '">' . $post->post_title    . '</a>',
          'value_raw' => $post->post_title
        ],
        'view_post'  => [
          'type'  => 'link-item',
          'value' => '<a class="button button-primary" style="width: 100px;" href="' . wpep_admin_url_statistics_content_type( $statisticsIntegration->get_alias(), [ 'post_id' => $post->ID ] ) . '">' . __( "View Report", "wpep" )    . '</a>'
        ]
      ];

      foreach( $statisticsIntegration->get_tracked_actions() as $event_type ) {
        $body[ 'event_' . $event_type . '_count_lifetime' ] = ( isset( $action_tracking_statistics_map[ $event_type ][ $post->ID ]->total_entries ) ? $action_tracking_statistics_map[ $event_type ][ $post->ID ]->total_entries : '-' );
        $body[ 'event_' . $event_type . '_first_date' ] = ( isset( $action_tracking_statistics_map[ $event_type ][ $post->ID ]->first_created_at ) ? wpep_format_date_with_time( $action_tracking_statistics_map[ $event_type ][ $post->ID ]->first_created_at ) : '-' );
        $body[ 'event_' . $event_type . '_last_date' ] = ( isset( $action_tracking_statistics_map[ $event_type ][ $post->ID ]->last_created_at ) ? wpep_format_date_with_time( $action_tracking_statistics_map[ $event_type ][ $post->ID ]->last_created_at ) : '-' );
      }

      $table_information['body'][] = $body;
    }

    $table_information['page_current'] = $current_page;
    $table_information['page_total']   = ceil( $post_wp_query->found_posts / $entries_per_page );

    return $table_information;
  }

}