<?php

namespace WPEP\ContentType\Skeleton\Integration\StatisticsFromActionTracking;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;
use WPEP\ContentType\Skeleton\Integration\StatisticsFromActionTracking as WPEP_StatisticsFromTracking;

class Single {

  /**
   * @param WPEP_StatisticsFromTracking $statisticsIntegration
   * @param $post_id
   * @param array $filter_by
   */
  public function display( $statisticsIntegration, $post_id, $filter_by = []  ) {
    $entries_per_page = 30;
    $current_page     = max( 1, ( isset( $_GET['pagination'] ) ? intval( $_GET['pagination'] ) : 1 ) );

    $bread_crumbs = [
      wpep_admin_url_statistics_content_type( $statisticsIntegration->get_alias() ) => sprintf( __( "%s List", "wpep" ), $statisticsIntegration->get_name() ),
      false => get_the_title( $post_id )
    ];

    echo wpep_breadcrumb( $bread_crumbs );

    $table_information = $this->_get_table_data( $statisticsIntegration, $post_id, $entries_per_page, $current_page, $filter_by );

    wpep_admin_display_table(
      sprintf( __( "%s Tracked Events", "wpep" ), get_the_title( $post_id ) ),
      $table_information['head'],
      $table_information['body'],
      true
    );
  }

  /**
   * @param WPEP_StatisticsFromTracking $statisticsIntegration
   * @param $post_id
   * @param array $filter_by
   */
  public function download( $statisticsIntegration, $post_id, $filter_by = [] ) {
    $table_information = $this->_get_table_data( $statisticsIntegration, $post_id, 1000000, 1, $filter_by );

    do_action(
      'wpep_deliver_csv_export_file',
      $statisticsIntegration->get_alias() . '-' . wpep_label_to_alias( get_the_title( $post_id ) ) . '-list-' . wpep_format_date(),
      $table_information['head'],
      $table_information['body']
    );
  }

  /**
   * @param WPEP_StatisticsFromTracking $statisticsIntegration
   * @param $post_id
   * @param $entries_per_page
   * @param $current_page
   * @param array $filter_by
   * @return array
   */
  private function _get_table_data( $statisticsIntegration, $post_id, $entries_per_page, $current_page, $filter_by = [] ) {
    $table_information = [
      'head'          => [],
      'body'          => [],
      'page_current'  => null,
      'page_total'    => null
    ];

    $tracked_events = WPEP_Entity_ActionTracking::instance()->get_tracked_events( [
      'post_id' => $post_id
    ], [
      'limit'       => $entries_per_page,
      'offset'      => ( $current_page - 1 ) * $entries_per_page,
      'count_total' => true
    ] );

    if( empty( $tracked_events['results'] ) )
      return $table_information;

    $table_information['head'] = [
      'name'          => __( "Name", 'wpep' ),
      'event_name'    => __( "Event", "wpep" ),
      'event_segment' => __( "Segment", "wpep" ),
      'created_at'    => __( "Date", "wpep" )
    ];


    foreach( $tracked_events['results'] as $tracked_event ) {
      $body = [
        'name'          => wpep_content_user_profile_format_name( $tracked_event->user_id ),
        'event_name'    => WPEP_Controller::instance()->actionTracking->get_event_name( $tracked_event->type ),
        'event_segment' => $tracked_event->segment,
        'created_at'    => wpep_format_date_with_time( $tracked_event->created_at )
      ];

      $table_information['body'][] = $body;
    }

    $table_information['page_current'] = $current_page;
    $table_information['page_total']   = ceil( $tracked_events['total'] / $entries_per_page );

    return $table_information;
  }

}