<?php

namespace WPEP\ContentType\Skeleton;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Discussion as WPEP_Entity_Discussion;

abstract class Skeleton {

  public $data_post_ignored_meta = [ '_edit_lock', '_edit_last', '_oembed_*' ];

  public function init() {}

  abstract public function get_post_type();
  abstract public function get_singular_name();
  abstract public function get_plural_name();

  public function get_label_all_items() {
    return __('All ' . $this->get_plural_name(), "wpep");
  }

  abstract public function get_cap_name();

  abstract public function register_post_type();

  public function get_statistics_instance() {
    return false;
  }

  /**
   * @return false|\WPEP\ContentType\Skeleton\Integration\LearningJourney
   */
  public function get_learning_journey_instance() {
    return false;
  }

  public function is_enabled() {
    return true;
  }

  /**
   * @param $args
   * @return array
   */
  public function data_user_activity_report( $args ) {
    global $wpdb;

    $sql = 'SELECT COUNT( * ) FROM `' . $wpdb->posts . '` 
                               WHERE post_type = "' . $this->get_post_type() . '"
                                 AND post_status IN ("' . implode( '","', wpep_post_monitored_post_status_list() ) . '")';

    if( isset( $args['time_from'] ) && isset( $args['time_to'] ) )
      $sql .= ' AND post_date >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '" AND post_date <= "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';

    return [
      sprintf( __( '%s Created', "wpep" ), $this->get_plural_name() ) => absint( $wpdb->get_var( $sql ) )
    ];
  }

  public function data_asset_map() {
    return [];
  }

  public function data_post_connection() {
    return [];
  }

  public function data_export_array() {
    $response = [];

    $post_list = get_posts( [ 'post_type' => $this->get_post_type(), 'numberposts' => -1, 'post_status' => wpep_post_monitored_post_status_list() ] );

    foreach( $post_list as $post )
      $response[ $post->ID ] = $this->data_export_single_post_array( $post );

    return $response;
  }

  public function data_import_array( $post_list, $id_mapping = [] ) {
    $response = [];

    foreach( $post_list as $post ) {
      if( !isset( $post['post_information'] ) )
        continue;

      if( isset( $post['post_id'] ) )
        $response[ $post['post_id'] ] = $this->data_import_single_post_array( $post, $id_mapping );
      else
        $this->data_import_single_post_array( $post, $id_mapping );
    }

    return $response;
  }

  public function data_import_array_post_connection( $post_list, $id_mapping = [] ) {
    $response = [];

    foreach( $post_list as $post ) {
      if( !isset( $post['post_connection'] ) )
        continue;

      $this->data_import_single_post_connection( $post, $id_mapping, $id_mapping[ $post['post_id'] ] );
    }

    return $response;
  }

  public function data_export_single_post_array( $post, $include_user_data = true ) {
    $post_meta_entries = get_post_meta( $post->ID );

    foreach( $this->data_post_ignored_meta as $meta_key_alias ) {
      if( wpep_string_ends_with( $meta_key_alias, '*' ) ) {
        $meta_key_alias = rtrim( $meta_key_alias, '*' );

        foreach( $post_meta_entries as $post_meta_entry_key => $post_meta_entry_value ) {
          if( strpos( $post_meta_entry_key, $meta_key_alias ) === 0 )
            unset( $post_meta_entries[ $post_meta_entry_key ] );
        }

        continue;
      }

      unset( $post_meta_entries[ $meta_key_alias ] );
    }

    foreach( wpep_content_type()->data_post_connection_array() as $post_connection ) {
      if( $post_connection['type'] != 'meta' )
        continue;

      unset( $post_meta_entries[ $post_connection[ 'key' ] ] );
    }

    $response = [
      'post_id'          => $post->ID,
      'post_information' => [
        'post_content'      => $post->post_content,
        'post_title'        => $post->post_title,
        'post_excerpt'      => $post->post_excerpt,
        'post_status'       => $post->post_status,
        'comment_status'    => $post->comment_status,
        'ping_status'       => $post->ping_status,
        'post_date'         => $post->post_date,
        'post_date_gmt'     => $post->post_date_gmt,
        'post_modified'     => $post->post_modified,
        'post_modified_gmt' => $post->post_modified_gmt,
        'post_type'         => $post->post_type,
        'post_mime_type'    => $post->post_mime_type
      ],
      'meta'             => $post_meta_entries,
      'taxonomies'       => []
    ];

    foreach( [ WPEP_TAXONOMY_CATEGORY ] as $taxonomy )
      $response['taxonomies'][$taxonomy] = wp_get_post_terms( $post->ID, $taxonomy, [ 'fields' => 'ids' ] );

    $post_connections = $this->data_post_connection();

    if( !empty( $post_connections ) ) {
      global $wpdb;

      $response[ 'post_connection' ] = [];

      foreach( $post_connections as $post_connection_key => $post_connection ) {
        if( $post_connection['type'] !== 'meta' )
          continue;

        if( $post_connection['location'] === 'external' ) {
          $response[ 'post_connection' ][ $post_connection_key ] = $wpdb->get_col(
            'SELECT post_id FROM `wp_postmeta` WHERE `meta_key` = "' . $post_connection['key'] . '" AND meta_value = ' . $post->ID
          );
        } else {
          $response[ 'post_connection' ][ $post_connection_key ] = get_post_meta( $post->ID, $post_connection['key'], false );
        }
      }
    }

    if( $include_user_data ) {
      $post_discussions = WPEP_Entity_Discussion::instance()->get_discussions( [ 'post_id' => $post->ID ] );

      if( !empty( $post_discussions ) )
        $response['discussions'] = $this->_data_export_single_post_attach_discussions( $post_discussions );
    }

    if( method_exists( $this, 'data_export_single_post_array_after' ) )
      $this->data_export_single_post_array_after( $response, $post, $include_user_data );

    return $response;
  }

  /**
   * @param $post_discussions
   * @return array
   */
  private function _data_export_single_post_attach_discussions( $post_discussions ) {
    $response = [];

    $user_list = wpep_controller()->dataSystem->get_users();

    foreach( $post_discussions as $post_discussion ) {
      $identifier_args = WPEP_Controller::instance()->discussion->get( $post_discussion->namespace )->get_discussion_identifier_args_from_string( $post_discussion->identifier );

      if( $identifier_args === false )
        continue;

      $response[ $post_discussion->identifier ] = [
        'identifier_args' => $identifier_args,
        'namespace'       => $post_discussion->namespace,
        'created_at'      => $post_discussion->created_at,
        'last_activity'   => $post_discussion->last_activity,
        'comments'        => [],
        'user_data'       => []
      ];

      $discussion_comments = WPEP_Entity_Discussion::instance()->get_discussion_comments( [
        'discussion_id' => $post_discussion->id,
      ], [
        'do_not_order_by_pinned' => true,
        'order_by'               => 'parent_id',
        'order'                  => 'ASC'
      ] );

      if( empty( $discussion_comments ) )
        continue;

      $discussion_user_data_list = WPEP_Entity_Discussion::instance()->get_discussion_user_data( false, $post_discussion->id, false, false );

      foreach( $discussion_comments as $discussion_comment ) {
        $response[ $post_discussion->identifier ][ 'comments' ][ $discussion_comment->id ] = [
          'id'        => $discussion_comment->id,
          'parent_id' => $discussion_comment->parent_id,
          'user_id'   => $discussion_comment->user_id,
          'user_email_address'  => $user_list[ $discussion_comment->user_id ]['email_address'],
          'content'       => $discussion_comment->content,
          'created_at'    => $discussion_comment->created_at,
          'last_activity' => $discussion_comment->last_activity,
          'is_pinned'     => $discussion_comment->is_pinned,
          'type'          => $discussion_comment->type
        ];
      }

      foreach( $discussion_user_data_list as $discussion_user_data ) {
        $response[ $post_discussion->identifier ][ 'user_data' ][] = [
          'user_id'            => $discussion_user_data->user_id,
          'user_email_address' => $user_list[ $discussion_user_data->user_id ]['email_address'],
          'key'                => $discussion_user_data->key,
          'value'              => $discussion_user_data->value
        ];
      }

    }

    return $response;
  }

  /**
   * The Discussions will not be imported automatically.
   * Every content type that integrates discussions should integrate data_import_single_post_discussion
   *
   * @param $post
   * @param array $id_mapping
   * @return int|\WP_Error
   */
  public function data_import_single_post_array( $post, $id_mapping = [] ) {
    $post['post_information']['post_type'] = $this->get_post_type();

    $new_post_id = wp_insert_post( $post['post_information'], true );

    if( $new_post_id == 0 )
      return 0;

    if( isset( $post['meta'] ) && !empty( $post['meta'] ) ) {
      foreach( $post['meta'] as $meta_key => $meta_values ) {
        if( !is_array( $meta_values ) )
          $meta_values = [ $meta_values ];

        foreach( $meta_values as $meta_value )
          add_post_meta( $new_post_id, $meta_key, $meta_value );
      }
    }

    if( isset( $post['taxonomies'] ) && !empty( $post['taxonomies'] ) ) {
      foreach( $post['taxonomies'] as $taxonomy => $taxonomy_information ) {
        if( !isset( $id_mapping[ $taxonomy ] ) )
          continue;

        $term_ids = [];

        foreach( $taxonomy_information as $taxonomy_item ) {
          $taxonomy_term_id = ( is_array( $taxonomy_item ) ? $taxonomy_item['id'] : $taxonomy_item );

          if( !isset( $id_mapping[ $taxonomy ][$taxonomy_term_id] ) )
            continue;

          $term_ids[] = $id_mapping[ $taxonomy ][$taxonomy_term_id];
        }

        if( !empty( $term_ids ) )
          wp_set_post_terms( $new_post_id, $term_ids, $taxonomy );
      }
    }

    return $new_post_id;
  }

  public function data_import_single_post_connection( $post, $post_id_map, $new_post_id = false ) {
    $post_connection_map = $this->data_post_connection();

    if( empty( $post_connection_map ) || $new_post_id == false || !isset( $post['post_connection'] ) )
      return false;

    foreach( $post['post_connection'] as $post_connection_key => $post_connection_data ) {
      if( !isset( $post_connection_map[ $post_connection_key ] ) )
        continue;

      if( $post_connection_map[ $post_connection_key ]['type'] !== 'meta' )
        continue;

      foreach( $post_connection_data as $post_connection_column_value ) {
        if( isset( $post_id_map[ $post_connection_column_value ] )
            && $post_connection_map[ $post_connection_key ]['value_type'] == 'post_id' ) {
          $post_connection_column_value = $post_id_map[ $post_connection_column_value ];
        }

        if( $post_connection_map[ $post_connection_key ]['location'] === 'external' )
          add_post_meta( $post_connection_column_value, $post_connection_map[ $post_connection_key ]['key'], $new_post_id );
        else
          add_post_meta( $new_post_id, $post_connection_map[ $post_connection_key ]['key'], $post_connection_column_value );
      }
    }

    return true;
  }

  /**
   * Ensure that you've changed the identifier_args to match the identifier after import.
   * @param $post_id
   * @param $discussion_data
   * @return int|false
   */
  public function data_import_single_post_discussion( $post_id, $discussion_data ) {
    $student_list = wpep_controller()->dataSystem->get_user_ids_mapped_by_email();
    $discussion_namespace_instance = WPEP_Controller::instance()->discussion->get( $discussion_data['namespace' ] );

    if( $discussion_namespace_instance === false )
      return false;

    $discussion_id = WPEP_Entity_Discussion::instance()->discussion_id_by_identifier(
      $discussion_namespace_instance->get_discussion_identifier( $discussion_data['identifier_args'] ),
      $post_id,
      $discussion_data['namespace']
    );

    WPEP_Entity_Discussion::instance()->wpdb->update( WPEP_Entity_Discussion::instance()->table_discussion, [
      'created_at'    => $discussion_data['created_at'],
      'last_activity' => $discussion_data['last_activity']
    ], [
      'id' => $discussion_id
    ] );

    if( isset( $discussion_data['comments'] ) && !empty( $discussion_data['comments'] ) ) {
      $discussion_comment_old_new_id_map = [];

      foreach( $discussion_data['comments'] as $discussion_comment_id => $discussion_comment ) {
        if( !isset( $student_list[ $discussion_comment['user_email_address'] ] ) )
          continue;

        if( !isset( $discussion_comment['id'] ) )
          $discussion_comment['id'] = $discussion_comment_id;

        $discussion_comment['parent_id'] = absint( $discussion_comment['parent_id'] );

        if( $discussion_comment['parent_id'] !== 0
            && !isset( $discussion_comment_old_new_id_map[ $discussion_comment['parent_id'] ] ) )
          continue;

        $discussion_comment_old_new_id_map[ $discussion_comment['id'] ] = WPEP_Entity_Discussion::instance()->add_discussion_comment( [
          'parent_id'     => ( $discussion_comment['parent_id'] === 0 ? 0 : $discussion_comment_old_new_id_map[ $discussion_comment['parent_id'] ] ),
          'discussion_id' => $discussion_id,
          'user_id'       => $student_list[ $discussion_comment['user_email_address'] ],
          'content'       => $discussion_comment['content'],
          'created_at'    => $discussion_comment['created_at'],
          'last_activity' => $discussion_comment['last_activity'],
          'is_pinned'     => $discussion_comment['is_pinned'],
          'type'          => $discussion_comment['type']
        ], false );
      }
    }

    if( isset( $discussion_data['user_data'] ) && !empty( $discussion_data['user_data'] ) ) {
      foreach( $discussion_data['user_data'] as $discussion_user_data ) {
        if( !isset( $student_list[ $discussion_user_data['user_email_address'] ] ) )
          continue;

        WPEP_Entity_Discussion::instance()->add_discussion_user_data(
          $discussion_user_data['key'],
          $discussion_user_data['value'],
          $discussion_id,
          $student_list[ $discussion_user_data['user_email_address'] ]
        );
      }
    }

    return $discussion_id;
  }

}