<?php

namespace WPEP\ContentType\Skeleton;

abstract class Standard extends Skeleton {

  abstract public function get_post_supports();

  public function is_publicly_queryable() {
    return true;
  }

  public function has_duplicate() {
    return true;
  }

  public function standard_get_post_meta_fields() {
    return [];
  }

  public function standard_get_post_meta_fields_context() {
    return 'normal';
  }

  public function register_post_type() {
    $post_labels = [
      'name'               => _x( $this->get_singular_name(), 'Post Type General Name', "wpep"),
      'singular_name'      => _x( $this->get_singular_name(), 'Post Type Singular Name', "wpep"),
      'menu_name'          => $this->get_label_all_items(),
      'name_admin_bar'     => __( $this->get_singular_name(), "wpep"),
      'parent_item_colon'  => __('Parent ' . $this->get_singular_name() . ':', "wpep"),
      'add_new'            => __('New ' . $this->get_singular_name(), "wpep"),
      'add_new_item'       => __('Add New ' . $this->get_singular_name(), "wpep"),
      'new_item'           => __('New ' . $this->get_singular_name(), "wpep"),
      'edit_item'          => __('Edit ' . $this->get_singular_name(), "wpep"),
      'update_item'        => __('Update ' . $this->get_singular_name(), "wpep"),
      'view_item'          => __('View ' . $this->get_singular_name(), "wpep"),
      'all_items'          => $this->get_label_all_items(),
      'search_items'       => __('Search ' . $this->get_singular_name(), "wpep"),
      'not_found'          => __('No ' . $this->get_singular_name() . ' found', "wpep"),
      'not_found_in_trash' => __('No ' . $this->get_singular_name() . ' found in Trash', "wpep")
    ];

    $taxonomies = [ WPEP_TAXONOMY_CATEGORY ];

    if( wpep_get_setting_flag( 'register-default-categories' ) )
      $taxonomies[] = 'category';

    $post_supports = $this->get_post_supports();

    if( wpep_get_setting_flag( 'enable-post-author-field' ) )
      $post_supports[] = "author";

    $post_params = [
      'supports'            => $post_supports,
      'taxonomies'          => $taxonomies,
      'hierarchical'        => false,
      'public'              => true,
      'show_ui'             => true,
      'show_in_menu'        => WPEP_ADMIN_BASE_MENU_SLUG,
      'show_in_nav_menus'   => true,
      'show_in_admin_bar'   => true,
      'menu_position'       => 2,
      'menu_icon'           => '',
      'can_export'          => true,
      'exclude_from_search' => false,
      'publicly_queryable'  => $this->is_publicly_queryable(),
      'map_meta_cap'        => true,
      'has_archive'         => ( wpep_get_setting_flag( 'content-type-' . $this->get_post_type() . '-enable-archive' )
        ? wpep_get_setting( 'content-type-' . $this->get_post_type()  . '-rewrite-slug', $this->get_post_type() )
        : false
      ),
      'singular_name'   => $this->get_singular_name(),
      'plural_name'     => $this->get_plural_name(),
      'description'     => __( $this->get_singular_name() . ' information pages', "wpep"),
      'label'           => __( strtolower( $this->get_singular_name() ), "wpep" ),
      'labels'          => $post_labels,
      'capability_type' => $this->get_cap_name()
    ];

    $post_params['rewrite'] = [
      'slug'  => wpep_get_setting( 'content-type-' . $this->get_post_type()  . '-rewrite-slug', $this->get_post_type() )
    ];

    register_post_type( $this->get_post_type(), $post_params);
  }

  public function data_asset_map() {
    $response = [];

    if( in_array( 'editor', $this->get_post_supports() ) )
      $response['post_information'] = [ 'post_content' ];

    if( in_array( 'thumbnail', $this->get_post_supports() ) )
      $response['meta'] = [ '_thumbnail_id' ];


    return $response;
  }

}