<?php

namespace WPEP;

class Controller {

  const SHOP_PRODUCT_ID = 35;

  protected static $instance;

  /**
   * @var Coordinator\ActionTracking
   */
  public $actionTracking;

  /**
   * @var Coordinator\AddOns
   */
  public $addOns;

  /**
   * @var Coordinator\Ajax
   */
  public $ajax;

  /**
   * @var Coordinator\Assets
   */
  public $assets;

  /**
   * @var Coordinator\Authentication
   */
  public $authentication;

  /**
   * @var Coordinator\Capabilities
   */
  public $capabilities;

  /**
   * @var Coordinator\ContentLibrary
   */
  public $contentLibrary;

  /**
   * @var Coordinator\Hooks
   */
  public $hooks;

  /**
   * @var Coordinator\LicenseLibrary
   */
  public $licenseLibrary;

  /**
   * @var Coordinator\Template
   */
  public $template;

  /**
   * @var Coordinator\TemplateStyle
   */
  public $templateStyle;

  /**
   * @var Coordinator\DataSystem
   */
  public $dataSystem;

  /**
   * @var Coordinator\Discussion
   */
  public $discussion;

  /**
   * @var Coordinator\Email
   */
  public $email;

  /**
   * @var Coordinator\EmailMarketing
   */
  public $emailMarketing;

  /**
   * @var Coordinator\Migration
   */
  public $migration;

  /**
   * @var Coordinator\MigrationLibrary
   */
  public $migrationLibrary;

  /**
   * @var Coordinator\Notifications
   */
  public $notifications;

  /**
   * @var Coordinator\PlatformHealth
   */
  public $platformHealth;

  /**
   * @var Coordinator\Content
   */
  public $content;

  /**
   * @var Coordinator\Request
   */
  public $request;

  /**
   * @var Coordinator\UserActivity
   */
  public $userActivity;

  /**
   * @var Coordinator\AssessmentActivity
   */
  public $assessmentActivity;

  /**
   * @var Coordinator\CourseActivity
   */
  public $courseActivity;

  /**
   * @var Coordinator\UserEngagement
   */
  public $userEngagement;

  /**
   * @var Admin\Controller
   */
  public $adminController;

  public static function instance() {
    if (!isset(self::$instance))
      self::$instance = new self();

    return self::$instance;

  }

  public function __construct() {
    require_once(WPEP_BASE_PATH . "/app/_functions/general.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/utilities.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/administration.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/auth.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/assessments.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/course.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/discussion.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/post.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/primary.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/user.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/user-profile.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/toolkit.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/shortcode-builder.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/image-builder.php" );
    require_once(WPEP_BASE_PATH . "/app/_functions/learning-journey.php" );
  }

  public function setup() {
    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\Controller->setup() - start' );

    $this->register_coordinators();

    // Migration Process need to be early.
    $this->migration->setup();

    do_action( "wpep_migration_setup" );

    add_action( 'plugins_loaded', [ $this, '_setup' ] );

    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\Controller->setup() - end' );
  }

  public function _setup() {
    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\Controller->_setup() - start' );

    $this->setup_coordinators();

    do_action( 'wpep_register_addon', $this->addOns );

    do_action( 'wpep_init_addon_before', $this->addOns );

    $this->addOns->_init_add_ons();

    do_action( 'wpep_init_addon_after', $this->addOns );

    do_action( 'wpep_loaded' );

    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\Controller->_setup() - end' );
  }

  public function register_coordinators() {
    $this->migrationLibrary = new Coordinator\MigrationLibrary();
    $this->migration        = new Coordinator\Migration();
    $this->notifications  = new Coordinator\Notifications();
    $this->platformHealth = new Coordinator\PlatformHealth();

    $this->addOns    = new Coordinator\AddOns();

    $this->actionTracking = new Coordinator\ActionTracking();
    $this->capabilities   = new Coordinator\Capabilities();
    $this->contentLibrary = new Coordinator\ContentLibrary();
    $this->ajax           = new Coordinator\Ajax();
    $this->hooks          = new Coordinator\Hooks();
    $this->licenseLibrary = new Coordinator\LicenseLibrary();
    $this->template       = new Coordinator\Template();
    $this->templateStyle  = new Coordinator\TemplateStyle();
    $this->assets         = new Coordinator\Assets();
    $this->authentication = new Coordinator\Authentication();
    $this->content        = new Coordinator\Content();
    $this->request        = new Coordinator\Request();
    $this->email          = new Coordinator\Email();
    $this->emailMarketing = new Coordinator\EmailMarketing();
    $this->dataSystem     = new Coordinator\DataSystem();
    $this->discussion     = new Coordinator\Discussion();
    $this->assessmentActivity = new Coordinator\AssessmentActivity();
    $this->userActivity   = new Coordinator\UserActivity();
    $this->userEngagement = new Coordinator\UserEngagement();
    $this->courseActivity = new Coordinator\CourseActivity();

    $this->adminController   = new Admin\Controller();
  }

  public function setup_coordinators() {
    $this->notifications->setup();
    $this->actionTracking->setup();
    $this->ajax->setup();
    $this->authentication->setup();
    $this->contentLibrary->setup();
    $this->email->setup();
    $this->hooks->setup();
    $this->licenseLibrary->setup();
    $this->template->setup();
    $this->templateStyle->setup();
    $this->assets->setup();
    $this->content->setup();
    $this->userActivity->setup();
    $this->dataSystem->setup();
    $this->discussion->setup();

    $this->adminController->dispatch();
  }

}
