<?php
namespace WPEP\Coordinator;

use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

class ActionTracking {

  private $_event_type  = 'wpep-action-event';

  public function setup() {
    if( isset( $_GET[ $this->_event_type ] ) )
      add_action( 'wpep_loaded', [ $this, '_handle_action_event' ] );

    add_action( 'delete_post', [ $this, "_on_delete_post" ] );
    add_action( 'delete_user', [ $this, "_on_delete_user" ] );
  }

  public function get_event_name( $event_type ) {
    if( $event_type === WPEP_ACTION_TRACKING_TYPE_ACCESS_CONTENT )
      return __( "Access Content", "wpep" );

    if( $event_type === WPEP_ACTION_TRACKING_TYPE_OPEN_LINK )
      return __( "Open Link", "wpep" );

    if( $event_type === WPEP_ACTION_TRACKING_TYPE_DOWNLOAD )
      return __( "Download", "wpep" );

    return $event_type;
  }

  public function get_event_url( $event_type, $post_id ) {
    $url = get_site_url();
    $url = add_query_arg( $this->_event_type, $event_type, $url );
    $url = add_query_arg( 'post_id', $post_id, $url );

    return $url;
  }

  public function _handle_action_event() {
    if( $_GET[ $this->_event_type ] == WPEP_ACTION_TRACKING_EVENT_TYPE_OPEN_LINK && isset( $_GET[ 'post_id' ] ) ) {
      $post_id   = absint( $_GET[ 'post_id' ] );
      $post_type = get_post_type( $post_id );

      $link = '';

      if( $post_type === WPEP_POST_TYPE_E_BOOK )
        $link = get_post_meta( $post_id, '_ebook_link', true );
      else if( $post_type === WPEP_POST_TYPE_LIST_ITEM )
        $link = get_post_meta( $post_id, 'wpep_content_link', true );
      else if( $post_type === WPEP_POST_TYPE_OFFER )
        $link = get_post_meta( $post_id, 'offer_link', true );

      if( $link !== '' && get_current_user_id() != 0 )
        WPEP_Entity_ActionTracking::instance()->track_event(
          $post_id,
          get_current_user_id(),
          WPEP_ACTION_TRACKING_TYPE_OPEN_LINK,
          $link
        );

      wpep_redirect( $link );
      exit;
    }
  }

  public function _on_delete_post( $post_id ) {
    WPEP_Entity_ActionTracking::instance()->wpdb->delete( WPEP_Entity_ActionTracking::instance()->table_action_tracking, [
      'post_id' => $post_id
    ] );
  }

  public function _on_delete_user( $user_id ) {
    WPEP_Entity_ActionTracking::instance()->wpdb->delete( WPEP_Entity_ActionTracking::instance()->table_action_tracking, [
      'user_id' => $user_id
    ] );
  }

}