<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WP_Query;

class AssessmentActivity {

  public function get_assessment_status_count( $assessment_id = false, $user_id = false, $filter_by = [] ) {
    global $wpdb;

    $where = ' aud.key = "status" ';

    if( $assessment_id != false )
      $where .= ' AND aud.post_id = ' . intval( $assessment_id );

    if( $user_id != false )
      $where .= ' AND aud.user_id = ' . intval ( $user_id );
    
    $sql = 'SELECT aud.value as status,
                   aud.user_id as user_id
              FROM ' . $wpdb->prefix . 'wpep_assessment_user_data aud
             WHERE ' . $where;

    $response = [
      WPEP_USER_ASSESSMENT_STATUS_PENDING        => 0,
      WPEP_USER_ASSESSMENT_STATUS_PENDING_REVIEW => 0,
      WPEP_USER_ASSESSMENT_STATUS_COMPLETED      => 0,
      WPEP_USER_ASSESSMENT_STATUS_FAILED         => 0
    ];

    foreach( $wpdb->get_results( $sql ) as $data ) {
      if( !wpep_user_filter_requirement_met( $data->user_id, $filter_by ) )
        continue;

      $response[ $data->status ]++;
    }

    return $response;
  }

  public function get_assessment_average_score( $assessment_id, $filter_by = []  ) {
    global $wpdb;

    $where = ' aud.post_id = ' . $assessment_id . ' AND aud.key = "score" ';

    $sql = 'SELECT aud.value,
                   aud.user_id
              FROM ' . $wpdb->prefix . 'wpep_assessment_user_data aud
             WHERE ' . $where;

    $entries = $wpdb->get_results( $sql );

    $entries = wpep_utility_map_to_array_assoc( $entries, 'user_id', 'value' );

    $entries = wpep_user_filter_map_entries( $entries, $filter_by );

    if( empty( $entries ) )
      return 0;

    return array_sum( $entries ) / count( $entries );
  }

  /**
   * @param int $interaction_number
   * @param int $interaction_offset
   * @param bool $pagination_information
   * @param array $filter_by
   * @return array
   */
  public function get_submission_history( $interaction_number = 30, $interaction_offset = 0, $pagination_information = false, $filter_by = [] ) {
    $assessment_list_query = new WP_Query( [
      'post_type'       => WPEP_POST_TYPE_ASSESSMENT,
      'posts_per_page'  => $interaction_number,
      'offset'          => $interaction_offset,
      'orderby'         => 'title',
      'order'           => 'ASC',
      'author'          => wpep_user_statistics_query_author_id()
    ] );

    $assessment_list = $assessment_list_query->get_posts();

    $response = [];

    foreach( $assessment_list as $assessment ) {
      $assessment_submissions = $this->get_assessment_status_count( $assessment->ID, false, $filter_by );

      $response[ $assessment->ID ] = [
        'assessment_id'     => $assessment->ID,
        'assessment_title'  => $assessment->post_title,
        'average_score'     => $this->get_assessment_average_score( $assessment->ID, $filter_by ),
        'submissions_total' => array_sum( $assessment_submissions )
      ];

      foreach( $assessment_submissions as $assessment_status => $assessment_status_entries )
        $response[$assessment->ID][ 'submissions_' . $assessment_status ] = $assessment_status_entries;
    }

    if( $pagination_information == false )
      return $response;

    return [
      'entries_total'   => $assessment_list_query->found_posts,
      'entries_number'  => $interaction_number,
      'entries_offset'  => $interaction_offset,
      'entries'         => $response,
    ];
  }

}