<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Container\FilterProxy as FilterProxy;

class Authentication {

  use FilterProxy;

  /**
   * @var Authentication\Request
   */
  public $request;

  public function setup() {
    $this->request = new Authentication\Request();

    add_shortcode( 'wpep_profile_authentication', [ $this, 'get_form' ] );
    add_shortcode( 'wpep_user_is_logged_in', [ $this, 'user_is_logged_in' ] );
    add_shortcode( 'wpep_user_not_logged_in', [ $this, 'user_not_logged_in' ] );
    add_shortcode( 'wpep_user_name', [ $this, 'user_name' ] );
    add_shortcode( 'wpep_user_first_name', [ $this, 'user_first_name' ] );
  }

  public function get_form($attributes, $content = null) {
    if( get_current_user_id() != 0 && !isset( $attributes[ 'ignore_auth' ] ) )
      return WPEP_Controller::instance()->content->userProfile->get_content( $attributes );

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    $filter_proxy = [];

    if( isset( $attributes[ 'success_type' ] ) ) {
      $filter_proxy[ 'wpep_user_profile_authentication_form_success_response_type' ] = $attributes[ 'success_type' ];

      if( $attributes[ 'success_type' ] == "redirect" && isset( $attributes[ 'url' ] ) )
        $filter_proxy[ 'wpep_user_profile_authentication_form_success_response_redirect' ] = $attributes[ 'url' ];
    }

    $this->_filter_proxy_setup( $filter_proxy );

    $response = WPEP_Controller::instance()->template->get_template( 'authentication.php',
      [
        'include_header'   => false,
        'include_footer'   => false,
        'lead_segment'     => ( isset( $attributes[ 'lead_segment' ] ) ? $attributes[ 'lead_segment' ] : '' )
      ]
    );

    $this->_filter_proxy_remove();

    return $response;
  }

  public function user_is_logged_in( $attributes, $content = '' ) {
    if( !is_user_logged_in() )
      return '';

    return do_shortcode( $content );
  }

  public function user_not_logged_in( $attributes, $content = '' ) {
    if( is_user_logged_in() )
      return '';

    return do_shortcode( $content );
  }

  public function user_name( $attributes, $content = '' ) {
    if( !is_user_logged_in() )
      return $content;

    return wpep_content_user_profile_format_name( wp_get_current_user() );
  }

  public function user_first_name( $attributes, $content = '' ) {
    if( !is_user_logged_in() )
      return $content;

    $user = wp_get_current_user();

    return ( isset( $user->first_name ) ? $user->first_name : '' );
  }


}