<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;

class Content {

  /**
   * @var Content\Library
   */
  public $library;
  /**
   * @var Content\Miscellaneous
   */
  public $miscellaneous;
  /**
   * @var Content\UserProfile
   */
  public $userProfile;
  /**
   * @var Content\Search
   */
  public $search;

  public $wpep_wrapper_content_depth = 0;

  public function setup() {
    $this->library        = new Content\Library();
    $this->miscellaneous  = new Content\Miscellaneous();
    $this->userProfile    = new Content\UserProfile();
    $this->search         = new Content\Search();

    add_action( 'wpep_before_main_content', [ $this, 'before_main_content' ] );
    add_action( 'wpep_after_main_content', [ $this, 'after_main_content' ] );

    if( wpep_get_setting_flag( 'enable-post-member-magnet-option' ) ) {
      add_action( 'wpep_user_profile_authentication_before', [ $this, 'authentication_action_before'] );
      add_action( 'wpep_user_profile_authentication_after', [ $this, 'authentication_action_after'] );
      add_filter( 'wpep_user_profile_authentication_form_allowed_fields', [ $this, 'authentication_form_fields' ] );
      add_filter( 'wpep_user_profile_authentication_form_view', [ $this, 'authentication_form_view' ] );
    }
  }

  public function before_main_content() {
    $this->wpep_wrapper_content_depth++;

    if( $this->wpep_wrapper_content_depth > 1 )
      return;

    echo '<div class="wpep-wrapper' . ( wpep_is_rtl() ? ' wpep-is-rtl' : '' ) . '" style="' . WPEP_Controller::instance()->templateStyle->get_wpep_wrapper_style() . '"><div class="wpep-container" style="clear:both;">';
  }

  public function after_main_content() {
    $this->wpep_wrapper_content_depth--;

    if( $this->wpep_wrapper_content_depth >= 1 )
      return;

    echo '</div></div>';
  }

  public function authentication_action_before() {
    if( get_the_ID() == 0 )
      return;

    if( !wpep_post_has_member_magnet_option( get_the_ID() ) )
      return;

    echo wpautop( get_post_meta( get_the_ID(), 'wpep_member_magnet_option_text_before', true ) );
  }

  public function authentication_form_view( $form_view ) {
    if( get_the_ID() == 0 )
      return $form_view;

    if( !wpep_post_has_member_magnet_option( get_the_ID() ) )
      return $form_view;

    $post_form_view = get_post_meta( get_the_ID(), 'wpep_member_magnet_option_form_view', true );

    return ( $post_form_view !== '' ? $post_form_view : $form_view );
  }

  public function authentication_form_fields( $form_fields ) {
    if( get_the_ID() == 0 )
      return $form_fields;

    if( !wpep_post_has_member_magnet_option( get_the_ID() ) )
      return $form_fields;

    if( wpep_get_setting( 'post-member-magnet-type' ) === WPEP_MEMBER_MAGNET_TYPE_QUICK )
      $form_fields = array_diff( $form_fields, [ 'first_name', 'last_name' ] );

    return $form_fields;
  }

  public function authentication_action_after() {
    if( get_the_ID() == 0 )
      return;

    if( !wpep_post_has_member_magnet_option( get_the_ID() ) )
      return;

    echo wpautop( get_post_meta( get_the_ID(), 'wpep_member_magnet_option_text_after', true ) );
  }

}
