<?php

namespace WPEP\Coordinator\Content\Library;

class GridItem {

  private $_post_locked_by_prerequisite = [];

  public function __construct() {
    add_filter( 'wpep_primary_content_item_link_title', [ $this, 'wpep_post_text_early' ], 5, 2 );
    add_filter( 'wpep_primary_content_item_link', [ $this, 'wpep_post_link_early' ], 5, 2 );
    add_filter( 'wpep_grid_item_button_style_args', [ $this, 'wpep_grid_item_button_style_args'], 5, 2 );

    // Early Events
    add_filter( 'wpep_grid_item_class', [ $this, 'grid_item_class' ], 10, 3 );
    add_filter( 'wpep_grid_item_hover_bar_style', [ $this, 'grid_item_hover_bar_style' ], 5, 3 );

    // Mid Events
    add_filter( 'wpep_widget_item_class', [ $this, 'wpep_general_item_class' ], 50, 3 );
    add_filter( 'wpep_grid_item_class', [ $this, 'wpep_general_item_class' ], 50, 3 );

    // Late Events
    add_filter( 'wpep_primary_content_item_link_title', [ $this, 'wpep_post_text' ], 1000, 2 );
    add_filter( 'wpep_primary_content_item_link',  [ $this, 'wpep_post_link' ], 1000, 2 );
  }

  public function wpep_post_text_early( $default, $post_id ) {
    $text = trim( get_post_meta( $post_id, 'wpep_post_button_text', true ) );

    if( $text !== '' )
      return $text;

    return $default;
  }

  public function wpep_post_text( $default, $post_id ) {
    if( wpep_post_coming_soon( $post_id ) )
      return wpep_get_setting_translated( 'opt-course-coming-soon' );

    if( $this->_post_is_locked_by_pre_requisites( $post_id ) )
      return wpep_get_setting_translated( 'prerequisite-course-button-lock-text' );

    return $default;
  }

  public function wpep_grid_item_button_style_args( $button_style_args, $post_id ) {
    $text_color   = get_post_meta( $post_id, 'wpep_post_button_text_color', true );
    $button_color = get_post_meta( $post_id, 'wpep_post_button_color', true );

    if( $text_color !== '' )
      $button_style_args['color'] = $text_color . ' !important';

    if( $button_color !== '' )
      $button_style_args['background'] = $button_color . ' !important';

    return $button_style_args;
  }

  public function wpep_post_link_early( $default, $post_id ) {
    $link = get_post_meta( $post_id, 'wpep_post_alternative_link', true );

    if( is_string($link) && strpos( $link, 'http' ) === 0 )
      return $link;

    return $default;
  }

  public function wpep_post_link( $default, $post_id ) {
    if( wpep_post_coming_soon( $post_id ) )
      return '#';

    if( wpep_post_has_member_magnet_option( $post_id, get_current_user_id() ) ) {
      if( get_post_type( $post_id ) !== WPEP_POST_TYPE_COURSE
        || !in_array( wpep_course_type_from_id( $post_id ), wpep_course_type_member_magnet_integrated() ) )
        return wpep_post_member_magnet_link( $post_id );
    }

    if( $this->_post_is_locked_by_pre_requisites( $post_id ) )
      return '#';

    return $default;
  }

  public function wpep_general_item_class( $class, $post_id, $post_type = "none" ) {
    if( wpep_post_coming_soon( $post_id ) )
      return $class . ' wpep-coming-soon-item';

    if( $this->_post_is_locked_by_pre_requisites( $post_id, $post_type ) )
      return $class . ' wpep-locked-prerequisite-item';

    return $class;
  }

  private function _post_is_locked_by_pre_requisites( $post_id, $post_type = false ) {
    if( !wpep_get_setting_flag( 'prerequisite-course-button-lock' ) )
      return false;

    if( isset( $this->_post_locked_by_prerequisite[$post_id] ) )
      return $this->_post_locked_by_prerequisite[$post_id];

    if( $post_type === false )
      $post_type = get_post_type( $post_id );

    if( $post_type !== WPEP_POST_TYPE_COURSE ) {
      $this->_post_locked_by_prerequisite[$post_id] = false;

      return false;
    }

    $prerequisites      = apply_filters( "wpep_single_course_prerequisites_list", get_post_meta( $post_id, 'wpep_prerequisite' ) );
    $prerequisite_mode  = wpep_get_prerequisite_mode( get_post_meta( $post_id, WPEP_PREREQUISITE_MODE_POST_META, true ) );

    if( empty( $prerequisites )
      || $prerequisite_mode[WPEP_PREREQUISITE_FIELD_ID] === WPEP_PREREQUISITE_MODE_NOTIFY ) {
      $this->_post_locked_by_prerequisite[$post_id] = false;

      return false;
    }

    $completed_prerequisites = [];

    foreach( $prerequisites as $prerequisite ) {
      if( !wpep_user_course_is_completed( $prerequisite, get_current_user_id(), true ) )
        continue;

      $completed_prerequisites[] = $prerequisite;
    }

    $requirement = ( $prerequisite_mode[WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT] === 'all' ? count( $prerequisites ) : $prerequisite_mode[WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT] );

    $this->_post_locked_by_prerequisite[$post_id] = ( $requirement != 0 && $requirement > count( $completed_prerequisites )  );

    return $this->_post_locked_by_prerequisite[$post_id];
  }

  public function course_grid_has_progress_bar() {
    return apply_filters( 'wpep_primary_content_grid_courses_progress_bars', wpep_get_setting_flag( 'index-page-progress-bars' ) );
  }

  public function grid_item_class( $class, $post_id ) {
    $class .= ' wpep-grid-item';

    $included_id = [];

    $post_categories = wp_get_post_terms($post_id, WPEP_TAXONOMY_CATEGORY, [ "fields" => "all" ] );

    foreach( $post_categories as $post_category ) {
      if( !in_array( $post_category->term_id, $included_id ) ) {
        $class .= ' course-grid-item-category-' . $post_category->term_id;

        $included_id[] = $post_category->term_id;
      }

      if( $post_category->parent != 0 && !in_array( $post_category->parent, $included_id ) ) {
        $class .= ' course-grid-item-category-' . $post_category->parent;

        $included_id[] = $post_category->parent;
      }
    }

    return $class;
  }

  public function grid_item_hover_bar_style( $style, $post_id, $post_type ) {
    if( $post_type != WPEP_POST_TYPE_COURSE )
      return $style;

    $identity_color = get_post_meta( $post_id, 'identity_color', true );

    if( $identity_color == '' || $identity_color == '#' )
      return $style;

    return 'background : ' . $identity_color . ';';
  }

}