<?php

namespace WPEP\Coordinator\Content;

use WPEP\Admin\Coordinator\Statistics\Courses\User;
use WPEP\Controller as WPEP_Controller;

class UserProfile {

  /**
   * @var UserProfile\Delivery
   */
  public $delivery;

  /**
   * @var UserProfile\Privacy
   */
  public $privacy;

  /**
   * @var UserProfile\PublicProfile
   */
  public $publicProfile;

  /**
   * @var UserProfile\Request
   */
  public $request;

  /**
   * @var \WP_User
   */
  protected $_current_display_user = null;

  /**
   * UserProfile constructor.
   */
  public function __construct() {
    $this->delivery = new UserProfile\Delivery();
    $this->privacy  = new UserProfile\Privacy();
    $this->request  = new UserProfile\Request();
    $this->publicProfile = new UserProfile\PublicProfile();
  }

  public function get_content($attributes, $content = null) {
    $user_information = WPEP_Controller::instance()->content->userProfile->get_display_user();

    if( $user_information == false )
      return WPEP_Controller::instance()->authentication->get_form( $attributes );

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    return WPEP_Controller::instance()->template->get_template(
      'user-profile.php', [
        'user_information' => $user_information,
        'include_header'   => false,
        'include_footer'   => false
      ]
    );
  }

  public function set_display_user( $user ) {
    $this->_current_display_user = $user;
  }

  /**
   * @return \WP_User|false
   */
  public function get_display_user() {
    if( $this->_current_display_user !== null )
      return $this->_current_display_user;

    if( is_user_logged_in() ) {
      $this->set_display_user( wp_get_current_user() );
    } else {
      $this->_current_display_user = false;
    }

    return $this->_current_display_user;
  }

}
