<?php

namespace WPEP\Coordinator\Content\UserProfile;

class Privacy {

  public function __construct() {
    add_filter( 'wp_privacy_personal_data_exporters', [ $this, 'register_exporters' ], 5 );
  }

  /**
   * Integrate this exporter implementation within the WordPress core exporters.
   *
   * @param array $exporters List of exporter callbacks.
   * @return array
   */
  public function register_exporters( $exporters = [] ) {
    $exporters[ 'wpep-student' ] = [
      'exporter_friendly_name' => wpep_get_setting_translated( 'user-account-export-student-information' ),
      'callback'               => [ $this, '_export_student_information' ],
    ];

    return $exporters;
  }

  public function _export_student_information( $email_address, $page ) {
    $user = get_user_by( 'email', $email_address );

    if ( ! $user ) {
      return [
        'data' => [],
        'done' => true,
      ];
    }

    $data = [];

    if( wpep_has_sid_support() )
      $data[] = [
        'name'  => wpep_get_setting_translated( 'user-auth-field-sid' ),
        'value' => get_user_meta( $user->ID, WPEP_USER_META_STUDENT_IDENTIFICATION, true )
      ];

    $location = get_user_meta( $user->ID, WPEP_USER_LOCATION_META, true );

    if( $location !== '' )
      $data[] = [
        'name'  => wpep_get_setting_translated( 'user-auth-field-location' ),
        'value' => $location
      ];

    if( $user->user_url !== '' )
      $data[] = [
        'name'  => wpep_get_setting_translated( 'user-auth-field-website' ),
        'value' => $user->user_url
      ];

    if( $user->description !== '' )
      $data[] = [
        'name'  => wpep_get_setting_translated( 'user-auth-field-about-me' ),
        'value' => sanitize_textarea_field( $user->description )
      ];

    $user_social_networks = get_user_meta( $user->ID, WPEP_USER_META_SOCIAL_NETWORK_PROFILES, true );
    $user_social_networks = ( is_array( $user_social_networks ) ? $user_social_networks : [] );

    foreach( wpep_user_profile_social_network_platform_list_active() as $social_network_key => $social_network_information ) {
      $current_value = ( isset( $user_social_networks[ $social_network_key ] ) ? $user_social_networks[ $social_network_key ] : '' );
      if( $current_value === '' ) continue;

      $data[] = [
        'name'  => $social_network_information['name'],
        'value' => $current_value
      ];
    }

    $data_to_export = [];

    $data_to_export[] = [
      'group_id'    => 'student',
      'group_label' => wpep_get_setting_translated( 'user-account-export-student-information' ),
      'item_id'     => "student-{$user->ID}",
      'data'        => $data,
    ];

    return [
      'data' => $data_to_export,
      'done' => true,
    ];
  }

}