<?php

namespace WPEP\Coordinator\Content\UserProfile;

class Request {

  public function __construct() {
    add_action('wp_ajax_wpep_update_user_profile', [ $this, 'update_user_profile' ] );
    add_action('wp_ajax_wpep_update_user_profile_cover_picture', [ $this, 'update_user_profile_cover_picture' ] );
    add_action('wp_ajax_wpep_update_user_profile_picture', [ $this, 'update_user_profile_picture' ] );
  }

  public function update_user_profile() {
    if( !isset( $_POST['information'] ) || !isset( $_POST['information'] ) )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    $information = $_POST['information'];

    $user_id = ( isset( $information['wpep_user_profile_id'] ) ? intval( $information['wpep_user_profile_id'] ) : get_current_user_id() );

    if( $user_id != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    $user = get_user_by( 'ID', $user_id );

    if( $user === false )
      exit ( json_encode( [
        'status'  => 'error'
      ] ) );

    $wp_user_update = [];

    if( isset( $information['wpep_user_url'] ) && $information['wpep_user_url'] != $user->user_url ) {
      $user->user_url = sanitize_text_field( $information['wpep_user_url'] );

      $wp_user_update['user_url'] = $user->user_url;
    }

    if( isset( $information['wpep_user_first_name'] ) && $information['wpep_user_first_name'] != $user->first_name ) {
      $user->first_name = sanitize_text_field( $information['wpep_user_first_name'] );

      $wp_user_update['first_name'] = $user->first_name;
    }

    if( isset( $information['wpep_user_last_name'] ) && $information['wpep_user_last_name'] != $user->last_name ) {
      $user->last_name = sanitize_text_field( $information['wpep_user_last_name'] );

      $wp_user_update['last_name'] = $user->last_name;
    }

    if( isset( $information['wpep_user_description'] ) && $information['wpep_user_description'] != $user->description ) {
      $user->description = wpep_utilities_sanitize_input( $information['wpep_user_description'] );

      $wp_user_update['description'] = $user->description;
    }

    if( isset( $information['wpep_user_student_identification'] ) && $information['wpep_user_student_identification'] != get_user_meta( $user_id, WPEP_USER_META_STUDENT_IDENTIFICATION, true ) ) {
      update_user_meta( $user_id, WPEP_USER_META_STUDENT_IDENTIFICATION, sanitize_text_field( $information['wpep_user_student_identification'] ) );
    }

    if( isset( $information['wpep_user_location'] ) && $information['wpep_user_location'] != get_user_meta( $user_id, WPEP_USER_LOCATION_META, true ) ) {
      update_user_meta( $user_id, WPEP_USER_LOCATION_META, sanitize_text_field( $information['wpep_user_location'] ) );
    }

    if( isset( $information['wpep_user_privacy_type'] ) && wpep_public_profiles_enabled() ) {
      if( in_array( $information['wpep_user_privacy_type'], [ WPEP_USER_PROFILE_PRIVACY_TYPE_PUBLIC, WPEP_USER_PROFILE_PRIVACY_TYPE_PRIVATE ] )
          && $information['wpep_user_privacy_type'] != get_user_meta( $user_id, WPEP_USER_META_PROFILE_PRIVACY_TYPE, true ) )
        update_user_meta( $user_id, WPEP_USER_META_PROFILE_PRIVACY_TYPE, sanitize_text_field( $information['wpep_user_privacy_type'] ) );
    }

    if( isset( $information['wpep_student_supervisor'] ) && wpep_get_setting_flag( 'enable-student-supervisor' ) ) {
      if( $information['wpep_student_supervisor'] != get_user_meta( $user_id, WPEP_USER_META_SUPERVISOR_USER_ID, true ) )
        update_user_meta( $user_id, WPEP_USER_META_SUPERVISOR_USER_ID, intval( $information['wpep_student_supervisor'] ) );
    }

    if( isset( $information['wpep_user_social_network'] ) ) {
      $current_social_networks = get_user_meta( $user_id, WPEP_USER_META_SOCIAL_NETWORK_PROFILES, true );

      if( !is_array( $current_social_networks ) )
        $current_social_networks = [];

      foreach( wpep_user_profile_social_network_platform_list_active() as $social_network_key => $social_network_information ) {
        if( !isset( $information['wpep_user_social_network'][$social_network_key] ) )
          continue;

        $current_val = trim( $information['wpep_user_social_network'][$social_network_key] );

        if( $current_val === '' ) {
          unset( $current_social_networks[$social_network_key] );
          continue;
        }

        $current_social_networks[$social_network_key] = (
        isset( $social_network_information['type'] ) && $social_network_information['type'] == 'link'
          ? esc_url( $current_val )
          : sanitize_text_field( $current_val )
        );
      }

      update_user_meta( $user_id, WPEP_USER_META_SOCIAL_NETWORK_PROFILES, $current_social_networks );
    }

    if( isset( $information['wpep_user_weekly_recap'] ) && $information['wpep_user_weekly_recap'] != get_user_meta( $user_id, 'wpep_disable_email_weekly_recap', true ) ) {
      update_user_meta( $user_id, 'wpep_disable_email_weekly_recap', intval( $information['wpep_user_weekly_recap'] ) );
    }

    do_action( 'wpep_user_profile_information_save', $information, $user );

    if( !empty( $wp_user_update ) )
      wp_update_user( [ 'ID' => $user_id ] + $wp_user_update );

    exit ( json_encode( [
      'status'        => 'ok',
      'display_name'  => wpep_content_user_profile_format_name( $user ),
      'description'   => wpautop( make_clickable( $user->description ) )
    ] ) );
  }

  public function update_user_profile_cover_picture() {
    if( !isset( $_POST['information'] ) || !isset( $_POST['information'] ) )
      exit ( json_encode( [
        'status'  => 'error',
        'error'   => __( "Missing request information", "wpep" )
      ] ) );

    $information = $_POST['information'];

    $user_id = ( isset( $information['wpep_user_id'] ) ? intval( $information['wpep_user_id'] ) : 0 );

    if( $user_id == false )
      $user_id = ( isset( $information['user_id'] ) ? intval( $information['user_id'] ) : get_current_user_id() );

    if( $user_id != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      exit ( json_encode( [
        'status'  => 'error',
        'error'   => __( "You don't have the permission to do this.", "wpep" )
      ] ) );

    $user = get_user_by( 'ID', $user_id );

    if( $user === false )
      exit ( json_encode( [
        'status'  => 'error',
        'error'   => __( "Invalid User ID Provided", "wpep" )
      ] ) );

    $information['picture'] = $this->_within_request_process_image_file( $information['picture'], 'cover-picture-' . $user_id );

    update_user_meta( $user_id, WPEP_USER_META_COVER_PICTURE, $information['picture'] );

    exit ( json_encode( [
      'status'        => 'ok',
      'picture'       => $information['picture']
    ] ) );
  }

  public function update_user_profile_picture() {
    if( !isset( $_POST['information'] ) || !isset( $_POST['information'] ) )
      exit ( json_encode( [
        'status'  => 'error',
        'error'   => __( "Missing request information", "wpep" )
      ] ) );

    $information = $_POST['information'];

    $user_id = ( isset( $information['wpep_user_id'] ) ? intval( $information['wpep_user_id'] ) : 0 );

    if( $user_id == false )
      $user_id = ( isset( $information['user_id'] ) ? intval( $information['user_id'] ) : get_current_user_id() );

    if( $user_id != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      exit ( json_encode( [
        'status'  => 'error',
        'error'   => __( "You don't have the permission to do this.", "wpep" )
      ] ) );

    $user = get_user_by( 'ID', $user_id );

    if( $user === false )
      exit ( json_encode( [
        'status'  => 'error',
        'error'   => __( "Invalid User ID Provided", "wpep" )
      ] ) );

    $information['picture'] = $this->_within_request_process_image_file( $information['picture'], 'profile-picture-' . $user_id );

    update_user_meta( $user_id, WPEP_USER_META_PROFILE_PICTURE, $information['picture'] );

    exit ( json_encode( [
      'status'        => 'ok',
      'picture'       => $information['picture']
    ] ) );
  }

  private function _within_request_process_image_file( $picture, $picture_reference ) {
    if ( !preg_match('/^data:image\/(\w+);base64,/', $picture, $type) )
      return $picture;

    $image_data = substr( $picture, strpos( $picture, ',' ) + 1 );
    $type = strtolower( $type[1] ); // jpg, png, gif

    if ( !in_array($type, [ 'jpg', 'jpeg', 'gif', 'png' ])) {
      exit ( json_encode( [
        'status'  => 'error',
        'error'   => sprintf( __( "Allowed image files are : %s", "wpep" ), [ 'jpg', 'jpeg', 'gif', 'png' ] )
      ] ) );
    }

    $image_data = base64_decode($image_data);

    if ($image_data === false)
      exit ( json_encode( [
        'status'  => 'error',
        'error'   => __( "Image File is corrupted", "wpep" )
      ] ) );

    $destination_directory = wpep_get_upload_dir() . '/' . 'user-profile-uploads';
    $destination_url       = wpep_get_upload_dir( "baseurl" ) . '/' . 'user-profile-uploads';
    $file_name             = $picture_reference . '-' . time() . '.' . $type;

    if( !is_dir( $destination_directory ) )
      mkdir($destination_directory, 0755, true);

    file_put_contents($destination_directory . '/' . $file_name, $image_data );

    $attachment = [
      'post_mime_type' => @mime_content_type( $destination_directory . '/' . $file_name ),
      'guid'           => $destination_url . '/' . $file_name,
      'post_parent'    => 0,
      'post_title'     => $picture_reference,
    ];

    $new_attachment_id = wp_insert_attachment( $attachment, $file_name, 0 );
    update_post_meta( $new_attachment_id, '_wp_attached_file', $destination_directory . '/' . $file_name );
    update_post_meta( $new_attachment_id, 'wpep_uploaded_file_user_id', get_current_user_id() );

    if( !function_exists( 'wp_generate_attachment_metadata' ) )
      require ( ABSPATH . 'wp-admin/includes/image.php' );

    $attach_data = wp_generate_attachment_metadata( $new_attachment_id, get_attached_file( $new_attachment_id ) );

    wp_update_attachment_metadata( $new_attachment_id,  $attach_data );

    $attachment_image_src = wp_get_attachment_image_src( $new_attachment_id, 'full' );

    return isset( $attachment_image_src[ 0 ] ) ? $attachment_image_src[ 0 ] : false;
  }

}