<?php

namespace WPEP\Coordinator;

class ContentLibrary {

  /**
   * @var \WPEP_Content_Library_Integration[]
   */
  protected $_integrations = [];

  public function setup() {
    add_action( 'set_current_user', [ $this, 'clear_access_storage' ] );
  }

  /**
   * @param \WPEP_Content_Library_Integration $content_library_integration
   */
  public function register( $content_library_integration ) {
    $this->_integrations[ $content_library_integration->prefix ] = $content_library_integration;
  }

  public function is_integrated() {
    return !empty( $this->_integrations );
  }

  /**
   * @return \WPEP_Content_Library_Integration[]
   */
  public function get_integrations(){
    return $this->_integrations;
  }

  /**
   * @param $post_id
   * @return bool
   */
  public function has_access_to_post( $post_id ) {
    $has_access = true;

    foreach( $this->_integrations as $integration ) {
      if( $integration->has_access( $post_id ) )
        continue;

      $has_access = false;
      break;
    }

    return $has_access;
  }

  /**
   * @param $post_id
   * @return bool
   */
  public function get_default_sales_page( $post_id ) {
    foreach( $this->_integrations as $integration ) {
      if( $integration->_has_access( $post_id ) )
        continue;

      $link = $integration->get_sell_link( $post_id);

      if( $link !== '' )
        return $link;
    }

    return get_permalink( $post_id );
  }

  public function access_to_post_map( $post_id ) {
    $response = [
      'has_access'     => true,
      'purchase_text'  => '',
      'purchase_link'  => ''
    ];

    foreach( $this->_integrations as $integration ) {
      if( $integration->_has_access( $post_id ) )
        continue;

      $response['has_access']     = false;
      $response['purchase_text']  = $integration->get_sell_text( $post_id );
      $response['purchase_link']  = $integration->get_sell_link( $post_id );

      return $response;
    }

    return $response;
  }

  public function clear_access_storage() {
    foreach( $this->_integrations as $integration )
      $integration->clear_access_storage();
  }

}