<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WP_Query;

class CourseActivity {

  public function get_course_started_users( $course_id ) {
    $course_completed_map = wpep_utility_map_to_array_assoc( WPEP_Controller::instance()->userActivity->get_course_user_data(
      WPEP_USER_COURSE_STARTED,
      $course_id,
      0,
      0,
      false,
      false
    ), 'user_id', "value" );

    return $course_completed_map;
  }

  public function get_period_completed_courses_list( $time_from, $time_to ) {
    $time_from = ( !is_numeric( $time_from ) ? strtotime( $time_from ) : $time_from );
    $time_to   = ( !is_numeric( $time_to )   ? strtotime ( $time_to )   : $time_to   );

    global $wpdb;

    $sql = 'SELECT user_data.user_id,
                   user_data.course_id,
                   user_data.value as completed_at
                   FROM ' . $wpdb->prefix . 'wpep_course_user_data user_data
                   WHERE user_data.key = "' . WPEP_USER_COURSE_COMPLETED_AT . '"
                     AND user_data.value >= ' . $time_from . '
                     AND user_data.value < ' . $time_to;

    return $wpdb->get_results( $sql );
  }

  /**
   * @param int $interaction_number
   * @param int $interaction_offset
   * @param bool $pagination_information
   * @param bool|string $filter_by
   * @return array
   */
  public function get_completed_history(
    $interaction_number     = 30,
    $interaction_offset     = 0,
    $pagination_information = false,
    $filter_by              = false
  ) {
    $course_list_query = new WP_Query( [
      'post_type'       => WPEP_POST_TYPE_COURSE,
      'posts_per_page'  => $interaction_number,
      'offset'          => $interaction_offset,
      'orderby'         => 'title',
      'order'           => 'ASC',
      'author'          => wpep_user_statistics_query_author_id()
    ] );

    $course_list = $course_list_query->get_posts();

    $response = [];

    foreach( $course_list as $course ) {
      $started_count   = 0;
      $completed_count = 0;

      foreach( WPEP_Controller::instance()->userActivity->get_course_user_data(
        WPEP_USER_COURSE_STARTED, $course->ID, 0, 0, false, false
      ) as $user_data_course_completed ) {
        if( isset( $filter_by['user_role'] ) && !wpep_user_has_role( $filter_by['user_role'], $user_data_course_completed->user_id ) )
          continue;

        $started_count++;
      }

      foreach( WPEP_Controller::instance()->userActivity->get_course_user_data(
        WPEP_USER_COURSE_COMPLETED, $course->ID, 0, 0, false, false
      ) as $user_data_course_completed ) {
        if( isset( $filter_by['user_role'] ) && !wpep_user_has_role( $filter_by['user_role'], $user_data_course_completed->user_id ) )
          continue;

        if( $user_data_course_completed->value == 1 )
          $completed_count++;
      }

      $response[] = [
        'course_id'               => $course->ID,
        'course_title'            => $course->post_title,
        'course_lessons'          => $this->get_course_lesson_count( $course->ID ),
        'course_users_started'    => $started_count,
        'course_users_completed'  => $completed_count
      ];
    }

    if( $pagination_information == false )
      return $response;

    return [
      'entries_total'   => $course_list_query->found_posts,
      'entries_number'  => $interaction_number,
      'entries_offset'  => $interaction_offset,
      'entries'         => $response,
    ];
  }

  public function get_course_lesson_count( $course_id ) {
    global $wpdb;

    $sql = 'SELECT COUNT( l.id ) 
              FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section s ON s.id = l.section_id
              WHERE s.post_id = ' . intval( $course_id );

    return intval( $wpdb->get_var( $sql ) );
  }

  /**
   * Get the number of users that have progressed a course until a certain lesson count
   * @param $course_id
   * @return mixed
   */
  public function get_course_progress_map( $course_id) {
    global $wpdb;

    $sql = 'SELECT lc.user_id, 
             COUNT( lc.id ) as completed_count 
             FROM ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
             LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson l ON l.id = lc.lesson_id
             LEFT JOIN ' . $wpdb->prefix . 'wpep_section s ON s.id = l.section_id
             WHERE s.post_id = ' . intval( $course_id ) . '
             GROUP BY lc.user_id';

    $result = $wpdb->get_results( $sql );
    $response = [];

    foreach( $result as $r )
      $response[ $r->user_id ] = $r->completed_count;

    return $response;
  }

}