<?php

namespace WPEP\Coordinator;

class Discussion {

  /**
   * @var \WPEP_Discussion_Integration[]
   */
  protected $_integrations = [];

  /**
   * @var Discussion\Delivery
   */
  public $delivery;
  /**
   * @var Discussion\EmailNotifications
   */
  public $emailNotifications;
  /**
   * @var Discussion\Events
   */
  public $events;
  /**
   * @var Discussion\Request
   */
  public $request;

  public function __construct() {
    $this->delivery = new Discussion\Delivery();
    $this->emailNotifications = new Discussion\EmailNotifications();
    $this->events   = new Discussion\Events();
    $this->request  = new Discussion\Request();
  }

  /**
   * @param \WPEP_Discussion_Integration $instance
   */
  public function register_integration( $instance ) {
    $this->_integrations[ $instance->get_namespace() ] = $instance;
  }

  public function all() {
    return $this->_integrations;
  }

  public function has_integrations() {
    return !empty( $this->_integrations );
  }

  /**
   * @param $namespace
   * @return \WPEP_Discussion_Integration|false
   */
  public function get( $namespace ) {
    return isset( $this->_integrations[ $namespace ] ) ? $this->_integrations[ $namespace ] : false;
  }

  public function setup() {
    $this->emailNotifications->setup();
    $this->events->setup();
    $this->request->setup();
  }

}