<?php

namespace WPEP\Coordinator\Discussion;

use WPEP\Entity\Discussion as Entity_Discussion;
use WPEP\Controller as WPEP_Controller;

class Delivery {

  /**
   * @param $discussion_id
   * @param int $pagination_current_page
   * @param array $args
   * @param \WPEP_Discussion_Integration|false $discussionIntegration
   * @return string
   */
  public function template_comments( $discussion_id, $pagination_current_page = 1, $args = [], $discussionIntegration = false ) {
    if( $discussionIntegration == false ) {
      $discussion = Entity_Discussion::instance()->get_discussion( [ 'id' => [ '=' => $discussion_id ] ] );

      if( isset( $discussion->namespace ) )
        $discussionIntegration = WPEP_Controller::instance()->discussion->get( $discussion->namespace );
    }

    $args = wp_parse_args( $args, [
      'show_pagination' => false,
      'structure'       => 'threaded',
      'search'          => '',
      '_search_id_map'  => [],
      'type'            => '',
      'parent__in'      => [],
      'date_query'      => null,
      'filter_by_user'  => false
    ] );

    $limit  = wpep_get_setting( 'comments-per-page' );
    $offset = $limit * ( $pagination_current_page - 1  );

    if( $args['search'] !== '' || $args['filter_by_user'] !== false ) {
      $args[ '_search_id_map' ] = Entity_Discussion::instance()->search_for_discussion_comment_ids( $discussion_id, $args['search'], $args['filter_by_user'] );

      if( empty( $args[ '_search_id_map' ] ) )
        return $this->_template_comments_content( [], $discussion_id, $args, $discussionIntegration );
    }

    $discussion_comments = Entity_Discussion::instance()->get_discussion_comments(
      $this->_query_where_array( $discussion_id, $args ),
      [
        'search'      => $args['search'],
        'count_total' => true,
        'limit'       => $limit,
        'offset'      => $offset
      ]
    );

    $response = $this->_template_comments_content( $discussion_comments['results'], $discussion_id, $args, $discussionIntegration );

    if( $args[ 'show_pagination' ] )
      $response .= wpep_discussion_comment_pagination_template( $pagination_current_page, ceil( $discussion_comments['total'] / wpep_get_setting( 'comments-per-page' ) ) );

    return $response;
  }

  /**
   * @param $discussion_comments
   * @param $discussion_id
   * @param array $args
   * @param \WPEP_Discussion_Integration|false $discussionIntegration
   * @return string
   */
  private function _template_comments_content( $discussion_comments, $discussion_id, $args = [], $discussionIntegration = false ) {
    $response =  '<ul class="wpep-post-comments-list">';

    foreach( $discussion_comments as $discussion_comment ) {
      $comment_class = 'wpep-single-comment' . ( $discussion_comment->parent_id == 0 ? '' : ' wpep-single-comment-reply' );

      $response .= '<li id="wpep-comment-' . $discussion_comment->id . '"';
      $response .=    ' data-wpep-comment-id="' . $discussion_comment->id . '"';
      $response .=    ' data-wpep-comment-parent="' . $discussion_comment->parent_id . '"';
      $response .=    ' data-wpep-single-comment-pin-status="' . $discussion_comment->is_pinned . '"';
      $response .=    ' class="' . $comment_class . '"';
      $response .=    '>';

      $response .= wpep_controller()->template->get_template( ( $discussion_comment->parent_id == 0 ? 'discussion/comment.php' : 'discussion/comment-reply.php' ), [
        'discussion_comment' => $discussion_comment,
        'can_reply_comment'  => true,
        'can_delete_comment' => ( $discussionIntegration != false ? $discussionIntegration->current_user_can_delete_comment( $discussion_comment ) : false ),
        'can_pin_comment'    => ( $discussionIntegration != false ? $discussionIntegration->current_user_can_pin_comment( $discussion_comment ) : false )
      ] );

      if( $args['structure'] == 'threaded' )
        $response .= $this->template_comment_replies( $discussion_id, $discussion_comment->id, $discussionIntegration, [
          'search'               => $args['search'],
          '_search_id_map'       => $args['_search_id_map'],
          'can_reply_comment'    => true,
          'last_activity_before' => false
        ] );

      $response .= '</li>';
    }

    $response .= '</ul>';

    return $response;
  }

  private function _query_where_array( $discussion_id, $args ) {
    $comments_where = [
      'discussion_id' => $discussion_id,
      'parent_id'     => ( !empty( $args['parent__in'] ) ? [
        'IN'  =>  $args['parent__in']
      ] : ( $args['structure'] == 'threaded' ? 0 : null ) )
    ];


    if( !empty( $args[ 'date_query' ] ) ) {
      $comments_where[ 'created_at' ] = $args[ 'date_query' ];
    }

    if( $args['type'] !== '' )
      $comments_where['type'] = $args['type'];

    if( isset( $args['_search_id_map'][0] ) && !empty( $args['_search_id_map'][0] ) )
      $comments_where[ 'id' ] = [ 'IN' => $args['_search_id_map'][0] ];

    return $comments_where;
  }

  /**
   * @param $discussion_id
   * @param $parent_id
   * @param \WPEP_Discussion_Integration|false $discussionIntegration
   * @param array $discussion_comment_replies_args
   * @return string
   */
  public function template_comment_replies(
    $discussion_id,
    $parent_id,
    $discussionIntegration = false,
    $discussion_comment_replies_args = []
  ) {
    $discussion_comment_replies_args = wp_parse_args( $discussion_comment_replies_args, [
      'search'                => '',
      '_search_id_map'        => [],
      'can_reply_comment'     => true,
      'last_activity_before'  => false
    ] );
    $discussion_comment_replies_where = [
      'discussion_id' => $discussion_id,
      'parent_id'     => $parent_id
    ];

    if( $discussion_comment_replies_args['last_activity_before'] != false )
      $discussion_comment_replies_where['last_activity'] = [
        '<' => $discussion_comment_replies_args['last_activity_before']
      ];

    if( isset( $discussion_comment_replies_args['_search_id_map'][ $parent_id ] ) )
      $discussion_comment_replies_where[ 'id' ] = [ 'IN' => $discussion_comment_replies_args['_search_id_map'][ $parent_id ] ];

    $discussion_comment_replies = Entity_Discussion::instance()->get_discussion_comments( $discussion_comment_replies_where, [
      'search'         => $discussion_comment_replies_args['search'],
      'order'          => 'DESC',
      'order_by'       => 'last_activity',
      'limit'          => wpep_get_setting_translated( 'comment-replies-per-page' ),
      'count_total'    => true
    ] );

    if( empty( $discussion_comment_replies['results'] ) )
      return '';

    $discussion_comment_replies['results'] = array_reverse( $discussion_comment_replies['results'] );

    $response = '';

    $response .= '<ul class="children">';

    if( $discussion_comment_replies['total'] > count( $discussion_comment_replies['results'] ) ) {
      $first_displayed_comment = $discussion_comment_replies['results'][ key( $discussion_comment_replies['results'] ) ];

      $response .= '<li data-wpep-comment-show-more="' . $parent_id . '" data-wpep-comment-show-more-last-activity="' . $first_displayed_comment->last_activity . '">';
      $response .=    sprintf(
        wpep_get_setting_translated( 'comment-replies-show-more' ),
        $discussion_comment_replies['total'] - ( count( $discussion_comment_replies['results'] ) )
      );
      $response .= '</li>';
    }

    foreach( $discussion_comment_replies['results'] as $discussion_comment_reply ) {
      $comment_reply_class = 'wpep-single-comment wpep-single-comment-reply';

      $response .= '<li id="wpep-comment-' . $discussion_comment_reply->id . '"';
      $response .=    ' data-wpep-comment-id="' . $discussion_comment_reply->id . '"';
      $response .=    ' data-wpep-comment-parent="' . $discussion_comment_reply->parent_id . '"';
      $response .=    ' data-wpep-single-comment-pin-status="' . $discussion_comment_reply->is_pinned . '"';
      $response .=    ' class="' . $comment_reply_class . '"';
      $response .=    '>';

      $response .= wpep_controller()->template->get_template( 'discussion/comment-reply.php', [
        'discussion_comment' => $discussion_comment_reply,
        'can_reply_comment'  => $discussion_comment_replies_args['can_reply_comment'],
        'can_delete_comment' => ( $discussionIntegration != false ? $discussionIntegration->current_user_can_delete_comment( $discussion_comment_reply ) : false ),
        'can_pin_comment'    => ( $discussionIntegration != false ? $discussionIntegration->current_user_can_pin_comment( $discussion_comment_reply ) : false )
      ] );

      $response .= $this->template_comment_replies( $discussion_id, $discussion_comment_reply->id, $discussionIntegration, [
        'search'               => $discussion_comment_replies_args['search'],
        '_search_id_map'       => $discussion_comment_replies_args['_search_id_map'],
        'can_reply_comment'    => false,
        'last_activity_before' => false
      ] );

      $response .= '</li>';
    }

    $response .= '</ul>';

    return $response;
  }

  public function wpep_discussion_comment_form_editor( $post_id, $discussion_namespace ) {
    add_filter( 'mce_buttons', [ $this, '_form_mce_buttons_row_1' ], 500 );
    add_filter( 'mce_buttons_2', [ $this, '_form_mce_buttons_remove' ], 500 );
    add_filter( 'mce_buttons_3', [ $this, '_form_mce_buttons_remove' ], 500 );
    add_filter( 'mce_buttons_4', [ $this, '_form_mce_buttons_remove' ], 500 );
    add_filter( 'mce_css', [ $this, '_form_mce_css' ], 500);

    $settings = wpep_controller()->discussion->get( $discussion_namespace )->get_comment_form_editor_args();

    if( !isset( $settings['textarea_name'] ) )
      $settings['textarea_name'] = 'wpep_comment_content';

    if( $settings['media_buttons'] )
      add_filter( 'user_has_cap', [ $this, '_form_user_has_cap' ], 100, 4 );

    wp_editor( '', 'wpep-post-comment-content-' . $post_id, $settings);

    if( $settings['media_buttons'] )
      remove_filter( 'user_has_cap', [ $this, '_form_user_has_cap' ], 100 );

    remove_filter( 'mce_buttons', [ $this, '_form_mce_buttons_row_1' ], 500 );
    remove_filter( 'mce_buttons_2', [ $this, '_form_mce_buttons_remove' ], 500 );
    remove_filter( 'mce_buttons_3', [ $this, '_form_mce_buttons_remove' ], 500 );
    remove_filter( 'mce_buttons_4', [ $this, '_form_mce_buttons_remove' ], 500 );
    remove_filter( 'mce_css', [ $this, '_form_mce_css' ], 500);
  }

  public function _form_user_has_cap( $capabilities, $caps, $args, $user_object ) {
    if( ! is_user_logged_in() )
      return $capabilities;

    $capabilities['upload_files'] = true;

    return $capabilities;
  }

  public function _form_mce_buttons_row_1( $buttons ) {
    return [ 'bold', 'italic', 'bullist', 'numlist', 'blockquote', 'alignleft', 'aligncenter', 'alignright', 'link' ];
  }

  public function _form_mce_buttons_remove( $buttons ) {
    return [];
  }

  public function _form_mce_css( $mce_css ) {
    $mce_css .= ',' . WPEP_BASE_URL_PATH . 'assets/front/discussion-editor.css?ver=' . WPEP_VERSION;

    return $mce_css;
  }

}