<?php

namespace WPEP\Coordinator\Discussion;

use WPEP\Entity\Discussion as WPEP_Discussion;

class Request {

  public function setup() {
    add_action( 'wp_ajax_wpep_discussions_manage_notifications', [ $this, 'request_discussions_manage_notifications' ] );

    add_action( 'wp_ajax_wpep_discussion_popup', [ $this, 'request_discussion_popup' ] );
    add_action( 'wp_ajax_wpep_discussion_comment_post', [ $this, 'request_post_comment' ] );
    add_action( 'wp_ajax_wpep_discussion_comment_pin', [ $this, 'request_pin_comment' ] );
    add_action( 'wp_ajax_wpep_discussion_comment_list', [ $this, 'request_list_comments' ] );
    add_action( 'wp_ajax_wpep_discussion_comment_replies_list', [ $this, 'request_list_comment_replies' ] );
    add_action( 'wp_ajax_wpep_discussion_comment_delete', [ $this, 'request_delete_comment' ] );
    add_action( 'wp_ajax_wpep_discussion_change_notifications_status', [ $this, 'request_change_notifications_status' ] );
    add_action( 'wp_ajax_wpep_discussion_user_list', [ $this, 'request_list_users' ] );
  }

  public function request_discussion_popup() {
    if( !isset( $_POST[ 'discussion_id' ] ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ] ) );

    $discussion  = WPEP_Discussion::instance()->get_discussion( [ 'id' => absint( $_POST[ 'discussion_id' ] ) ] );
    $integration = wpep_controller()->discussion->get( $discussion->namespace );

    if( $integration === false )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Discussion namespace.", "wpep" )
      ] ) );

    if( !$integration->current_user_can_view( $discussion->id ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Current user does not have access to post.", "wpep" )
      ] ) );

    exit ( json_encode( [
      'status'      => 'ok',
      'content'     => wpep_controller()->template->get_template( 'discussion/template-popup.php', [
        'post_id'              => $discussion->post_id,
        'discussion_id'        => $discussion->id,
        'discussion'           => $discussion,
        'discussion_namespace' => $integration->get_namespace()
      ] ),
    ] ) );
  }

  public function request_post_comment() {
    if( !isset( $_POST[ 'namespace' ] )
      || !isset( $_POST[ 'discussion_id' ] )
      || !isset( $_POST[ 'comment_parent' ] )
      || !isset( $_POST[ 'content' ] ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ] ) );

    $namespace         = $_POST[ 'namespace' ];
    $discussion_id     = intval( $_POST[ 'discussion_id' ] );
    $comment_parent_id = intval( $_POST[ 'comment_parent' ] );
    $content           = $_POST[ 'content' ];

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Discussion namespace.", "wpep" )
      ] ) );

    if( !$integration->current_user_can_post( $discussion_id ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Current user does not have access to post.", "wpep" )
      ] ) );

    $content = wpep_utilities_sanitize_input( $content );
    $comment_type = isset( $_POST['comment_type'] ) ? $_POST['comment_type'] : '';
    $comment_type = $integration->has_comment_type( $comment_type ) ? $comment_type : '';
    $current_date_time = current_time( 'mysql' );
    $can_reply_comment = true;

    if( $comment_parent_id != 0 ) {
      $discussion_comment_parent = WPEP_Discussion::instance()->get_discussion_comment( [ 'id' => $comment_parent_id ] );

      if( $discussion_comment_parent->discussion_id != $discussion_id )
        exit ( json_encode( [
          'status'  =>  'error',
          'error'   =>  __( "Parent Comment Discussion ID doesn't match the current discussion.", "wpep" )
        ] ) );

      if( $discussion_comment_parent->parent_id != 0 )
        $can_reply_comment = false;
    }

    $discussion_comment_id = WPEP_Discussion::instance()->add_discussion_comment( [
      'parent_id'     => $comment_parent_id,
      'discussion_id' => $discussion_id,
      'user_id'       => get_current_user_id(),
      'content'       => $content,
      'created_at'    => $current_date_time,
      'last_activity' => $current_date_time,
      'type'          => $comment_type
    ] );

    if( $discussion_comment_id === false )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Comment could not be saved.", "wpep" )
      ] ) );

    $discussion_comment = WPEP_Discussion::instance()->get_discussion_comment( [
      'id' => $discussion_comment_id
    ] );

    if( isset( $discussion_comment_parent ) ) {
      $sql =  'UPDATE `' . WPEP_Discussion::instance()->table_discussion_comment . '` SET last_activity = "' . $current_date_time . '"';
      $sql .= ' WHERE id = ' . $discussion_comment_parent->id;

      if( $discussion_comment_parent->parent_id != 0 )
        $sql .= ' OR id = ' . $discussion_comment_parent->parent_id;

      WPEP_Discussion::instance()->wpdb->query( $sql );
    }

    $comment_class = 'wpep-single-comment' . ( $discussion_comment->parent_id == 0 ? '' : ' wpep-single-comment-reply' );

    $comment_content = '<li id="wpep-comment-' . $discussion_comment->id . '"';
    $comment_content .=    ' data-wpep-comment-id="' . $discussion_comment->id . '"';
    $comment_content .=    ' data-wpep-comment-parent="' . $discussion_comment->parent_id . '"';
    $comment_content .=    ' data-wpep-single-comment-pin-status="' . $discussion_comment->is_pinned . '"';
    $comment_content .=    ' class="' . $comment_class . '"';
    $comment_content .=    '>';
    $comment_content .=     wpep_controller()->template->get_template( ( $discussion_comment->parent_id == 0 ? 'discussion/comment.php' : 'discussion/comment-reply.php' ), [
      'discussion_comment' => $discussion_comment,
      'can_reply_comment'  => $can_reply_comment,
      'can_delete_comment' => $integration->current_user_can_delete_comment( $discussion_comment ),
      'can_pin_comment'    => $integration->current_user_can_pin_comment( $discussion_comment )
    ] );
    $comment_content .= '</li>';

    exit ( json_encode( [
      'status'      => 'ok',
      'content'     => $comment_content,
      'parent_id'   => $comment_parent_id,
      'comment_id'  => $discussion_comment_id
    ] ) );
  }

  public function request_pin_comment() {
    if( !isset( $_POST[ 'namespace' ] ) || !isset( $_POST[ 'discussion_id' ] ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ] ) );

    $namespace             = $_POST[ 'namespace' ];
    $discussion_comment_id = intval( $_POST[ 'discussion_comment_id' ] );
    $pin_status            = isset( $_POST[ 'pin_status' ] ) && intval( $_POST[ 'pin_status' ] ) ? 1 : 0;

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Discussion namespace.", "wpep" )
      ] ) );

    if( !$integration->current_user_can_pin_comment( $discussion_comment_id ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Your user is not allowed to pin to the top this comment.", "wpep" )
      ] ) );

    WPEP_Discussion::instance()->pin_discussion_comment_by_id( $discussion_comment_id, $pin_status );

    exit ( json_encode( [
      'status'                => 'ok',
      'discussion_comment_id' => $discussion_comment_id,
      'pin_status'            => $pin_status
    ] ) );
  }

  public function request_list_comments() {
    if( !isset( $_POST[ 'namespace' ] ) || !isset( $_POST[ 'discussion_id' ] ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ] ) );

    $namespace     = $_POST[ 'namespace' ];
    $discussion_id = intval( $_POST[ 'discussion_id' ] );

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      exit ( json_encode( [
        'status'  => 'error',
        'error'   => __( "Invalid Discussion namespace.", "wpep" )
      ] ) );

    if( !$integration->current_user_can_view( $discussion_id ) )
      exit ( json_encode( [
        'status'  => 'error',
        'error'   => __( "Current user does not have access to post.", "wpep" )
      ] ) );

    exit ( json_encode( [
      'status'      => 'ok',
      'timestamp'   => time(),
      'content'     => $integration->get_comments_template( $discussion_id, true )
    ] ) );
  }

  public function request_list_comment_replies() {
    if( !isset( $_POST[ 'namespace' ] )
        || !isset( $_POST[ 'discussion_id' ] )
        || !isset( $_POST[ 'comment_parent_id' ] ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ] ) );

    $namespace            = $_POST[ 'namespace' ];
    $discussion_id        = intval( $_POST[ 'discussion_id' ] );
    $comment_parent_id    = intval( $_POST[ 'comment_parent_id' ] );
    $last_activity_before = isset( $_POST[ 'last_activity_before' ] ) && $_POST[ 'last_activity_before' ] != false ? $_POST[ 'last_activity_before' ] : false;

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Discussion namespace.", "wpep" )
      ] ) );

    if( !$integration->current_user_can_view( $discussion_id ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Current user does not have access to post.", "wpep" )
      ] ) );

    $can_reply_comment = true;

    if( $comment_parent_id !== 0 ) {
      $discussion_comment_parent = WPEP_Discussion::instance()->get_discussion_comment( [ 'id' => $comment_parent_id ] );

      if( $discussion_comment_parent->discussion_id != $discussion_id )
        exit ( json_encode( [
          'status'  =>  'error',
          'error'   =>  __( "Parent Comment Discussion ID doesn't match the current discussion.", "wpep" )
        ] ) );

      $can_reply_comment = ( intval( $discussion_comment_parent->parent_id ) === 0 );
    }

    exit ( json_encode( [
      'status'      => 'ok',
      'timestamp'   => time(),
      'content'     => wpep_controller()->discussion->delivery->template_comment_replies( $discussion_id, $comment_parent_id, $integration, [
        'last_activity_before' => $last_activity_before,
        'can_reply_comment'    => $can_reply_comment
      ] )
    ] ) );
  }

  public function request_delete_comment() {
    if( !isset( $_POST[ 'namespace' ] ) || !isset( $_POST[ 'discussion_comment_id' ] ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ] ) );

    $namespace = $_POST[ 'namespace' ];
    $discussion_comment_id = intval( $_POST[ 'discussion_comment_id' ] );

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Discussion namespace.", "wpep" )
      ] ) );

    if( !$integration->current_user_can_delete_comment( $discussion_comment_id ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Your user is not allowed to delete this comment.", "wpep" )
      ] ) );

    WPEP_Discussion::instance()->delete_discussion_comment_by_id( $discussion_comment_id );

    exit ( json_encode( [
      'status'      => 'ok',
      'timestamp'   => time()
    ] ) );
  }

  public function request_discussions_manage_notifications() {
    $user_id     = get_current_user_id();
    $discussions = [];

    foreach( wpep_controller()->discussion->all() as $post_comments_integration )
      $discussions = array_merge( $discussions, $post_comments_integration->get_discussions_by_user_id( $user_id, [
        'limit' => false
      ] ) );

    exit ( json_encode( [
      'status'  => 'ok',
      'content' => wpep_template_get( 'discussion/user-notifications.php', [
        'discussions'              => isset( $discussions['entries'] ) ? $discussions['entries'] : [],
        'discussion_notifications' => wpep_utility_map_to_array_assoc(
          WPEP_Discussion::instance()->get_discussion_user_data(
            WPEP_DISCUSSION_USER_DATA_NOTIFICATION_STATUS,
            false,
            $user_id,
            false
          ),
          'discussion_id',
          'value'
        ),
        'user_id' => $user_id
      ] )
    ] ) );
  }

  public function request_change_notifications_status() {
    if( !isset( $_POST[ 'discussion_id' ] ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ] ) );

    $discussion  = WPEP_Discussion::instance()->get_discussion( [ 'id' => absint( $_POST[ 'discussion_id' ] ) ] );
    $user_id     = ( isset( $_POST['user_id'] ) ? absint( $_POST['user_id'] ) : get_current_user_id() );
    $notification_status = ( isset( $_POST[ 'notification_status' ] ) ? absint( $_POST[ 'notification_status' ] ) : 0 );

    if( $user_id != get_current_user_id()
      && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      exit ( json_encode( [
        'status' => 'error',
        'error'  => __( "You are not allowed to modify the discussion settings for this User.", "wpep" )
      ] ) );

    $integration = wpep_controller()->discussion->get( $discussion->namespace );

    if( $integration === false )
      exit ( json_encode( [
        'status' => 'error',
        'error'  => __( "Invalid Discussion namespace.", "wpep" )
      ] ) );

    if( !$integration->current_user_can_view( $discussion->id ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Current user does not have access to the discussion.", "wpep" )
      ] ) );

    WPEP_Discussion::instance()->set_discussion_user_data( WPEP_DISCUSSION_USER_DATA_NOTIFICATION_STATUS, $notification_status, $discussion->id, $user_id );

    exit ( json_encode( [
      'status'      => 'ok',
      'timestamp'   => time(),
      'notification_status' => $notification_status
    ] ) );
  }

  public function request_list_users() {
    if( !isset( $_POST[ 'namespace' ] ) || !isset( $_POST[ 'discussion_id' ] ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ] ) );

    $namespace     = $_POST[ 'namespace' ];
    $discussion_id = intval( $_POST[ 'discussion_id' ] );
    $page          = isset( $_POST[ 'page' ] ) ? absint( $_POST[ 'page' ] ) : 1;

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Discussion namespace.", "wpep" )
      ] ) );

    if( !$integration->current_user_can_view( $discussion_id ) )
      exit ( json_encode( [
        'status'  =>  'error',
        'error'   =>  __( "Current user does not have access to the discussion.", "wpep" )
      ] ) );

    $discussion_users = $integration->get_discussion_users( $discussion_id, [
      'limit'   => WPEP_DISCUSSION_USER_LIST_PER_PAGE,
      'offset'  => ( $page == 1 ? 0 : ( $page - 1 ) * WPEP_DISCUSSION_USER_LIST_PER_PAGE )
    ] );

    exit ( json_encode( [
      'status'      => 'ok',
      'timestamp'   => time(),
      'content'     => wpep_template_get( 'discussion/user-list-navigation.php', [
        'discussion_id'     => $discussion_id,
        'discussion_users'  => isset( $discussion_users['entries'] ) ? $discussion_users['entries'] : [],
        'pagination_page'   => $page,
        'total_users'       => $discussion_users['total'],
        'entries_per_page'  => WPEP_DISCUSSION_USER_LIST_PER_PAGE
      ] )
    ] ) );
  }

}