<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;

class Email {

  private $_event_tracking_type  = 'wpep-email-event';
  private $_event_tracking_token = 'wpep-email-token';

  public function setup() {
    add_action( 'delete_post', [ $this, "_on_delete_post" ] );
    add_action( 'delete_user', [ $this, "_on_delete_user" ] );

    if( isset( $_GET[ $this->_event_tracking_type ] ) && isset( $_GET[ $this->_event_tracking_token ] ) )
      add_action( 'wpep_loaded', [ $this, '_handle_event_tracking' ] );

    if( isset( $_GET['wpep-preview-email'] ) )
      add_action( 'init', [ $this, '_init_preview_email' ] );
  }

  public function _init_preview_email() {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return;

    if( $_GET['wpep-preview-email'] === 'weekly-recap-email' ) {
      $user_ids = get_option( 'wpep_weekly_recap_que', [] );

      $user_ids = [ get_current_user_id() => get_current_user_id() ] + $user_ids;

      update_option( 'wpep_weekly_recap_que', $user_ids );

      wp_redirect( wpep_admin_url_settings( [
        'tab'       => 'learner_engagement',
        'tab_inner' => 'weekly_recap'
      ] ) );
      exit;
    } elseif( $_GET['wpep-preview-email'] === 'weekly-recap-instant-email' ) {
      WPEP_Controller::instance()->userEngagement->send_email(
        get_current_user_id(),
        wpep_get_setting( 'email-weekly-recap-title' ),
        wpep_get_setting( 'email-weekly-recap-content' )
      );

      wp_redirect( wpep_admin_url_settings( [
        'tab'       => 'learner_engagement',
        'tab_inner' => 'weekly_recap'
      ] ) );
      exit;
    } elseif( $_GET['wpep-preview-email'] === 'weekly-recap-browser' ) {
      add_filter( 'wpep_email_track_events', '__return_false' );
      add_filter( 'wpep_email_content_html', function( $content ) {
        echo $content;exit;
      }, 100 );

      WPEP_Controller::instance()->userEngagement->send_email(
        get_current_user_id(),
        wpep_get_setting( 'email-weekly-recap-title' ),
        wpep_get_setting( 'email-weekly-recap-content' )
      );

      exit;
    }

    add_filter( 'wpep_email_track_events', '__return_false' );
    add_filter( 'wpep_email_content_html', function( $content ) {
      echo $content;exit;
    }, 100 );

    WPEP_Controller::instance()->userEngagement->send_email(
      get_current_user_id(),
      __( "Email Preview", "wpep" ),
      urldecode( $_GET['wpep-preview-email'] )
    );

    exit;
  }

  public function _handle_event_tracking() {
    $token = sanitize_text_field( $_GET[ $this->_event_tracking_token ] );
    $event = sanitize_text_field( $_GET[ $this->_event_tracking_type ] );

    global $wpdb;

    $email_notification = $wpdb->get_row( 'SELECT * FROM ' . $wpdb->prefix . 'wpep_email_notification WHERE token = "' . $token . '"');

    if( $event == 'open-pixel' ) {
      if( isset( $email_notification->id ) && intval( $email_notification->is_opened ) === 0 ) {
        $wpdb->update( $wpdb->prefix . 'wpep_email_notification',
          [
            'is_opened' =>  1,
            'opened_at' => current_time( 'mysql' )
          ],
          [
            'id' => $email_notification->id
          ]
        );
      }

      header('Content-Type: image/png');
      echo base64_decode('iVBORw0KGgoAAAANSUhEUgAAAAEAAAABAQMAAAAl21bKAAAAA1BMVEUAAACnej3aAAAAAXRSTlMAQObYZgAAAApJREFUCNdjYAAAAAIAAeIhvDMAAAAASUVORK5CYII=');
      exit;
    }

    exit;
  }

  public function send( $user_email, $email_title, $email_content, $wpautop = true ) {
    $headers =  'Content-Type: text/html; charset=UTF-8' . "\r\n";

    if( wpep_get_setting( 'email-from-name' ) !== '' && wpep_get_setting( 'email-from-email' ) !== '' )
      $headers .= 'From: ' . wpep_get_setting( 'email-from-name' ) . ' <' . wpep_get_setting( 'email-from-email' ) . '>' . "\r\n";

    if( isset( $user_email->user_email ) ) {
      $this->_send( $user_email, $email_title, $email_content, $headers, $wpautop );
    } elseif( is_array( $user_email ) ) {
      foreach( $user_email as $email )
        $this->_send( $email, $email_title, $email_content, $headers, $wpautop );
    } else {
      $this->_send( $user_email, $email_title, $email_content, $headers, $wpautop );
    }
  }

  private function _send( $email_address, $email_title, $email_content, $headers, $wpautop = true ) {
    $user = ( isset( $email_address->ID ) ? $email_address : get_user_by( 'email', $email_address ) );
    $email_address = isset( $user->user_email ) ? $user->user_email : $email_address;

    if( $wpautop )
      $email_content = wpautop( $email_content );

    if( apply_filters( "wpep_email_track_events", isset( $user->ID ) )  ) {
      $notification_data = $this->_track_email_sent( $user->ID, $email_title );

      $email_content .= $this->action_tracking( $notification_data['token'], 'open-pixel' );
    }

    $email_content = WPEP_Controller::instance()->template->get_template( 'email-template/layout.php', [
      'email_title'   => $email_title,
      'email_content' => $email_content,
      'email_logo'    => wpep_get_setting( 'email-logo', wpep_get_placeholder_image() )
    ] );

    $email_content = apply_filters( 'wpep_email_content_html', $email_content );

    wp_mail( $email_address, $email_title, $email_content, $headers);
  }

  private function _track_email_sent( $user_id, $email_title ) {
    $salt = wpep_get_setting( 'email-tracking-salt' );

    if( $salt === '' ) {
      $salt = wpep_generate_random_token( 24 );

      wpep_set_setting( 'email-tracking-salt', $salt );
    }

    $data = [
      'post_id'   => 0,
      'user_id'   => $user_id,
      'title'     => $email_title,
      'is_opened' => 0,
      'created_at' => current_time( 'mysql' ),
      'token'      => sha1( $salt . wpep_generate_random_token( 32 ) )
    ];

    global $wpdb;

    $wpdb->insert( $wpdb->prefix . 'wpep_email_notification', $data );

    $data['id'] = $wpdb->insert_id;

    return $data;
  }

  public function _on_delete_post( $post_id ) {
    global $wpdb;

    $email_notification_ids = $wpdb->get_col(
      'SELECT id FROM ' . $wpdb->prefix . 'wpep_email_notification WHERE post_id = ' . $post_id,
      0
    );

    if( !empty ( $email_notification_ids ) ) {
      $wpdb->query( "DELETE FROM " . $wpdb->prefix . 'wpep_email_tracking WHERE email_notification_id IN ( ' . implode( ',', $email_notification_ids ). ' ) ' );
      $wpdb->delete( $wpdb->prefix . 'wpep_email_notification', [ 'post_id' => $post_id ] );
    };
  }

  public function _on_delete_user( $user_id ) {
    global $wpdb;

    $wpdb->delete( $wpdb->prefix . 'wpep_email_tracking', [ 'user_id' => $user_id ] );
    $wpdb->delete( $wpdb->prefix . 'wpep_email_notification', [ 'user_id' => $user_id ] );
  }

  /**
   * @param int $interaction_number
   * @param int $interaction_offset
   * @param bool $pagination_information
   * @param bool|string $filter_by
   * @return array
   */
  public function get_delivery_history(
    $interaction_number     = 30,
    $interaction_offset     = 0,
    $pagination_information = false,
    $filter_by              = false
  ) {
    global $wpdb;

    $query_entries_total = 'SELECT COUNT( * ) FROM `' . $wpdb->prefix . 'wpep_email_notification`';

    if( isset( $filter_by['title'] ) )
      $query_entries_total .= ' WHERE title = "' . esc_sql( $filter_by['title'] ) . '"';

    $query_entries = 'SELECT wpep_em.title,
                             wpep_em.token,
                             wpep_em.is_opened,
                             wpep_em.created_at,
                             wpep_em.opened_at,
                             wpep_em.user_id,
                             wp_u.user_email
                      FROM `' . $wpdb->prefix . 'wpep_email_notification` wpep_em
                      LEFT JOIN `' . $wpdb->users . '` wp_u ON wp_u.ID = wpep_em.user_id';

    if( isset( $filter_by['title'] ) )
      $query_entries .= ' WHERE wpep_em.title = "' . esc_sql( $filter_by['title'] ) . '"';

    $query_entries .= ' ORDER BY wpep_em.created_at DESC
                        LIMIT ' . $interaction_offset . ',' . $interaction_number;

    $total_entries = intval( $wpdb->get_var( $query_entries_total ) );
    $entries = $wpdb->get_results( $query_entries, ARRAY_A );

    if( $pagination_information == false )
      return $entries;

    return [
      'entries_total'   => $total_entries,
      'entries_number'  => $interaction_number,
      'entries_offset'  => $interaction_offset,
      'entries'         => $entries,
    ];
  }

  public function get_delivery_count(
    $filter_by = false
  ) {
    global $wpdb;

    $query_entries_total = 'SELECT COUNT( * ) FROM `' . $wpdb->prefix . 'wpep_email_notification` WHERE 1 = 1';

    if( isset( $filter_by['title'] ) )
      $query_entries_total .= ' AND title = "' . esc_sql( $filter_by['title'] ) . '"';

    if( isset( $filter_by['time_from'] ) )
      $query_entries_total .= ' AND `created_at` >= "' . date( "Y-m-d H:i:s", $filter_by['time_from'] ) . '"';

    if( isset( $filter_by['time_to'] ) )
      $query_entries_total .= ' AND `created_at` < "' . date( "Y-m-d H:i:s", $filter_by['time_to'] ) . '"';

    return absint( $wpdb->get_var( $query_entries_total ) );
  }

  /**
   * @param int $interaction_number
   * @param int $interaction_offset
   * @param bool $pagination_information
   * @param bool|string $filter_by
   * @return array
   */
  public function get_delivery_history_by_title(
    $interaction_number     = 30,
    $interaction_offset     = 0,
    $pagination_information = false,
    $filter_by              = false
  ) {
    global $wpdb;

    $query_entries_total = 'SELECT COUNT( DISTINCT title) FROM `' . $wpdb->prefix . 'wpep_email_notification`';
    $total_entries = intval( $wpdb->get_var( $query_entries_total ) );

    $query_entries = 'SELECT title, 
                             COUNT( * ) as sent_count,
                             SUM( is_opened ) as open_count, 
                             min( created_at ) as first_date,
                             max(created_at) as last_date 
                      FROM `' . $wpdb->prefix . 'wpep_email_notification`
                      GROUP BY title';
    $query_entries = 'SELECT wpep_en.* FROM ( ' . $query_entries . ' ) wpep_en ORDER BY wpep_en.last_date DESC LIMIT ' . $interaction_offset . ',' . $interaction_number;

    $entries = $wpdb->get_results( $query_entries, ARRAY_A );

    if( $pagination_information == false )
      return $entries;

    return [
      'entries_total'   => $total_entries,
      'entries_number'  => $interaction_number,
      'entries_offset'  => $interaction_offset,
      'entries'         => $entries,
    ];
  }

  /**
   * @param $tracking_token
   * @param string $type
   * @param array $options
   * @return string
   */
  public function action_tracking( $tracking_token, $type = 'open-pixel', $options = [] ) {
    $link = get_site_url();

    $link = add_query_arg( $this->_event_tracking_type, $type, $link );
    $link = add_query_arg( $this->_event_tracking_token, $tracking_token, $link );

    if( $type == 'open-pixel' )
      return '<img src="' . $link . '" alt="" width="1" height="1" border="0"/>';

    return $link;
  }

}