<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;
use WPEP\Container\Migration as WPEP_Migration_Container;

class Migration {

  use WPEP_Migration_Container;

  public function setup() {
    $current_version_available = '2.0.4';
    $migration_list = [
      '1.3.3'  => 'version-1.3.3.php',
      '1.4.0'  => 'version-1.4.0.php',
      '1.7.1'  => 'version-1.7.1.php',
      '1.7.2'  => 'version-1.7.2.php',
      '1.8.0'  => 'version-1.8.0.php',
      '1.8.1'  => 'version-1.8.1.php',
      '1.8.2'  => 'version-1.8.2.php',
      '1.8.3'  => 'version-1.8.3.php',
      '1.8.4'  => 'version-1.8.4.php',
      '1.8.5'  => 'version-1.8.5.php',
      '1.8.6'  => 'version-1.8.6.php',
      '1.8.7'  => 'version-1.8.7.php',
      '1.8.8'  => 'version-1.8.8.php',
      '1.9.0'  => 'version-1.9.0.php',
      '1.9.1'  => 'version-1.9.1.php',
      '1.9.2'  => 'version-1.9.2.php',
      '1.9.3'  => 'version-1.9.3.php',
      '1.9.4'  => 'version-1.9.4.php',
      '1.9.5'  => 'version-1.9.5.php',
      '2.0.0'  => 'version-2.0.0.php',
      '2.0.1'  => 'version-2.0.1.php',
      '2.0.2'  => 'version-2.0.2.php',
      '2.0.3'  => 'version-2.0.3.php',
      '2.0.4'  => 'version-2.0.4.php'
    ];

    $this->trait_setup( [
      'has_admin'                    => true,
      'namespace'                    => 'wpep',
      'plugin_file_path'             => WPEP_BASE_FILE_PATH,
      'option_name'                  => 'wpep_migration_version',
      'option_progress_name'         => 'wpep_migration_progress',
      'current_version_available'    => $current_version_available,
      'plugin_migration_folder_path' => WPEP_BASE_PATH . '/lib/migrations/',
      'migration_list'               => $migration_list,
    ] );
  }

  public function get_table_structure() {
    global $wpdb;

    $charset_collate = $wpdb->get_charset_collate();

    $action_tracking_table_name = $wpdb->prefix . 'wpep_action_tracking';
    $section_table_name = $wpdb->prefix . 'wpep_section';
    $lesson_table_name = $wpdb->prefix . 'wpep_section_lesson';
    $lesson_completed_table_name = $wpdb->prefix . 'wpep_section_lesson_completed';
    $course_user_data_table_name = $wpdb->prefix . 'wpep_course_user_data';
    $assessment_question_table_name = $wpdb->prefix . 'wpep_assessment_question';
    $assessment_question_answer_table_name = $wpdb->prefix . 'wpep_assessment_question_answer';
    $assessment_user_data = $wpdb->prefix . 'wpep_assessment_user_data';
    $email_table = $wpdb->prefix . 'wpep_email_notification';
    $email_tracking_table = $wpdb->prefix . 'wpep_email_tracking';
    $discussion_table = $wpdb->prefix . 'wpep_discussion';
    $discussion_data_table = $wpdb->prefix . 'wpep_discussion_data';
    $discussion_comment_table = $wpdb->prefix . 'wpep_discussion_comment';

    $response = [];

    $response[ $action_tracking_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $action_tracking_table_name . '` (
                                              `id`       INT NOT NULL AUTO_INCREMENT,
                                              `post_id`  INT NOT NULL DEFAULT 0,
                                              `user_id`  INT NOT NULL DEFAULT 0,
                                              `type`     VARCHAR(500) NOT NULL DEFAULT "",
                                              `segment`  VARCHAR(500) NOT NULL DEFAULT "",
                                              `created_at` DATETIME DEFAULT NULL,
                                              PRIMARY KEY (id),
                                              INDEX `' . $action_tracking_table_name . '_post_id` (`post_id`),
                                              INDEX `' . $action_tracking_table_name . '_user_id` (`user_id`),
                                              INDEX `' . $action_tracking_table_name . '_type` (`type`)
                                              ) ' . $charset_collate;

    $response[ $section_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $section_table_name . '` (
                                              `id`       INT NOT NULL AUTO_INCREMENT,
                                              `post_id`  INT NOT NULL DEFAULT 0,
                                              `title`    VARCHAR(500) NOT NULL,
                                              `content`  LONGTEXT NOT NULL,
                                              `journey_access_type`    VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_access_segment` VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_expiration_type`    VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_expiration_segment` VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_execution_type`    VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_execution_segment` VARCHAR(100) NOT NULL DEFAULT "",
                                              `order`  INT NOT NULL DEFAULT 0,
                                              `options` TEXT NOT NULL,
                                              PRIMARY KEY (id),
                                              INDEX `' . $section_table_name . '_post_id` (`post_id`)
                                              ) ' . $charset_collate;

    $response[ $lesson_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $lesson_table_name . '` (
                                              `id`         INT NOT NULL AUTO_INCREMENT,
                                              `post_id`    INT NOT NULL DEFAULT 0,
                                              `section_id` INT NOT NULL DEFAULT 0,
                                              `title`      VARCHAR(500) NOT NULL,
                                              `content`    LONGTEXT NOT NULL,
                                              `journey_access_type`    VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_access_segment` VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_expiration_type`    VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_expiration_segment` VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_execution_type`    VARCHAR(100) NOT NULL DEFAULT "",
                                              `journey_execution_segment` VARCHAR(100) NOT NULL DEFAULT "",
                                              `order`    INT NOT NULL DEFAULT 0,
                                              `options`  TEXT NOT NULL,
                                              PRIMARY KEY (id),
                                              INDEX `' . $lesson_table_name . '_post_id` (`post_id`),
                                              INDEX `' . $lesson_table_name . '_section_id` (`section_id`)
                                              ) ' . $charset_collate;

    $response[ $lesson_completed_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $lesson_completed_table_name . '` (
                                                        `id`         INT NOT NULL AUTO_INCREMENT,
                                                        `lesson_id`  INT NOT NULL DEFAULT 0,
                                                        `user_id`    INT NOT NULL DEFAULT 0,
                                                        `created_at` DATETIME DEFAULT NULL,
                                                        PRIMARY KEY (id),
                                                        INDEX `' . $lesson_completed_table_name . '_lesson_id` (`lesson_id`),
                                                        INDEX `' . $lesson_completed_table_name . '_user_id` (`user_id`)
                                              ) ' . $charset_collate;

    $response[ $course_user_data_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $course_user_data_table_name . '` (
                                                        `id`         INT NOT NULL AUTO_INCREMENT,
                                                        `user_id`    INT NOT NULL DEFAULT 0,
                                                        `course_id`  INT NOT NULL DEFAULT 0,
                                                        `section_id` INT NOT NULL DEFAULT 0,
                                                        `lesson_id`  INT NOT NULL DEFAULT 0,
                                                        `key`       VARCHAR(100) NOT NULL DEFAULT "",
                                                        `value`     VARCHAR(500) NOT NULL,
                                                        PRIMARY KEY ( `id` ),
                                                        INDEX `' . $course_user_data_table_name . '_user_id` (`user_id`),
                                                        INDEX `' . $course_user_data_table_name . '_course_id` (`course_id`),
                                                        INDEX `' . $course_user_data_table_name . '_section_id` (`section_id`),
                                                        INDEX `' . $course_user_data_table_name . '_lesson_id` (`lesson_id`)
                                              ) ' . $charset_collate;

    $response[ $assessment_question_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $assessment_question_table_name . '` (
                                                           `id`      INT NOT NULL AUTO_INCREMENT,
                                                           `post_id` INT NOT NULL DEFAULT 0,
                                                           `title`   VARCHAR(500) NOT NULL DEFAULT "",
                                                           `content` LONGTEXT NOT NULL DEFAULT "",
                                                           `type`    VARCHAR(500) NOT NULL DEFAULT "",
                                                           `weight`  INT NOT NULL DEFAULT 0,
                                                           `order`   INT NOT NULL DEFAULT 0,
                                                           PRIMARY KEY (id),
                                                           INDEX `' . $assessment_question_table_name . '_post_id` (`post_id`)
                                              ) ' . $charset_collate;

    $response[ $assessment_question_answer_table_name ] = 'CREATE TABLE IF NOT EXISTS `' . $assessment_question_answer_table_name . '` (
                                                                  `id`          INT NOT NULL AUTO_INCREMENT,
                                                                  `post_id`     INT NOT NULL DEFAULT 0,
                                                                  `question_id` INT NOT NULL DEFAULT 0,
                                                                  `title`       VARCHAR(500) NOT NULL DEFAULT "",
                                                                  `content`     LONGTEXT NOT NULL DEFAULT "",
                                                                  `is_correct`  TINYINT NOT NULL DEFAULT 0,
                                                                  `order`       INT NOT NULL DEFAULT 0,
                                                                  PRIMARY KEY (id),
                                                                  INDEX `' . $assessment_question_answer_table_name . '_post_id` (`post_id`),
                                                                  INDEX `' . $assessment_question_answer_table_name . '_question_id` (`question_id`)
                                              ) ' . $charset_collate;

    $response[ $assessment_user_data ] = 'CREATE TABLE IF NOT EXISTS `' . $assessment_user_data . '` (
                                                 `id`          INT NOT NULL AUTO_INCREMENT,
                                                 `user_id`     INT NOT NULL DEFAULT 0,
                                                 `post_id`     INT NOT NULL DEFAULT 0,
                                                 `question_id` INT NOT NULL DEFAULT 0,
                                                 `question_answer_id` INT NOT NULL DEFAULT 0,
                                                 `key`         VARCHAR(100) NOT NULL DEFAULT "",
                                                 `value`       LONGTEXT NOT NULL DEFAULT "",
                                                 `created_at`  DATETIME DEFAULT NULL,
                                                 PRIMARY KEY (id),
                                                 INDEX `' . $assessment_user_data . '_user_id` (`user_id`),
                                                 INDEX `' . $assessment_user_data . '_post_id` (`post_id`),
                                                 INDEX `' . $assessment_user_data . '_question_id` (`question_id`),
                                                 INDEX `' . $assessment_user_data . '_question_answer_id` (`question_answer_id`)
                                              ) ' . $charset_collate;

    $response[ $email_table ] = 'CREATE TABLE IF NOT EXISTS `' . $email_table . '` (
                                        `id`      INT NOT NULL AUTO_INCREMENT,
                                        `post_id` INT NOT NULL DEFAULT 0,
                                        `user_id` INT NOT NULL DEFAULT 0,
                                        `title`   VARCHAR(500) NOT NULL DEFAULT "",
                                        `token`   VARCHAR(500) NOT NULL DEFAULT "",
                                        `is_opened`  INT NOT NULL DEFAULT 0,
                                        `created_at`  DATETIME DEFAULT NULL,
                                        `opened_at`   DATETIME DEFAULT NULL,
                                        PRIMARY KEY (id),
                                        INDEX `' . $email_table . '_post_id` ( `post_id` ),
                                        INDEX `' . $email_table . '_user_id` ( `user_id` ),
                                        INDEX `' . $email_table . '_is_opened` ( `is_opened` )
                                              ) ' . $charset_collate;

    $response[ $email_tracking_table ] = 'CREATE TABLE IF NOT EXISTS `' . $email_tracking_table . '` (
                                                 `id`          INT NOT NULL AUTO_INCREMENT,
                                                 `user_id`     INT NOT NULL DEFAULT 0,
                                                 `email_notification_id` INT NOT NULL DEFAULT 0,
                                                 `action`      VARCHAR(100) NOT NULL DEFAULT "",
                                                 `target`      VARCHAR(500) NOT NULL DEFAULT "",
                                                 `created_at`  DATETIME DEFAULT NULL,
                                                 PRIMARY KEY (id),
                                                 INDEX `' . $email_tracking_table . '_user_id` (`user_id`),
                                                 INDEX `' . $email_tracking_table . '_email_notification_id` (`email_notification_id`)
                                              ) ' . $charset_collate;

    $response[ $discussion_table ] = 'CREATE TABLE IF NOT EXISTS `' . $discussion_table . '` (
                                             `id`            INT NOT NULL AUTO_INCREMENT,
                                             `post_id`       INT NOT NULL DEFAULT 0,
                                             `identifier`    VARCHAR(500) NOT NULL DEFAULT "",
                                             `namespace`     VARCHAR(500) NOT NULL DEFAULT "",
                                             `created_at`    DATETIME DEFAULT NULL,
                                             `last_activity` DATETIME DEFAULT NULL,
                                             PRIMARY KEY (id),
                                             INDEX `' . $discussion_table . '_post_id` ( `post_id` ),
                                             INDEX `' . $discussion_table . '_identifier` ( `identifier` )
                                              ) ' . $charset_collate;

    $response[ $discussion_data_table ] = 'CREATE TABLE IF NOT EXISTS `' . $discussion_data_table . '` (
                                                  `id`            INT NOT NULL AUTO_INCREMENT,
                                                  `discussion_id` INT NOT NULL DEFAULT 0,
                                                  `user_id`       INT NOT NULL DEFAULT 0,
                                                  `key`           VARCHAR(100) NOT NULL DEFAULT "",
                                                  `value`         VARCHAR(500) NOT NULL,
                                                  PRIMARY KEY (id),
                                                  INDEX `' . $discussion_data_table . '_discussion_id` ( `discussion_id` ),
                                                  INDEX `' . $discussion_data_table . '_user_id` ( `user_id` )
                                              ) ' . $charset_collate;

    $response[ $discussion_comment_table ] = 'CREATE TABLE IF NOT EXISTS `' . $discussion_comment_table . '` (
                                                     `id`            INT NOT NULL AUTO_INCREMENT,
                                                     `parent_id`     INT NOT NULL DEFAULT 0,
                                                     `discussion_id` INT NOT NULL DEFAULT 0,
                                                     `user_id`       INT NOT NULL DEFAULT 0,
                                                     `content`       LONGTEXT NOT NULL,
                                                     `created_at`    DATETIME DEFAULT NULL,
                                                     `last_activity` DATETIME DEFAULT NULL,
                                                     `is_pinned`     TINYINT NOT NULL DEFAULT 0,
                                                     `type`          VARCHAR(500) NOT NULL,
                                                     PRIMARY KEY (id),
                                                     INDEX `' . $discussion_comment_table . '_parent_id` (`parent_id`),
                                                     INDEX `' . $discussion_comment_table . '_discussion_id` (`discussion_id`),
                                                     INDEX `' . $discussion_comment_table . '_user_id` (`user_id`)
                                              ) ' . $charset_collate;

    return $response;
  }

  public function get_table_list() {
    global $wpdb;

    $response = [
      $wpdb->prefix . 'wpep_action_tracking',
      $wpdb->prefix . 'wpep_course_user_data',
      $wpdb->prefix . 'wpep_section',
      $wpdb->prefix . 'wpep_section_lesson',
      $wpdb->prefix . 'wpep_assessment_question',
      $wpdb->prefix . 'wpep_assessment_question_answer',
      $wpdb->prefix . 'wpep_assessment_user_data',
      $wpdb->prefix . 'wpep_email_notification',
      $wpdb->prefix . 'wpep_email_tracking',
      $wpdb->prefix . 'wpep_discussion',
      $wpdb->prefix . 'wpep_discussion_data',
      $wpdb->prefix . 'wpep_discussion_comment'
    ];

    return $response;
  }

  public function _after_activation() {
    WPEP_Settings::instance()->update_settings( [
      'configuration-setup-rules-flushed'  => 0,
    ], true );
  }

  public function _after_background_migration() {
    WPEP_Settings::instance()->update_settings( [
      'show-lesson-hover-color'      => '1',
      'wpep-theme'                   => WPEP_Controller::instance()->templateStyle->default_theme
    ], true );
  }

  public function handle_migration_in_background() {
    if( $this->current_version != '0.0.0' )
      return false;

    return true;
  }

}