<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\Course as WPEP_Course;

class UserActivity {

  /**
   * @var UserActivity\Assessment
   */
  public $assessment;

  /**
   * @var UserActivity\Statistics
   */
  public $statistics;

  private $_user_data_cache = [];

  private $_course_section_current_time = 0;
  private $_course_sections_user_started_at = 0;

  public function setup() {
    $this->assessment = new UserActivity\Assessment();
    $this->statistics = new UserActivity\Statistics();
  }

  public function get_recommended_content( $user_id = false, $limit = 4 ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $courses_started   = wpep_utility_map_to_array_assoc( $this->get_course_user_data( WPEP_USER_COURSE_STARTED, false, false, false, $user_id, false ), 'course_id', 'value' );
    $courses_completed = wpep_utility_map_to_array_assoc( $this->get_course_user_data( WPEP_USER_COURSE_COMPLETED_AT, false, false, false, $user_id, false ), 'course_id', 'value' );
    $response = array_diff_key( $courses_started, $courses_completed);

    if( empty( $response ) )
      return apply_filters( 'wpep_user_activity_recommended_course_ids', [], $user_id, $limit, $courses_started, $courses_completed );

    arsort( $response, SORT_NUMERIC );

    $response = array_keys( $response );

    return apply_filters( 'wpep_user_activity_recommended_course_ids', array_slice( $response, 0, $limit ), $user_id, $limit, $courses_started, $courses_completed );
  }

  public function last_course_interaction_by_flag( $key = WPEP_USER_COURSE_STARTED, $user_id = false, $plain = true ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    global $wpdb;

    $sql = 'SELECT * FROM ' . $wpdb->prefix . 'wpep_course_user_data cud 
                    WHERE cud.user_id = ' . intval( $user_id ) . '
                      AND cud.key = "' . wpep_utilities_sanitize_input( $key ) . '"
                 ORDER BY cud.value DESC';

    $entry = $wpdb->get_row( $sql );

    if( $entry == null )
      return '-';

    if( $plain )
      return get_the_title( $entry->course_id ) . ' ' . wpep_format_date_with_time( $entry->value );

    return '<span data-wpep-information="title"><a href="' . get_permalink( $entry->course_id ) . '">' . get_the_title( $entry->course_id ) . '</a></span> ' .
           '<span data-wpep-information="date-time">' . wpep_format_date_with_time( $entry->value ) . '</span>';
  }

  public function get_course_bookmark_status( $course_id, $user_id = false ) {
    return $this->get_course_user_data_value(
      WPEP_USER_COURSE_IS_BOOKMARKED,
      $course_id,
      false,
      false,
      $user_id,
      true
    );
  }


  public function set_course_bookmark_status( $course_id, $is_bookmarked, $user_id = false ) {
    return $this->set_course_user_data(
      WPEP_USER_COURSE_IS_BOOKMARKED,
      intval( $is_bookmarked ),
      $course_id,
      false,
      false,
      $user_id
    );
  }

  public function set_lesson_status( $lesson_id, $is_completed = true, $user_id = false, $created_at = false, $set_progress_flags = true ) {
    $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

    if( $user_id == 0 ) {
      do_action( "wpep_user_activity_lesson_status", $lesson_id, $is_completed, $user_id );
      return;
    }

    $lesson_id    = intval( $lesson_id );
    $is_completed = intval( $is_completed );
    $course_id    = WPEP_Course::instance()->get_course_id_by_lesson_id( $lesson_id );

    global $wpdb;

    $wpdb->delete( $wpdb->prefix . 'wpep_section_lesson_completed', [
      'user_id'   => $user_id,
      'lesson_id' => $lesson_id
    ] );

    if( $is_completed ) {
      $wpdb->insert( $wpdb->prefix . 'wpep_section_lesson_completed', [
        'user_id'    => $user_id,
        'lesson_id'  => $lesson_id,
        'created_at' => ( $created_at == false ? current_time( 'mysql' ) : $created_at )
      ] );
    }

    if( $set_progress_flags ) {
      $this->set_course_user_data(
        WPEP_USER_COURSE_PROGRESS,
        $this->course_progress_percentage_raw( $course_id, $user_id ),
        $course_id,
        0,
        0,
        $user_id
      );

      if( $this->course_progress_percentage( $course_id, $user_id ) == 100 ) {
        $this->set_course_status_completed( $course_id, $user_id, false );
      } else {
        $this->set_course_status_incomplete( $course_id, $user_id, false );
      }

    }
    
    $started_at = $this->get_course_user_data_value( WPEP_USER_COURSE_STARTED, $course_id, 0, 0, $user_id, true );

    if( $started_at === null )
      $this->set_course_user_data( WPEP_USER_COURSE_STARTED, time(), $course_id, 0, 0, $user_id );

    $this->set_course_user_data( WPEP_USER_COURSE_LATEST_INTERACTION, time(), $course_id, 0, 0, $user_id );

    do_action( "wpep_user_activity_lesson_status", $lesson_id, $is_completed, $user_id );
  }

  public function set_course_status_completed( $course_id, $user_id, $update_progress = true ) {
    $this->set_course_user_data(
      WPEP_USER_COURSE_COMPLETED,
      1,
      $course_id,
      0,
      0,
      $user_id
    );

    $this->set_course_user_data(
      WPEP_USER_COURSE_COMPLETED_AT,
      time(),
      $course_id, 0, 0, $user_id
    );

    if( $update_progress )
      $this->set_course_user_data(WPEP_USER_COURSE_PROGRESS, 100, $course_id, 0, 0, $user_id);
  }

  public function set_course_status_incomplete( $course_id, $user_id, $update_progress = true ) {
    $this->set_course_user_data(
      WPEP_USER_COURSE_COMPLETED,
      0,
      $course_id,
      0,
      0,
      $user_id
    );

    $this->remove_course_user_data(
      WPEP_USER_COURSE_COMPLETED_AT,
      null,
      $course_id, 0, 0, $user_id
    );

    if( $update_progress )
      $this->set_course_user_data(WPEP_USER_COURSE_PROGRESS, 0, $course_id, 0, 0, $user_id);
  }

  public function set_section_learning_path( $course_id, $section_id, $learning_path_index, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return;

    $this->set_course_user_data( 'learning_path', $learning_path_index, $course_id, $section_id, 0, $user_id );
    $this->set_course_user_data( WPEP_USER_COURSE_LATEST_INTERACTION, time(), $course_id, 0, 0, $user_id );

    do_action( "wpep_user_set_section_learning_path", $course_id, $section_id, $learning_path_index, $user_id );
  }

  public function has_completed_course( $course_id = false, $user_id = false, $including_assessments = false ) {
    $course_id = ( $course_id == false ? get_the_ID()          : $course_id );
    $user_id   = ( $user_id   == false ? get_current_user_id() : $user_id );

    if( $course_id == 0 || $user_id == 0 )
      return false;

    $is_completed = $this->get_course_user_data_value( WPEP_USER_COURSE_COMPLETED, $course_id, 0, 0, $user_id, true );

    if( ( $is_completed === null ? false : intval( $is_completed ) ) == false )
      return false;

    if( $including_assessments == false )
      return true;

    $remaining_assessments = wpep_get_assessment_requirements_for_course( $course_id, $user_id );

    if( empty( $remaining_assessments ) )
      return true;

    return false;
  }

  public function get_course_user_data_value( $key = false, $course_id = false, $section_id = false, $lesson_id = false, $user_id = false, $single_row = true ) {
    if( $single_row ) {
      $response = null;

      $user_data = $this->get_course_user_data( $key, $course_id, $section_id, $lesson_id, $user_id, true );

      if( $user_data !== null )
        return $user_data->value;

      return $response;
    }

    $response = [];

    foreach( $this->get_course_user_data( $key, $course_id, $section_id, $lesson_id, $user_id, false ) as $user_data )
      $response[$user_data->id] = $user_data->value;

    return $response;
  }

  public function get_course_user_data( $key = false, $course_id = false, $section_id = false, $lesson_id = false, $user_id = false, $single_row = true, $value = false ) {
    if ( $single_row ) {
      $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

      if ($user_id == 0 )
        return null;
    }

    if( $single_row
        && $section_id === false
        && $lesson_id === false
        && isset( $this->_user_data_cache[ $user_id . '_' . $key . '_' . $course_id ] ) )
      return $this->_user_data_cache[ $user_id . '_' . $key . '_' . $course_id ];

    global $wpdb;

    $sql = 'SELECT * FROM ' . $wpdb->prefix . 'wpep_course_user_data cud WHERE 1 = 1 ';

    if( $user_id != 0 )
      $sql .= ' AND cud.user_id = ' . intval( $user_id );

    if( $key !== false )
      $sql .= ' AND cud.key = "' . wpep_utilities_sanitize_input( $key ) . '"';

    if( $course_id !== false )
      $sql .= ' AND cud.course_id  = ' . intval( $course_id );

    if( $section_id !== false )
      $sql .= ' AND cud.section_id = ' . intval( $section_id );

    if( $lesson_id !== false )
      $sql .= ' AND cud.lesson_id  = ' . intval( $lesson_id );

    if( $value !== false ) {
      if( is_array( $value ) && isset( $value[0] ) && isset( $value[1] )  ) {
        if( $value[0] !== false )
          $sql .= ' AND cud.value  >= ' . intval( $value[0] );
        if( $value[1] !== false )
          $sql .= ' AND cud.value  <= ' . intval( $value[1] );
      } else {
        $sql .= ' AND cud.value  = ' . intval( $value );
      }
    }

    if( $single_row ) {
      $response = $wpdb->get_row( $sql );

      if( $section_id === false && $lesson_id === false )
        $this->_user_data_cache[ $user_id . '_' . $key . '_' . $course_id ] = $response;

      return $response;
    }

    return $wpdb->get_results( $sql );
  }

  public function set_course_user_data( $key, $value, $course_id, $section_id, $lesson_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return false;

    $course_id  = intval( $course_id );
    $section_id = intval( $section_id );
    $lesson_id  = intval( $lesson_id );

    $existent_entry = $this->get_course_user_data( $key, $course_id, $section_id, $lesson_id, $user_id, true );

    $value_updated = true;

    if( $existent_entry === null
        || $existent_entry === false )
      $this->add_course_user_data( $key, $value, $course_id, $section_id, $lesson_id, $user_id );
    else if( $existent_entry->value != $value ) {
      global $wpdb;

      $wpdb->update(
        $wpdb->prefix . 'wpep_course_user_data',
        [ 'value' => $value ],
        [ 'id'    => $existent_entry->id ]
      );
    } else {
      $value_updated = false;
    }

    if( !$value_updated )
      return true;

    if( isset( $this->_user_data_cache[ $user_id . '_' . $key . '_' . $course_id ] ) )
      unset( $this->_user_data_cache[ $user_id . '_' . $key . '_' . $course_id ] );

    do_action( "wpep_user_set_course_data", $key, $value, $course_id, $section_id, $lesson_id, $user_id, $existent_entry );

    return true;
  }

  public function add_course_user_data( $key, $value, $course_id, $section_id = 0, $lesson_id = 0, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return;

    $course_id  = intval( $course_id );
    $section_id = intval( $section_id );
    $lesson_id  = intval( $lesson_id );

    global $wpdb;

    $wpdb->insert( $wpdb->prefix . 'wpep_course_user_data', [
      'user_id'    => $user_id,
      'course_id'  => $course_id,
      'section_id' => $section_id,
      'lesson_id'  => $lesson_id,
      'key'        => $key,
      'value'      => $value,
    ] );

    do_action( "wpep_user_add_course_data", $key, $value, $course_id, $section_id, $lesson_id, $user_id );
  }

  public function remove_course_user_data( $key = null, $value = null, $course_id = null, $section_id = null, $lesson_id = null, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return null;

    $delete_where = [ 'user_id' => $user_id ];

    if( $key !== null )
      $delete_where['key'] = wpep_utilities_sanitize_input( $key );

    if( $value !== null )
      $delete_where['value'] = wpep_utilities_sanitize_input( $value );

    if( $course_id !== null )
      $delete_where['course_id'] = intval( $course_id );

    if( $section_id !== null )
      $delete_where['section_id'] = intval( $section_id );

    if( $lesson_id !== null )
      $delete_where['lesson_id'] = intval( $lesson_id );

    global $wpdb;

    $wpdb->delete(
      $wpdb->prefix . 'wpep_course_user_data',
      $delete_where
    );

    if( isset( $this->_user_data_cache[ $user_id . '_' . $key . '_' . $course_id ] ) )
      unset( $this->_user_data_cache[ $user_id . '_' . $key . '_' . $course_id ] );

    do_action( "wpep_user_remove_course_data", $key, $value, $course_id, $section_id, $lesson_id, $user_id );
  }

  /**
   * @param $section_id
   * @param bool $user_id
   * @return array
   */
  public function get_course_section( $section_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $section_completed = true;
    $user_section = apply_filters( 'wpep_user_activity_course_section', $this->get_section_map( $section_id, $user_id ) );

    if( $user_section === null )
      return null;

    $response     = [
      'id'           => $section_id,
      'title'        => $user_section->title,
      'content'      => $user_section->content,
    ];

    foreach( $user_section->lessons as $lesson_key => $lesson_information ) {

      $is_completed = apply_filters( 'wpep_user_activity_lesson_is_completed', $lesson_information->is_completed, $lesson_information->id, $user_id, $lesson_information );

      $response['lessons'][$lesson_information->id] = [
        'title'        => $lesson_information->title,
        'content'      => $lesson_information->content,
        'is_completed' => $is_completed
      ];

      if( $is_completed == false )
        $section_completed = false;

    }

    $response['is_completed'] = $section_completed;

    return $response;
  }

  /**
   * This function will return content as needed based on the Timing Options if enabled.
   * @param $course_id
   * @param bool $user_id
   * @return array
   */
  public function get_course_sections( $course_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $user_sections = $this->course_map( $course_id, $user_id, true );
    $response      = [];

    $remaining_sections = count( $user_sections );

    $args  = apply_filters( 'wpep_user_activity_course_sections_args', [
      'available_sections' => [],
      'available_lessons'  => []
    ] );

    $this->_course_section_current_time = time();

    try {
      if( $user_id === 0 )
        $user_started_at = $this->_course_section_current_time;
      else
        $user_started_at = intval( $this->get_course_user_data_value( WPEP_USER_COURSE_STARTED, $course_id, 0, 0, $user_id, true ) );

      $this->_course_sections_user_started_at = $user_started_at;
    } catch ( \Exception $e ) {
      $this->_course_sections_user_started_at = $this->_course_section_current_time;
    }

    $_allow_learning_journey = wpep_get_setting_flag( 'enable-course-timing-option' );

    if( $_allow_learning_journey ) {
      if( get_post( $course_id )->post_author == get_current_user_id() || current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
        $_allow_learning_journey = false;
    }

    foreach( $user_sections as $user_section ) {
      $section_locked = ( !empty( $args['available_sections' ] ) && !in_array( $user_section->id, $args['available_sections' ] ) );

      if( $_allow_learning_journey && !$section_locked ) {
        $journey_status = $this->_get_learning_journey_status( $user_section, $user_id );

        if( $journey_status === 'lock' )
          $section_locked = true;
      }

      $response[$user_section->id] = [
        'title'        => $user_section->title,
        'content'      => $user_section->content,
        'lessons'      => [],
        'options'      => ( isset( $user_section->options )   ? ( is_object( $user_section->options ) ? get_object_vars( $user_section->options ) : $user_section->options ) : [] ),
        'user_data'    => ( isset( $user_section->user_data ) ? ( is_object( $user_section->user_data ) ? get_object_vars( $user_section->user_data ) : $user_section->user_data ) : [] ),
        'is_final'     => ( $remaining_sections == 1 )
      ];

      if( wpep_assessments_enabled() )
        $response[$user_section->id]['assessment_id'] = wpep_user_assessment_id_by_course_id( $course_id, $user_section->id, $user_id );

      $response[$user_section->id] = $this->_get_course_sections_attach_lessons( $course_id, $response[$user_section->id], $user_section->lessons, $user_id, [
        'remaining_sections' => $remaining_sections,
        'available_lessons'  => ( $section_locked ? false : $args['available_lessons'] )
      ] );

      $remaining_sections--;
    }

    $response = apply_filters( 'wpep_user_activity_course_sections', $response, $course_id );

    return $response;
  }

  private function _get_course_sections_attach_lessons( $course_id, $current_section, $lessons, $user_id, $args = [ 'remaining_sections' => 1, 'available_lessons' => [] ] ) {
    $learning_path     = isset( $current_section['user_data']['learning_path'] ) ? $current_section['user_data']['learning_path'] : false;
    $remaining_section_lessons = count( $lessons );

    $_allow_learning_journey = wpep_get_setting_flag( 'enable-course-timing-option' ) && empty( $args['available_lessons' ] );

    if( $_allow_learning_journey ) {
      if( get_post( $course_id )->post_author == get_current_user_id() || current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
        $_allow_learning_journey = false;
    }

    $_indicator_section_completed = true;

    $current_section['options']['disabled_lessons'] = 0;

    foreach( $lessons as $lesson_key => $lesson_information ) {
      $is_completed = apply_filters( 'wpep_user_activity_lesson_is_completed', $lesson_information->is_completed, $lesson_information->id, $user_id, $lesson_information );

      $lesson_options = ( isset( $lesson_information->options ) ? ( is_object( $lesson_information->options ) ? get_object_vars( $lesson_information->options ) : $lesson_information->options ) : [] );
      $lesson_locked   = $args['available_lessons' ] === false ? false : ( empty( $args['available_lessons' ] ) || in_array( $lesson_information->id, $args['available_lessons'] ) );

      if( $lesson_locked && !$is_completed && $_allow_learning_journey ) {
        $journey_status = $this->_get_learning_journey_status( $lesson_information, $user_id );

        if( $journey_status === 'lock' )
          $lesson_locked = false;
      }

      $current_section['lessons'][$lesson_information->id] = [
        'title'           => $lesson_information->title,
        'content'         => '',
        'is_completed'    => false,
        'completed_at'    => $lesson_information->completed_at,
        'options'         => $lesson_options,
        'is_final'        => ( $args['remaining_sections'] == 1 && $remaining_section_lessons == 1 )
      ];

      if( !$lesson_locked ) {
        $current_section['lessons'][$lesson_information->id]['options']['is_disabled'] = true;

        if( isset( $current_section['lessons'][$lesson_information->id]['options']['downloads_list'] ) )
          unset( $current_section['lessons'][$lesson_information->id]['options']['downloads_list'] );


        $current_section['options']['disabled_lessons']++;

        $_indicator_section_completed = false;

        continue;
      }

      $current_section['lessons'][$lesson_information->id]['content'] = $lesson_information->content;

      if( $is_completed )
        $current_section['lessons'][$lesson_information->id]['is_completed'] = $is_completed;

      $remaining_section_lessons--;

      if( $is_completed == true )
        continue;

      if( $learning_path !== false ) {
        if( isset( $lesson_options['question_answer'] )
          && intval( $lesson_options['question_answer'] ) !== -1
          && $lesson_options['question_answer'] !== $learning_path
        )
          continue;
      }

      $_indicator_section_completed = false;
    }

    if( $current_section['options']['disabled_lessons'] == count( $current_section['lessons'] ) ) {
      $current_section['options']['is_disabled'] = true;
      $current_section['content'] = '';

      if( isset( $current_section['options']['downloads_list'] ) )
        unset( $current_section['options']['downloads_list'] );
    }

    $current_section['is_completed'] = $_indicator_section_completed;

    return $current_section;
  }

  /**
   * @param $object
   * @param $user_id
   * @return string
   */
  private function _get_learning_journey_status( $object, $user_id ) {
    if( !$this->_is_learning_journey_condition_met( $object, 'journey_access', WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_INSTANT ) )
      return 'lock';

    if( $this->_is_learning_journey_condition_met( $object, 'journey_expiration', WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_NONE ) )
      return 'lock';

    return 'ok';
  }

  private function _is_learning_journey_condition_met( $object, $type_field_prefix, $default_type = WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_INSTANT ) {
    $type_field_key    = $type_field_prefix . '_type';
    $segment_field_key = $type_field_prefix . '_segment';

    if( $object->$type_field_key === '' )
      $object->$type_field_key = $default_type;

    if( $object->$type_field_key === WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_NONE )
      return false;

    if( $object->$type_field_key === WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_INSTANT )
      return true;

    if( in_array( $object->$type_field_key, [ WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_SPECIFIC_DATE, WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_DRIPPED, WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_DRIPPED_SECONDS ] )
        && $object->$segment_field_key === '' )
      return true;

    if( $this->_course_sections_user_started_at === 0 )
      return false;

    if( $object->$type_field_key === WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_DRIPPED ) {
      try {
        $current_allow_action = new \DateTime( '@' . $this->_course_sections_user_started_at );
      } catch ( \Exception $e ) {
        return false;
      }

      $current_allow_action->modify( $object->$segment_field_key );

      return ( $this->_course_section_current_time > $current_allow_action->setTime(0,0 )->getTimestamp() );
    }

    if( $object->$type_field_key === WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_SPECIFIC_DATE ) {
      try {
        $current_allow_action = new \DateTime( $object->$segment_field_key );
      } catch ( \Exception $e ) {
        return false;
      }

      return ( $this->_course_section_current_time > $current_allow_action->getTimestamp() );
    }

    if( $object->$type_field_key === WPEP_LEARNING_JOURNEY_DELIVERY_TYPE_DRIPPED_SECONDS )
      return $this->_course_sections_user_started_at + intval( $object->$segment_field_key ) < time();

    return false;
  }

  /**
   * @param $course_id
   * @param bool $user_id
   * @param bool $include_lessons
   * @return array
   */
  public function course_map( $course_id, $user_id = false, $include_lessons = true ) {
    global $wpdb;

    if( $user_id === false )
      $user_id = get_current_user_id();

    $response = [];

    $sql = 'SELECT * FROM ' . $wpdb->prefix . 'wpep_section WHERE post_id = ' . $course_id . ' ORDER BY `order` ASC';

    $course_sections = $wpdb->get_results($sql);

    foreach( $course_sections as $course_section ) {
      $response[$course_section->id] = $course_section;
      $response[$course_section->id]->user_data = wpep_utility_map_to_object_assoc ( $this->get_course_user_data( false, $course_id, $course_section->id, false, $user_id, false ), 'key', 'value' );
      $response[$course_section->id]->options = ( $course_section->options == '' ? new \stdClass() : json_decode( $course_section->options ) );

      if( $include_lessons )
        $response[$course_section->id]->lessons = $this->get_section_lessons_map( $course_section->id, $user_id );
    }

    return $response;
  }

  /**
   * This function will return content as needed based on the Timing Options if enabled.
   * @param $section_id
   * @param bool $user_id
   * @return array|null|object
   */
  public function get_section_map( $section_id, $user_id = false ) {
    global $wpdb;

    if( $user_id === false )
      $user_id = get_current_user_id();

    $sql = 'SELECT * FROM ' . $wpdb->prefix . 'wpep_section WHERE id = ' . $section_id . ' ORDER BY `order` ASC';

    $course_section = $wpdb->get_row($sql);

    if( is_null( $course_section ) )
      return null;

    $course_section->lessons = $this->get_section_lessons_map( $section_id, $user_id );

    return $course_section;
  }

  /**
   * @param $lesson_id
   * @param bool $user_id
   * @param bool $fetch_next_previous
   * @return array|null|object
   */
  public function get_course_lesson( $lesson_id, $user_id = false, $fetch_next_previous = true ) {
    global $wpdb;

    if( !is_numeric( $lesson_id ) )
      return null;

    if( $user_id === false )
      $user_id = get_current_user_id();

    $sql = 'SELECT l.*,
                   s.post_id as course_id,
                   s.id      as section_id,
                   s.title   as section_title,
                   IF(lc.id IS NULL, 0, 1) as is_completed 
              FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section s ON s.id = l.section_id ';

    $sql .=  'WHERE l.id = ' . $lesson_id . ' LIMIT 1';

    $course_lesson = $wpdb->get_row($sql);

    if( $course_lesson == null )
      return null;

    if( $fetch_next_previous ) {
      $raw_query = ' SELECT inner_lesson.* FROM ' . $wpdb->prefix . 'wpep_section_lesson inner_lesson
                         LEFT JOIN ' . $wpdb->prefix . 'wpep_section inner_section
                                ON inner_lesson.section_id = inner_section.id
                             WHERE inner_lesson.post_id = ' . $course_lesson->post_id;

      $previous_lesson = $wpdb->get_row( $raw_query . ' AND inner_lesson.order < ' . $course_lesson->order . ' ORDER BY inner_lesson.order DESC' );
      $next_lesson = $wpdb->get_row( $raw_query . ' AND inner_lesson.order > ' . $course_lesson->order . ' ORDER BY inner_lesson.order ASC' );

      $course_lesson->previous_lesson_id         = ( $previous_lesson == null ? null : $previous_lesson->id );
      $course_lesson->previous_lesson_title      = ( $previous_lesson == null ? null : $previous_lesson->title );
      $course_lesson->previous_lesson_section_id = ( $previous_lesson == null ? null : $previous_lesson->section_id );
      $course_lesson->next_lesson_id             = ( $next_lesson == null ? null : $next_lesson->id );
      $course_lesson->next_lesson_title          = ( $next_lesson == null ? null : $next_lesson->title );
      $course_lesson->next_lesson_section_id     = ( $next_lesson == null ? null : $next_lesson->section_id );
    }

    $course_lesson->is_completed = apply_filters( 'wpep_user_activity_lesson_is_completed', $course_lesson->is_completed, $course_lesson->id, $user_id, $course_lesson );

    return $course_lesson;
  }

  /**
   * @param $lesson_id
   * @param bool $user_id
   * @return array|null|object
   */
  public function get_course_next_lesson_id( $lesson_id, $user_id = false ) {
    global $wpdb;

    if( $user_id === false )
      $user_id = get_current_user_id();

    $sql = 'SELECT l.*
              FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section s ON s.id = l.section_id ';

    $sql .=  'WHERE l.id = ' . $lesson_id . ' LIMIT 1';

    $course_lesson = $wpdb->get_row($sql);

    if( $course_lesson == null )
      return null;

    $raw_query = ' SELECT inner_lesson.* FROM ' . $wpdb->prefix . 'wpep_section_lesson inner_lesson
                       LEFT JOIN ' . $wpdb->prefix . 'wpep_section inner_section
                              ON inner_lesson.section_id = inner_section.id
                           WHERE inner_lesson.post_id = ' . $course_lesson->post_id;

    $next_lesson = $wpdb->get_row( $raw_query . ' AND inner_lesson.order > ' . $course_lesson->order . ' ORDER BY inner_lesson.order ASC' );

    return ( $next_lesson == null ? 0 : $next_lesson->id );
  }

  /**
   * @param $course_id
   * @param bool $user_id
   * @return string|null
   */
  public function get_delivered_course_section_id( $course_id, $user_id = false ) {
    return $this->_get_delivered_course_variable( $course_id, $user_id, 'l.section_id' );
  }

  /**
   * @param $course_id
   * @param bool $user_id
   * @return string|null
   */
  public function get_delivered_course_lesson_id( $course_id, $user_id = false ) {
    return $this->_get_delivered_course_variable( $course_id, $user_id, 'l.id' );
  }

  /**
   * @param $course_id
   * @param bool $user_id
   * @param string $param
   * @return string|null
   */
  private function _get_delivered_course_variable( $course_id, $user_id = false, $param = 'l.id' ) {
    global $wpdb;

    if( $user_id === false )
      $user_id = get_current_user_id();

    $sql = 'SELECT ' . $param . '
                  FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section s
                     ON s.id = l.section_id
              WHERE lc.id IS NULL
                AND l.post_id = ' . intval( $course_id ) . ' ORDER BY l.order ASC';


    $resp = $wpdb->get_var($sql);

    if( is_null( $resp ) ) {
      $sql = 'SELECT ' . $param . '
                  FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section s
                     ON l.section_id = s.id
              WHERE l.post_id = ' . intval( $course_id ) . ' ORDER BY l.order ASC';

      $resp = $wpdb->get_var($sql);
    }

    return $resp;
  }

  /**
   * This function will return content as needed based on the Timing Options if enabled.
   * @param $section_id
   * @param bool $user_id
   * @return array|null|object
   */
  public function get_section_lessons_map( $section_id, $user_id = false ) {
    global $wpdb;

    if( $user_id === false )
      $user_id = get_current_user_id();

    $section_id = intval( $section_id );

    $sql = 'SELECT l.*, 
                   IF(lc.id IS NULL, 0, 1) as is_completed,
                   IF(lc.created_at IS NULL, 0, lc.created_at) as completed_at
                FROM ' . $wpdb->prefix . 'wpep_section_lesson l
               LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                    ON l.id       = lc.lesson_id
                   AND lc.user_id = ' . intval( $user_id ) . '
               WHERE l.section_id = ' . $section_id;

    $sql .= ' ORDER BY l.`order` ASC';

    $response = $wpdb->get_results($sql);

    foreach( $response as $response_key => $response_value )
      $response[$response_key]->options = ( $response_value->options == '' ? new \stdClass() : json_decode( $response_value->options ) );

    return $response;
  }

  public function get_course_lesson_completed_date( $course_id, $user_id = false, $order = 'ASC' ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    global $wpdb;

    $sql = 'SELECT lc.created_at
                  FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              WHERE l.post_id = ' . intval( $course_id ) . '      
              ORDER BY lc.created_at ' . ( $order == 'ASC' ? 'ASC' : 'DESC' );

    $date = $wpdb->get_var( $sql );

    return ( is_string( $date ) ? strtotime( $date ) : 0 );
  }

  public function course_progress_percentage( $course_id, $user_id = false, $option_section_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    $course_progress = $this->get_course_user_data( WPEP_USER_COURSE_PROGRESS, $course_id, false, false, $user_id );

    if( $course_progress !== null && isset( $course_progress->value ) && $option_section_id == false ) {
      $course_progress = floatval( $course_progress->value );

    } else if( get_post_meta( $course_id, 'wpep_learning_path_enabled', true ) ) {
      $course_progress = floatval( 0 );
    } else {
      $course_progress = $this->course_progress_percentage_raw( $course_id, $user_id, $option_section_id );
    }


    return apply_filters( 'wpep_user_activity_course_progress_percentage', $course_progress, $course_id, $user_id, $course_progress );
  }

  public function course_progress_percentage_raw( $course_id, $user_id = false, $option_section_id = false ) {
    global $wpdb;
    $section_id_query = 'IN (
        SELECT id FROM ' . $wpdb->prefix . 'wpep_section WHERE post_id = ' . intval( $course_id ) . '
      )';

    if( is_numeric( $option_section_id ) && $option_section_id != false )
      $section_id_query = '= ' . $option_section_id;
    else if( is_array( $option_section_id ) )
      $section_id_query = 'IN ( ' . implode( ',', $option_section_id ) . ' )';

    $sql = 'SELECT ( SUM(IF(lc.id IS NULL, 0, 1)) / COUNT(l.id) * 100 ) as completed_percent, 
                   COUNT(l.id) as lesson_count
              FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              WHERE section_id ' . $section_id_query . '
              ORDER BY `order` ASC';

    return floatval( $wpdb->get_var( $sql ) );
  }

  public function course_progress_reset( $course_id, $user_id = false, $option_section_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return false;

    global $wpdb;

    $section_id_query = 'IN (
      SELECT id FROM ' . $wpdb->prefix . 'wpep_section WHERE post_id = ' . intval( $course_id ) . '
    )';

    if( is_numeric( $option_section_id ) && $option_section_id != false )
      $section_id_query = '= ' . $option_section_id;
    else if( is_array( $option_section_id ) )
      $section_id_query = 'IN ( ' . implode( ',', $option_section_id ) . ' )';


    $sql = 'DELETE lc FROM ' . $wpdb->prefix . 'wpep_section_lesson l
              LEFT JOIN ' . $wpdb->prefix . 'wpep_section_lesson_completed lc
                     ON l.id       = lc.lesson_id
                    AND lc.user_id = ' . intval( $user_id ) . '
              WHERE section_id ' . $section_id_query;

    $wpdb->query( $sql );

    $this->remove_course_user_data( WPEP_USER_COURSE_COMPLETED_AT, null, $course_id, null, null, $user_id );
    $this->remove_course_user_data( WPEP_USER_COURSE_COMPLETED, null, $course_id, null, null, $user_id );
    $this->remove_course_user_data( WPEP_USER_COURSE_PROGRESS, null, $course_id, null, null, $user_id );
    $this->remove_course_user_data( WPEP_USER_COURSE_STARTED, null, $course_id, null, null, $user_id );

    $this->add_course_user_data(WPEP_USER_COURSE_REPEAT_TRACKER, time(), $course_id, 0, 0, $user_id );

    $assessment_ids = wpep_get_course_assessment_ids( $course_id );

    if( empty( $assessment_ids ) )
      return true;

    foreach( $assessment_ids as $assessment_id ) {
      $this->assessment->delete_user_data(
        false,
        $assessment_id,
        false,
        false,
        $user_id
      );
    }

    return true;
  }

  public function get_registration_count( $filter_by = false ) {
    global $wpdb;

    $query_entries_total = 'SELECT COUNT( * ) FROM `' . $wpdb->users . '` WHERE 1 = 1';

    if( isset( $filter_by['time_from'] ) )
      $query_entries_total .= ' AND `user_registered` >= "' . date( "Y-m-d H:i:s", $filter_by['time_from'] ) . '"';

    if( isset( $filter_by['time_to'] ) )
      $query_entries_total .= ' AND `user_registered` < "' . date( "Y-m-d H:i:s", $filter_by['time_to'] ) . '"';

    return absint( $wpdb->get_var( $query_entries_total ) );
  }

}