<?php

namespace WPEP\Coordinator\UserEngagement;

use WPEP\Controller as WPEP_Controller;

class Assessment {

  public function __construct() {
    if( wpep_get_setting( 'assessment-admin-email-notification-deliver-to' ) != 0 )
      add_action( "wpep_user_assessment_after_grading", [ $this, '_event_user_assessment_after_grading' ], 10, 2 );
  }

  public function _event_user_assessment_after_grading( $assessment_id, $user_id ) {
    $email_title = wpep_get_setting( 'assessment-admin-email-notification-title' );
    $email_content = wpep_get_setting( 'assessment-admin-email-notification-content' );

    $replace_data = [
      'assessment_name'             => get_the_title( $assessment_id ),
      'assessment_student_name'     => wpep_content_user_profile_format_name( $user_id ),
      'assessment_student_status'   => wpep_get_setting_translated( 'assessment-status-label-' . wpep_user_assessment_status( $assessment_id, $user_id ) ),
      'assessment_student_score'    => intval( WPEP_Controller::instance()->userActivity->assessment->get_user_data_value( WPEP_USER_ASSESSMENT_SCORE, $assessment_id, false, false, $user_id ) ),
      'assessment_student_answers'  => wpep_template_get( 'email-template/assessment-answer-list.php', [
        'assessment_id' => $assessment_id,
        'user_id'       => $user_id
      ] ),
      'assessment_student_submission_link' => wpep_admin_url_statistics( [ 'tab' => 'assessments', 'assessment_id' => $assessment_id, 'user_id' => $user_id ] ),
    ];

    foreach( $replace_data as $replace_key => $replace_value ) {
      $email_title = str_replace( '[' . $replace_key . ']', $replace_value, $email_title );
      $email_content = str_replace( '[' . $replace_key . ']', $replace_value, $email_content );
    }

    WPEP_Controller::instance()->userEngagement->send_email(
      wpep_get_setting( 'assessment-admin-email-notification-deliver-to' ),
      $email_title,
      $email_content,
      true
    );
  }


}
