<?php

namespace WPEP\Coordinator\UserEngagement;

use WPEP\Controller as WPEP_Controller;

class AutoLogin {

  public function __construct() {
    if( !wpep_get_setting_flag( 'enable-auto-login-link' ) )
      return;

    if( isset( $_GET[ wpep_get_setting( 'auto-login-link-token' ) ] ) )
      add_action( 'init', [ $this, '_init_handle_login_tokens' ] );
  }

  public function _init_handle_login_tokens() {
    $url_param = wpep_get_setting( 'auto-login-link-token' );

    if( !isset( $_GET[ $url_param ] ) )
      return;

    $token = $_GET[ $url_param ];

    $user = get_users( [
      'meta_key'    => WPEP_USER_META_AUTO_LOGIN_TOKEN,
      'meta_value'  => $token,
      'number'      => 1,
      'count_total' => false
    ] );

    if( !empty( $user ) ) {
      $user = array_shift( $user );

      wp_set_current_user( $user->ID );
      wp_set_auth_cookie( $user->ID );
      do_action( 'wp_login', $user->user_login, $user );
    }

    $current_url = wpep_get_current_url();

    $current_url = remove_query_arg( $url_param, $current_url );

    wpep_redirect( $current_url );
    exit;
  }

  /**
   * @param bool $regenerate_tokens
   * @return array
   */
  public function get_data_export( $regenerate_tokens = false ) {
    $salt           = wpep_get_setting( 'auto-login-link-salt' );
    $url_identifier = wpep_get_setting( 'auto-login-link-token' );
    $site_url       = get_site_url();

    $users    = get_users( [ 'paged' => -1, 'orderby' => 'ID' ] );
    $response = [];

    foreach( $users as $user ) {
      $user_token = get_user_meta( $user->ID, WPEP_USER_META_AUTO_LOGIN_TOKEN, true );

      if( $user_token === '' || $regenerate_tokens ) {
        $user_token = sha1( $salt . wpep_generate_random_token( 32 ) );

        update_user_meta( $user->ID, WPEP_USER_META_AUTO_LOGIN_TOKEN, $user_token );
      }

      $response[ $user->ID ] = [
        'name'            => wpep_content_user_profile_format_name( $user ),
        'email_address'   => $user->user_email,
        'user_token'      => $user_token,
        "home_auto_login" => add_query_arg( $url_identifier, $user_token, $site_url )
      ];
    }
    if( wpep_get_setting( 'auto-login-link-last-generate' ) === 0 || $regenerate_tokens )
      wpep_set_setting( 'auto-login-link-last-generate', time() );

    return $response;
  }

}
