<?php

namespace WPEP\Entity\CourseType;

use WPEP\Controller as WPEP_Controller;

class LearnMap extends ExecutionPlan {

  public $has_next_suggestions = true;
  public $has_next_suggestions_after_completion = false;

  /**
   * LearnMap constructor.
   * @param $current_course_id
   */
  public function __construct( $current_course_id ) {
    $this->_skeleton_init( $current_course_id );

    $this->show_quick_continue = wpep_get_setting_flag( 'show-lesson-continue-button' );
    $this->load_lesson_content_on_demand = intval( get_post_meta( $current_course_id, 'wpep_lesson_content_on_demand', true ) );
  }

  public function get_alias() {
    return WPEP_COURSE_TYPE_LEARN_MAP;
  }

  public function init() {
    parent::init();
    add_filter( 'wpep_single_content_sidebar_active', [ $this, '_return_false' ], 50 );
  }

  public function close() {
    parent::close();
    remove_filter( 'wpep_single_content_sidebar_active', [ $this, '_return_false' ], 50 );
  }

  public function content() {
    $form_args = $this->get_form_args();

    $response = '';

    if( apply_filters('wpep_course_single_content_include_header', true ) )
      $response .= $this->get_header();

    $response .= apply_filters(
      'wpep_single_content_form_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/learn-map.php', $form_args ),
      $form_args
    );

    if( $this->top_level_request && wpep_course_has_discussion( $this->course_id ) )
      add_filter( 'wpep_single_content_learn_map_html', [ $this, '_content_attach_discussion_for_course' ], 1 );

    $response = apply_filters( 'wpep_single_content_learn_map_html', $response );

    if( $this->top_level_request && wpep_course_has_discussion( $this->course_id ) )
      remove_filter( 'wpep_single_content_learn_map_html', [ $this, '_content_attach_discussion_for_course' ], 1 );

    return $response;
  }

}