<?php

namespace WPEP\Entity\CourseType;

use WPEP\Controller as WPEP_Controller;

class Standard extends Skeleton {

  public $current_item    = 'course';
  public $current_item_id = 0;

  public $has_next_suggestions = true;
  public $has_next_suggestions_after_completion = true;

  private $_course_sections = [];

  /**
   * Standard constructor.
   * @param $current_course_id
   */
  public function __construct( $current_course_id ) {
    $this->_skeleton_init( $current_course_id );

    $this->course_has_index_page             = wpep_get_setting_flag( 'course-standard-allow-index-page' );
    $this->course_has_redirect_on_completion = wpep_get_setting_flag( 'course-standard-redirect-on-completion' );

    $course_has_index_page = get_post_meta( $this->course_id, 'wpep_standard_course_overview_page', true );

    if( $course_has_index_page != 'default' )
      $this->course_has_index_page = $course_has_index_page;

    add_action( 'wpep_course_single_content_standard_lessons_display', [ $this, 'display_lesson_list' ], 10, 3 );

    $this->_set_item_data();
  }

  public function has_sidebar_content() {
    return ( $this->current_item == $this->lesson_slug );
  }

  public function get_alias() {
    return "standard";
  }

  public function init() {
    parent::init();

    $this->_course_sections = WPEP_Controller::instance()->userActivity->get_course_sections( $this->course_id, get_current_user_id() );

    do_action( 'wpep_course_single_content_standard_coordinator_before_init', $this );

    $this->_init_item_based_course( false );
    
    do_action( 'wpep_course_single_content_standard_coordinator_after_init', $this );

    if( $this->has_sidebar_content() ) {
      add_filter( 'wpep_single_content_sidebar_active', [ $this, '_return_true' ], 20 );
      add_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
    } else if( wpep_get_setting_flag( 'course-standard-allow-index-page-widgets' ) == false ) {
      add_filter( 'wpep_single_content_sidebar_active', [ $this, '_return_false' ], 20 );
    }
  }

  public function close() {
    parent::close();

    $this->_course_sections = [];

    if( $this->has_sidebar_content() ) {
      remove_filter( 'wpep_single_content_sidebar_active', [ $this, '_return_true' ], 20 );
      remove_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
    } else if( wpep_get_setting_flag( 'course-standard-allow-index-page-widgets' ) == false ) {
      remove_filter( 'wpep_single_content_sidebar_active', [ $this, '_return_false' ], 20 );
    }
  }
  
  public function content() {
    if( $this->current_item == $this->lesson_slug )
      return $this->get_content_lesson( $this->current_item_id );

    return $this->get_content();
  }

  public function get_content() {
    $args = [
      'course_id'         => $this->course_id,
      'course_sections'   => $this->_course_sections,
      'course_title'      => get_the_title( $this->course_id ),
      'course_content'    => wpep_format_content( get_post_meta( $this->course_id , 'wpep_course_description', true) ),
    ];

    $args = apply_filters( 'wpep_course_single_content_standard_args', $args );

    return apply_filters(
      'wpep_single_content_standard_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/standard-content.php', $args ),
      $args
    );
  }

  public function get_content_lesson( $lesson_id ) {
    $course_lesson = WPEP_Controller::instance()->userActivity->get_course_lesson( $lesson_id, get_current_user_id(), true );

    if( $course_lesson == null )
      return '';

    if( isset( $this->_course_sections[ $course_lesson->section_id ][ 'lessons' ][ $course_lesson->id ][ 'options' ][ 'is_disabled' ] )
        && $this->_course_sections[ $course_lesson->section_id ][ 'lessons' ][ $course_lesson->id ][ 'options' ][ 'is_disabled' ] )
      return '';

    $lesson_content = wpautop( $course_lesson->content );

    if( !wpep_course_has_the_content_filter() || $this->top_level_request == false )
      $lesson_content = do_shortcode( $lesson_content );

    if( isset( $course_lesson->options ) ) {
      $course_lesson->options = ( is_string( $course_lesson->options ) ? json_decode( $course_lesson->options, true ) : $course_lesson->options );

      if( isset( $course_lesson->options['downloads_list'] ) && !empty( $course_lesson->options['downloads_list'] ) ) {
        $downloads_content = WPEP_Controller::instance()->template->get_template( 'utilities/downloads-list.php', [
          'downloads_list'  => $course_lesson->options['downloads_list']
        ] );

        if( wpep_get_setting( 'course-downloads-position' ) == 'top' )
          $lesson_content = $downloads_content . $lesson_content;
        else
          $lesson_content .= $downloads_content;

        unset( $downloads_content );
      }
    }

    $args = [
      'course_id'             => $this->course_id,
      'course_read_only'      => $this->read_only,
      'course_name'           => get_the_title( $this->course_id ),
      'course_has_index_page' => $this->course_has_index_page,
      'section_id'            => $course_lesson->section_id,
      'section_title'         => $course_lesson->section_title,
      'lesson_id'             => $course_lesson->id,
      'lesson_title'          => $course_lesson->title,
      'lesson_content'        => $lesson_content,
      'lesson_is_completed'       => $course_lesson->is_completed,
      'lesson_completed_message'  => wpep_get_setting_translated( 'course-standard-lesson-completed' ),
      'next_lesson_id'            => $course_lesson->next_lesson_id,
      'next_lesson_title'         => $course_lesson->next_lesson_title,
      'previous_lesson_id'        => $course_lesson->previous_lesson_id,
      'previous_lesson_title'     => $course_lesson->previous_lesson_title,
      'complete_lesson_text'      => wpep_get_setting_translated( 'course-standard-complete-lesson' ),
    ];

    if( $course_lesson->previous_lesson_id !== null
        && isset( $this->_course_sections[ $course_lesson->previous_lesson_section_id ][ 'lessons' ][ $course_lesson->previous_lesson_id ][ 'options' ][ 'is_disabled' ] )
        && $this->_course_sections[ $course_lesson->previous_lesson_section_id ][ 'lessons' ][ $course_lesson->previous_lesson_id ][ 'options' ][ 'is_disabled' ]
    )
      $args[ 'previous_lesson_id' ] = null;

    if( $course_lesson->next_lesson_id !== null
        && isset( $this->_course_sections[ $course_lesson->next_lesson_section_id ][ 'lessons' ][ $course_lesson->next_lesson_id ][ 'options' ][ 'is_disabled' ] )
        && $this->_course_sections[ $course_lesson->next_lesson_section_id ][ 'lessons' ][ $course_lesson->next_lesson_id ][ 'options' ][ 'is_disabled' ]
    )
      $args[ 'next_lesson_id' ] = null;

    $args = apply_filters( 'wpep_course_single_content_standard_lesson_args', $args );

    if( $this->top_level_request && wpep_course_has_discussion( $this->course_id ) )
      add_filter( 'wpep_single_content_standard_lesson_html', [ $this, '_content_attach_discussion_for_lesson' ], 1, 2 );

    $response = apply_filters(
      'wpep_single_content_standard_lesson_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/standard-lesson-content.php', $args ),
      $args
    );

    if( $this->top_level_request && wpep_course_has_discussion( $this->course_id ) )
      remove_filter( 'wpep_single_content_standard_lesson_html', [ $this, '_content_attach_discussion_for_lesson' ], 1 );

    return $response;
  }

  public function display_lesson_list(
    $lesson_list,
    $course_id = false,
    $assessment_section_id = false,
    $is_minimal = false,
    $is_visible = true
  ) {
    if( $course_id == false )
      $course_id = $this->course_id;

    $args = [
      'course_id'         => $course_id,
      'lesson_list'       => $lesson_list,
      'is_minimal'        => $is_minimal,
      'is_visible'        => $is_visible,
      'current_lesson_id' => ( $this->current_item == $this->lesson_slug ? $this->current_item_id : 0 )
    ];

    if( wpep_assessments_enabled() && $assessment_section_id !== false ) {
      $args['assessments'] = [];

      if( !is_array( $assessment_section_id ) )
        $assessment_section_id = [$assessment_section_id];

      foreach( $assessment_section_id as $section_id ) {
        $assessment_id = wpep_user_assessment_id_by_course_id( $course_id, $section_id );

        if( $assessment_id == 0 )
          continue;

        $args['assessments'][] = [
            'id'        => $assessment_id,
            'title'     => get_the_title( $assessment_id ),
            'permalink' => get_the_permalink( $assessment_id ),
            'status'    => wpep_user_assessment_status( $assessment_id )
        ];
      }
    }

    echo apply_filters(
      'wpep_course_single_content_standard_lessons_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/_standard_lesson_list.php', $args ),
      $args
    );
  }

  public function display_sidebar_content( $course_id ) {
    $course_id = ( $course_id == false ? get_the_ID() : $course_id );

    if( $course_id != $this->course_id )
      return;

    if( wpep_course_has_progress_bar( $this->course_id ) ) {
      echo '<div class="wpep-widget">';
      echo    '<div class="wpep-widget-wrapper">';
      echo      '<div class="standard-course-progress-widget">';
      echo        '<h2>' . wpep_get_setting_translated( 'course-standard-course-progress' ) . '</h2>';

      echo         wpep_progress_bar( WPEP_Controller::instance()->userActivity->course_progress_percentage( $this->course_id, get_current_user_id() )  );

      echo      '</div>';
      echo    '</div>';
      echo '</div>';
    }

    $course_section_count = count( $this->_course_sections );

    echo '<div class="wpep-widget">';
    echo    '<div class="wpep-widget-wrapper">';
    echo      '<div class="standard-course-widget">';

    echo        '<h2>' . wpep_get_setting_translated( 'course-standard-course-navigation' ) . '</h2>';
    echo        '<ul>';

    $current_section_index = 1; foreach( $this->_course_sections as $course_section_id => $course_section ) {
      $class = '';

      if( isset( $course_section['lessons'][$this->current_item_id] ) ) {
        $class .= 'current-section';
      }

      echo '<li class="' . $class . '">';
      echo    '<p>' . $course_section['title'] . '</p>';

      $this->display_lesson_list(
        $course_section['lessons'],
        $this->course_id,
        ( $course_section_count != $current_section_index ? $course_section_id : [ $course_section_id, 0 ] ),
        true,
        ( isset( $course_section['lessons'][$this->current_item_id] ) )
      );

      echo '</li>';
      $current_section_index++;
    }

    echo        '</ul>';
    echo      '</div>';
    echo    '</div>';
    echo '</div>';
  }

}