<?php

function wpep_administration_platform_health_table( $row_list, $table_title = '', $table_class = 'wpep-modules-table', $tab_operation = '&nbsp;' ) {
  echo '<table class="wp-list-table widefat striped ' . $table_class . '">';
  echo    '<thead>';
  echo      '<tr>';
  echo        '<th>' . $table_title . '</th>';
  echo        '<th class="wpep-row-button-container">' . $tab_operation . '</th>';
  echo      '</tr>';
  echo    '</thead>';
  echo    '<tbody>';

  $admin_url = admin_url();

  foreach( $row_list as $service_key => $service_information ) {
    $class = 'wpep-row-highlight';

    if( isset( $service_information['status'] ) && $service_information['status'] !== '' )
      $class .= ' wpep-row-highlight-' . $service_information['status'];

    echo      '<tr class="' . $class . '">';

    if( isset( $service_information[ 'operation' ] ) && ( !isset( $service_information[ 'operation_allowed'] ) || $service_information['operation_allowed'] == true ) ) {
      echo        '<td class="wpep-row-highlight-icon">' . $service_information[ 'title' ] . '</td>';
      echo        '<td class="wpep-row-button-container">';
      echo            '<a class="button button-primary" href="' . $service_information[ 'operation' ]  . '"';
      if( strpos( $service_information[ 'operation' ], $admin_url ) !== 0 )
        echo          ' target="_blank"';

      echo            '>' . $service_information[ 'operation_text' ] . '</a>';
      echo        '</td>';
    } else {
      echo '<td class="wpep-row-highlight-icon" colspan="2">' . $service_information[ 'title' ] . '</td>';
    }

    echo      '</tr>';
  }

  echo    '</tbody>';
  echo '</table>';
}

function wpep_administration_platform_health_confirmation_form( $list_items, $operation_name ) {
  echo '<form method="POST" class="wpep-confirm-danger-action-form">';
  echo   '<input type="hidden" name="wpep-operation" value="' . $operation_name . '"/>';
  echo   '<h2>' . __( "This Action cannot be undone", "wpep" ) . '</h2>';
  echo   '<ul>';

  foreach( $list_items as $list_item )
    echo '<li>' . $list_item . '</li>';

  echo   '</ul>';

  echo   '<button class="button">' . __( "Confirm", "wpep" ) . '</button>';
  echo '</form>';
}

function wpep_administration_migration_success_notice() {
  echo '<p class="wpep-migration-complete">' . __( "Database successfully updated. Continue to : ", 'wpep' ) . '</p>';
  echo '<p>';
  echo    '<a href="' . wpep_admin_url_dashboard() . '" class="button button-primary">' . __( "Dashboard", "wpep" ) . '</a> ';
  echo    '<a href="' . wpep_admin_url_settings() . '" class="button button-primary">' . __( "Manage Settings", "wpep" ) . '</a> ';
  echo '</p>';
}

if( !function_exists( 'wpep_admin_url_dashboard' ) ) {
  function wpep_admin_url_dashboard() {
    $url = admin_url( 'edit.php?post_type=' . WPEP_POST_TYPE_COURSE );

    return $url;
  }
}

if( !function_exists( 'wpep_admin_url' ) ) {
  function wpep_admin_url( $page, $params = [] ) {
    $url = admin_url( 'admin.php?page=' . $page );

    if( is_array( $params ) ) {
      foreach( $params as $key => $value )
        $url = add_query_arg( $key, $value, $url );

    } else {
      $url .= $params;
    }

    return $url;
  }
}

if( !function_exists( 'wpep_admin_url_statistics' ) ) {
  function wpep_admin_url_statistics( $params = [] ) {
    return apply_filters( 'wpep_administration_statistics_url', wpep_admin_url( 'wpep-statistics', $params ), $params );
  }
}

if( !function_exists( 'wpep_admin_url_statistics_content_type' ) ) {
  function wpep_admin_url_statistics_content_type( $content_type, $params = [] ) {
    return wpep_admin_url_statistics( [ 'tab' => 'content-types', 'content-type' => $content_type ] + $params );
  }
}

if( !function_exists( 'wpep_admin_url_settings' ) ) {
  function wpep_admin_url_settings( $params = [] ) {
    return apply_filters( 'wpep_administration_settings_url', wpep_admin_url( 'wpep-settings', $params ), $params );
  }
}

if( !function_exists( 'wpep_admin_url_platform_health' ) ) {
  function wpep_admin_url_platform_health( $params = [] ) {
    return apply_filters( 'wpep_administration_platform_health_url', wpep_admin_url( 'wpep-platform-health', $params ), $params );
  }
}

if( !function_exists( 'wpep_admin_url_discussions_management' ) ) {
  function wpep_admin_url_discussions_management( $params = [] ) {
    return apply_filters( 'wpep_administration_discussions_management_url', wpep_admin_url( 'wpep-discussion-management', $params ), $params );
  }
}

if( !function_exists( 'wpep_admin_url_learning_center' ) ) {
  function wpep_admin_url_learning_center( $params = [] ) {
    return apply_filters( 'wpep_administration_learning_center_url', wpep_admin_url( 'wpep-learning-center', $params ), $params );
  }
}

if( !function_exists( 'wpep_admin_url_addons' ) ) {
  function wpep_admin_url_addons( $params = [] ) {
    return apply_filters( 'wpep_administration_addons_url', wpep_admin_url( 'wpep-addons', $params ), $params );
  }
}

if( !function_exists( 'wpep_admin_register_menu' ) ) {

  /**
   * @param $page_title
   * @param $menu_title
   * @param $capability
   * @param $menu_slug
   * @param $function
   * @param int $top_level_position
   */
  function wpep_admin_register_menu( $page_title, $menu_title, $capability, $menu_slug, $function, $top_level_position = 4 ) {
    if( WPEP_ADMIN_BASE_MENU_SLUG === true ) {
      add_menu_page(
        $page_title,
        $menu_title,
        $capability,
        $menu_slug,
        $function,
        '',
        $top_level_position
      );
    } else {
      add_submenu_page(
        WPEP_ADMIN_BASE_MENU_SLUG,
        $page_title,
        $menu_title,
        $capability,
        $menu_slug,
        $function
      );
    }
  }

}