<?php

/**
 * @param $discussion_id
 * @param int $pagination_current_page
 * @param array $args
 * @param \WPEP_Discussion_Integration|false $integrationInstance
 * @return string
 */
function wpep_discussion_comments_template( $discussion_id, $pagination_current_page = 1, $args = [], $integrationInstance = false ) {
  return wpep_controller()->discussion->delivery->template_comments( $discussion_id, $pagination_current_page, $args, $integrationInstance );
}

/**
 * @param $current_page
 * @param int $available_pages
 * @return array|string
 */
function wpep_discussion_comment_pagination_template( $current_page, $available_pages = 1 ) {
  return '<div class="wpep-post-comments-list-pagination">' .
            paginate_links( [
              'format'       => '',
              'base'         => '?wpep-post-comments-page=%#%',
              'current'      => $current_page,
              'total'        => $available_pages,
              'add_fragment' => '',
              'prev_text'    => wpep_get_setting_translated( 'comments-navigation-previous' ),
              'next_text'    => wpep_get_setting_translated( 'comments-navigation-next' ),
            ] ) .
         '</div>';
}

function wpep_discussion_comment_content( $discussion_comment ) {
  $comment_content = $discussion_comment->content;

  if( user_can( $discussion_comment->user_id, WPEP_CAP_MANAGE_SETTINGS ) ) {
    $comment_content = do_shortcode( $comment_content );
  } else {
    add_filter( 'strip_shortcodes_tagnames', '_wpep_discussion_comment_content_stripped_shortcodes', 100 );

    $comment_content = strip_shortcodes( $comment_content );

    $comment_content = do_shortcode( $comment_content );

    remove_filter( 'strip_shortcodes_tagnames', '_wpep_discussion_comment_content_stripped_shortcodes', 100 );
  }

  $comment_content = wpep_format_content_autoembed( $comment_content );
  $comment_content = make_clickable( nl2br( $comment_content ) );

  return $comment_content;
}

function _wpep_discussion_comment_content_stripped_shortcodes( $tags_to_remove ) {
  $tags_to_remove = array_diff( $tags_to_remove, [ 'video', 'wpep_video', 'audio' ] );

  return $tags_to_remove;
}

function wpep_discussion_format_date( $to_format ) {
  $to_format = is_numeric( $to_format ) ? $to_format : strtotime( $to_format );

  $time = wpep_format_date( $to_format, get_option( 'time_format' ) );
  $date = wpep_format_date( $to_format, get_option( 'date_format' ) );

  return '<span class="wpep-post-comments-date">' . $date . '</span> <span class="wpep-post-comments-time">' . $time . '</span>';
}

function wpep_discussion_comment_form_editor( $post_id, $discussion_namespace ) {
  wpep_controller()->discussion->delivery->wpep_discussion_comment_form_editor( $post_id, $discussion_namespace );
}

function wpep_discussion_notifications_intervals() {
  return [
    300   => __( "Every 5 Minutes", "wpep" ),
    600   => __( "Every 10 Minutes", "wpep" ),
    900   => __( "Every 15 Minutes", "wpep" ),
    1800  => __( "Every 30 Minutes", "wpep" ),
    3600  => __( "Every Hour", "wpep" ),
    28800 => __( "Every 8 Hours", "wpep" ),
    57600 => __( "Every 16 Hours", "wpep" ),
    86400 => __( "Every Day", "wpep" )
  ];
}