<?php

use WPEP\Controller as WPEP_Controller;
use WPEP\ContentType as WPEP_ContentType;
use WPEP\ContentTaxonomy as WPEP_ContentTaxonomy;
use WPEP\Settings as WPEP_Settings;

/**
 * @return WPEP_Controller
 */
function wpep_controller() {
  return WPEP_Controller::instance();
}

/**
 * @return WPEP_ContentType
 */
function wpep_content_type() {
  return WPEP_ContentType::instance();
}

/**
 * @return WPEP_ContentType\Course
 */
function wpep_content_type_course_instance() {
  if( !isset( WPEP_ContentType::instance()->contentTypes[ WPEP_POST_TYPE_COURSE ] ) )
    exit( "wpep_content_type_course_instance() requires " . WPEP_POST_TYPE_COURSE . ' to be registered.' );

  return WPEP_ContentType::instance()->contentTypes[ WPEP_POST_TYPE_COURSE ];
}

/**
 * @return WPEP_ContentTaxonomy
 */
function wpep_content_taxonomy() {
  return WPEP_ContentTaxonomy::instance();
}

/**
 * @return WPEP_Settings
 */
function wpep_settings_instance() {
  return WPEP_Settings::instance();
}

/**
 * @param $option_name
 * @param string $option_default
 * @return string|array
 */
function wpep_get_setting( $option_name, $option_default = '' ) {
  return WPEP_Settings::instance()->get_option( $option_name, $option_default );
}

/**
 * @param $option_name
 * @return string
 */
function wpep_get_setting_translated( $option_name ) {
  return WPEP_Settings::instance()->get_option_translated( $option_name );
}

/**
 * @param $option_name
 * @return string
 */
function wpep_get_setting_flag( $option_name ) {
  return intval( WPEP_Settings::instance()->get_option( $option_name ) );
}

/**
 * @param $option_name
 * @param $option_value
 * @return bool
 */
function wpep_set_setting( $option_name, $option_value ) {
  return WPEP_Settings::instance()->update_settings( [
    $option_name => $option_value
  ] );
}

/**
 * @param $option_name
 * @return bool
 */
function wpep_delete_setting( $option_name ) {
  return WPEP_Settings::instance()->delete_setting( $option_name );
}


/**
 * @param $template_name
 * @param array $params
 * @return string
 */
function wpep_template_get( $template_name, $params = []) {
  return WPEP_Controller::instance()->template->get_template( $template_name, $params );
}

/**
 * @param $template_name
 * @param array $params
 */
function wpep_template_load( $template_name, $params = []) {
  WPEP_Controller::instance()->template->load_template( $template_name, $params );
}

/**
 * WPEP Proxy of WordPress get_header()
 */
function wpep_template_header() {
  WPEP_Controller::instance()->template->get_header();
}

/**
 * WPEP Proxy of WordPress get_footer()
 */
function wpep_template_footer() {
  WPEP_Controller::instance()->template->get_footer();
}

function wpep_has_sid_support() {
  return wpep_get_setting_flag( 'enable-sid-support' );
}

function wpep_has_embed_support() {
  return apply_filters( "wpep_has_embed_support_enabled", false );
}

function wpep_has_engagement_embed_support() {
  return wpep_has_embed_support() && wpep_get_setting_flag( 'user-content-embed-box-enabled' );
}

function wpep_get_placeholder_image() {
  return wpep_get_setting( 'wpep-placeholder-image' );
}

/**
 * @param $tracking_code
 * @param null|int $tracking_id
 * @return string
 */
function wpep_prepare_tracking_code( $tracking_code, $tracking_id = null ) {
  if( !is_string( $tracking_code ) || $tracking_code == '' )
    return '';

  $tracking_id = ( $tracking_id === null ? rand( 100, 1000 ) : $tracking_id );

  return '<div id="wpep-tracking-code-' . $tracking_id . '" class="wpep-tracking-code">' . $tracking_code . '</div>';
}

/**
 * @param array $data
 * @return string
 */
function wpep_breadcrumb( $data = [] ) {
  $response = '';

  if( empty( $data ) )
    return $response;

  $response .= '<div class="wpep-breadcrumb-container">';
  $response .=    '<ul class="wpep-breadcrumb">';

  foreach( $data as $link => $label ) {
    $response .= '<li>';

    if( $link == false )
      $response .=  '<span>' . $label . '</span>';
    else
      $response .=  '<a href="' . $link . '">' . $label . '</a>';
    $response .= '</li>';
  }

  $response .=    '</ul>';
  $response .=    '<div class="clear"></div>';
  $response .= '</div>';

  return $response;
}

function wpep_frontend_button( $link, $attributes = [] ) {
  $text           = isset($attributes['text']) ? $attributes['text'] : $link;
  $position       = isset($attributes['position']) ? $attributes['position'] : 'left';
  $force_download = isset($attributes['force_download']) ? $attributes['force_download'] : wpep_string_ends_with( $link, [ 'mp3', 'pdf', 'doc', 'mov', 'mp4' ] );
  $download_name  = isset($attributes['download_name'])  ? $attributes['download_name'] : basename( $link );

  $link = ( strpos( $link, "http" ) !== 0 ? 'http://' . $link : $link );

  $style = '';

  if( isset( $attributes['color'] ) )
    $style .= 'color : ' . $attributes['color'] . ' !important;';

  if( isset( $attributes['background_color'] ) )
    $style .= 'background-color : ' . $attributes['background_color'] . ' !important;';

  $response = '';

  $response .= '<p class="wpep-link-container wpep-link-container-' . $position . '">';
  $response .=    '<a href="' . $link . '" ';
  $response .=        ( $force_download ? ' download="' . $download_name . '" ' : '' );
  $response .=        ( $style !== '' ? ' style="' . $style . '" ' : '' );
  $response .=     '>';
  $response .=      $text;
  $response .=    '</a>';
  $response .= '</p>';

  return $response;
}

/**
 * @param int $progress_percent
 * @return string
 */
function wpep_progress_bar( $progress_percent = 0 ) {
  $response = '';
  $response .= '<div class="wpep-progress-bar">';
  $response .=    '<div class="wpep-progress-bar-inner" role="progressbar" aria-valuenow="' . $progress_percent . '" aria-valuemin="0" aria-valuemax="100" style="width: ' . $progress_percent . '%"></div>';
  $response .= '</div>';

  return $response;
}

function wpep_migration_backup( $version_number, $table_list = [] ) {
  if( empty( $table_list ) )
    return;

  global $wpdb;

  foreach( $table_list as $table_name ) {
    $destination = wpep_get_upload_dir_migration_backup() . '/' . $version_number . '-' . $table_name. '.json';

    $results = $wpdb->get_results( 'SELECT * FROM `' . $wpdb->prefix . $table_name . '`');

    if( !empty( $results ) ) {
      $information = json_encode( $results  );

      file_put_contents( $destination, $information );
    }
  }
}

/**
 * @param $current_step
 * @param $step_count
 * @return string
 */
function wpep_navigation_bar( $current_step, $step_count ) {
  $response = '';
  $response .= '<div class="wpep-navigation-bar-container">';
  $response .=    '<ul class="wpep-navigation-bar wpep-navigation-bar-items-' . $step_count . ' wpep-navigation-bar-current-item-' . $current_step . '">';

  for( $i = 1; $i <= $step_count; $i++ ) {
    $class  = 'wpep-navigation-bar-point';
    $class .= ( $i < $current_step ? ' wpep-navigation-bar-point-passed' : '' );
    $class .= ( $i == $current_step ? ' wpep-navigation-bar-point-current' : '' );

    $response .=    '<li class="' . $class . '">' . $i . '</li>';
  }

  $response .=    '</ul>';
  $response .= '</div>';

  return $response;
}

/**
 * @param string|bool $title
 * @param string $container_class
 * @param string $container_tag
 * @return string
 */
function wpep_spinner_loader( $title = false, $container_class = 'wpep-spinner-loader-container', $container_tag = 'div' ) {
  $content = '';

  $content  .= '<' . $container_tag . ' class="' . $container_class . '">';

  if( $title != false )
    $content .=       '<h2>' . $title . '</h2>';

  $content .=       '<span class="wpep-spinner-loader wpep-loading-container">' . __( "Loading...", "wpep" ) . '</span>';

  $content .= '</' . $container_tag . '>';

  return $content;
}

function wpep_is_rtl() {
  return is_rtl() || wpep_get_setting_flag( 'enforce-rtl-support' ) == true;
}

/**
 * @param $content
 * @param $id
 * @param bool $field_name
 * @param $editor_args
 * @return bool | true if it uses the wp_editor
 */
function wpep_editor( $content, $id, $field_name = false, $editor_args = [] ) {
  $editor_mode = WPEP_Settings::instance()->get_option( 'wpep-editor-setup' );
  $field_name  = ( $field_name == false ? $id : $field_name );

  if( $editor_mode == 'textarea' ) {
    echo '<textarea id="' . $id . '" 
                    class="wpep-editor wpep-editor-setup-textarea"
                    style="width:100%;max-width:960px;min-height:200px;"
                    name="' . $field_name . '">' . $content . '</textarea>';

    return false;
  }

  wpep_editor_wp( $content, $id, $field_name, $editor_args );
  
  return true;
}


/**
 * @param $content
 * @param $id
 * @param bool $field_name
 * @param $wp_editor_args
 * @return bool | true if it uses the wp_editor
 */
function wpep_editor_wp( $content, $id, $field_name = false, $wp_editor_args = [] ) {
  $field_name  = ( $field_name == false ? $id : $field_name );
  $editor_mode = WPEP_Settings::instance()->get_option( 'wpep-editor-setup' );

  if( !isset( $wp_editor_args['textarea_name'] ) )
    $wp_editor_args['textarea_name'] = $field_name;

  if( !isset( $wp_editor_args['textarea_rows'] ) )
    $wp_editor_args['textarea_rows'] = 10;

  if( $editor_mode == 'visual' ) {
    $wp_editor_args['quicktags']      = [];
    $wp_editor_args['default_editor'] = 'tinymce';
  }

  $wp_editor_args['editor_class'] = ( isset( $wp_editor_args['editor_class'] ) ? $wp_editor_args['editor_class'] . ' ' : '' ) . 'wpep-editor wpep-editor-wp-editor wpep-editor-setup-' . $editor_mode;

  wp_editor( $content, $id, $wp_editor_args );

  return true;
}

/**
 * @param $title
 * @param array $head_row_data
 * @param array $body_row_list
 * @param bool $display_download_button
 * @param null $pagination_current
 * @param null $pagination_total
 * @param string $table_class
 */
function wpep_admin_display_table(
  $title,
  $head_row_data = [],
  $body_row_list = [],
  $display_download_button = false,
  $pagination_current = null,
  $pagination_total   = null,
  $table_class        = 'wp-list-table widefat fixed striped wpep-statistics-table'
) {
  if( !empty( $title ) ) {
    echo '<div class="wpep-statistics-table-information-top' . ( is_array( $title ) ? ' wpep-with-nav-tab' : '' ) . '">';
    if( is_array( $title )  ) {
      echo '<div class="wpep-nav-links">';
      foreach( $title as $tab_link => $tab_name ) {
        if( $tab_link == false ) {
          echo '<a class="wpep-nav-link wpep-nav-link-active" href="#">' . $tab_name . '</a>';
          continue;
        }

        echo '<a class="wpep-nav-link" href="' . $tab_link . '">' . $tab_name . '</a>';
        continue;
      }
      echo '</div>';
    } else {
      echo '<h2 class="wpep-statistics-title">' . $title . '</h2>';
    }

    echo    '<div class="wpep-operation-container">';
    echo      ( $display_download_button && !empty( $body_row_list ) ? '<form method="POST"><input type="submit" class="wpep-button wpep-button-primary" name="wpep_download_statistics" value="' . __( "Download", "wpep" ) . '"></form>' : '' );
    echo    '</div>';
    echo '</div>';
  }

  if( empty( $body_row_list ) ) {
    echo '<div class="wpep-notification wpep-notification-warning">';
    echo    '<p>' . __( "There are no statistics available right now, please check back later.", "wpep" ) . '</p>';
    echo '</div>';

    return;
  }

  echo '<table class="' . $table_class . '">';
  echo    '<thead>';
  echo      '<tr>';

  foreach( $head_row_data as $current_entry_column ) {
    if( !is_array( $current_entry_column ) )
      $current_entry_column = [ 'value' => $current_entry_column ];

    echo    '<th class="' . ( isset( $current_entry_column['type'] ) ? $current_entry_column['type'] : 'normal' ) . '"' .
                ( isset( $current_entry_column['style'] ) ? 'style="' . $current_entry_column['style'] . '" ' : '' ) .
                ' >' .
                $current_entry_column['value'] .
            '</th>';
  }

  echo      '</tr>';
  echo    '<tbody>';


  foreach( $body_row_list as $current_entry_row ) {
    echo '<tr>';

    foreach( $head_row_data as $current_entry_index => $current_entry_title ) {
      if( !isset( $current_entry_row[$current_entry_index] ) ) {
        echo '<td>-</td>';

        return;
      }

      $current_entry_column = $current_entry_row[$current_entry_index];

      if( !is_array( $current_entry_column ) )
        $current_entry_column = [ 'value' => $current_entry_column ];

      echo    '<td class="' . ( isset( $current_entry_column['type'] ) ? $current_entry_column['type'] : 'normal' ) . '" ' .
                ( isset( $current_entry_column['style'] ) ? 'style="' . $current_entry_column['style'] . '" ' : '' ) .
                ' >' .
        $current_entry_column['value'] .
        '</td>';
    }

    echo '</tr>';
  }

  echo    '</tbody>';
  echo '</table>';

  if( $pagination_current === null || $pagination_total === null )
    return;

  echo '<div class="wpep-pagination-links">';

  echo paginate_links( [
    'base'    => wpep_admin_url_statistics() . '%_%',
    'format'  => '&pagination=%#%',
    'current' => $pagination_current,
    'total'   => $pagination_total
  ] );

  echo '</div>';
}

function wpep_get_external_page_embed_copy_button(
  $post_id,
  $permalink = false,
  $class     = 'button button-primary',
  $button_text        = '',
  $button_text_copied = ''
) {
  $permalink          = ( $permalink === false ? get_permalink( $post_id ) : $permalink );
  $button_text        = ( $button_text === ''  ? __( "Copy Embed Code", "wpep" ) : $button_text );
  $button_text_copied = ( $button_text_copied === '' ? __( "Copied !", "wpep" ) : $button_text_copied );

  return '<a class="' . $class . '"
               data-wpep-embed-id="wpep-embed-' . $post_id . '"
               data-wpep-embed-link-copy="' . add_query_arg( 'wpep-embed', 'true', $permalink ) . '"
               data-wpep-embed-link-copied="' . $button_text_copied . '">' . $button_text . '</a>';
}

/**
 * This is not by default used on the frontend, and the option Enforce Ordering is required to be checked.
 * @param $term_list
 * @return mixed
 */
function wpep_term_list_ordering( $term_list ) {
  if( !is_array( $term_list ) )
    return $term_list;

  if( empty( $term_list ) )
    return $term_list;

  foreach( $term_list as $term_index => $term )
    $term_list[$term_index]->wpep_order = intval( get_term_meta( $term->term_id, 'wpep_order', true ) );

  usort( $term_list, function( $a, $b ) {
    if ( $a->wpep_order == $b->wpep_order )
      return 0;

    return ( $a->wpep_order < $b->wpep_order ) ? -1 : 1;
  });

  return $term_list;
}

function wpep_get_theme_list_as_entity_layout_option() {
  $theme_list = WPEP_Controller::instance()->templateStyle->get_theme_list();

  $response = [
    ''  =>  sprintf( __( "Default Theme - %s", "wpep" ), $theme_list[ WPEP_Controller::instance()->templateStyle->theme ]['name'] )
  ];

  foreach( $theme_list as $theme_key => $theme_information )
    $response[ $theme_key ] = $theme_information['name'];

  return $response;
}

/**
 * @return array
 */
function wpep_get_custom_style_old_files() {
  $upload_dir = wpep_get_upload_dir();

  $files = array_diff( scandir( $upload_dir ) , [ '.', '..' ] );

  foreach( $files as $file_index => $file ) {
    if( strpos( $file, 'custom-style-' . WPEP_VERSION ) === 0 || strpos( $file, 'custom-style' ) !== 0 ) {
      unset( $files[$file_index] );
    } elseif( is_dir( $upload_dir . '/' . $file ) ) {
      unset( $files[$file_index] );
    }
  }

  if( !empty( $files ) )
    return $files;

  if( !is_dir( $upload_dir . '/custom-stylesheet' ) )
    return [];

  $files = array_diff( scandir( $upload_dir . '/custom-stylesheet' ) , [ '.', '..' ] );

  foreach( $files as $file_index => $file )
    if( strpos( $file, 'style-' . WPEP_VERSION ) === 0
      || strpos( $file, 'style' ) !== 0 )
      unset( $files[$file_index] );

  return $files;
}

/**
 * @param string $prefix
 * @return bool|string
 */
function wpep_get_custom_style_file( $prefix = 'baseurl' ) {
  if( !file_exists( wpep_get_upload_dir() . '/' . wpep_get_custom_style_file_name() ) ) {
    $file_exists = wpep_refresh_custom_style_file();

    if( !$file_exists )
      return false;
  }

  return wpep_get_upload_dir( $prefix ) . '/custom-stylesheet/' . wpep_get_custom_style_file_name();
}

/**
 * @param bool $wpep_theme
 * @return bool|string
 */
function wpep_refresh_custom_style_file( $wpep_theme = false ) {
  if( $wpep_theme === true ) {
    foreach( WPEP_Controller::instance()->templateStyle->get_theme_list() as $theme_key => $theme_information )
      wpep_refresh_custom_style_file( $theme_key );

    return true;
  }

  $wpep_style_dir = wpep_get_upload_dir() . '/custom-stylesheet';

  if( !is_dir( $wpep_style_dir ) )
    mkdir($wpep_style_dir, 0755, true);

  $file_path = $wpep_style_dir . '/' . wpep_get_custom_style_file_name( $wpep_theme );

  if( file_exists( $file_path ) )
    @unlink( $file_path );

  file_put_contents( $file_path, wpep_controller()->templateStyle->get_custom_style( $wpep_theme ) );

  if( !file_exists( $file_path ) )
    return false;

  return $file_path;
}

/**
 * @param string $index
 * @return string
 */
function wpep_get_upload_dir( $index = 'basedir' ) {
  $upload_dir = wp_upload_dir();

  return $upload_dir[ $index ] . '/wpep';
}

function wpep_get_upload_dir_migration_backup( $index = 'basedir' ) {
  $dir_path = wpep_get_upload_dir() . '/migration-backup';

  if( !is_dir( $dir_path ) ) {
    mkdir($dir_path, 0755, true);

    file_put_contents(
      $dir_path . '/.htaccess',
      "Options -Indexes" . "\n" .
      "deny from all" . "\n" .
      "<FilesMatch '\.(jpg|jpeg|png|gif|mp3|ogg)$'>" . "\n" .
      "Order Allow,Deny" . "\n" .
      "Allow from all" . "\n" .
      "</FilesMatch>"
    );
  }

  return wpep_get_upload_dir( $index ) . '/migration-backup';
}

/**
 * @param bool|string $wpep_theme
 * @return string
 */
function wpep_get_custom_style_file_name( $wpep_theme = false ) {
  $wpep_theme = ( $wpep_theme === false ? WPEP_Controller::instance()->templateStyle->theme : $wpep_theme );

  return 'style-' . WPEP_VERSION . '-' . $wpep_theme . '.css';
}

function wpep_custom_style_format_legacy_rules( $custom_css ) {
  $map = [
    '#course-grid-category-list'  => '.course-grid-category-list',
    '#course-grid'                => '.course-grid-container',
    '#course-primary'             => '.course-primary-container',
    '#course-content'             => '.course-primary-content',
    '#wpep-secondary'             => '.course-sidebar-content',
    '#course-grid-main-head'      => '.course-grid-container-header',
    '#course-grid-sub-head'       => '.course-grid-container-content',
    '#course-single'              => '.wpep-course-single',
  ];

  foreach( $map as $key => $value )
    $custom_css = str_replace( $key, $value, $custom_css );

  return $custom_css;
}

function wpep_flag_setting_field_options() {
  return [
    0         => __( 'Disabled', "wpep" ),
    1         => __( 'Enabled', "wpep" ),
  ];
}