<?php

use WPEP\Entity\Course as Entity_Course;

/**
 * @param $tag
 * @param array $options
 * @param string $label
 * @param bool $button
 * @return string
 */
function wpep_admin_shortcode_builder( $tag, $options = [], $label = 'Configure ShortCode', $button = true) {
  add_thickbox();

  return '<a href="' . wpep_admin_shortcode_builder_url( $tag, $options ) . '" 
             class="wpep-thickbox thickbox' . ( $button ? ' button button-primary' : '' ) . '">' .
            $label .
         '</a>';
}

/**
 * @param $tag
 * @param array $options
 * @return string
 */
function wpep_admin_shortcode_builder_url(
  $tag,
  $options = []
) {
  $url = add_query_arg( 'wpep-shortcode-builder', $tag, admin_url() );

  if( !empty( $options ) ) {
    $url = add_query_arg( 'wpep-sb-options', urlencode( json_encode( $options ) ), $url );
  }
  return $url;
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_index_options() {
  $category_list = _wpep_admin_shortcode_builder_category_select_field_list();

  $wpep_index_options = [
    'categories' => [
      'type'        => 'multiple_select',
      'options'     => $category_list,
      'description' => __( "Display only certain categories within this Index Page", "wpep" )
    ],
    'categories_per_row' => [
      'type'    => 'select',
      'options' => [ '' => __( "Default", "wpep" ) ] + wpep_content_categories_row_options()
    ],
    'all_categories_filter' => [
      'type'    => 'select',
      'options' => [
          ''  => sprintf( __( "Default : %s" ), wpep_get_setting_flag( 'index-page-enable-filter-all-categories' ) ? 'Enabled' : 'Disabled' )
        ] + wpep_flag_setting_field_options()
    ],
    'single_category_view' => [
      'type'    => 'select',
      'options' => [
        ''  => sprintf( __( "Default : Show %s category navigation" ), wpep_get_setting_flag( 'index-page-single-category-primary-view' ) ? 'primary' : 'secondary' ),
        0   => __( "Show secondary category navigation" ),
        1   => __( "Show primary category navigation" )
      ]
    ],
    'display_header' => [
      'type'    => 'select',
      'options' => [
        '1' => 'Yes',
        '0' => 'No'
      ]
    ],
    'order' => [
      'type'    => 'select',
      'options' => wpep_content_primary_order_options()
    ],
    'order_by' => [
      'type'    => 'select',
      'options' => wpep_content_primary_order_by_options()
    ],
  ];

  if( wpep_controller()->contentLibrary->is_integrated() ) {
    $wpep_index_options[ 'display_restriction' ] = [
      'type'    => 'select',
      'options' => [
        'none'    =>  __( "Standard", "wpep" ),
        'access'  =>  __( "Require Access", "wpep" )
      ],
      'description' => __( 'Selecting the "Require Access" option will make it mandatory for student to have access, this functionality may be provided already by your membership plugin.', "wpep" )
    ];
  }

  foreach( wpep_content_type()->libraryContentTypes as $libraryContentType )
    $wpep_index_options += $libraryContentType->get_grid_content_options();

  if( defined( "WPEP_BETA_TESTING" ) || defined( "WPEP_BETA_PAGINATION" ) ) {
    $wpep_index_options['pagination_per_page'] = [
      'type'        => 'text',
      'placeholder' => ( wpep_get_setting_flag( 'index-page-enable-pagination' ) ? sprintf( __( "Using Settings Rule : %s", "wpep" ), wpep_get_setting( 'index-page-entries-per-page' ) ) : __( "Pagination is disabled from settings" ) ),
      'description' => __( "Set to 0 to disable pagination, leave empty for default behaviour.", "wpep" )
    ];
  }

  $wpep_index_options[ 'layout' ] = [
    'type'    => 'select',
    'options' => [
      WPEP_INDEX_PAGE_LAYOUT_TOP_NAVIGATION   => __( "Top Navigation", "wpep" ),
      WPEP_INDEX_PAGE_LAYOUT_LEFT_NAVIGATION  => __( "Left Sidebar Navigation", "wpep" ),
      WPEP_INDEX_PAGE_LAYOUT_RIGHT_NAVIGATION => __( "Right Sidebar Navigation", "wpep" )
    ],
    'description' => __( "Advanced Option : Choose a different Index Page Layout.", "wpep" )
  ];

  $wpep_index_options[ 'layout-category' ] = [
    'type'    => 'select',
    'options' => [
      0 =>  __( "Top Level", "wpep" )
    ] + $category_list,
    'description' => __( "Advanced Option : Require all the Content to be part of a Category, but don't offer the filters from it.", "wpep" )
  ];

  return $wpep_index_options;
}

function _wpep_admin_shortcode_builder_category_select_field_list() {
  $category_list = wpep_utility_map_to_array_assoc(
    get_terms( [ 'taxonomy' => WPEP_TAXONOMY_CATEGORY, 'hide_empty' => false, 'parent' => 0 ] ),
    'term_id',
    'name'
  );

  foreach( $category_list as $category_id => $category_name ) {
    $sub_category_list = wpep_utility_map_to_array_assoc(
      get_terms( [ 'taxonomy'   => WPEP_TAXONOMY_CATEGORY, 'hide_empty' => false, 'parent' => $category_id ] ),
      'term_id',
      'name'
    );

    if( empty( $sub_category_list ) )
      continue;

    foreach( $sub_category_list as $k => $v )
      $sub_category_list[ $k ] = ' - ' . $v;

    $category_list[ $category_id ] = [
      'group_title'   => $category_name,
      'group_options' => [ $category_id => $category_name] + $sub_category_list
    ];
  }

  return $category_list;
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_course_grid_options() {
  return [
    'courses'         => [
      'type'    => 'multiple_select',
      'options' => [],
      'description' => __( "Leaving this empty will display all the courses.", "wpep" ),
      'attributes'    => [
        'data-wpep-select-search' => json_encode( [
          'action'    => 'wpep_admin_post_list_search',
          'post_type' => WPEP_POST_TYPE_COURSE
        ] )
      ]
    ],
    'container_class' => [
      'name'    => __('Container', "wpep"),
      'type'    => 'select',
      'options' => [
        ''                                    => 'Default',
        'wpep-3d-carousel-container-shadowed' => 'Box Shadow on Sides'
      ]
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_course_options() {
  $options = [];

  foreach( wpep_course_layout_types() as $course_type => $course_type_label ) {
    $group_options = wpep_get_post_list_assoc( WPEP_POST_TYPE_COURSE, 'ID', 'post_title', 'post_title', [
      [
        'key'   => 'course_type',
        'value' => $course_type,
      ]
    ] );

    if( empty( $group_options ) )
      continue;

    $options[] = [
      'group_title'   => $course_type_label,
      'group_options' => $group_options
    ]; ;
  }

  return [
    'id'          => [
      'name'    => __('Course', "wpep"),
      'type'    => 'searchable_select',
      'options' => $options
    ],
    'show_header' => [
      'type'        => 'checkbox',
      'is_checked'  => 1
    ],
    'read_only'   => [
      'name'    => __( 'Read Only', "wpep"),
      'type'    => 'select',
      'options' => [
        ''    => __( 'Course Default', "wpep" ),
        '0'   => __( 'No', "wpep" ),
        '1'   => __( 'Yes', "wpep" )
      ]
    ],
    'course_type' => [
      'name'    => __('Course Type', "wpep"),
      'type'    => 'select',
      'options' => [
          "" => __( "Option Selected in Course Settings", "wpep" )
        ] + wpep_course_layout_types()
    ]
  ];
}

function wpep_admin_shortcode_builder_wpep_assessment_options() {
  $options = [];

  foreach( wpep_utility_map_to_array_assoc(
             get_posts( [
               'post_type'   => WPEP_POST_TYPE_ASSESSMENT,
               'numberposts' => -1,
               'orderby'     => 'post_title',
               'order'       => 'ASC'
             ] ),
             'ID',
             'post_title'
           ) as $assessment_id => $assessment_title ) {
    $course_id = wpep_get_assessment_course_id( $assessment_id );

    if( !isset( $options[$course_id] ) ) {
      $options[$course_id] = [
        'group_title'   => get_the_title( $course_id ),
        'group_options' => []
      ];
    }

    $options[$course_id]['group_options'][ $assessment_id ] = $assessment_title;
  }

  return [
    'id'          => [
      'name'    => __('Assessment', "wpep"),
      'type'    => 'searchable_select',
      'options' => $options
    ]
  ];
}

function wpep_admin_shortcode_builder_wpep_course_preview_options( $builder_base_options = [] ) {
  if( !isset( $builder_base_options[ 'id' ] ) )
    return [];

  $course_map = Entity_Course::instance()->get_with_lessons( $builder_base_options['id'] );
  $available_lessons_options = [];

  foreach( $course_map as $section_information ) {
    $available_lessons_options[] = [
      'group_title'   => $section_information->title,
      'group_options' => wpep_utility_map_to_array_assoc( $section_information->lessons, 'id', 'title' )
    ]; ;
  }

  return [
    'show_header' => [
      'type'        => 'checkbox',
      'is_checked'  => 1
    ],
    'course_lessons' => [
      'name'    => __('Available Lessons', "wpep"),
      'type'    => 'multiple_select',
      'options' => $available_lessons_options,
      'desc'    => __( "Select each lesson that will be available under this Course Preview", "wpep" )
    ],
    'course_type'       => [
      'name'    => __('Course Type', "wpep"),
      'type'    => 'select',
      'options' => wpep_course_layout_types( [
          WPEP_COURSE_TYPE_EXECUTION_PLAN,
          WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED,
          WPEP_COURSE_TYPE_LEARN_MAP,
          WPEP_COURSE_TYPE_STANDARD,
          WPEP_COURSE_TYPE_VIDEO,
          WPEP_COURSE_TYPE_CHALLENGE
      ] )
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_link_options() {
  return [
    'content' => [
      'name'        => __( "Link" ),
      'type'        => 'shortcode_content'
    ],
    'text'  => [
      'name'    => __('Button Text', "wpep"),
      'type'    => 'text'
    ],
    'background_color' => [
      'name'    => __('Button Color', "wpep"),
      'type'    => 'color'
    ],
    'color' => [
      'name'    => __('Text Color', "wpep"),
      'type'    => 'color'
    ],
    'position' => [
      'name'    => __( "Alignment", "wpep" ),
      'type'    => 'select',
      'options' => [
        'left'    => __( "Left", "wpep" ),
        'center'  => __( "Center", "wpep" ),
        'right'   => __( "Right", "wpep" )
      ]
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_notification_options() {
  return [
    'content' => [
      'name'        => __( "Content" ),
      'type'        => 'shortcode_content'
    ],
    'type' => [
      'name'    => __('Type', "wpep"),
      'type'    => 'select',
      'options' => [
        "standard" => __( "Standard", "wpep" ),
        "info"     => __( "Informative", "wpep" ),
        "success"  => __( "Success", "wpep" ),
        "warning"  => __( "Warning", "wpep" ),
        "danger"   => __( "Danger", "wpep" ),
      ]
    ]
  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_course_ids_selection_options() {
  return [
    'content' => [
      'name'        => __( "Content" ),
      'type'        => 'shortcode_content',
      'default'     => 'You can add any content here.'
    ],
    'id'          => [
      'name'    => __('Course', "wpep"),
      'type'    => 'multiple_select',
      'options' => wpep_get_post_list_assoc( WPEP_POST_TYPE_COURSE )
    ]
  ];
}

function wpep_admin_shortcode_builder_wpep_search_advanced_options() {
  $category_list = wpep_utility_map_to_array_assoc(
    get_terms( [ 'taxonomy'   => 'course_category', 'hide_empty' => true, 'parent' => 0 ] ),
    'term_id',
    'name'
  );

  return [
    'post_type'         => [
      'type'        => 'multiple_select',
      'options'     => wpep_content_type()->library_post_type_list(),
      'description' => __( "Display only certain Post Types", "wpep" )
    ],
    'include_categories' => [
      'type'        => 'multiple_select',
      'options'     => $category_list,
      'description' => __( "Include only these Categories in this search", "wpep" )
    ],
    'exclude_categories' => [
      'type'        => 'multiple_select',
      'options'     => $category_list,
      'description' => __( "Exclude these Categories in this search", "wpep" )
    ],

  ];
}

/**
 * @return array
 */
function wpep_admin_shortcode_builder_wpep_lesson_content_options() {
  $options = [];
  $courses = wpep_get_post_list_assoc( WPEP_POST_TYPE_COURSE );

  $options[0] = __( "Please Select...", "wpep" );

  foreach( $courses as $course_id => $course_title ) {
    if( $course_title === '' )
      continue;

    $options[ $course_id ] = [
      'group_title'   => $course_title,
      'group_options' => Entity_Course::instance()->get_lesson_title_map( $course_id )
    ];
  }

  return [
    'lesson' => [
      'type'    => 'searchable_select',
      'options' => $options
    ]
  ];
}

function wpep_admin_shortcode_builder_wpep_profile_authentication_options() {
  $options = [
    'refresh'  => __( "Default - Refresh Page", "wpep" ),
    'redirect' => __( "Redirect", "wpep" )
  ];

  if( wpep_public_profiles_enabled() )
    $options[ 'redirect-to-public-profile' ] = __( "Redirect to Public Profile", "wpep" );

  return [
    'success_type' => [
      'type'        => 'select',
      'options'     => $options,
      'description' => __( "What happens after the user is logged in ? This also includes registration.", "wpep" ),
      'placeholder' => __( "Leave this empty to refresh the page instead", "wpep" )
    ],
    'url'          => [
      'type'        => 'text',
      'placeholder' => __( "The Page Link where you want to redirect", "wpep" )
    ],
    'lead_segment' => [
      'type'        => 'text',
      'placeholder' => 'This field is optional',
      'description' => __( "Assign a segment to this lead, this will be tracked in the Statistics Panel & Dashboard Report.", "wpep" )
    ],
  ];
}