<?php

/**
 * @return array
 */
function wpep_toolkit_list_map() {
  $wpep_search_section_items = [
    'wpep_search_content' => [
      'type'        => 'plain',
      'name'        => __( "Content", "wpep" ),
      'before'      => '[wpep_search]'
    ]
  ];

  foreach( wpep_content_type()->libraryContentTypes as $libraryContentType ) {
    $wpep_search_section_items[ 'wpep_search_' . $libraryContentType->get_post_type() ] = [
      'type'        => 'plain',
      'name'        => $libraryContentType->get_plural_name(),
      'before'      => '[wpep_search post_type="' . $libraryContentType->get_post_type() . '"]'
    ];
  }

  $wpep_search_section_items['wpep_search_advanced'] = [
    'type'        => 'builder',
    'name'        => __( "Advanced Settings", "wpep" ),
    'url'         => wpep_toolkit_list_item_url( 'wpep_search', [ 'wpep_admin_shortcode_builder_wpep_search_advanced_options' ] )
  ];

  $response = [
    'wpep_elements' => [
      'type'        => 'menu',
      'name'        => __( "Elements", "wpep" ),
      'items'       => [
        'wpep_notification' => [
          'type'        => 'builder',
          'name'        => __( "Notification", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( 'wpep_notification', [ 'wpep_admin_shortcode_builder_wpep_notification_options' ], 0 )
        ],
        'wpep_link'         => [
          'type'        => 'builder',
          'name'        => __( "Link Button", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( 'wpep_link', [ 'wpep_admin_shortcode_builder_wpep_link_options' ], 0 )
        ],
        'wpep_video' => [
          'type'        => 'plain',
          'name'        => __( "Video", "wpep" ),
          'before'      => '[wpep_video]',
          'placeholder' => __( "video-url-goes-here", "wpep" ),
          'after'       => '[/wpep_video]'
        ],
        'wpep_hero_section' => [
          'type'        => 'plain',
          'name'        => __( "Hero Section", "wpep" ),
          'before'      => '[wpep_hero_section]',
          'placeholder' => __( "Hero Section Content", "wpep" ),
          'after'       => '[/wpep_hero_section]'
        ]
      ]
    ],
    'wpep_index'             => [
      'type'        => 'builder',
      'name'        => __( "Index Page", "wpep" ),
      'url'         => wpep_toolkit_list_item_url( 'wpep_index', [ 'wpep_admin_shortcode_builder_wpep_index_options' ] )
    ],
    'wpep_course'            => [
      'type'        => 'builder',
      'name'        => __( "Course Embed", "wpep" ),
      'url'         => wpep_toolkit_list_item_url( 'wpep_course', [ 'wpep_admin_shortcode_builder_wpep_course_options' ] )
    ],
    'wpep_assessment'        => [
      'type'        => 'builder',
      'name'        => __( "Assessment Embed", "wpep" ),
      'url'         => wpep_toolkit_list_item_url( 'wpep_assessment', [ 'wpep_admin_shortcode_builder_wpep_assessment_options' ] )
    ],
    'wpep_course_grid' => [
      'type'        => 'builder',
      'name'        => __( "Course 3D Grid", "wpep" ),
      'url'         => wpep_toolkit_list_item_url( 'wpep_course_grid', [ 'wpep_admin_shortcode_builder_wpep_course_grid_options' ] )
    ],
    'wpep_profile_section' => [
      'type'        => 'builder',
      'name'        => __( "User Profile Page", "wpep" ),
      'url'         => wpep_toolkit_list_item_url( 'wpep_profile', [] )
    ],
    'wpep_search_section' => [
      'type'        => 'menu',
      'name'        => 'Search Bar',
      'items'       => $wpep_search_section_items
    ],
    'wpep_misc'     => [
      'type'  => 'menu',
      'name'  => __( "Miscellaneous", "wpep" ),
      'items' => [
        'wpep_center'            => [
          'type'        => 'plain',
          'name'        => __( "Content Center", "wpep" ),
          'before'      => '[wpep_center]',
          'placeholder' => __( "Content that will be centered", "wpep" ),
          'after'       => '[/wpep_center]'
        ],
        'wpep_profile_authentication'     => [
          'type'        => 'builder',
          'name'        => __( "Authentication Form", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( 'wpep_profile_authentication', [ 'wpep_admin_shortcode_builder_wpep_profile_authentication_options' ] )
        ],
        'wpep_lesson_content' => [
          'type'        => 'builder',
          'name'        => __( "Lesson Content", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( 'wpep_lesson_content', [ 'wpep_admin_shortcode_builder_wpep_lesson_content_options' ] )
        ],
        'wpep_user_is_logged_in' => [
          'type'        => 'plain',
          'name'        => __( "User Logged in", "wpep" ),
          'before'      => '[wpep_user_is_logged_in]',
          'placeholder' => __( "Content visible for logged in users", "wpep" ),
          'after'       => '[/wpep_user_is_logged_in]'
        ],
        'wpep_user_is_logged_out' => [
          'type'        => 'plain',
          'name'        => __( "User Logged Out", "wpep" ),
          'before'      => '[wpep_user_not_logged_in]',
          'placeholder' => __( "Content visible for logged out users", "wpep" ),
          'after'       => '[/wpep_user_not_logged_in]'
        ],
        'wpep_user_name' => [
          'type'        => 'plain',
          'name'        => __( "User Full Name", "wpep" ),
          'before'      => '[wpep_user_name]'
        ],
        'wpep_user_first_name' => [
          'type'        => 'plain',
          'name'        => __( "User First Name", "wpep" ),
          'before'      => '[wpep_user_first_name]'
        ],
        'wpep_course_is_completed' => [
          'type'        => 'builder',
          'name'        => __( "Course(s) Is Completed", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( 'wpep_course_is_completed', [ 'wpep_admin_shortcode_builder_wpep_course_ids_selection_options' ] )
        ],
        'wpep_course_not_completed' => [
          'type'        => 'builder',
          'name'        => __( "Course(s) Not Completed", "wpep" ),
          'url'         => wpep_toolkit_list_item_url( 'wpep_course_not_completed', [ 'wpep_admin_shortcode_builder_wpep_course_ids_selection_options' ] )
        ]
      ]
    ]
  ];

  if( wpep_public_profiles_enabled() )
    $response[ 'wpep_misc' ][ 'items' ][ 'wpep_public_profile_link' ] = [
      'type'        => 'plain',
      'name'        => __( "Public Profile Link", "wpep" ),
      'before'      => '[wpep_public_profile_link]',
      'placeholder' => __( "Link Name", "wpep" ),
      'after'       => '[/wpep_public_profile_link]'
    ];

  return apply_filters( 'wpep_toolkit_list_items_map', $response );
}

/**
 * @param $tag
 * @param array $options
 * @param int $layout_options
 * @return string
 */
function wpep_toolkit_list_item_url( $tag, $options = [], $layout_options = 1 ) {
  $url = wpep_admin_shortcode_builder_url( $tag, $options );
  $url = add_query_arg( 'wpep-sb-init', 0, $url );
  $url = add_query_arg( 'wpep-sb-button-text', urlencode( __( "Use ShortCode", "wpep" ) ), $url );
  $url = add_query_arg( 'wpep-sb-button-description', urlencode( __( "The shortcode will be displayed in the editor", "wpep" ) ) , $url );
  $url = add_query_arg( 'wpep-sb-layout-options', intval( $layout_options ) , $url );

  return $url;
}