<?php

function wpep_public_profiles_enabled() {
  return apply_filters( 'wpep_public_profiles_enabled', wpep_get_setting_flag( 'user-profile-public-active' ) );
}

/**
 * @param bool $user_id
 * @return array
 */
function wpep_content_user_profile_tabs( $user_id = false ) {
  return wpep_controller()->content->userProfile->delivery->get_tab_list( $user_id );
}

function wpep_user_profile_current_user_can_view( $user_id ) {
  if( $user_id == get_current_user_id() )
    return true;

  if( current_user_can( WPEP_CAP_MANAGE_MEMBERS ) || current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
    return true;

  if( wpep_get_setting_flag( 'enable-student-supervisor' ) && current_user_can( WPEP_CAP_SUPERVISE_STUDENTS ) ) {
    $supervisor_ids = get_user_meta( $user_id, WPEP_USER_META_SUPERVISOR_USER_ID, false );

    if( in_array( get_current_user_id(), $supervisor_ids ) )
      return true;
  }

  return false;
}

function wpep_user_profile_privacy_type( $user_id ) {
  if( !wpep_public_profiles_enabled() )
    return WPEP_USER_PROFILE_PRIVACY_TYPE_PRIVATE;

  $user_meta = get_user_meta( $user_id, WPEP_USER_META_PROFILE_PRIVACY_TYPE, true );

  if( $user_meta == '' )
    return wpep_get_setting( 'user-profile-privacy-setting-default' );

  return $user_meta;
}

function wpep_user_profile_is_public( $user_id ) {
  return wpep_user_profile_privacy_type( $user_id ) == WPEP_USER_PROFILE_PRIVACY_TYPE_PUBLIC;
}

/**
 * @param \WP_User|int $user
 * @return mixed
 */
function wpep_content_user_profile_format_name( $user ) {
  if( is_numeric( $user ) )
    $user = get_user_by( 'ID', intval( $user ) );
  else if( !isset( $user->first_name ) || !isset( $user->first_name ) )
    $user = ( isset( $user->ID ) ? get_user_by( 'ID', $user->ID ) : false );

  if( $user === false )
    return ( current_user_can( WPEP_CAP_MANAGE_MEMBERS ) ? 'This Account was deleted' : '' );


  $name = trim ( $user->first_name . ' ' . $user->last_name );

  return ( $name == '' ? $user->display_name : $name );
}

/**
 * @return array
 */
function wpep_user_profile_social_network_platform_list_active() {
  $active_social_networks = wpep_get_setting( 'user-profile-enabled-social-networks' );
  $response = wpep_user_profile_social_network_platform_list();

  foreach( $response as $key => $r )
    if( !in_array( $key, $active_social_networks ) )
      unset( $response[ $key ] );


  return $response;
}

function wpep_user_profile_public_link( $id = false ) {
  $user = ( is_object( $id ) ? $id : get_user_by( 'id', intval( $id === false ? get_current_user_id() : $id ) ) );

  if( $user === false )
    return false;

  return get_site_url() . '/' . wpep_controller()->content->userProfile->publicProfile->user_identifier . '/' . urlencode( $user->user_login ) . '/';
}

function wpep_user_profile_picture( $user_id, $size = 256 ) {
  return get_avatar( $user_id, $size, wpep_get_setting( 'user-profile-default-profile-picture' ) );
}

function wpep_user_profile_picture_url( $user_id, $size = 256 ) {
  $profile_picture = get_user_meta( $user_id, WPEP_USER_META_PROFILE_PICTURE, true );

  if( $profile_picture !== '' )
    return esc_url( $profile_picture );

  return get_avatar_url( $user_id, [
    'size'    => $size,
    'default' => wpep_get_setting( 'user-profile-default-profile-picture' )
  ] );
}

function wpep_user_profile_description_excerpt( $user_id ) {
  $excerpt = get_the_author_meta('description', $user_id );
  $excerpt = strip_shortcodes( $excerpt );
  $excerpt = strip_tags( $excerpt );
  $excerpt = wp_trim_words( $excerpt, 15 );

  return htmlentities( $excerpt, ENT_QUOTES, "UTF-8" );
}

/**
 * @return array
 */
function wpep_user_profile_social_network_platform_list() {
  return [
    'facebook'  => [
      'name'   => "Facebook",
      'icon'   => "wpep-social-media-icon-facebook",
      'type'   => 'link'
    ],
    'twitter'   => [
      'name'   => "Twitter",
      'icon'   => "wpep-social-media-icon-twitter",
      'type'   => 'link'
    ],
    'instagram'   => [
      'name'   => "Instagram",
      'icon'   => "wpep-social-media-icon-instagram",
      'type'   => 'link'
    ],
    'linkedin'    => [
      'name'   => "Linkedin",
      'icon'   => "wpep-social-media-icon-linkedin",
      'type'   => 'link'
    ],
    'youtube'     => [
      'name'   => "Youtube",
      'icon'   => "wpep-social-media-icon-youtube",
      'type'   => 'link'
    ],
    'pinterest'   => [
      'name'   => "Pinterest",
      'icon'   => "wpep-social-media-icon-pinterest",
      'type'   => 'link'
    ],
    'skype' => [
      'name'   => "Skype",
      'icon'   => "wpep-social-media-icon-skype"
    ],
    'snapchat' => [
      'name'   => "Snapchat",
      'icon'   => "wpep-social-media-icon-snapchat"
    ],
    'vk' => [
      'name'   => "VKontakte",
      'icon'   => "wpep-social-media-icon-vk",
      'type'   => 'link'
    ],
    'reddit' => [
      'name'   => "Reddit",
      'icon'   => "wpep-social-media-icon-reddit",
      'type'   => 'link'
    ],
    'deviant_art' => [
      'name'   => "Deviant Art",
      'icon'   => "wpep-social-media-icon-deviant-art",
      'type'   => 'link'
    ],
    'behance' => [
      'name'   => "Behance",
      'icon'   => "wpep-social-media-icon-behance",
      'type'   => 'link'
    ],
    'xing' => [
      'name'   => "Xing",
      'icon'   => "wpep-social-media-icon-xing",
      'type'   => 'link'
    ],
  ];
}