<?php

echo '<p>' . __( "WPEP Certifications AddOn is now part of native WPEP.", "wpep" ) . '</p>';

if( function_exists( 'is_plugin_active' ) && is_plugin_active( 'wpep-certifications/wpep-certifications.php' ) ) {

  wpep_set_setting('enable-certifications-support', 1);

  echo '<p>' . __("WPEP Certifications are now enabled.", "wpep") . '</p>';
}

$migrate_options = [
  'wpep_addon_certifications_download_box_placement' => 'certification-download-box-place',
  'wpep_addon_certifications_download_box_title'     => 'certification-download-box-title',
  'wpep_addon_certifications_download_box_button'    => 'certification-download-box-button',
  'wpep_addon_certifications_user_profile_title'     => 'certification-user-profile-title',
  'wpep_addon_certifications_user_profile_public_certificates' => 'certification-user-profile-public'
];

$options_migrated = 0;

foreach( $migrate_options as $addon_option => $wpep_option ) {
  if( wpep_get_setting( $wpep_option ) == get_option( $addon_option ) )
    continue;

  if( get_option( $addon_option ) == '' )
    continue;

  wpep_set_setting( $wpep_option, get_option( $addon_option ) );

  delete_option( $addon_option );
  $options_migrated++;
}


if( $options_migrated !== 0 ) {
  echo '<p>' . sprintf( __( "Certification Settings - OK ( %s migrated %s defaults )", "wpep" ), $options_migrated, ( count( $migrate_options ) - $options_migrated ) ). '</p>';
} else {
  echo '<p>' . sprintf( __( "Certification Settings - OK", "wpep" ), $options_migrated ). '</p>';
}

$post_meta_swap_map = [
  'wpep_addon_certification_status'     => 'wpep_certificate_status',
  'wpep_addon_certification_reset_date' => 'wpep_certificate_reset_date',
  'wpep_addon_certification_image'      => 'wpep_certificate_image',
  'wpep_addon_certification_content'    => 'wpep_certificate_content_map'
];


$courses_changed = 0; foreach( get_posts( [
  'post_type'   => WPEP_POST_TYPE_COURSE,
  'numberposts' => -1,
  'fields'      => 'ids',
  'meta_key'    => 'wpep_addon_certification_status'
] ) as $post_id ) {

  foreach( $post_meta_swap_map as $previous_post_meta => $new_post_meta ) {
    $post_meta_list = get_post_meta( $post_id, $previous_post_meta );

    if( empty( $post_meta_list ) )
      continue;

    foreach( $post_meta_list as $post_meta ) {
      if( is_string( $post_meta ) && wpep_string_ends_with( trim( $post_meta ), 'assets/default-certificate.jpg' ) )
        $post_meta = WPEP_BASE_URL_PATH . 'assets/img/default-certificate.jpg';

      add_post_meta( $post_id, $new_post_meta, $post_meta );
    }

    delete_post_meta( $post_id, $previous_post_meta );
  }
}

if( $courses_changed != 0 )
  echo '<p>' . sprintf( __( "Certification Settings Migrated for %s courses", "wpep" ), $courses_changed ). '</p>';

if( function_exists( 'is_plugin_active' ) && is_plugin_active( 'wpep-certifications/wpep-certifications.php' ) ) {
  deactivate_plugins( [ 'wpep-certifications/wpep-certifications.php' ] );

  echo '<p>' . __( "WPEP Certifications AddOn has been deactivated", "wpep" ) . '</p>';

  try {
    delete_plugins( [ 'wpep-certifications/wpep-certifications.php' ] );

    echo '<p>' . __( "WPEP Certifications AddOn has been deleted", "wpep" ) . '</p>';
  } catch ( \Exception $e ) {
    echo '<p>' . __( "WPEP Certifications AddOn has not been deleted, please remove it manually", "wpep" ) . '</p>';
  }
}
